Package: voluModel
Title: Modeling Species Distributions in Three Dimensions
Version: 0.2.5
Authors@R: 
    c(person(given = "Hannah L.",
             family = "Owens",
             role = c("aut", "cre", "cph"),
             email = "hannah.owens@gmail.com",
             comment = c(ORCID = "0000-0003-0071-1745")),
      person(given = "Emmaline",
             family = "Sheahan",
             role = "aut",
             email = "sheahane@ufl.edu",
             comment = c(ORCID = "0000-0003-3358-9758")),
      person(given = "Carsten", 
             family = "Rahbek",,,
             role="aut",
             comment = c(ORCID = "0000-0003-4585-0300")))
Maintainer: Hannah L. Owens <hannah.owens@gmail.com>
Description: Facilitates modeling species' ecological niches and 
  geographic distributions based on occurrences and environments that 
  have a vertical as well as horizontal component, and projecting models 
  into three-dimensional geographic space. Working in three dimensions is 
  useful in an aquatic context when the organisms one wishes to model can 
  be found across a wide range of depths in the water column. The package
  also contains functions to automatically generate marine training
  model training regions using machine learning, and interpolate and smooth
  patchily sampled environmental rasters using thin plate splines.
  Davis Rabosky AR, Cox CL, Rabosky DL, Title PO, Holmes IA, Feldman A, McGuire JA (2016) <doi:10.1038/ncomms11484>.
  Nychka D, Furrer R, Paige J, Sain S (2021) <doi:10.5065/D6W957CT>.
  Pateiro-Lopez B, Rodriguez-Casal A (2022) <https://CRAN.R-project.org/package=alphahull>.
License: GPL-3
URL: https://hannahlowens.github.io/voluModel/,
        https://github.com/hannahlowens/voluModel
BugReports: https://github.com/hannahlowens/voluModel/issues
Encoding: UTF-8
Depends: R (>= 4.0.0)
Imports: dplyr, fields, ggplot2, ggtext, grDevices, methods, metR,
        predicts, rangeBuilder (>= 2.0), rnaturalearth, terra,
        viridisLite, sf
Suggests: testthat (>= 3.0.0), nlme, knitr, covr, gridExtra, lattice,
        rJava, rmarkdown, rnaturalearthdata, tibble
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-23 15:52:43 UTC; HannahOwens
Author: Hannah L. Owens [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-0071-1745>),
  Emmaline Sheahan [aut] (ORCID: <https://orcid.org/0000-0003-3358-9758>),
  Carsten Rahbek [aut] (ORCID: <https://orcid.org/0000-0003-4585-0300>)
Repository: CRAN
Date/Publication: 2026-07-23 16:50:02 UTC

# Generated by roxygen2: do not edit by hand

export(MESS3D)
export(areColors)
export(blendColor)
export(bottomRaster)
export(centerPointRasterTemplate)
export(cleanDepth)
export(columnParse)
export(depthMatch)
export(diversityStack)
export(downsample)
export(env_stack_transform)
export(interpolateRaster)
export(mSampling2D)
export(mSampling3D)
export(marineBackground)
export(maxent_3D)
export(oneRasterPlot)
export(partition_3D)
export(plotLayers)
export(pointCompMap)
export(pointMap)
export(rasterComp)
export(smoothRaster)
export(testIntersection)
export(threshold_3D)
export(transectPlot)
export(transpColor)
export(verticalSample)
export(xyzSample)
import(ggplot2)
import(predicts)
import(sf)
import(terra)
importFrom(dplyr,"%>%")
importFrom(dplyr,anti_join)
importFrom(dplyr,inner_join)
importFrom(dplyr,mutate)
importFrom(fields,Tps)
importFrom(fields,fastTps)
importFrom(ggplot2,ggplot)
importFrom(ggtext,element_markdown)
importFrom(grDevices,col2rgb)
importFrom(grDevices,recordPlot)
importFrom(grDevices,rgb)
importFrom(graphics,legend)
importFrom(metR,scale_x_latitude)
importFrom(methods,"slot<-")
importFrom(methods,as)
importFrom(methods,is)
importFrom(predicts,mess)
importFrom(rangeBuilder,getDynamicAlphaHull)
importFrom(rnaturalearth,ne_countries)
importFrom(stats,complete.cases)
importFrom(terra,extract)
importFrom(terra,rast)
importFrom(terra,rasterize)
importFrom(terra,resample)
importFrom(viridisLite,viridis)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)

## ----packages, message=FALSE, warning=FALSE-----------------------------------
library(voluModel) # Because of course
library(ggplot2) # For fancy plotting
library(rangeBuilder) # To compare marineBackground to getDynamicAlphaHull
library(dplyr) # To filter data
library(terra) # Now being transitioned in
library(sf) # Now being transitioned in

## ----occurrence dataset, message=FALSE, warning = FALSE-----------------------
# Get points
occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv", 
                             package='voluModel'))
occurrences <- occs %>% 
  dplyr::select(decimalLongitude, decimalLatitude, depth) %>%
  dplyr::distinct() %>% 
  dplyr::filter(depth %in% 1:2000)

## ----environmental data loading, eval=T, asis=T, message = FALSE, warning=FALSE----
# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Looking at the dataset
as.data.frame(temperature[1:5,1:10])

## ----temperature to compatible RasterBrick------------------------------------
# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal

rm(tempTerVal)

## ----column interpretations, message=TRUE, warning=TRUE-----------------------
occsTest <- occurrences[19:24,]
xyzSample(occs = occsTest, envBrick = temperature)

colnames(occsTest) <- c("x", "y", "z")
xyzSample(occs = occsTest, envBrick = temperature)

rm(occsTest)

## ----downsample to voxel, eval=TRUE, warning=FALSE, message=FALSE-------------
# Gets the layer index for each occurrence by matching to depth
layerNames <- as.numeric(names(temperature))
occurrences$index <- unlist(lapply(occurrences$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occurrences$index)
downsampledOccs <- data.frame()
for(i in indices){
  tempPoints <- occurrences[occurrences$index==i,]
  tempPoints <- downsample(tempPoints, temperature[[1]])
  tempPoints$depth <- rep(layerNames[[i]], times = nrow(tempPoints))
  downsampledOccs <- rbind(downsampledOccs, tempPoints)
}

occurrences <- downsampledOccs
head(occurrences)

print(paste0("Original number of points: ", nrow(occs), "; number of downsampled occs: ", nrow(occurrences)))

## ----plot downsample, warning=FALSE-------------------------------------------
land <- rnaturalearth::ne_countries(scale = "small", returnclass = "sf")[1]

pointCompMap(occs1 = occs, occs2 = occurrences, 
             occs1Name = "original", occs2Name = "cleaned", 
             spName = "Steindachneria argentea", 
             land = land, verbose = FALSE)

## ----temperature extraction---------------------------------------------------
# Extraction
occurrences$temperature <- xyzSample(occs = occurrences, envBrick = temperature)

# Add "response" column for modeling
occurrences$response <- rep(1, times = nrow(occurrences))
occurrences <- occurrences[complete.cases(occurrences),]

head(occurrences)

## ----alpha hull demonstration, message=FALSE, warning=FALSE, eval=T-----------
trainingRegion <- marineBackground(occurrences, 
                                   fraction = 1, 
                                   partCount = 1, 
                                   buff = 1000000,
                                   clipToOcean = F)
plot(trainingRegion, border = F, col = "gray",
     main = "100% Points,\nMax 1 Polygon Permitted, 100 km Buffer",
     axes = T)
plot(land, col = "black", add = T)
points(occurrences[,c("decimalLongitude", "decimalLatitude")], 
       pch = 20, col = "red", cex = 1.5)

## ----clipToOcean demo, message=FALSE, warning=FALSE, eval=T-------------------
trainingRegion <- marineBackground(occurrences, 
                                   buff = 1000000, 
                                   clipToOcean = T)
plot(trainingRegion, border = F, col = "gray",
     main = "100 km Buffer,\nClipped to Occupied Polygon",
     axes = T)
plot(land, col = "black", add = T)
points(occurrences[,c("decimalLongitude", "decimalLatitude")], 
       pch = 20, col = "red", cex = 1.5)

## ----meridian wrap demo, warning=FALSE, message=FALSE, eval=T-----------------
# Fictional example occurrences
pacificOccs <- occurrences
pacificOccs$decimalLongitude <- pacificOccs$decimalLongitude - 100
for (i in 1:length(pacificOccs$decimalLongitude)){
  if (pacificOccs$decimalLongitude[[i]] < -180){
    pacificOccs$decimalLongitude[[i]] <- pacificOccs$decimalLongitude[[i]] + 360
  }
}

# marine Background
pacificTrainingRegion <- marineBackground(pacificOccs, 
                                          fraction = 0.95, partCount = 3,
                                          buff = 1000000,
                                          clipToOcean = T)
plot(pacificTrainingRegion, border = F, col = "gray",
     main = "marineBackground\nAntimeridian Wrap",
     axes = T)
plot(land, col = "black", add = T)
points(pacificOccs[,c("decimalLongitude", "decimalLatitude")], 
       pch = 20, col = "red", cex = 1.5)

## ----training points----------------------------------------------------------
# Background
backgroundVals <- mSampling3D(occs = occurrences, 
                              envBrick = temperature, 
                              mShp = trainingRegion, 
                              depthLimit = c(50, 1500))
backgroundVals$temperature <- xyzSample(occs = backgroundVals, temperature)

#Remove incomplete cases
backgroundVals <- backgroundVals[complete.cases(backgroundVals),]

## -----------------------------------------------------------------------------
# Add "response" column for modeling
backgroundVals$response <- rep(0, times = nrow(backgroundVals))

head(backgroundVals)

## ----glm data prep part 2-----------------------------------------------------
# Sample background points weighted by distance from mean of occurrence temperatures
meanTemp <- mean(occurrences[,c("temperature")])
backgroundVals$distance <- abs(backgroundVals[,"temperature"] - meanTemp)
backgroundVals$sampleWeight <- (backgroundVals$distance - 
                                  min(backgroundVals$distance))/(max(backgroundVals$distance) -
                                                                  min(backgroundVals$distance))
sampleForAbsence <- sample(x = rownames(backgroundVals), 
                           size = nrow(occurrences) * 100, 
                           prob = backgroundVals$backgroundVals)
backgroundVals <- backgroundVals[match(sampleForAbsence, 
                                       rownames(backgroundVals)),]

backgroundVals$response <- as.factor(backgroundVals$response)

# Unite datasets and see how things look
dataForModeling <- rbind(occurrences, backgroundVals[,colnames(occurrences)])
ggplot(dataForModeling, aes(x = temperature, fill = response, color = response)) +
  geom_density(alpha = .6) +
  scale_color_manual(values=c("#999999", "#E69F00"), 
                     labels = c("Pseudoabsence", "Presence")) +
  scale_fill_manual(values=c("#999999", "#E69F00"), 
                    labels = c("Pseudoabsence", "Presence")) +
  labs(title="Temperature Sampling Density,\nOccurrences vs. Pseudoabsences",
       x="Temperature (C)", y = "Density")+
  theme_classic()

## ----cleanup temporary directory----------------------------------------------
unlink(td, recursive = T)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)

## ----packages, message=FALSE, warning=FALSE-----------------------------------
library(voluModel) # Because of course
library(dplyr) # To filter data
library(ggplot2) # For fancy plotting
library(terra) # Now being transitioned in

## ----show points, warning=FALSE, echo=TRUE, message=FALSE, eval=TRUE----------
occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv", 
                             package='voluModel'))

# Filter points
occurrences <- occs %>% 
  dplyr::select(decimalLongitude, decimalLatitude, depth) %>%
  dplyr::distinct() %>% 
  filter(depth %in% 1:2000)

head(occurrences)
land <- rnaturalearth::ne_countries(scale = "small", returnclass = "sf")[1]
pointMap(occs = occurrences, ptCol = "orange", landCol = "black",
             spName = "Steindachneria argentea", ptSize = 3,
             land = land)

## ----loading temperature data, message=FALSE, warning=FALSE, include=TRUE-----
# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Creating a bottom raster
temperatureBottom <- bottomRaster(temperature)

# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- terra::rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal
rm(tempTerVal)

## ----plotting temperature, eval = TRUE----------------------------------------
# How do these files look?
par(mfrow=c(1,2))
p1 <- oneRasterPlot(temperature[[1]], land = land, landCol = "black", 
              title= "Surface Temperature (C)")

p2 <- oneRasterPlot(temperatureBottom,land = land, landCol = "black", 
              title = "Bottom Temperature (C)")

## ----loading oxygen data, message=FALSE, warning=FALSE, include=TRUE----------
# Oxygen processing, pre-baked to save time
oxygenSmooth <- rast(system.file("extdata/oxygenSmooth.tif", 
                                 package='voluModel'))
oxygenBottom <- rast(system.file("extdata/oxygenBottom.tif", 
                                 package="voluModel"))
names(oxygenSmooth) <- names(temperature)

## ----oxygen plotting, echo=TRUE, eval = TRUE, message=FALSE, warning=FALSE----
par(mfrow=c(1,2))
p3 <- oneRasterPlot(oxygenSmooth[[1]], land = land, landCol = "black", 
              title= "Surface Apparent Oxygen Utilization (µmol/kg),\ninterpolated and smoothed")
p4 <- oneRasterPlot(oxygenBottom, land = land, landCol = "black",
     title = "Bottom Apparent Oxygen Utilization (µmol/kg),\ninterpolated and smoothed")

## ----occurrence and depth matchup---------------------------------------------
# Gets the layer index for each occurrence by matching to depth
layerNames <- as.numeric(names(temperature))
occurrences$index <- unlist(lapply(occurrences$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occurrences$index)

# Downsamples occurrences in each depth layer
downsampledOccs <- data.frame()
for(i in indices){
  tempPoints <- occurrences[occurrences$index==i,]
  tempPoints <- downsample(tempPoints, temperature[[1]], verbose = FALSE)
  tempPoints$depth <- rep(layerNames[[i]], times = nrow(tempPoints))
  downsampledOccs <- rbind(downsampledOccs, tempPoints)
}
occurrences <- downsampledOccs
occurrences <- occurrences[,c("decimalLatitude", "decimalLongitude", "depth")]

rm(indices, layerNames, tempPoints, i, downsampledOccs, occs)

## ----data extraction, echo=FALSE, message=FALSE, warning=FALSE----------------
# Extract temperature
threeDimTemp <- xyzSample(occs = occurrences, temperature)
threeDimTemp <- cbind(rep("X, Y,\nZ", length(threeDimTemp)), threeDimTemp)
surfTemp <- extract(x = temperature[[1]], 
                    occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
surfTemp <- cbind(rep("X, Y,\nSurface", length(surfTemp)), surfTemp)
bottomTemp <- extract(x = temperatureBottom, 
                      occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
bottomTemp <- cbind(rep("X, Y,\nBottom", length(bottomTemp)), bottomTemp)

# Extract AOU
threeDimOxy <- xyzSample(occs = occurrences, oxygenSmooth)
threeDimOxy <- cbind(rep("X, Y,\nZ", length(threeDimOxy)), threeDimOxy)
surfOxy <- extract(x = oxygenSmooth[[1]], 
                   occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
surfOxy <- cbind(rep("X, Y,\nSurface", length(surfOxy)), surfOxy)
bottomOxy <- extract(x = oxygenBottom, 
                     occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
bottomOxy <- cbind(rep("X, Y,\nBottom", length(bottomOxy)), bottomOxy)

## ----temperature violin plots, echo=FALSE, message=FALSE, warning=FALSE-------
# Collation
allTemp <- rbind(threeDimTemp, surfTemp, bottomTemp)
colnames(allTemp) <- c("Group", "Temperature")
allTemp <- as.data.frame(allTemp)
allTemp <- allTemp[complete.cases(allTemp),]
allTemp$Temperature <- as.numeric(allTemp$Temperature)

allOxy <- rbind(threeDimOxy, surfOxy, bottomOxy)
colnames(allOxy) <- c("Group", "Oxygen")
allOxy <- as.data.frame(allOxy)
allOxy <- allOxy[complete.cases(allOxy),]
allOxy$Oxygen <- as.numeric(allOxy$Oxygen)

# Plotting
groups <- c("X, Y,\nSurface", "X, Y,\nBottom", "X, Y,\nZ")
tempPlot <- ggplot(allTemp, aes(x=Group, y=Temperature)) + 
  geom_boxplot(fill="#b2182b", notch = FALSE) +
  theme_classic(base_size = 15) +
  theme(axis.title.x = element_blank(), 
        text = element_text(family = "Arial"), 
        axis.text = element_text(size = rel(1.1))) +
  scale_x_discrete(limits = groups) +
  ylab("Temperature (C)")

oxyPlot <- ggplot(allOxy, aes(x=Group, y=Oxygen)) + 
  geom_boxplot(fill="#2166ac", notch = FALSE) +
  theme_classic(base_size = 15) +
  theme(axis.title.x = element_blank(), 
        text = element_text(family = "Arial"), 
        axis.text = element_text(size = rel(1.1))) +
  scale_x_discrete(limits = groups) +
  ylab("Apparent Oxygen Utilization (µmol/kg)")

gridExtra::grid.arrange(tempPlot, oxyPlot, nrow = 1)

## ----study region, message=FALSE, warning=FALSE-------------------------------
studyRegion <- marineBackground(occurrences,
                                buff = 1000000,
                                clipToOcean = TRUE,
                                alpha = 1,
                                partCount = 1)
landVect <- vect(land)
landVect <- terra::project(landVect, y = studyRegion)
plot(studyRegion, border = F, col = "gray",
     main = "Points and Background Sampling",
     axes = T)
plot(landVect, col = "black", add = T)
points(occurrences[,c("decimalLongitude", "decimalLatitude")], 
       pch = 20, col = "red", cex = 1.5)

## ----sampling-----------------------------------------------------------------
# Surface Presences
oxyVals <- extract(x = oxygenSmooth[[1]], occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
tempVals <- extract(x = temperature[[1]], occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
vals <- cbind(occurrences, oxyVals, tempVals)
colnames(vals) <- c("decimalLongitude", "decimalLatitude", "depth", "AOU", "Temperature")
vals <- vals[complete.cases(vals),]
row.names(vals) <- NULL
occsWdataSurface <- vals

# Bottom Presences
oxyVals <- extract(x = oxygenBottom, occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
tempVals <- extract(x = temperatureBottom, occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
vals <- cbind(occurrences, oxyVals, tempVals)
colnames(vals) <- c("decimalLongitude", "decimalLatitude", "depth", "AOU", "Temperature")
vals <- vals[complete.cases(vals),]
row.names(vals) <- NULL
occsWdataBottom <- vals

# 3D Presences
oxyVals <- xyzSample(occs = occurrences, oxygenSmooth)
tempVals <- xyzSample(occs = occurrences, temperature)
vals <- cbind(occurrences, oxyVals, tempVals)
colnames(vals) <- c("decimalLongitude", "decimalLatitude", "depth", "AOU", "Temperature")
vals <- vals[complete.cases(vals),]
row.names(vals) <- NULL
occsWdata3D <- vals

rm(oxyVals, tempVals, vals)

## ----generate 2D envelope niche models----------------------------------------
# Get limits, surface
tempLims <- quantile(occsWdataSurface$Temperature,c(0, 1))
aouLims <- quantile(occsWdataSurface$AOU,c(0, 1))

# Reclassify environmental bricks to presence/absence, surface
temperaturePresence <- classify(temperature[[1]], 
                                  rcl = matrix(c(-Inf,tempLims[[1]],0, 
                                                 tempLims[[1]], tempLims[[2]], 1,
                                                 tempLims[[2]], Inf, 0), 
                                               ncol = 3, byrow = TRUE))
AOUpresence <- classify(oxygenSmooth[[1]], 
                          rcl = matrix(c(-Inf, aouLims[[1]],0,
                                  aouLims[[1]], aouLims[[2]], 1,
                                  aouLims[[2]], Inf, 0), ncol = 3, byrow = TRUE))

# Put it all together, surface
envelopeModelSurface <- temperaturePresence * AOUpresence
envelopeModelSurface <- mask(crop(envelopeModelSurface, studyRegion), 
                             mask = studyRegion)

# Get limits, bottom
tempLims <- quantile(occsWdataBottom$Temperature,c(0, 1))
aouLims <- quantile(occsWdataBottom$AOU,c(0, 1))

# Reclassify environmental bricks to presence/absence, bottom
temperaturePresence <- classify(temperatureBottom, 
                                  rcl = matrix(c(-Inf,tempLims[[1]],0,
                                          tempLims[[1]], tempLims[[2]], 1,
                                          tempLims[[2]], Inf, 0), byrow = TRUE, ncol=3))
AOUpresence <- classify(oxygenBottom, 
                          rcl = matrix(c(-Inf, aouLims[[1]],0,
                                  aouLims[[1]], aouLims[[2]], 1,
                                  aouLims[[2]], Inf, 0), byrow = TRUE, ncol = 3))

# Put it all together, bottom
envelopeModelBottom <- temperaturePresence * AOUpresence
envelopeModelBottom <- mask(crop(envelopeModelBottom, studyRegion), mask = studyRegion)

## ----comparing 2D maps--------------------------------------------------------
rasterComp(rast1 = envelopeModelSurface, rast2 = envelopeModelBottom, 
           rast1Name = "Surface", rast2Name = "Bottom", 
           land = land, landCol = "black", 
           title = "Comparison between surface and bottom envelope models")

## ----generate 3D envelope model-----------------------------------------------
# Get limits
tempLims <- quantile(occsWdata3D$Temperature,c(0, 1))
aouLims <- quantile(occsWdata3D$AOU,c(0, 1))

# Reclassify environmental bricks to presence/absence
temperaturePresence <- classify(temperature, 
                                  rcl = matrix(c(-Inf,tempLims[[1]],0,
                                                 tempLims[[1]], tempLims[[2]], 1,
                                                 tempLims[[2]], Inf, 0), 
                                               byrow = TRUE, ncol = 3))
AOUpresence <- classify(oxygenSmooth, 
                          rcl = matrix(c(-Inf, aouLims[[1]],0,
                                         aouLims[[1]], aouLims[[2]], 1,
                                         aouLims[[2]], Inf, 0), 
                                       byrow = TRUE, ncol = 3))

# Put it all together
envelopeModel3D <- temperaturePresence * AOUpresence
envelopeModel3D <- mask(crop(envelopeModel3D, studyRegion), 
                        mask = studyRegion)
names(envelopeModel3D) <- names(temperature)

## ----plot 3D envelope model---------------------------------------------------
# Get indices of model-relevant layers
layerNames <- as.numeric(names(envelopeModel3D))
occurrences$index <- unlist(lapply(occurrences$depth, 
                                   FUN = function(x) 
                                     which.min(abs(layerNames - x))))
indices <- sort(unique(occurrences$index))

plotLayers(envelopeModel3D[[min(indices):max(indices)]],
           title = "Envelope Model of Luminous Hake,\n 20 to 700m",
           land = land)

## ----cleanup temporary directory----------------------------------------------
unlink(td, recursive = T)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)

## ----load packages, message=FALSE, warning=FALSE------------------------------
library(voluModel) # Since this is the package this vignette is about.
library(tibble) # For data organization
library(ggplot2) # For supplementary visualization
library(fields) # For raster interpolation
library(terra) # Now being transitioned in

## ----environmental data loading temperature, eval=T, message=FALSE, warning=FALSE, asis=T----
# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Looking at the dataset
as.data.frame(temperature[1:5,1:10])

# Plotting the dataset
layout(matrix(c(1, 2), ncol=2, byrow=TRUE), widths=c(4, 1))
land <- rnaturalearth::ne_countries(scale = "small", 
                                    returnclass = "sf")[1]
temperatureForPlot <- temperature
crs(temperatureForPlot) <- crs(land) 
ext <- ext(temperatureForPlot)
plot(temperatureForPlot, main = "Distribution of voluModel Subset\nof WOA Temperature 2018",
     pch = 20, col = "red", xlim = ext[1:2], ylim = ext[3:4], cex = .6, mar = c(2,2,3,2))
plot(land, col = "black", add = T)

# What does the WOA depth structure look like?
depths <- names(temperatureForPlot)
depths <- as.numeric(gsub(depths[-1], pattern = "[d,M]", replacement = ""))
plot(0, xlim = c(0,1), ylim = c(0-max(depths), 0), axes=FALSE, type = "n", xlab = "", ylab = "Depth Intervals (m)")
axis(2, at = 0-depths, labels = depths)

## ----temperature processing, eval=TRUE----------------------------------------
# Creating a bottom raster
temperatureBottom <- bottomRaster(temperature)

# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal
rm(tempTerVal)

# How do these files look?
par(mfrow=c(1,2))
p1 <- oneRasterPlot(temperature[[1]], land = land, landCol = "black", 
              title= "Surface Temperature (C)")

p2 <- oneRasterPlot(temperatureBottom,land = land, landCol = "black", 
              title = "Bottom Temperature (C)")

## ----environmental data loading oxygen, eval=T, message=FALSE, warning=FALSE, asis=T----
td <- tempdir()
unzip(system.file("extdata/woa18_all_A00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/oxygen"), junkpaths = T)

oxygen <- vect(paste0(td, "/oxygen/woa18_all_A00mn01_cropped.shp")) 

plot(oxygen, main = "Distribution of voluModel subset of WOA AOU 2018",
     pch = 20, col = "red", xlim = ext[1:2], ylim = ext[3:4], cex = .6)
plot(land, col = "black", add = T)

## ----interpolate oxygen, warning=FALSE, eval = T------------------------------
# Creating a SpatRaster vector for the first 10 depth layers
oxygen <- oxygen[,1:10] # Remove this line if you want to process the whole file
oxygen <- rasterize(x = oxygen, y = template,
                   field = names(oxygen)) #Uses same raster template as temperature

for (i in 1:nlyr(oxygen)){ 
  oxygen[[i]] <- interpolateRaster(oxygen[[i]], lon.lat = T, fast = T, aRange = 30) #Thin plate spline interpolation
  oxygen[[i]] <- crop(mask(x = oxygen[[i]], 
                           mask = temperature[[i]]), 
                      temperature[[i]])
}

# Change names to match tempT
names(oxygen) <- envtNames[1:nlyr(oxygen)]

## ----smoothing oxygen, eval=TRUE----------------------------------------------
# Smoothing tempO and saving
oxygenSmooth <- oxygen
for (i in 1:nlyr(oxygenSmooth)){
  oxygenSmooth[[i]] <- smoothRaster(oxygenSmooth[[i]], lon.lat = T) #Thin plate spline interpolation
  oxygenSmooth[[i]] <- crop(mask(x = oxygenSmooth[[i]], mask = temperature[[i]]), temperature[[i]])
}

# Change names to match tempT and save
names(oxygenSmooth) <- names(oxygen)
oxygenSmooth <- oxygenSmooth

par(mfrow=c(1,2))
p3 <- oneRasterPlot(oxygen[[1]], land = land, landCol = "black", 
              title= "Surface Apparent Oxygen Utilization\n(µmol/kg), interpolated")
p4 <- oneRasterPlot(oxygenSmooth[[1]], land = land, landCol = "black",
     title = "Surface Apparent Oxygen Utilization\n(µmol/kg), interpolated and smoothed")

## ----cleanup temporary directory----------------------------------------------
unlink(td, recursive = T)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)

## ----generating data for plotting---------------------------------------------
library(voluModel) # Because of course
library(ggplot2) # For fancy plotting
library(viridisLite) # For high-contrast plotting palettes
library(dplyr) # To filter data
library(terra) # Now being transitioned in
library(sf) # Now being transitioned in

# Load data
oxygenSmooth <- rast(system.file("extdata/oxygenSmooth.tif", 
                                 package='voluModel'))

occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv", 
                             package='voluModel'))

# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal

# Oxygen processing
names(oxygenSmooth) <- names(temperature)

# Clean points ----
occurrences <- occs %>% dplyr::select(decimalLongitude, decimalLatitude, depth) %>%
  distinct() %>% filter(dplyr::between(depth, 1, 2000))

# Gets the layer index for each occurrence by matching to depth
layerNames <- as.numeric(names(temperature))
occurrences$index <- unlist(lapply(occurrences$depth, 
                                   FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occurrences$index)
downsampledOccs <- data.frame()
for(i in indices){
  tempPoints <- occurrences[occurrences$index==i,]
  tempPoints <- downsample(tempPoints, temperature[[1]], verbose = FALSE)
  tempPoints$depth <- rep(layerNames[[i]], times = nrow(tempPoints))
  downsampledOccs <- rbind(downsampledOccs, tempPoints)
}
occsWdata <- downsampledOccs[,c("decimalLatitude", "decimalLongitude", "depth")] 

# Extract data ----
occsWdata$temperature <- xyzSample(occs = occsWdata, temperature)
occsWdata$AOU <- xyzSample(occs = occsWdata, oxygenSmooth)
occsWdata <- occsWdata[complete.cases(occsWdata),]

# Land shapefile
land <- rnaturalearth::ne_countries(scale = "small", returnclass = "sf")[1]

# Study region
studyRegion <- marineBackground(occsWdata, buff = 1000000)

# Get limits
tempLims <- quantile(occsWdata$temperature,c(0, 1))
aouLims <- quantile(occsWdata$AOU,c(0, 1))

# Reclassify environmental bricks to presence/absence
temperaturePresence <- classify(temperature, 
                                rcl = matrix(c(-Inf,tempLims[[1]],0,
                                               tempLims[[1]], tempLims[[2]], 1,
                                               tempLims[[2]], Inf, 0),
                                             ncol = 3, byrow = TRUE))
AOUpresence <- classify(oxygenSmooth, 
                        rcl = matrix(c(-Inf, aouLims[[1]],0,
                                       aouLims[[1]], aouLims[[2]], 1,
                                       aouLims[[2]], Inf, 0), 
                                     ncol = 3, byrow = TRUE))

# Put it all together
envelopeModel3D <- temperaturePresence * AOUpresence
envelopeModel3D <- mask(crop(envelopeModel3D, studyRegion), 
                        mask = studyRegion)
names(envelopeModel3D) <- names(temperature)
rm(AOUpresence, downsampledOccs, occurrences, temperaturePresence, 
   tempPoints, aouLims, envtNames, i, indices, layerNames, tempLims)

## ----pointMap, warning=FALSE, message=FALSE, eval=TRUE------------------------
pointMap(occs = occs, land = land, landCol = "black", spName = "Steindachneria argentea", 
         ptSize = 2, ptCol = "orange")

## ----pointCompMap, warning=FALSE, message=FALSE-------------------------------
pointCompMap(occs1 = occs, occs1Col = "red", occs1Name = "Raw", 
             occs2 = occsWdata, occs2Col = "orange", occs2Name = "Clean",
             spName = "Steindachneria argentea", agreeCol = "purple",
             land = land, landCol = "black", ptSize = 2, verbose = FALSE)

## ----oneRasterPlot------------------------------------------------------------
oneRasterPlot(rast = temperature[[1]],
              land = land, title = "Sea Surface Temperature, WOA 2018",
              landCol = "black", n = 11, option = "mako",
              varName = "Temperature")

## ----rasterComp---------------------------------------------------------------
rasterComp(rast1 = envelopeModel3D[[1]], rast1Name = "Surface",
           rast2 = envelopeModel3D[[10]], rast2Name = "45m", 
           land = land, landCol = "black", 
           title = "Suitability of Habitat for Luminous Hake\nAt Two Different Depths")

## ----plotLayers---------------------------------------------------------------
layerNames <- as.numeric(names(envelopeModel3D))
occsWdata$index <- unlist(lapply(occsWdata$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occsWdata$index)

layerPlot <- plotLayers(envelopeModel3D[[min(indices):max(indices)]], 
                        title = "Envelope Model of Luminous Hake,\n 20 to 700m",
                        land = land, landCol = "black")

## ----cleanup temporary directory----------------------------------------------
unlink(td, recursive = T)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)

## ----packages, warning=FALSE--------------------------------------------------
library(voluModel) # Because of course
library(dplyr) # For occurrence data filtration
library(ggplot2) # For fancy plotting
library(terra) # Now being transitioned in
library(sf) # Now being transitioned in
library(viridisLite) # For high-contrast plotting palettes

## ----occurrence data, eval=TRUE-----------------------------------------------
occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv", 
                             package='voluModel'))
summary(occs$depth)
boxplot.stats(occs$depth)

## ----clean occurrence data, eval=TRUE-----------------------------------------
occsClean <- occs %>% 
  dplyr::select(decimalLongitude, decimalLatitude, depth) %>%
  dplyr::distinct() %>% 
  filter(dplyr::between(depth, 1, 2000))
head(occsClean)

ggplot(occsClean, aes(x = 1, y=depth)) +
  geom_violin(fill="yellow", ) +
  theme_classic(base_size = 15) +
  theme(axis.title = element_blank(),
        text = element_text(family = "Arial"),
        axis.text = element_text(size = rel(1.1))) +
  ggtitle("Depth") +
  geom_boxplot(width=0.1)

## ----environmental data loading, eval=T, asis=T, message=FALSE----------------
# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal

## ----environmental data loading oxygen, eval=T, asis=T, warning=FALSE---------
oxygenSmooth <- rast(system.file("extdata/oxygenSmooth.tif", 
                                 package='voluModel'))

# Change names to match temperature
names(oxygenSmooth) <- names(temperature)

## ----downsample to voxel, eval=TRUE, warning=FALSE, message=FALSE-------------
# Gets the layer index for each occurrence by matching to depth
layerNames <- as.numeric(names(temperature))
occsClean$index <- unlist(lapply(occsClean$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occsClean$index)

# Downsamples occsClean in each depth layer
downsampledOccs <- data.frame()
for(i in indices){
  tempPoints <- occsClean[occsClean$index==i,]
  tempPoints <- downsample(tempPoints, temperature[[1]], 
                           verbose = FALSE)
  tempPoints$depth <- rep(layerNames[[i]], times = nrow(tempPoints))
  downsampledOccs <- rbind(downsampledOccs, tempPoints)
}

occsClean <- downsampledOccs

print(paste0("Original number of points: ", nrow(occs), "; number of downsampled occs: ", nrow(occsClean)))
land <- rnaturalearth::ne_countries(scale = "small", returnclass = "sf")[1]
pointCompMap(occs1 = occs, occs2 = occsClean, 
             occs1Name = "Original", occs2Name = "Cleaned", 
             spName = "Steindachneria argentea", 
             land = land, verbose = FALSE)

## ----environmental background sampling, warning=FALSE, eval = T---------------
backgroundSamplingRegions <- marineBackground(occsClean,
                                buff = 1000000,
                                clipToOcean = TRUE,
                                alpha = 1,
                                partCount = 1)
crs(backgroundSamplingRegions) <- crs(land)
plot(backgroundSamplingRegions, border = F, col = "gray",
     main = "Points and Background Sampling",
     axes = T)
plot(land, col = "black", add = T)
points(occsClean[,c("decimalLongitude", "decimalLatitude")], 
       pch = 20, col = "red", cex = 1.5)

## ----presence sampling--------------------------------------------------------
# Presences
oxyVals <- xyzSample(occs = occsClean, oxygenSmooth)
tempVals <- xyzSample(occs = occsClean, temperature)
vals <- cbind(occsClean, oxyVals, tempVals)
colnames(vals) <- c("decimalLongitude", "decimalLatitude", "depth", "AOU", "Temperature")
vals <- vals[complete.cases(vals),]
row.names(vals) <- NULL
occsWdata <- vals

# Add response as a column
occsWdata$response <- rep(1, times = nrow(occsWdata))

## ----background sampling------------------------------------------------------
# Background
backgroundVals <- mSampling3D(occs = occsClean, 
                              envBrick = temperature, 
                              mShp = backgroundSamplingRegions, 
                              depthLimit = c(5, 800))
oxyVals <- xyzSample(occs = backgroundVals, oxygenSmooth)
tempVals <- xyzSample(occs = backgroundVals, temperature)
vals <- cbind(backgroundVals, oxyVals, tempVals)
colnames(vals) <- c("decimalLongitude", "decimalLatitude", "depth", "AOU", "Temperature")
vals <- vals[complete.cases(vals),]
row.names(vals) <- NULL
backgroundWdata <- vals

# Add response as a column
backgroundWdata$response <- rep(0, times = nrow(backgroundWdata))

## ----glm data prep part 2-----------------------------------------------------
# Sample background points weighted by distance from centroid of occurrence environments
suitableCentroid <- apply(occsWdata[,c("Temperature", "AOU")], 
                          MARGIN = 2, FUN = mean)
backgroundWdata$distance <- apply(backgroundWdata[,c("Temperature", "AOU")], MARGIN = 1, 
                                  FUN = function(x) dist(rbind(suitableCentroid, x)))
backgroundWdata$sampleWeight <- (backgroundWdata$distance - 
                                 min(backgroundWdata$distance))/(max(backgroundWdata$distance)-
                                                                 min(backgroundWdata$distance))
sampleForAbsence <- sample(x = rownames(backgroundWdata), 
                           size = nrow(occsWdata) * 100, 
                           prob = backgroundWdata$sampleWeight)
backgroundWdata <- backgroundWdata[match(sampleForAbsence, 
                                         rownames(backgroundWdata)),]

## ----uniting datasets---------------------------------------------------------
# Unite datasets
datForMod <- rbind(occsWdata, backgroundWdata[,colnames(occsWdata)])
rm(suitableCentroid, sampleForAbsence, backgroundWdata, occsWdata)

## ----generate glm niche model-------------------------------------------------
glmModel <- glm(formula = response ~ Temperature *  AOU, 
                  family = binomial(link = "logit"),  data = datForMod)
summary(glmModel)

## ----project glm niche model--------------------------------------------------
layerNames <- as.numeric(names(temperature))
index <- seq(from = match(min(datForMod$depth), layerNames), 
             to = match(max(datForMod$depth), layerNames), by = 1)
depthPred <- NULL
for(j in index){
  depthPreds <- c(temperature[[j]], oxygenSmooth[[j]])
  crs(depthPreds) <- "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs"
  names(depthPreds) <- c("Temperature", "AOU")
  depthPred[[j]] <- mask(predict(depthPreds, glmModel), backgroundSamplingRegions)
  depthPred[[j]] <- crop(depthPred[[j]], backgroundSamplingRegions)
  names(depthPred[[j]]) <- layerNames[[j]]
}
glmPred <- rast(depthPred[!unlist(lapply(depthPred, FUN = function(x) is.null(x)))])

## ----glm niche threshold------------------------------------------------------
glmThreshold <- quantile(xyzSample(datForMod[datForMod$response == 1,
                                             c("depth", "decimalLongitude", "decimalLatitude")], 
                                   glmPred), .1, na.rm = T)[[1]] # MS90
glmThresholded <- glmPred > glmThreshold
rclMatrix <- matrix(c(NA, NA, 0), ncol = 3, byrow = TRUE)
glmThresholded <- classify(glmThresholded, rclMatrix, include.lowest = T)

## ----thresholded glm niche model plotted--------------------------------------
plotLayers(glmThresholded, 
          land = land, landCol = "black",
          title = "Areas of suitable habitat for \n Luminous Hake, 5m to 800m")

## ----calculate MESS, warning=FALSE--------------------------------------------
# Prepare environmental data
layerNames <- as.numeric(names(temperature))
datForMod$index <- unlist(lapply(datForMod$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(datForMod$index)

projList <- list("AOU" = oxygenSmooth[[min(indices):max(indices)]], 
                 "Temperature" = temperature[[min(indices):max(indices)]])

# Explicitly formatting calibration data
calibration <- data.frame(
  AOU = datForMod$AOU,
  Temperature = datForMod$Temperature)

# Calculate MESS
messBrick <- MESS3D(calibration = calibration, projection = projList)

## ----reclassify and plot extrapolation----------------------------------------
# Reclassify MESS 
extrapolation <- 0 >= messBrick

# Plot Extrapolation
plotLayers(extrapolation, land = land, landCol = "black", 
           title = "Areas of extrapolation according to MESS,\n 5m to 800m")

## ----reclassify MESS and plot GLM with no extrapolation, warning=FALSE--------
# Reclassify MESS 
noExtrapolation <- messBrick > 0

glmThreshNoExtrapolation <- NULL
for(i in 1:nlyr(noExtrapolation)){
  extrapCut <- crop(noExtrapolation[[i]], y = glmThresholded[[i]])
  croppedLayer <- glmThresholded[[i]] * extrapCut
  glmThreshNoExtrapolation[[i]] <- croppedLayer
}

glmThreshNoExtrapolation <- rast(glmThreshNoExtrapolation)

# Plot MESS
plotLayers(glmThreshNoExtrapolation, land = land, 
           landCol = "black", 
           title = "Areas of suitable habitat with no model extrapolation,\n 5m to 800m")

## ----cleanup temporary directory----------------------------------------------
unlink(td, recursive = T)

#' @title Coverts a list of 'SpatRaster' stacks where the elements are environmental
#' variables and the layers are depths to a list of SpatRaster stacks where the elements
#' are depths and the layers are environmental variables
#'
#' @description A user will likely already have environmental data organized into a list
#' of 'spatRaster' stacks where each element is a different environmental variable and each
#' layer corresponds to a depth slice for background sampling and extracting. However, the
#' maxent_3D function requires the data be input as a list of 'spatRaster' stacks where
#' each element corresponds to a depth slice and each layer is an environmental variable
#' for the purpose of projecting the model back into geographic space, and this function
#' has been developed for ease of conversion.
#'
#' @param envs_all a list of 'spatRaster' stacks where each element is a different
#' environmental variable and each layer is a depth slice.
#'
#' @param envs_names A 'character' vector of the names of the environmental variables
#' to be applied to the layers of the resulting list of 'spatRaster' stacks
#'
#' @details The extents of each environmental variable layer should match on a depth slice
#' by depth slice basis.
#'
#' @return A list of 'spatRaster' stacks where each list element is a depth slice and
#' each layer is an environmental variable.
#'
#' @examples
#' library(terra)
#'
#' # creating a list of spatRaster stacks where each element is an environmental variable
#' # and each layer is a depth
#'
#' env1_d1 <- rast(ncol = 50, nrow = 50)
#' values(env1_d1) <- sample(c(1:100), size = 2500, replace = TRUE)
#' env2_d1 <- rast(ncol = 50, nrow = 50)
#' values(env2_d1) <- sample(c(1:100), size = 2500, replace = TRUE)
#' env1_d2 <- rast(ncol = 50, nrow = 50)
#' values(env1_d2) <- sample(c(1:100), size = 2500, replace = TRUE)
#' env2_d2 <- rast(ncol = 50, nrow = 50)
#' values(env2_d2) <- sample(c(1:100), size = 2500, replace = TRUE)
#'
#' env1 <- c(env1_d1, env1_d2)
#' env2 <- c(env2_d1, env2_d2)
#' envs <- list(env1, env2)
#' envnames <- c("env1", "env2")
#'
#' # Here's the function
#' result <- env_stack_transform(envs_all = envs, envs_names = envnames)
#'
#' @importFrom terra rast
#'
#' @keywords transform list SpatRaster stack
#'
#' @export


env_stack_transform <- function(envs_all, envs_names) {
  env_call <- vector(length = length(envs_all))
  for (i in 1:length(envs_all)) {
    place <- paste0("envs_all[[", i, "]][[i]]")
    env_call[i] <- place
  }

  stack_list_all <- vector("list", length = dim(envs_all[[1]])[3])
  for (i in 1:dim(envs_all[[1]])[3]) {
    j_list <- vector("list", length = length(env_call))
    for (j in 1:length(env_call)) {
      j_list[[j]] <- eval(parse(text = env_call[j]))
    }
    st <- rast(j_list)
    names(st) <- envs_names
    stack_list_all[[i]] <- st
  }
  return(stack_list_all)
}

# test of maxent modelling functions (partition_3D, maxent_3D, threshold_3D)
library(terra)
library(predicts)
library(rnaturalearth)
library(sf)
library(voluModel)

# Initial Settings & Base Data
target_crs <- "+proj=longlat +datum=WGS84"
set.seed(0)

occs_raw <- read.csv(system.file("extdata/Steindachneria_argentea.csv", package='voluModel'))

# Clean and sample raw occurrences
occtemp <- occs_raw[,c("decimalLatitude", "decimalLongitude", "depth")]
occtemp <- occtemp[complete.cases(occtemp[, c("decimalLatitude", "decimalLongitude", "depth")]),]
occurrences <- occtemp[sample(seq_len(nrow(occtemp)), size = 50, replace = FALSE), ]

# Re-structure occurrences and assign a balanced depth scheme
occurrences <- data.frame(
  longitude = occurrences$decimalLongitude,
  latitude  = occurrences$decimalLatitude,
  depth     = rep(c(1, 2), length.out = nrow(occurrences))
)

# Create Environmental Layers
land <- project(ne_countries(returnclass = "sv"), target_crs)
occs_v <- terra::vect(occurrences, geom = c("longitude", "latitude"), crs = target_crs)

# Generate uniform environmental rasters spanning the data coordinates
r_base <- terra::rast(extent = ext(occs_v), crs = target_crs, resolution = 0.5)

r1d1 <- terra::init(r_base, fun = function(x) as.numeric(sample(0:10000, x, replace = TRUE)))
r2d1 <- terra::init(r_base, fun = function(x) as.numeric(sample(0:100, x, replace = TRUE)))
r1d2 <- terra::init(r_base, fun = function(x) as.numeric(sample(20:10020, x, replace = TRUE)))
r2d2 <- terra::init(r_base, fun = function(x) as.numeric(sample(20:120, x, replace = TRUE)))

envs <- list(
  mask(c(r1d1, r2d1), land, inverse = TRUE),
  mask(c(r1d2, r2d2), land, inverse = TRUE)
)
names(envs[[1]]) <- names(envs[[2]]) <- c("r1", "r2")

# Create background point pools directly from ocean cells
bg_cells_d1 <- crds(envs[[1]][[1]])
bg_cells_d2 <- crds(envs[[2]][[1]])

bgsd1 <- data.frame(bg_cells_d1[sample(nrow(bg_cells_d1), size = 1000, replace = FALSE), ], depth = 1)
bgsd2 <- data.frame(bg_cells_d2[sample(nrow(bg_cells_d2), size = 1000, replace = FALSE), ], depth = 2)

colnames(bgsd1) <- colnames(bgsd2) <- c("longitude", "latitude", "depth")

# Coordinate dataframe containing all potential test points
all_coords <- rbind(
  data.frame(occurrences, p = 1),
  data.frame(bgsd1, p = 0),
  data.frame(bgsd2, p = 0)
)

# Extract raster values matching each row's explicit depth tier
extracted_vals <- matrix(NA_real_, nrow = nrow(all_coords), ncol = 2)
colnames(extracted_vals) <- c("r1", "r2")

for (d in c(1, 2)) {
  idx <- which(all_coords$depth == d)
  if (length(idx) > 0) {
    # extract returns a data frame where cols 2:3 are the raster values
    vals <- terra::extract(envs[[d]], all_coords[idx, c("longitude", "latitude")])
    extracted_vals[idx, ] <- as.matrix(vals[, 2:3])
  }
}

# Bind everything into a single evaluation dataframe
eval_df <- cbind(all_coords, extracted_vals)

# Drop any rows missing environmental variables (e.g., points falling on landmask)
valid_rows <- complete.cases(eval_df[, c("r1", "r2")])
eval_df  <- eval_df[valid_rows, ]

# Split the pristine dataframe into perfectly mirrored test inputs
maxent_df     <- eval_df[, c("p", "r1", "r2")]
maxent_coords <- eval_df[, c("longitude", "latitude", "depth")]

# Cleanup temporary variables from global testing environment
rm(eval_df, all_coords, extracted_vals, valid_rows, bgsd1, bgsd2)

# Create "bad" data
baddf <- maxent_df
colnames(baddf) <- c("a", "b", "c")
badcoords <- maxent_coords
colnames(badcoords) <- c("lon", "lat", "dep")

# testing partition_3D
test_that("partition_3D warnings behave as expected", {
  expect_error(partition_3D())
  expect_warning(partition_3D(maxent_df = baddf, coord_df = maxent_coords,
                              which_partition = 'k.fold', kfolds = 5))
  expect_warning(partition_3D(maxent_df = maxent_df, coord_df = badcoords,
                              which_partition = 'k.fold', kfolds = 5))
  expect_warning(partition_3D(maxent_df = maxent_df,
                              coord_df = maxent_coords,
                              which_partition = 'spam', kfolds = 5))
  expect_warning(partition_3D(maxent_df = maxent_df, coord_df = maxent_coords,
                              which_partition = 'k.fold', kfolds = "h"))
}
)

test_that("partition_3D k.fold outputs as expected", {
  test_result1 <- partition_3D(maxent_df = maxent_df,
                               coord_df = maxent_coords,
                               which_partition = 'k.fold',
                               kfolds = 3, return_format = "list")
  expect_equal(length(unique(test_result1$occ_partitions)), 3)
  expect_equal(length(unique(test_result1$bg_partitions)), 3)
  test_result2 <- partition_3D(maxent_df = maxent_df,
                               coord_df = maxent_coords,
                               which_partition = 'k.fold',
                               kfolds = 5, return_format = "vector")
  expect_equal(length(unique(test_result2)), 5)
  expect_equal(nrow(maxent_df), length(test_result2))
}
)

test_that("partition_3D k.fold outputs as expected", {
  skip_on_cran() # this is way too slow for CRAN
  test_result3 <- partition_3D(maxent_df = maxent_df,
                               coord_df = maxent_coords,
                               which_partition = 'block',
                               orientation = "lat_lon",
                               return_format = "list",
                               max_attempts = 10)
  expect_equal(length(unique(test_result3$occ_partitions)), 4)
  expect_equal(length(unique(test_result3$bg_partitions)), 4)
  test_result4 <- partition_3D(maxent_df = maxent_df, coord_df = maxent_coords,
                               which_partition = 'block', orientation = "lon_lat",
                               return_format = "list", max_attempts = 10)
  expect_equal(length(unique(test_result4$occ_partitions)), 4)
  expect_equal(length(unique(test_result4$bg_partitions)), 4)
  expect_equal(length(test_result3$bg_partitions),
               length(test_result4$bg_partitions))
}
)

# Testing maxent_3D
test_partition <- partition_3D(maxent_df = maxent_df, coord_df = maxent_coords,
                               which_partition = 'k.fold', kfolds = 3, return_format = "list")
test_proj <- env_stack_transform(envs, c("r1", "r2"))
depths <- c(1, 2)

bad_depths <- c(-1, -2)
bad_partition1 <- vector()
bad_partition2 <- list("a", "b")
bad_partition3 <- test_partition
bad_partition3$occ_partitions <- c(bad_partition3$occ_partitions, 2, 1, 3)
bad_occs <- occurrences
colnames(bad_occs) <- c("a", "b", "c")
bad_layers <- test_proj
names(bad_layers[[1]]) <- c("a", "b")
names(bad_layers[[2]]) <- c("a", "b")

test_that("maxent_3D warnings behave as expected", {
  expect_error(maxent_3D())
  skip_if_not_installed("rJava")
  expect_warning(maxent_3D(maxent_df = baddf, wanted_fc = c("L"), wanted_rm = c(1),
                           projection_layers = test_proj, occs = occurrences,
                           depth_list = depths))
  expect_warning(maxent_3D(maxent_df = maxent_df, wanted_fc = c("L"), wanted_rm = c(1),
                           projection_layers = test_proj, occs = bad_occs,
                           depth_list = depths))
  expect_warning(maxent_3D(maxent_df = maxent_df, wanted_fc = c("L"),
                           wanted_rm = c("1"),
                           projection_layers = test_proj, occs = occurrences,
                           depth_list = depths))
  expect_warning(maxent_3D(maxent_df = maxent_df, wanted_fc = c("L"),
                           wanted_rm = c(1),
                           projection_layers = bad_layers, occs = occurrences,
                           depth_list = depths))
  expect_warning(maxent_3D(maxent_df = maxent_df, wanted_fc = c("L"),
                           wanted_rm = c(1),
                           projection_layers = test_proj, occs = occurrences,
                           depth_list = bad_depths))
  expect_warning(maxent_3D(maxent_df = maxent_df, wanted_fc = c("spam"),
                           wanted_rm = c(1),
                           projection_layers = test_proj, occs = occurrences,
                           depth_list = depths))
  expect_warning(maxent_3D(maxent_df = maxent_df, wanted_fc = c("L"),
                           wanted_rm = c(1), wanted_partition = bad_partition1,
                           projection_layers = test_proj, occs = occurrences,
                           depth_list = depths))
  expect_warning(maxent_3D(maxent_df = maxent_df, wanted_fc = c("L"),
                           wanted_rm = c(1), wanted_partition = bad_partition2,
                           projection_layers = test_proj, occs = occurrences,
                           depth_list = depths))
  expect_warning(maxent_3D(maxent_df = maxent_df, wanted_fc = c("L"),
                           wanted_rm = c(1), wanted_partition = bad_partition3,
                           projection_layers = test_proj, occs = occurrences,
                           depth_list = depths))
}
)

test_that("maxent_3D outputs as expected", {
  skip_if_not_installed("rJava")
  # Look for maxent.jar in the predicts package space
  jar_path <- system.file("java/maxent.jar", package = "predicts")

  # If it doesn't exist, skip the test gracefully without failing
  if (jar_path == "") {
    skip("maxent.jar is not installed in the 'predicts' package directory. Skipping test.")
  }

  result1 <- maxent_3D(maxent_df = maxent_df, wanted_fc = c("L"), wanted_rm = c(1),
                       projection_layers = test_proj, occs = maxent_coords,
                       depth_list = depths)
  expect_equal(class(result1), "list")
  expect_equal(class(result1$results), "data.frame")
  expect_contains(class(result1$models[[1]]), "MaxEnt_model")
  expect_contains(class(result1$predictions[[1]]), "SpatRaster")
  result2 <- maxent_3D(maxent_df = maxent_df, wanted_fc = c("L"), wanted_rm = c(1),
                       wanted_partition = test_partition,
                       projection_layers = test_proj, occs = maxent_coords,
                       depth_list = depths)
  expect_equal(class(result2$partition_results), "list")
  expect_equal(class(result2$partition_results[[1]]), "data.frame")
  result3 <- maxent_3D(maxent_df = maxent_df, wanted_fc = c("L", "LQ"),
                       wanted_rm = c(1:2),
                       wanted_partition = test_partition,
                       projection_layers = test_proj, occs = maxent_coords,
                       depth_list = depths)
  expect_equal(nrow(result3$results), 4)
}
)

test_that("threshold_3D warnings and outputs behave as expected", {
  skip_if_not_installed("rJava")
  maxent_result <- maxent_3D(maxent_df = maxent_df, wanted_fc = c("L"), wanted_rm = c(1),
                             projection_layers = test_proj, occs = maxent_coords,
                             depth_list = depths)
  predicted_layers <- maxent_result$predictions[[1]]
  expect_error(threshold_3D())
  expect_warning(threshold_3D(predicted_layers = predicted_layers,
                              thresholding_vals = c("0.95", "0.9"),
                              maxent_df = maxent_df, coord_df = maxent_coords))
  expect_warning(threshold_3D(predicted_layers = predicted_layers,
                              thresholding_vals = c(95, 90),
                              maxent_df = maxent_df, coord_df = maxent_coords))
  expect_warning(threshold_3D(predicted_layers = predicted_layers,
                              thresholding_vals = c(0.95, 0.90),
                              maxent_df = maxent_df[,2:3], coord_df = maxent_coords))
  expect_warning(threshold_3D(predicted_layers = predicted_layers,
                              thresholding_vals = c(0.95, 0.90),
                              maxent_df = maxent_df, coord_df = badcoords))
  result1 <- threshold_3D(predicted_layers = predicted_layers,
                          thresholding_vals = c(0.95, 0.90),
                          maxent_df = maxent_df, coord_df = maxent_coords)
  expect_equal(class(result1), "list")
  expect_contains(class(result1$threshold_layers), "SpatRaster")
  expect_equal(class(result1$tss_results), "data.frame")
  result2 <- threshold_3D(predicted_layers = predicted_layers,
                          thresholding_vals = c(0.95, 0.90),
                          maxent_df = maxent_df, coord_df = maxent_coords,
                          weights = 1/3)
  expect_true(max(result1$tss_results$TSS) != max(result2$tss_results$TSS))
}
)

library(terra)

# creating a list of spatRaster stacks where each element is an environmental variable
# and each layer is a depth

env1_d1 <- rast(ncol = 50, nrow = 50)
values(env1_d1) <- sample(c(1:100), size = 2500, replace = T)
env2_d1 <- rast(ncol = 50, nrow = 50)
values(env2_d1) <- sample(c(1:100), size = 2500, replace = T)
env1_d2 <- rast(ncol = 50, nrow = 50)
values(env1_d2) <- sample(c(1:100), size = 2500, replace = T)
env2_d2 <- rast(ncol = 50, nrow = 50)
values(env2_d2) <- sample(c(1:100), size = 2500, replace = T)

env1 <- c(env1_d1, env1_d2)
env2 <- c(env2_d1, env2_d2)
envs <- list(env1, env2)
envnames <- c("env1", "env2")

# env_stack_transform() test
test_that("env_stack_transform behaves as expected", {
  expect_error(env_stack_transform())
  test_result <- env_stack_transform(envs_all = envs, envs_names = envnames)
  expect_equal(class(test_result), "list")
  expect_length(test_result, 2)
  expect_equal(dim(test_result[[1]])[3], 2)
  expect_equal(class(test_result[[1]])[1], "SpatRaster")
})

#' @title Interpolate patchily sampled rasters
#'
#' @description Uses thin plate spline regression from
#' `fields` package to interpolate missing two-dimensional
#' raster values.
#'
#' @param inputRaster An object of class `SpatRaster`
#' @param fast A logical operator. Setting to `TRUE` triggers use
#' of `fastTps` instead of `Tps`.
#' @param ... For any additional arguments passed to `Tps` or `fastTps`
#'
#' @details Missing data values from original raster
#' are replaced with interpolated values. User has the
#' option of choosing `fastTps` to speed calculation,
#' but be advised that this is only an approximation
#' of a true thin plate spline.
#'
#' @return An object of class raster
#'
#' @examples
#' \donttest{
#' library(terra)
#' library(fields)
#' # Create sample raster
#' r <- rast(ncol=50, nrow=50)
#' values(r) <- 1:2500
#'
#' # Introduce a "hole"
#' values(r)[c(117:127, 167:177, 500:550)] <- NA
#' plot(r)
#'
#' # Patch hole with interpolateRaster
#' interpolatedRaster <- interpolateRaster(r)
#' plot(interpolatedRaster)
#' fastInterp <- interpolateRaster(r, fast = TRUE, aRange = 3.0)
#' plot(fastInterp)
#' }
#'
#' @import terra
#' @importFrom fields Tps fastTps
#'
#' @seealso \code{\link[fields]{Tps}}, \code{\link[fields]{fastTps}}
#'
#' @keywords dataPrep
#' @export

# Interpolates patchily sampled rasters (2d)
interpolateRaster <- function(inputRaster, fast = FALSE, ...){
  args <- list(...)

  if("lon.lat" %in% names(args)){
    lon.lat <- args$lon.lat
  } else{
    lon.lat <- FALSE
  }

  if("aRange" %in% names(args)){
    aRange <- args$aRange
  } else{
    aRange <- NA
  }

  if("theta" %in% names(args)){
    aRange <- args$theta # Theta and aRange are same, per fields docs
  }

  if("REML" %in% names(args)){
    REML <- args$REML
  } else{
    REML <- FALSE
  }

  if("method" %in% names(args)){
    method <- args$method
  } else{
    method <- "GCV"
  }

  # Input error check
  if (!is(inputRaster, "SpatRaster")) {
    warning(paste0("inputRaster is not of class 'SpatRaster'.\n"))
    return(NULL)
  }
  if (!is.logical(fast)) {
    warning(paste0("Argument 'fast' is not of type 'logical'.\n"))
    return(NULL)
  }
  if (!is.logical(lon.lat)) {
    warning(paste0("Argument 'lon.lat' is not of type 'logical'.\n"))
    return(NULL)
  }
  if(fast && is.na(aRange)){
    warning(paste0("'aRange' or 'theta' must be specified when using `fastTps()`.\n"))
    return(NULL)
  }
  if(fast && !is.numeric(aRange)){
    warning(paste0("'aRange' must be numeric.\n"))
    return(NULL)
  }
  if (!is.logical(REML)) {
    warning(paste0("Argument 'REML' is not of type 'logical'.\n"))
    return(NULL)
  }
  if (!(method %in% c("GCV", "GCV.model", "GCV.one", "RMSE", "pure error", "REML"))) {
    warning(paste0("Argument 'method' is not a recognized value.\n"))
    return(NULL)
  }

  # Prepare input data
  r <- inputRaster
  ra <- terra::aggregate(r, 4, na.rm = TRUE)
  xy <- data.frame(crds(ra, na.rm = FALSE))
  v <- values(ra)
  # remove NAs
  i <- -c(which(is.na(v)))
  if(length(i) == 0){
    i <- 1:nrow(xy)
  }
  xy <- xy[i,]
  v <- v[i]

  #### Thin plate spline model
  if(fast){
    tps <- try(fastTps(xy, v, lon.lat = lon.lat, aRange = aRange, REML = REML,
                   verbose = F),
               silent = TRUE)
  } else {
    tps <- try(Tps(xy, v, lon.lat = lon.lat, method = method, verbose = FALSE),
               silent = TRUE)
  }

  if("try-error" %in% class(tps)){
    tps <- try(Tps(xy, v, lon.lat = lon.lat, method = method, verbose = FALSE),
               silent = TRUE)
  }

  p <- rast(r)

  # use model to predict values at all locations
  p <- interpolate(p, tps, verbose = FALSE)

  # Fill in holes in original raster
  complete <- cover(r, p)

  return(complete)
}

#' @title Smooth rasters
#'
#' @description Uses thin plate spline regression from
#' `fields` package to smooth raster values.
#'
#' @param inputRaster An object of class `SpatRaster`
#' @param fast A logical operator. Setting to `TRUE` triggers use
#' of `fastTps` instead of `Tps`.
#' @param ... For any additional arguments passed to `Tps` or `fastTps`
#'
#' @details Original raster is smoothed using a thin
#' plate spline. This may be desirable in cases where
#' the user has a reasonable expectation of spatial autocorrelation,
#' but observes putative measurement errors in a raster. The user has
#' the option of choosing `fastTps` to speed calculation,
#' but be advised that this is only an approximation
#' of a true thin plate spline.
#'
#' @return An object of class `SpatRaster`
#'
#' @examples
#' library(terra)
#' library(fields)
#' # Create sample raster
#' r <- rast(ncol=100, nrow=100)
#' values(r) <- 1:10000
#'
#' # Introduce a "bubble"
#' values(r)[720:725] <- 9999
#' plot(r)
#'
#' # Smooth bubble with smoothRaster
#' fastSmooth <- smoothRaster(r, fast = TRUE, aRange = 10.0)
#' plot(fastSmooth)
#'
#' @import terra
#' @importFrom fields Tps fastTps
#'
#' @seealso \code{\link[fields]{Tps}}, \code{\link[fields]{fastTps}}
#'
#' @keywords dataPrep
#' @export

smoothRaster <- function(inputRaster, fast = FALSE, ...){
  args <- list(...)

  if("lon.lat" %in% names(args)){
    lon.lat <- args$lon.lat
  } else{
    lon.lat <- FALSE
  }

  if("aRange" %in% names(args)){
    aRange <- args$aRange
  } else{
    aRange <- NA
  }

  if("theta" %in% names(args)){
    aRange <- args$theta # Theta and aRange are same, per fields docs
  }

  if("REML" %in% names(args)){
    REML <- args$REML
  } else{
    REML <- FALSE
  }

  if("method" %in% names(args)){
    method <- args$method
  } else{
    method <- "GCV"
  }

  # Input error check
  if (!is(inputRaster, "SpatRaster")) {
    warning(paste0("inputRaster is not of class 'SpatRaster'.\n"))
    return(NULL)
  }
  if (!is.logical(fast)) {
    warning(paste0("Argument 'fast' is not of type 'logical'.\n"))
    return(NULL)
  }
  if (!is.logical(lon.lat)) {
    warning(paste0("Argument 'lon.lat' is not of type 'logical'.\n"))
    return(NULL)
  }
  if(fast && is.na(aRange)){
    warning(paste0("'aRange' or 'theta' must be specified when using `fastTps()`.\n"))
    return(NULL)
  }
  if(fast && !is.numeric(aRange)){
    warning(paste0("'aRange' must be numeric.\n"))
    return(NULL)
  }
  if (!is.logical(REML)) {
    warning(paste0("Argument 'REML' is not of type 'logical'.\n"))
    return(NULL)
  }
  if (!(method %in% c("GCV", "GCV.model", "GCV.one", "RMSE", "pure error", "REML"))) {
    warning(paste0("Argument 'method' is not a recognized value.\n"))
    return(NULL)
  }

  # Prepare input data
  r <- inputRaster
  ra <- terra::aggregate(r, 4, na.rm = TRUE)
  xy <- data.frame(crds(ra, na.rm = FALSE))
  v <- values(ra)
  # remove NAs
  i <- -c(which(is.na(v)))
  if(length(i) == 0){
    i <- 1:nrow(xy)
  }
  xy <- xy[i,]
  v <- v[i]

  #### Thin plate spline model
  if(fast){
    tps <- fastTps(xy, v, lon.lat = lon.lat, aRange = aRange,
                   REML = REML,
                   verbose = FALSE)
  } else {
    tps <- Tps(xy, v, lon.lat = lon.lat, method = method,
               verbose = FALSE)
  }

  p <- rast(r)

  # use model to predict values at all locations
  p <- interpolate(p, tps, verbose = FALSE)

  return(p)
}

library(terra)
library(fields)

# interpolateRaster() tests ----
# Create sample raster
r <- rast(ncol=10, nrow=10)
values(r) <- 1:100

completeRaster <- r

test_that("interpolateRaster input warnings behave as expected", {

  # Tests
  expect_error(interpolateRaster())
  expect_warning(interpolateRaster(inputRaster = "a"))
  expect_warning(interpolateRaster(inputRaster = r, fast = "a"))
  expect_warning(interpolateRaster(inputRaster = r, fast = F,
                                lon.lat = "a"))
  expect_warning(interpolateRaster(inputRaster = r, fast = T))
  expect_warning(interpolateRaster(inputRaster = r, fast = T,
                                aRange = "a"))
  expect_warning(interpolateRaster(inputRaster = r, fast = T,
                                   theta = "a"))
  expect_warning(interpolateRaster(inputRaster = r, fast = F,
                                   REML = "a"))
  expect_warning(interpolateRaster(inputRaster = r, fast = F,
                                   REML = T, method = "guess"))
})

# Introduce a "hole"
values(r)[c(45:47, 55:57)] <- NA

test_that("interpolateRaster returns appropriate information", {
 # Execute function
  interpolatedRaster <- interpolateRaster(r)
  expect_equal(class(r), class(interpolatedRaster))
})

test_that("interpolateRaster interpolates as expected", {
  # Execute function
  interpolatedRaster <- interpolateRaster(r)

  # Examine results
  expect_true(global(interpolatedRaster, "isNA") == 0)
  expect_equal(global(completeRaster, max),
               global(interpolatedRaster, max))
  expect_equal(global(completeRaster, min),
               global(interpolatedRaster, min))
})

# Create sample raster
r <- rast(ncol=100, nrow=100)
values(r) <- rep(1:100, times = 100)
completeRaster <- r

# Introduce a "hole"
values(r)[c(5030:5040)] <- NA

# Execute function
interpolatedRaster <- interpolateRaster(r, fast = T, aRange = 1)

test_that("fast version of interpolateRaster returns appropriate information", {
  expect_equal(class(r), class(interpolatedRaster))
})

test_that("fast version of interpolateRaster interpolates as expected", {
  # Examine results
  expect_true(global(completeRaster, "isNA") == 0)
  expect_equal(global(completeRaster, max),
               global(interpolatedRaster, max))
  expect_equal(global(completeRaster, min),
               global(interpolatedRaster, min))
})

# smoothRaster() tests ----
# Introduce a "bubble"
values(r)[c(5030:5050,5130:5150)] <- 99

test_that("smoothRaster input warnings behave as expected", {

  # Tests
  expect_error(smoothRaster())
  expect_warning(smoothRaster(inputRaster = "a"))
  expect_warning(smoothRaster(inputRaster = r, fast = "a"))
  expect_warning(smoothRaster(inputRaster = r, fast = F,
                                   lon.lat = "a"))
  expect_warning(smoothRaster(inputRaster = r, fast = T))
  expect_warning(smoothRaster(inputRaster = r, fast = T,
                                   aRange = "a"))
  expect_warning(smoothRaster(inputRaster = r, fast = T,
                              theta = "a"))
  expect_warning(smoothRaster(inputRaster = r, fast = T,
                              REML = "a"))
  expect_warning(smoothRaster(inputRaster = r, fast = T,
                              REML = T, method = "guess"))
})

# Execute function
smoothedRaster <- smoothRaster(r, method = "REML")

test_that("smoothRaster returns appropriate information", {
  expect_equal(class(r), class(smoothedRaster))
})

test_that("smoothRaster interpolates as expected", {
  # Examine results
  expect_true(global(smoothedRaster, "isNA") == 0)
  expect_equal(global(r, max),
               round(global(smoothedRaster, max), digits = 0))
})

# Create sample raster
r <- rast(ncol=100, nrow=100)
values(r) <- 1:10000
completeRaster <- r

# Introduce a "hole"
values(r)[100:105] <- 200

# Execute function
smoothedRaster <- smoothRaster(r, fast = T, aRange = 1)

test_that("fast version of smoothRaster returns appropriate information", {
  expect_equal(class(r), class(smoothedRaster))
})

test_that("fast version of smoothRaster interpolates as expected", {
  # Examine results
  expect_true(global(smoothedRaster, "isNA") == 0)
  expect_equal(global(smoothedRaster, max),
               global(smoothedRaster, max))
  expect_equal(global(smoothedRaster, min),
               global(smoothedRaster, min))
})

# Visualization ----
#' @title Are Colors
#'
#' @description Checks to see if a given vector can be
#' interpreted by R as a color or colors
#'
#' @param col A vector of anything to be interpreted by `rgb`
#' as a color.
#'
#' @return A logical vector stating whether inputs
#' can be interpreted as colors.
#'
#' @examples
#'
#' areColors(col = c("red", "prairie_chicken", 2))
#'
#' @importFrom grDevices col2rgb
#'
#' @keywords internal plotting
#'
#' @export

areColors <- function(col) {
  if(is.null(col)){
    warning(paste0("'col' cannot be NULL.\n"))
    return(NULL)
  } else{
    result <- sapply(col, function(X) {
      tryCatch(is.matrix(col2rgb(X)),
               error = function(e) FALSE)
    })
    return(result)
  }
}

#' @title Test Intersection
#'
#' @description Tests whether two rasters overlap. Used in
#' `\code{\link[voluModel:diversityStack]{diversityStack}}`
#' function to verify all rasters in list overlap with the
#' template raster.
#'
#' @param a The first `SpatRaster` object
#'
#' @param b The second `SpatRaster` object
#'
#' @return A logical vector stating whether the two
#' inputs overlap
#'
#' @examples
#'
#' library(terra)
#' rast1 <- rast(ncol=10, nrow=10)
#' values(rast1) <- rep(0:1, 50)
#'
#' rast2 <- rast(ncol=10, nrow=10)
#' values(rast2) <- c(rep(0, 50), rep(1,50))
#'
#' testIntersection(rast1, rast2)
#'
#' rast1 <- crop(rast1, ext(10, 20, 30, 40))
#' rast2 <- crop(rast2, ext(-20, -10, -40, -30))
#'
#' testIntersection(rast1, rast2)
#'
#' @importFrom grDevices col2rgb
#'
#' @keywords internal plotting
#'
#' @export

testIntersection <- function(a,b){
  #reads in two rasters and tests for overlap T or F
  # if returns TRUE then there is overlap
  # try error is included b/c errors has come up with other test data
  !(inherits(try(intersect(a,b),T ), what ='try-error') || is.null(intersect(a,b)))
}

#' @title Point mapping
#'
#' @description A convenient wrapper around ggplot
#' to generate formatted occurrence point plots.
#'
#' @param occs A `data.frame` with at least two columns
#' named "longitude" and "latitude" or that
#' can be coerced into this format.
#'
#' @param spName A character string with the species
#' name to be used in the plot title.
#'
#' @param land An optional coastline polygon shapefile
#' of types `sf` or `SpatRaster` to provide geographic
#' context for the occurrence points.
#'
#' @param ptCol Color for occurrence points on map
#'
#' @param landCol Color for land on map
#'
#' @param waterCol Color for water on map
#'
#' @param ptSize `numeric` value for `cex`;
#' size of occurrence points on map.
#'
#' @param verbose `logical`. Switching to `FALSE` mutes message describing
#' which columns in `occs` are interpreted as x and y coordinates.
#'
#' @param ... Additional optional arguments to pass to
#' `ggplot` initial plot object.
#'
#' @return A `ggplot` plot object.
#'
#' @examples
#' occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv",
#'                              package='voluModel'))
#' spName <- "Steindachneria argentea"
#' pointMap(occs = occs, spName = spName,
#'          land = rnaturalearth::ne_countries(scale = "small",
#'                                             returnclass = "sf")[1])
#'
#' @import ggplot2
#'
#' @seealso \code{\link[ggplot2:ggplot]{ggplot}}
#'
#' @keywords plotting
#' @export

pointMap <- function(occs, spName, land = NA,
                     ptCol = "#bd0026", landCol = "gray",
                     waterCol = "steelblue", ptSize = 1,
                     verbose = TRUE,
                     ...){
  args <- list(...)

  if("mapping" %in% names(args)){
    mapping <- args$mapping
  } else{
    mapping <- aes()
  }

  if("alpha" %in% names(args)){
    alpha <- args$alpha
  } else{
    alpha <- 2/3
  }

  if (!is.logical(verbose)) {
    warning(message("Argument 'verbose' is not of type 'logical'.\n"))
    return(NULL)
  }

  # Input checking
  if(!is.data.frame(occs)){
    warning(paste0("'occs' must be an object of class 'data.frame'.\n"))
    return(NULL)
  }

  if(!is.character(spName)){
    warning(paste0("'spName' must be an object of class 'character'.\n"))
    return(NULL)
  }

  if(!any(is.na(land[[1]]),
          "sf" %in% class(land),
          "SpatVector" %in% class(land))){
    warning(paste0("'land' must be NA or of class 'sf' or 'SpatVector'."))
    return(NULL)
  }

  if(!any(is.na(land))){
    if(!("sf" %in% class(land))){
      land <- sf::st_as_sf(land)
    }
  }

  colVec <- c(ptCol, landCol, waterCol)
  colTest <- areColors(colVec)

  if(!any(c(all(colTest), length(colTest) < 3))){
    warning(paste0("'ptCol', 'landCol', and 'waterCol' must
                   be recognized colors.\n"))
    return(NULL)
  }

  if(!is.numeric(ptSize)){
    warning(paste0("'ptSize' must be numeric.\n"))
  }

  # Parse columns
  colNames <- colnames(occs)
  colParse <- columnParse(occs)
  if(is.null(colParse)){
    return(NULL)
  }
  xIndex <- colParse$xIndex
  yIndex <- colParse$yIndex
  interp <- colParse$reportMessage

  if(verbose){
    message(interp)
  }

  # Where the actual function happens
  if(any(is.na(land))){
      point_map <- ggplot(mapping = mapping) +
        geom_point(data = occs, aes(x = occs[[xIndex]], y = occs[[yIndex]]),
                   colour = ptCol, cex = ptSize, shape = 20, alpha = alpha) +
        theme(panel.background = element_rect(fill = waterCol),
              panel.grid = element_blank()) +
        coord_sf(xlim = c(min(occs[[xIndex]]), max(occs[[xIndex]])),
                 ylim = c(min(occs[[yIndex]]), max(occs[[yIndex]])),
                 expand = TRUE, ) +
        xlab("") +
        ylab("") +
        ggtitle(paste0(spName, ", ", nrow(occs), " points"))
  }else{
    point_map <- ggplot(mapping = mapping) +
      geom_sf(data = land, color = landCol, fill = landCol) +
      geom_point(data = occs, aes(x = occs[[xIndex]], y = occs[[yIndex]]),
                 colour = ptCol, cex = ptSize, shape = 20, alpha = alpha) +
      theme(panel.background = element_rect(fill = waterCol),
            panel.grid = element_blank()) +
      coord_sf(xlim = c(min(occs[[xIndex]]), max(occs[[xIndex]])),
               ylim = c(min(occs[[yIndex]]), max(occs[[yIndex]])),
               expand = TRUE, ) +
      xlab("") +
      ylab("") +
      ggtitle(paste0(spName, ", ", nrow(occs), " points"))
  }
  return(point_map)
}

#' @title Comparative point mapping
#'
#' @description A convenient wrapper around `ggplot`
#' to generate formatted plots comparing two sets of
#' occurrence point plots.
#'
#' @param occs1 A `data.frame` with at least two columns
#' named "longitude" and "latitude" or that
#' can be coerced into this format.
#'
#' @param occs2 A `data.frame` with at least two columns
#' named "longitude" and "latitude" or that
#' can be coerced into this format.
#'
#' @param spName A character string with the species
#' name to be used in the plot title.
#'
#' @param land An optional coastline polygon shapefile
#' of types `sf` or `SpatRaster` to provide geographic
#' context for the occurrence points.
#'
#' @param occs1Col Color for occurrence points on map
#'
#' @param occs2Col Color for occurrence points on map
#'
#' @param occs1Name An optional name for the first set
#' of occurrences, which will be color-coded to
#' `occs1Col` in the resulting plot.
#'
#' @param occs2Name An optional name for the first set
#' of occurrences, which will be color-coded to
#' `occs2Col` in the resulting plot.
#'
#' @param agreeCol Color for occurrence points shared
#' between `occs1` and `occs2`.
#'
#' @param landCol Color for land on map
#'
#' @param waterCol Color for water on map
#'
#' @param ptSize `numeric` value for `cex`;
#' size of occurrence points on map.
#'
#' @param verbose `logical`. Switching to `FALSE` mutes message describing
#' which columns in `occs1` and `occs2` are interpreted as x and y coordinates.
#'
#' @param ... Additional optional arguments to pass to
#' `ggplot` initial plot object.
#'
#' @return A `ggplot` plot object.
#'
#' @note The x and y column names of `occs1` and `occs2`
#' must match.
#'
#' @examples
#' set.seed(5)
#' occs <- data.frame(cbind(decimalLatitude = sample(seq(7,35), 24),
#'                          decimalLongitude = sample(seq(-97, -70), 24)))
#'
#' set.seed(0)
#' occs1 <- occs[sample(1:nrow(occs),
#'                      size = 12, replace = FALSE),]
#' set.seed(10)
#' occs2 <- occs[sample(1:nrow(occs),
#'                      size = 12, replace = FALSE),]
#'
#' pointCompMap(occs1 = occs1, occs2 = occs2,
#'              occs1Col = "red", occs2Col = "orange",
#'              agreeCol = "purple",
#'              occs1Name = "2D",
#'              occs2Name = "3D",
#'              waterCol = "steelblue",
#'              spName = "Steindachneria argentea",
#'              ptSize = 2,
#'              verbose = FALSE)
#'
#' @import ggplot2
#' @importFrom dplyr inner_join anti_join %>% mutate
#' @importFrom ggtext element_markdown
#'
#' @seealso \code{\link[ggplot2:ggplot]{ggplot}}
#'
#' @keywords plotting
#' @export

pointCompMap <- function(occs1, occs2,
                         spName, land = NA,
                         occs1Col = "#bd0026",
                         occs2Col = "#fd8d3c",
                         agreeCol = "black",
                         occs1Name = "Set 1",
                         occs2Name = "Set 2",
                         landCol = "gray",
                         waterCol = "steelblue",
                         ptSize = 1,
                         verbose = TRUE,
                         ...){
  args <- list(...)

  if("mapping" %in% names(args)){
    mapping <- args$mapping
  } else{
    mapping <- aes()
  }

  # Input checking
  if(!is.data.frame(occs1)){
    warning(paste0("'occs1' must be an object of class 'data.frame'.\n"))
    return(NULL)
  }

  if(!is.data.frame(occs2)){
    warning(paste0("'occs2' must be an object of class 'data.frame'.\n"))
    return(NULL)
  }

  if(!all(c(is.character(spName),
            is.character(occs1Name),
            is.character(occs2Name)))){
    warning(paste0("'spName', 'occs1Name', and 'occs2Name'
                   must be 'character' strings.\n"))
    return(NULL)
  }

  if(!any(is.na(land[[1]]),
          "sf" %in% class(land),
          "SpatVector" %in% class(land))){
    warning(paste0("'land' must be NA or of class 'sf' or 'SpatVector'."))
    return(NULL)
  }

  if(!any(is.na(land))){
    if(!("sf" %in% class(land))){
      land <- sf::st_as_sf(land)
    }
  }

  if (!is.logical(verbose)) {
    warning(message("Argument 'verbose' is not of type 'logical'.\n"))
    return(NULL)
  }

  colVec <- c(occs1Col, occs2Col, agreeCol, landCol, waterCol)
  colTest <- areColors(colVec)

  if(!any(c(all(colTest), length(colTest) < 5))){
    warning(paste0("'pt1Col', 'pt2Col', 'agreeCol', 'landCol',
                    and 'waterCol' must be recognized colors.\n"))
    return(NULL)
  }

  if(!is.numeric(ptSize)){
    warning(paste0("'ptSize' must be numeric.\n"))
    return(NULL)
  }

  # Parse columns
  colNames1 <- colnames(occs1)
  colParse1 <- columnParse(occs1)
  if(is.null(colParse1)){
    return(NULL)
  }
  xIndex1 <- colParse1$xIndex
  yIndex1 <- colParse1$yIndex
  interp1 <- colParse1$reportMessage

  if(verbose){
    message(paste("First set of occurrences: ", interp1))
  }

  colNames2 <- colnames(occs2)
  colParse2 <- columnParse(occs2)
  if(is.null(colParse2)){
    return(NULL)
  }
  xIndex2 <- colParse2$xIndex
  yIndex2 <- colParse2$yIndex
  interp2 <- colParse2$reportMessage

  if(verbose){
    message(paste("Second set of occurrences: ", interp2))
  }

  if(!all(c(colnames(occs1)[[xIndex1]] == colnames(occs2)[[xIndex2]],
           colnames(occs1)[[yIndex1]] == colnames(occs2)[[yIndex2]]))){
    warning(paste0("x and y column names in the two occurrence datasets
                   do not match.\n"))
    return(NULL)
  }

  # Where the function actually starts
  occsBoth <- NA
  occsBoth <- suppressMessages(dplyr::inner_join(occs2[,c(xIndex2, yIndex2)],
                                occs1[,c(xIndex1, yIndex1)], multiple = "all") %>%
    dplyr::distinct() %>%
    dplyr::mutate(source = "both"))
  occs1 <- suppressMessages(dplyr::anti_join(occs1[,c(xIndex1, yIndex1)],occsBoth) %>%
    dplyr::distinct() %>%
    dplyr::mutate(source = occs1Name))
  occs2 <- suppressMessages(dplyr::anti_join(occs2[,c(xIndex2, yIndex2)],occsBoth) %>%
    dplyr::distinct() %>%
    dplyr::mutate(source = occs2Name))

  colParse1 <- columnParse(occs1)
  xIndex1 <- colParse1$xIndex
  yIndex1 <- colParse1$yIndex

  colParse2 <- columnParse(occs2)
  xIndex2 <- colParse2$xIndex
  yIndex2 <- colParse2$yIndex

  if(nrow(occs2)==0) {
    occs1 <- unique(occs1[,c(xIndex1, yIndex1)])
    occs1$source <- rep_len(occs1Name, length.out = nrow(occs1))
  }
  if (nrow(occs1)==0){
    occs2 <- unique(occs2[,c(xIndex2, yIndex2)])
    occs2$source <- rep_len(occs2Name, length.out = nrow(occs2))
  }

  allDat <- list(occsBoth[,c(colnames(occs1)[c(xIndex1,yIndex1)], "source")],
                 occs1[,c(colnames(occs1)[c(xIndex1,yIndex1)], "source")],
                 occs2[,c(colnames(occs2)[c(xIndex2,yIndex2)], "source")])
  occ_dat <- do.call("rbind", allDat[!is.na(allDat)])

  # Now add the occurrence points
  cols <- NULL
  for (x in occ_dat$source){
    if (x == occs1Name){
      cols <- c(cols, occs1Col)
    } else if (x == occs2Name){
      cols <- c(cols, occs2Col)
    } else{
      cols <- c(cols, agreeCol)
    }
  }

  occ_datIndices <- columnParse(occ_dat)

  if(any(is.na(land))){
    comparison_map <- ggplot() +
    geom_point(data = occ_dat, aes(x = occ_dat[[occ_datIndices$xIndex]],
                                   y = occ_dat[[occ_datIndices$yIndex]]),
               colour = cols, cex = ptSize, shape = 20, alpha = 1) +
    theme(panel.background = element_rect(fill = waterCol)) +
    theme(panel.grid = element_blank()) +
    coord_sf(xlim = c(min(occ_dat[[occ_datIndices$xIndex]]),
                      max(occ_dat[[occ_datIndices$xIndex]])),
             ylim = c(min(occ_dat[[occ_datIndices$yIndex]]),
                      max(occ_dat[[occ_datIndices$yIndex]])),
             expand = TRUE, ) +
    xlab("") +
    ylab("") +
    labs(
      title = paste0("***", spName, "***<br>",
                     "<span style='color:", agreeCol, ";'>Overlapping</span>, ",
                     "<span style='color:", occs1Col, ";'>in ", occs1Name,
                     " dataset only</span>, and ",
                     "<span style='color:", occs2Col, ";'>in ", occs2Name,
                     " dataset only</span>")) +
    theme(plot.title = element_markdown(lineheight = .4))}
  else{
    comparison_map <- ggplot() +
      geom_sf(data = land, color = landCol, fill = landCol) +
      geom_point(data = occ_dat, aes(x = occ_dat[[occ_datIndices$xIndex]],
                                     y = occ_dat[[occ_datIndices$yIndex]]),
                 colour = cols, cex = ptSize, shape = 20, alpha = 1) +
      theme(panel.background = element_rect(fill = waterCol)) +
      theme(panel.grid = element_blank()) +
      coord_sf(xlim = c(min(occ_dat[[occ_datIndices$xIndex]]),
                        max(occ_dat[[occ_datIndices$xIndex]])),
               ylim = c(min(occ_dat[[occ_datIndices$yIndex]]),
                        max(occ_dat[[occ_datIndices$yIndex]])),
               expand = TRUE, ) +
      xlab("") +
      ylab("") +
      labs(title = paste0("***", spName, "***<br>",
                          "<span style='color:", agreeCol, ";'>Overlapping</span>, ",
                          "<span style='color:", occs1Col, ";'>in ", occs1Name,
                          " dataset only</span>, and ",
                          "<span style='color:", occs2Col, ";'>in ", occs2Name,
                          " dataset only</span>")) +
      theme(plot.title = element_markdown(lineheight = .4))
  }
  return(comparison_map)
}

#' @title Transparent Color
#'
#' @description Generates transparent colors
#'
#' @param color Anything that can be interpreted by `rgb`
#' as a color.
#'
#' @param percent A whole number between 0 and 100 specifying
#' how transparent the color should be.
#'
#' @return A `character` string with hex color, including
#' adjustment for transparency.
#'
#' @examples
#'
#' transpColor(color = "red", percent = 50)
#'
#' @importFrom grDevices rgb
#'
#' @keywords internal plotting
#'
#' @export

transpColor <- function(color, percent = 50) {
  colTest <- areColors(color)

  if(!colTest){
    warning(paste0("'color' must be a recognized color.\n"))
    return(NULL)
  }

  if(!is.numeric(percent)){
    warning(paste0("'percent' must be numeric.\n"))
    return(NULL)
  }

  if(!all(percent >= 0, 100 >= percent)){
    warning(paste0("'percent' must be between 0 and 100.\n"))
    return(NULL)
  }

  ## Get RGB values for named color
  rgb.val <- col2rgb(color)

  ## Make new color using input color as base and transparency set by alpha
  t.col <- rgb(rgb.val[1], rgb.val[2], rgb.val[3],
               maxColorValue = 255,
               alpha = (100 - percent) * 255 / 100)

  ## Save the color
  return(t.col)
}

#' @title Blend Colors
#'
#' @description Generates a blended color from two transparent colors
#'
#' @param col1 Anything that can be interpreted by `rgb`
#' as a color.
#'
#' @param col2 Anything that can be interpreted by `rgb`
#' as a color.
#'
#' @return A `character` string with hex color, including
#' adjustment for transparency.
#'
#' @examples
#'
#' blendColor(col1 = "#1B9E777F", col2 = "#7570B37F")
#'
#' @importFrom grDevices rgb
#'
#' @keywords internal plotting
#'
#' @export

blendColor <- function( col1 = "#1b9e777F", col2 = "#7570b37F") {
  colVec <- c(col1, col2)
  colTest <- areColors(colVec)

  if(!any(c(all(colTest), length(colTest) < 2))){
    warning(paste0("'col1' and 'col2'
                   must be recognized colors.\n"))
    return(NULL)
  }

  ## Get RGB values for colors
  rgb.val1 <- col2rgb(col1, alpha = TRUE)
  rgb.val2 <- col2rgb(col2, alpha = TRUE)
  alphaVal <- rgb.val1["alpha",] + rgb.val2["alpha",]
  if(alphaVal >= 255){alphaVal <- 255}
  ## Make new color using input color as base and transparency set by alpha
  t.col <- rgb(red = mean(c(rgb.val1["red",], rgb.val2["red",])),
               green = mean(c(rgb.val1["green",], rgb.val2["green",])),
               blue = mean(c(rgb.val1["blue",], rgb.val2["blue",])),
               alpha = alphaVal,
               maxColorValue = 255)

  ## Save the color
  return(t.col)
}

#' @title Comparative raster mapping
#'
#' @description A convenient wrapper around `terra::plot`
#' to generate formatted plots comparing two rasters.
#' This is used in the context of voluModel to
#' overlay semi-transparent distributions (coded as 1)
#' in two different `RasterLayers`.
#'
#' @param rast1 A single `SpatRaster` showing the
#' distribution of the species corresponding to
#' `rast1Name`. Should have values of 0 (absence)
#' and 1 (presence). Can also be `NULL`.
#'
#' @param rast2 A single `SpatRaster` showing the
#' distribution of the species corresponding to
#' `rast2Name`. Should have values of 0 (absence)
#' and 1 (presence). Must match the extent and
#' resolution of `rast1`. Can also be `NULL.`
#'
#' @param col1 Color for `rast1` presences
#'
#' @param col2 Color for `rast2` presences
#'
#' @param rast1Name An optional name for the first set
#' of occurrences, which will be color-coded to
#' `occs1Col` in the resulting plot.
#'
#' @param rast2Name An optional name for the first set
#' of occurrences, which will be color-coded to
#' `occs2Col` in the resulting plot.
#'
#' @param landCol Color for land on map.
#'
#' @param land An optional coastline polygon shapefile
#' of types `sf` or `SpatRaster` to provide geographic
#' context for the occurrence points.
#'
#' @param title A title for the plot.
#'
#' @param graticule Do you want a grid of lon/lat lines?
#'
#' @param ... Additional optional arguments to pass to
#' `terra::plot()`.
#'
#' @return A plot of class `recordedplot` overlaying mapped,
#' semitransparent extents of the input rasters
#'
#' @note The extents of `rast1` and `rast2`
#' must match.
#'
#' @examples
#' library(terra)
#' rast1 <- rast(ncol=10, nrow=10)
#' values(rast1) <- rep(0:1, 50)
#'
#' rast2 <- rast(ncol=10, nrow=10)
#' values(rast2) <- c(rep(0, 50), rep(1,50))
#'
#' rasterComp(rast1 = rast1, rast2 = rast2)
#'
#' @import terra
#' @importFrom grDevices recordPlot
#' @importFrom graphics legend
#'
#' @seealso \code{\link[terra:plot]{plot}}
#'
#' @keywords plotting
#' @export

rasterComp <- function(rast1 = NULL, rast2 = NULL,
                       col1 = "#1b9e777F", col2 = "#7570b37F",
                       rast1Name = "Set 1", rast2Name = "Set 2",
                       land = NA, landCol = "black",
                       title = "A Raster Comparison",
                       graticule = TRUE, ...){

  args <- list(...)

  colVec <- c(col1, col2, landCol)
  colTest <- areColors(colVec)

  if(!any(c(all(colTest), length(colTest) < 3))){
    warning(paste0("'col1', 'col2', and 'landCol'
                   must be recognized colors.\n"))
    return(NULL)
  }

  if(!any(is.na(land[[1]]),
          "sf" %in% class(land),
          "SpatVector" %in% class(land))){
    warning(paste0("'land' must be NA or of class 'sf' or 'SpatVector'."))
    return(NULL)
  }

  if(!all(c(is.character(rast1Name),
            is.character(rast2Name),
            is.character(title)))){
    warning(paste0("'rast1Name', 'rast2Name', and 'title'
                   must be 'character' strings.\n"))
    return(NULL)
  }

  if(!is.logical(graticule)){
    warning(paste0("'graticule' must be 'TRUE' or 'FALSE'.\n"))
    return(NULL)
  }


  if(any(all(!inherits(rast1, what = "SpatRaster"), !is.null(rast1)),
         all(!inherits(rast2, what = "SpatRaster"), !is.null(rast2)))){
    warning(paste0("'rast1' and 'rast2', must either be objects of type
                   'SpatRaster' or NULL.\n"))
    return(NULL)
  }

  if(any(is.null(rast1), is.null(rast2))){
    if(all(is.null(rast1),!is.null(rast2))){
      rast1 <- rast2
      values(rast1) <- 0
      message(paste0("'rast1' was null. Replaced with blank raster."))
    }else if(all(is.null(rast2),!is.null(rast1))){
      rast2 <- rast1
      values(rast2) <- 0
      message(paste0("'rast2' was null. Replaced with blank raster."))
    } else{
      rast1 <- rast(ncol=10, nrow=10)
      values(rast1) <- 0
      rast2 <- rast(ncol=10, nrow=10)
      values(rast2) <- 0
      message(paste0("Both 'rast1' and 'rast2' were null.\n",
                     "They were replaced with blank rasters."))
    }
  }

  # Here is where the function actually starts
  myCols <- c(transpColor("white", percent = 100),
              transpColor(col1, percent = 50),
              transpColor(col2, percent = 50),
              blendColor(transpColor(col1, percent = 50),
                         transpColor(col2, percent = 50)))

  plot(rast1, col = c(myCols[1], myCols[2]), legend = FALSE, axes = TRUE, mar = c(2,2,3,2))
  plot(rast2, col = c(myCols[1], myCols[3]), legend = FALSE, axes = FALSE, add = T)

  if(graticule){
    grat <- graticule(lon = seq(-180, 180, 10), lat = seq(-90,90,10), crs = crs(rast1))
    plot(crop(grat, rast1), col="gray50", labels = FALSE, add = TRUE)
  }

  if(!any(is.na(land))){
    if(!("SpatVector" %in% class(land))){
      land <- vect(land)
    }
    plot(land, col = landCol, add = TRUE)
  }

  title(main = title, cex.main = 1.1)

  # Legend plotting
  if(sum(terra::minmax(rast1), terra::minmax(rast2)) == 0){
    legend(x = round(xmax(rast1)) + 1, y = round(ymax(rast1)) + 1,
           bty = "n",
           legend = c("Neither"),
           fill = myCols[1])
  }else if(terra::minmax(rast1)[2] == 0){
    legend(x = round(xmax(rast1)) + 1, y = round(ymax(rast1)) + 1,
           bty = "n",
           legend = c("Neither", rast2Name),
           fill = myCols[c(-2, -4)])
  }else if(terra::minmax(rast2)[2] == 0){
    legend(x = round(xmax(rast1)) + 1, y = round(ymax(rast1)) + 1,
           bty = "n",
           legend = c("Neither", rast1Name),
           fill = myCols[c(-3, -4)])
  } else{
    legend(x = round(xmax(rast1)) + 1, y = round(ymax(rast1)) + 1,
           bty = "n",
           legend = c("Neither", rast1Name, rast2Name, "Both"),
           fill = myCols)
  }

  plotResult <- recordPlot()
  return(plotResult)
}

#' @title Diversity stack
#'
#' @description Takes list of rasters of species distributions
#' (interpreted as 1 = presence, 0 = absence), which do not
#' have to have the same extents, and stack them to create an
#' estimate of species richness that matches the extent and
#' resolution of a template.
#'
#' @param rasterList A `list` of `SpatRaster` objects, which
#' are interpreted as species distributions (1 = presence,
#' 0 = absence).
#'
#' @param template A `SpatRaster` with the desired extent
#'
#' @return A `SpatRaster`
#'
#' @examples
#' library(terra)
#' rast1 <- rast(ncol=10, nrow=10)
#' values(rast1) <- rep(0:1, 50)
#'
#' rast2 <- rast(ncol=10, nrow=10)
#' values(rast2) <- c(rep(0, 50), rep(1,50))
#'
#' rastList <- list(rast1, rast2)
#' result <- diversityStack(rasterList = rastList,
#'                          template = rast2)
#' result
#' plot(result)
#'
#' @importFrom terra rast resample
#' @keywords plotting
#' @export

diversityStack <- function(rasterList, template){
  if(!inherits(rasterList, what = "list")){
    warning(paste0("'rasterList' must be of class 'list'.\n"))
    return(NULL)
  }

  if(!all(unlist(lapply(rasterList,
                        function(X){grepl("SpatRaster", class(X))})))){
    warning(paste0("All objects in 'rasterList' must be of class 'SpatRaster'.\n"))
    return(NULL)
  }

  if(!grepl("SpatRaster", class(template))){
    warning(paste0("'template' must be of class 'SpatRaster'.\n"))
    return(NULL)
  }

  diversityRaster <- template
  values(diversityRaster) <- 0

  for(i in rasterList){
    temp <- i
    if(testIntersection(temp,template)){
      temp <- terra::resample(temp, template, method = "near")
      temp <- subst(temp, NA, 0)
      diversityRaster <- diversityRaster + temp
    }
  }
  return(diversityRaster)
}

#' @title Single raster plot
#'
#' @description A convenient wrapper around `ggplot`
#' to generate a formatted plot of a single raster.
#'
#' @param rast A single `SpatRaster` layer on a continuous
#' scale.
#'
#' @param land An optional coastline polygon shapefile
#' of types `sf` or `SpatRaster` to provide geographic
#' context for the occurrence points.
#'
#' @param landCol Color for land on map.
#'
#' @param scaleRange Optional numeric vector containing
#' maximum and minimum values for color scale. Helpful
#' when making multiple plots for comparison. Defaults
#' to minimum and maximum of input `rast`.
#'
#' @param title A title for the plot.
#'
#' @param verbose `logical`. Switching to `FALSE` mutes message alerting
#' user if input `rast` values exceed a specified `scaleRange`.
#'
#' @param graticule `logical`. Do you want a grid of lon/lat lines?
#'
#' @param ... Additional optional arguments to pass to
#' `plot` initial plot object or `viridis`.
#'
#' @return A plot of mapping the values of the input raster layer
#'
#' @examples
#' library(terra)
#' rast <- rast(ncol=10, nrow=10)
#' values(rast) <- seq(0,99, 1)
#'
#' oneRasterPlot(rast = rast)
#'
#' @import terra
#' @importFrom viridisLite viridis
#'
#' @seealso \code{\link[viridisLite:viridis]{viridis}} \code{\link[ggplot2:ggplot]{ggplot}}
#'
#' @keywords plotting
#' @export

oneRasterPlot <- function(rast,
                          land = NA, landCol = "black",
                          scaleRange = NA,
                          graticule = TRUE,
                          title = "A Raster",
                          verbose = TRUE, ...){
  #Input processing
  args <- list(...)

  if("alpha" %in% names(args)){
    alpha <- args$alpha
  } else{
    alpha <- 1
  }

  if("option" %in% names(args)){
    option <- args$option
  } else{
    option <- "plasma"
  }

  if("plotLegend" %in% names(args)){
    plotLegend <- args$plotLegend
  } else{
    plotLegend <- FALSE
  }

  if("legendMargin" %in% names(args)){
    legendMargin <- args$legendMargin
  } else{
    legendMargin <- 3
  }

  if("n" %in% names(args)){
    n <- args$n
  } else{
    n = 11
  }

  if("varName" %in% names(args)){
    varName <- args$varName
  } else{
    varName <- "Variable"
  }

  if("legendRound" %in% names(args)){
    legendRound <- args$legendRound
  } else{
    legendRound <- 2
  }

  # Input error checking
  if(!grepl("SpatRaster", class(rast))){
    warning(paste0("'rast' must be of class 'SpatRaster'.\n"))
    return(NULL)
  }

  if(!any(is.na(land[[1]]),
          "sf" %in% class(land),
          "SpatVector" %in% class(land))){
    warning(paste0("'land' must be NA or of class 'sf' or 'SpatVector'."))
    return(NULL)
  }

  if(!any(is.na(land))){
    if(!("sf" %in% class(land))){
      land <- sf::st_as_sf(land)
    }
  }

  if(!all(is.na(scaleRange))){
    if(!all(any("numeric" %in% class(scaleRange)),
            length(scaleRange) == 2)){
      warning(paste0("'scaleRange' must either be NA or\na numeric vector of length 2."))
      return(NULL)
    }
  }

  if (!is.logical(graticule)) {
    warning(message("Argument 'graticule' is not of type 'logical'.\n"))
    return(NULL)
  }

  colTest <- areColors(landCol)

  if(!any(c(all(colTest), length(colTest) < 1))){
    warning(paste0("'landCol' must be a recognized color.\n"))
    return(NULL)
  }

  if (!is.logical(verbose)) {
    warning(message("Argument 'verbose' is not of type 'logical'.\n"))
    return(NULL)
  }

  #Function body
  if(any(is.na(scaleRange))){
    begin <- unlist(global(rast, min, na.rm = TRUE))
    end <- unlist(global(rast, max, na.rm = TRUE))
  } else if(any(global(rast, min, na.rm = TRUE) < min(scaleRange),
                global(rast, max, na.rm = TRUE) > max(scaleRange))){
    begin <- unlist(global(rast, min, na.rm = TRUE))
    end <- unlist(global(rast, max, na.rm = TRUE))

    if(verbose){
      message(paste0("Input raster extremes exceed specified scale range.\n",
                     "Using input raster max and min instead."))
    }

  } else{
    begin <- min(scaleRange)
    end <- max(scaleRange)
  }

  colVals <- seq(from = begin, to = end, by = (end-begin)/n)
  colVals <- round(colVals, legendRound)
  colVals <- data.frame("from" = colVals[1:(length(colVals)-1)],
                        "to" = colVals[2:length(colVals)],
                        "color" = viridisLite::viridis(n = length(colVals)-1, alpha = 1, option = option))

  if(is.na(title)){
    adjustedMargin <- c(2,2,0,2)
  } else{
    adjustedMargin <- c(2,2,3,2)
  }

  if(plotLegend){
    adjustedMargin[[4]] <- legendMargin
  }

  plot(rast,
       col = colVals,
       legend = plotLegend, mar = adjustedMargin,
       axes = TRUE)

  if(graticule){
    grat <- graticule(lon = seq(-180, 180, 10), lat = seq(-90,90,10), crs = crs(rast))
    plot(grat, col="gray50", labels = FALSE, add = TRUE)
  }

  if(!any(is.na(land))){
    if(!("SpatVector" %in% class(land))){
      land <- vect(land)
    }
    plot(land, col = landCol, add = TRUE)
  }

  if(plotLegend){
    legend(x = round(xmax(rast)) + 1,
           yjust = 0,
           title = varName,
           bty = "n",
           legend = paste0(round(colVals[,1], legendRound),
                           " - ", round(colVals[,2], legendRound)),
           fill = colVals[,3])
  }

  if(!is.na(title)){
    title(main = title, cex.main = 1.1)
  }

  finalPlot <- recordPlot()

  return(finalPlot)
}

#' @title Plotting 3D model in 2D
#'
#' @description This script plots a semitransparent layer
#' of suitable habitat for each depth layer. The redder
#' the color, the shallower the layer, the bluer, the
#' deeper. The more saturated the color, the more layers
#' with suitable habitat.
#'
#' @param rast A `SpatRaster` vector with the 3D presence/absence
#' distribution of a species (interpreted as 1 = presence,
#' 0 = absence).
#'
#' @param land An optional coastline polygon shapefile
#' of types `sf` or `SpatRaster` to provide geographic
#' context for the occurrence points.
#'
#' @param landCol Color for land on map.
#'
#' @param title A title for the plot. If not title is
#' supplied, the title "Suitability from (MINIMUM
#' DEPTH) to (MAXIMUM DEPTH)" is inferred from
#' names of `rast`.
#'
#' @param graticule Do you want a grid of lon/lat lines?
#'
#' @param ... Additional optional arguments.
#'
#' @note Only include the depth layers that you actually
#' want to plot.
#'
#' @examples
#' library(terra)
#'
#' rast1 <- rast(ncol=10, nrow=10)
#' values(rast1) <- rep(0:1, 50)
#'
#' rast2 <- rast(ncol=10, nrow=10)
#' values(rast2) <- c(rep(0, 50), rep(1,50))
#'
#' rast3 <- rast(ncol=10, nrow=10)
#' values(rast3) <- rep(c(1,0,0,1), 25)
#'
#' distBrick <- c(rast1, rast2, rast3)
#'
#' plotLayers(distBrick)
#'
#' @import terra
#' @importFrom grDevices recordPlot
#'
#' @seealso \code{\link[viridisLite:viridis]{viridis}}
#'
#' @keywords plotting
#'
#' @return A plot of class `recordedplot`
#'
#' @export

plotLayers <- function(rast,
                      land = NA, landCol = "black",
                      title = NULL,
                      graticule = TRUE, ...){
  #Input processing
  args <- list(...)

  # Input error checking
  if(!grepl("SpatRaster", class(rast))){
    warning(paste0("'rast' must be of class 'SpatRaster'.\n"))
    return(NULL)
  }

  if(!any(is.na(land[[1]]),
          "sf" %in% class(land),
          "SpatVector" %in% class(land))){
    warning(paste0("'land' must be NA or of class 'sf' or 'SpatVector'."))
    return(NULL)
  }

  colTest <- areColors(landCol)

  if(!any(c(all(colTest), length(colTest) < 1))){
    warning(paste0("'landCol' must be a recognized color.\n"))
    return(NULL)
  }

  if (!is.logical(graticule)) {
    warning(message("Argument 'graticule' is not of type 'logical'.\n"))
    return(NULL)
  }

  if(is.null(title)){
   title <- paste0("Suitability from ",
           names(rast)[[1]]," to ",
           names(rast)[[nlyr(rast)]])
  }


  #Function body
  redVal <- 1
  blueVal <- 0
  stepSize <- 1/(nlyr(rast) + 1)

  plot(rast[[1]],
       col = c(rgb(0,0,0,0),
               rgb(redVal,0,blueVal,stepSize)),
       legend = FALSE, mar = c(2,2,3,2), axes = TRUE)

  for(i in 2:nlyr(rast)){
    redVal <- redVal - stepSize
    blueVal <- blueVal + stepSize
    plot(rast[[i]], col = c(rgb(0,0,0,0),
                            rgb(redVal,0,blueVal,stepSize)),
         legend = FALSE, axes = FALSE, add = TRUE)
  }

  if(graticule){
    grat <- graticule(lon = seq(-180, 180, 10), lat = seq(-90,90,10), crs = crs(rast))
    plot(grat, col="gray50", labels = FALSE, add = TRUE)
  }

  if(!any(is.na(land))){
    if(!("SpatVector" %in% class(land))){
      land <- vect(land)
    }
    plot(land, col = landCol, add = TRUE)
  }

  title(main = title, cex.main = 1.1)

  finalPlot <- recordPlot()

  return(finalPlot)
}

#' @title Vertical sample
#'
#' @description Samples data along a vertical transect
#'
#' @param x A multilayer `SpatRaster` object
#'
#' @param sampleAxis Specifies whether a latitudinal ("lat") or
#' longitudinal ("long") transect is desired.
#'
#' @param axisValue Numeric value specifying transect postion.
#'
#' @note Only unprojected `SpatRaster` files are supported.
#'
#' @return A `data.frame` with values sampled across vertical
#' transect.
#'
#' @keywords internal plotting
#'
#' @export

verticalSample <- function(x = NULL, sampleAxis = "lon", axisValue = NA){

  # Input error checking
  if(any(!grepl("SpatRaster", class(x)), is.null(x))){
    warning(paste0("'x' must be of class 'SpatRaster'.\n"))
    return(NULL)
  }

  if(all(!is.numeric(axisValue), !is.na(axisValue))){
    warning(paste0("'axisValue' must be single numeric value."))
    return(NULL)
  }

  if(!(sampleAxis %in% c("lat", "lon"))){
    warning(paste0("'sampleAxis' must be 'lon' or 'lat'."))
    return(NULL)
  }

  samplePoints <- as.data.frame(x, xy = TRUE, na.rm=FALSE)

  # Takes mean axis value if none specified
  if(is.na(axisValue)){
    if(sampleAxis == "lon"){
      axisValue <-  mean(c(xmin(x), xmax(x)))
    } else{
      axisValue <-  mean(c(ymin(x), ymax(x)))
    }
  }

  if(sampleAxis == "lon"){
    if(all(axisValue > xmin(x), axisValue < xmax(x))){
      axisValue <- samplePoints[which.min(abs(samplePoints[,"x"] - axisValue)), "x"]
    } else{
      warning(paste0("'axisValue' falls outside the extent of 'x'."))
      return(NULL)
    }
  } else{
    if(all(axisValue > ymin(x), axisValue < ymax(x))){
      axisValue <- samplePoints[which.min(abs(samplePoints[,"y"] - axisValue)), "y"]
    } else{
      warning(paste0("'axisValue' falls outside the extent of 'x'."))
      return(NULL)
    }
  }

  # Select coordinates to sample
  if(sampleAxis == "lon"){
    samplePoints <- samplePoints[samplePoints[,"x"] == axisValue,]
  } else{
    samplePoints <- samplePoints[samplePoints[,"y"] == axisValue,]
  }

  samplePoints <- lapply(1:nrow(samplePoints), FUN = function(x){
    cbind(rep(samplePoints[x,"x"], n = (ncol(samplePoints)-2)),
          rep(samplePoints[x,"y"], n = (ncol(samplePoints)-2)),
          as.numeric(colnames(samplePoints[c(-1,-2)])),
          t(samplePoints[x,])[c(-1,-2),])
  })

  samplePoints <- as.data.frame(do.call(rbind, samplePoints))
  colnames(samplePoints) <- c("x", "y", "z", "value")

  if(all(is.na(samplePoints$z))){
    samplePoints <- samplePoints[, !names(samplePoints) == "z"]
  } else{
    zVals <- sort(unique(samplePoints$z))
    zHeights <- vector(mode = "numeric",
                       length = length(zVals))
    for(i in 1:length(zHeights)){
      if ( i == length(zHeights)){
        zHeights[[i]] <- abs(zVals[[i]] - zVals[[i-1]])
      } else{
        zHeights[[i]] <- abs(zVals[[i]] - zVals[[i+1]])
      }
    }

    height <- vector(mode = "numeric", length = nrow(samplePoints))
    for(i in 1:nrow(samplePoints)){
      height[[i]] <- zHeights[which(zVals %in% samplePoints$z[[i]])]
    }
    samplePoints$height <- height
  }

  samplePoints <- samplePoints[complete.cases(samplePoints),]

  if(sampleAxis == "lon"){
    samplePoints <- samplePoints[,2:5]
  } else{
    samplePoints <- samplePoints[,-2]
  }

  colnames(samplePoints) <- c("x", "y", "value", "height")

  return(samplePoints)
}

#' @title Plot vertical sample
#'
#' @description Plots cell values along a vertical transect
#'
#' @param rast A multilayer `SpatRaster` object, with names
#' corresponding to the z coordinate represented by the layer.
#' These names must be interpretable by `as.numeric`.
#'
#' @param sampleAxis Specifies whether a latitudinal ("lat") or
#' longitudinal ("long") transect is desired.
#'
#' @param axisValue Numeric value specifying transect postion.
#'
#' @param scaleRange A numeric vector of length 2, specifying
#' the range that should be used for the plot color scale.
#'
#' @param plotLegend `logical`, controls whether legend is plotted.
#'
#' @param depthLim A single vector of class `numeric`. How deep
#' should the plot go?
#'
#' @param transRange A `numeric` vector of lenghth 2. How far
#' along the transect should be plotted?
#'
#' @param transTicks `numeric`, spacing between breaks on x axis.
#'
#' @param verbose `logical`. Switching to `FALSE` mutes message
#' alerting user if input `rast` values exceed specified `scaleRange`.
#'
#' @param ... Additional optional arguments to pass to `viridis`.
#'
#' @note Only unprojected `SpatRaster` files are supported.
#'
#' @return A `ggplot` showing a vertical slice through the `SpatRaster`.
#'
#' @examples
#' library(terra)
#'
#' rast1 <- rast(ncol=10, nrow=10)
#' values(rast1) <- rep(0:3, 50)
#'
#' rast2 <- rast(ncol=10, nrow=10)
#' values(rast2) <- c(rep(0, 50), rep(1,25), rep(2,25))
#'
#' rast3 <- rast(ncol=10, nrow=10)
#' values(rast3) <- rep(c(1,3,2,1), 25)
#'
#' distBrick <- c(rast1, rast2, rast3)
#' names(distBrick) <- c(0:2)
#'
#' transectPlot(distBrick, depthLim = 3)
#'
#'
#' @importFrom ggplot2 ggplot
#' @importFrom metR scale_x_latitude
#'
#' @keywords plotting
#'
#' @export

transectPlot <- function(rast = NULL,
                         sampleAxis = "lon",
                         axisValue = NA,
                         scaleRange = NA,
                         plotLegend = TRUE,
                         depthLim = as.numeric(max(names(rast))),
                         transRange = c(-90,90),
                         transTicks = 20,
                         verbose = FALSE,
                         ...){
  #Input processing
  args <- list(...)

  if("option" %in% names(args)){
    option <- args$option
  } else{
    option <- "plasma"
  }

  if("n" %in% names(args)){
    n <- args$n
  } else{
    n <- 11
  }

  if("legendRound" %in% names(args)){
    legendRound <- args$legendRound
  } else{
    legendRound <- 2
  }

  # Input error checking
  if(any(!grepl("SpatRaster", class(rast)), is.null(rast))){
    warning(paste0("'rast' must be of class 'SpatRaster'.\n"))
    return(NULL)
  }

  nameTest <- suppressWarnings(as.numeric(names(rast)))
  if(any(is.na(nameTest))){
    warning(paste0("Names of 'rast' must be interpretable as 'numeric'.\n"))
    return(NULL)
  }

  if(!(sampleAxis %in% c("lat", "lon"))){
    warning(paste0("'sampleAxis' must be 'lon' or 'lat'."))
    return(NULL)
  }

  if(all(!is.numeric(axisValue), !is.na(axisValue))){
    warning(paste0("'axisValue' must be single numeric value."))
    return(NULL)
  }

  if(!all(is.na(scaleRange))){
    if(!all(any("numeric" %in% class(scaleRange)),
            length(scaleRange) == 2)){
      warning(paste0("'scaleRange' must either be NA or\na numeric vector of length 2."))
      return(NULL)
    }
  }

  # Get vertical transect
  verticalTransect <- try(verticalSample(x = rast, sampleAxis = sampleAxis,
                                     axisValue = axisValue))
  if(is.null(verticalTransect)){
    return(NULL)
  } else {
    # Get information for color scale
    if(any(is.na(scaleRange))){
      begin <- min(verticalTransect$value)
      end <- max(verticalTransect$value)
      scaleRange <- c(begin, end)
    } else if(any(min(verticalTransect$value) < min(scaleRange),
                  max(verticalTransect$value) > max(scaleRange))){
      begin <- min(verticalTransect$value)
      end <- max(verticalTransect$value)

      if(verbose){
        message(paste0("Input extremes exceed specified scale range.\n",
                       "Using input max and min instead."))
      }
    } else{
      begin <- min(scaleRange)
      end <- max(scaleRange)
    }

    at <- seq(from = begin, to = end, by = (end-begin)/n)
    at <- round(at, legendRound)[c(-1,-length(at))]

    begin <- (begin - min(verticalTransect$value)/diff(scaleRange))
    end <- max(verticalTransect$value)/end

    # Tiled heatmap
    # Kluge to get rid of annoying note
    x <- NA
    y <- NA
    value <- NA
    height <- NA

    transect.tile <- ggplot(verticalTransect) +
      geom_tile(aes(x = x, y = y, fill = value, height = height), na.rm = TRUE) +
      coord_cartesian(expand = FALSE) +
      labs(x = NULL, y = "Water Depth (m)") +
      scale_fill_viridis_b(option = option, begin = begin,
                           end = end, breaks = at) +
      theme_bw() +
      theme(panel.background = element_rect(fill = "grey90"),
            panel.grid.major = element_line(linetype = 3, colour = "grey60"),
            axis.text = element_text(colour = 1, size = 10),
            axis.title = element_text(colour = 1, size = 12)) +
      scale_y_reverse() +
      scale_x_latitude(ticks = transTicks, position = "bottom",
                       breaks = seq(transRange[[1]], transRange[[2]],
                                    by = transTicks)) +
      guides(fill = "none") +
      theme(plot.margin = margin(.5,.5,.5,0, "cm")) +
      ylim(depthLim, -0.5) + # Allows plot at surface
      xlim(transRange)

    if(plotLegend){
      transect.tile <- transect.tile +
        guides(fill = guide_legend(title.position = "right", direction = "vertical",
                                   title.theme = element_text(angle = 90, size = 12,
                                                              colour = "black"),
                                   barheight = .5, barwidth = .95,
                                   title.hjust = 0.5,
                                   title = "Value"))
    }
    return(transect.tile)
  }
}

library(rnaturalearth)
library(ggplot2)
library(terra)

occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv",
                             package='voluModel'))
spName <- "Steindachneria argentea"
land <- ne_countries(scale = "medium", returnclass = "sf")[1]

test_that("areColors works as expected", {
  expect_error(voluModel:::areColors())
  expect_warning(voluModel:::areColors(NULL))
  expect_true(is.logical(voluModel:::areColors("red")))
})

test_that("pointMap checks", {
  expect_error(pointMap())
  expect_warning(pointMap(occs = "a",
                          spName = spName))
  expect_warning(pointMap(occs = occs[,c(1,3)],
                          spName = spName))
  expect_warning(pointMap(occs=occs, spName = 2))
  expect_warning(pointMap(occs = occs,
                          spName = spName, land = "a"))
  expect_warning(pointMap(occs = occs,
                          spName = spName, land = land,
                          ptCol = "eggs"))
  expect_warning(pointMap(occs = occs,
                          spName = spName, land = land,
                          ptSize = "a"))
  expect_warning(pointMap(occs = occs,
                          spName = spName, land = land,
                          verbose = "banana"))
  point_map <- pointMap(occs = occs, alpha = 1,
                        spName = spName, land = land)
  expect_true(is_ggplot(point_map))
  point_map <- pointMap(occs = occs,
                        spName = spName, land = NA)
  expect_true(is_ggplot(point_map))
})

set.seed(0)
occs1 <- occs[sample(1:nrow(occs),
                size = 24, replace = FALSE),]

set.seed(10)
occs2 <- occs[sample(1:nrow(occs),
                size = 24, replace = FALSE),]

test_that("pointCompMap checks", {
  expect_error(pointCompMap())
  expect_warning(pointCompMap(occs1 = "a", occs2 = occs2,
                              spName = spName))
  expect_warning(pointCompMap(occs1 = occs1[,c(1,3)], occs2 = occs2,
                              spName = spName))
  expect_warning(pointCompMap(occs2 = "a", occs1 = occs1,
                              spName = spName))
  expect_warning(pointCompMap(occs2 = occs2[,c(1,3)], occs1 = occs1,
                              spName = spName))
  expect_warning(pointCompMap(occs1=occs1, occs2 = occs2,
                              spName = 2))
  expect_warning(pointCompMap(occs1 = occs1, occs2 = occs2,
                              spName = spName, land = "a"))
  expect_warning(pointCompMap(occs1 = occs1, occs2 = occs2,
                              spName = spName, land = land,
                          occs1Col = "eggs"))
  expect_warning(pointCompMap(occs1 = occs1, occs2 = occs2,
                              spName = spName, land = land,
                          ptSize = "a"))
  expect_warning(pointCompMap(occs1 = occs1, occs2 = occs2,
                              spName = spName, land = land,
                              verbose = "banana"))

  badColNames <- occs1
  colnames(badColNames)[1:2] <- c("x", "y")
  expect_warning(pointCompMap(occs1 = badColNames, occs2 = occs2,
                              spName = spName, land = land))

  point_map <- pointCompMap(occs1 = occs1, occs2 = occs2,
                            spName = spName, land = land)
  expect_true(is_ggplot(point_map))
  point_map <- pointCompMap(occs1 = occs1, occs2 = occs2,
                            spName = spName, land = vect(land))
  expect_true(is_ggplot(point_map))
  point_map <- pointCompMap(occs1 = occs1, occs2 = occs2,
                            spName = spName, land = NA)
  expect_true(is_ggplot(point_map))
  point_map <- pointCompMap(occs1 = rbind(occs1,occs2), occs2 = occs2,
                            spName = spName, land = NA)
  expect_true(is_ggplot(point_map))
  point_map <- pointCompMap(occs1 = occs1, occs2 = rbind(occs1,occs2),
                            spName = spName, land = NA)
  expect_true(is_ggplot(point_map))
})

test_that("transpColor checks", {
  expect_error(voluModel:::transpColor())
  expect_warning(voluModel:::transpColor(color = "eggs"))
  expect_warning(voluModel:::transpColor(color = "red",
                                         percent = "blue"))
  expect_warning(voluModel:::transpColor(color = "red",
                                         percent = 200))
  expect_equal(class(voluModel:::transpColor(color = "red",
                                             percent = 20)), "character")
})

test_that("blendColor checks", {
  expect_warning(voluModel:::blendColor(col1 = "eggs"))
  expect_warning(voluModel:::blendColor(col1 = "#1B9E777F",
                                        col2 = "eggs"))
  expect_equal(class(voluModel:::blendColor(col1 = "#1B9E777F",
                                        col2 = "black")), "character")
  expect_equal(class(voluModel:::blendColor(col1 = "#1B9E777F",
                                            col2 = "#7570B37F")), "character")
})

# Set up tests for rasterComp function
rast1 <- rast(ncol=10, nrow=10)
values(rast1) <- rep(0:1, 50)

rast2 <- rast(ncol=10, nrow=10)
values(rast2) <- c(rep(0, 50), rep(1,50))

test_that("rasterComp works", {
  expect_warning(rasterComp(rast1 = "a"))
  expect_warning(rasterComp(rast1 = rast1, rast1Name = 2))
  expect_warning(rasterComp(rast1 = rast1, rast1Name = "First Raster",
                            rast2 = rast2, rast2Name = "Second Raster",
                            graticule = "b"))
  expect_warning(rasterComp(rast1 = rast1, rast1Name = "First Raster",
                            rast2 = rast2, rast2Name = "Second Raster",
                            land = "b"))
  expect_warning(rasterComp(rast1 = rast1, rast1Name = "First Raster",
                            rast2 = rast2, rast2Name = "Second Raster",
                            col1 = "ukulele"))
 expect_equal(class(rasterComp(rast1 = rast1)), "recordedplot")
 expect_equal(class(rasterComp(rast2 = rast2)), "recordedplot")
expect_equal(class(rasterComp(graticule = FALSE)), "recordedplot")
expect_equal(class(rasterComp(rast1 = rast1, rast2 = rast2)), "recordedplot")
 expect_equal(class(rasterComp(rast1 = rast1, land = land)), "recordedplot")
 expect_equal(class(rasterComp(rast2 = rast2, land = land)), "recordedplot")
 expect_equal(class(rasterComp(land = land)), "recordedplot")
  expect_equal(class(rasterComp(rast1 = rast1, rast2 = rast2, land = land)),
               "recordedplot")
})

test_that("oneRasterPlot works", {
  divStack <- diversityStack(list(rast1, rast2), template = rast2)
  expect_warning(oneRasterPlot(rast = "a"))
  expect_warning(oneRasterPlot(rast = divStack, land = "a"))
  expect_warning(oneRasterPlot(rast = divStack, graticule = "a"))
  expect_warning(oneRasterPlot(rast = divStack, landCol = "a"))
  expect_warning(oneRasterPlot(rast = divStack, scaleRange = "a"))
  expect_warning(oneRasterPlot(rast = divStack, verbose = "a"))
  expect_warning(oneRasterPlot(rast = divStack, scaleRange = c(1)))
  expect_message(oneRasterPlot(rast = divStack, scaleRange = c(1,10), verbose = TRUE))
  expect_equal(class(oneRasterPlot(rast = divStack)), c("recordedplot"))
  expect_equal(class(oneRasterPlot(rast = divStack,
                                   scaleRange = c(-1, 80))), c("recordedplot"))
  expect_equal(class(oneRasterPlot(divStack, land = land)), c("recordedplot"))
  expect_equal(class(oneRasterPlot(divStack, land = land,
                                   alpha = .5)), c("recordedplot"))
  expect_equal(class(oneRasterPlot(divStack, land = land,
                                   option = "mako")), c("recordedplot"))
  expect_equal(class(oneRasterPlot(divStack, land = land,
                                   plotLegend = TRUE)), c("recordedplot"))
  expect_equal(class(oneRasterPlot(divStack, land = land,
                                   n = 3)), c("recordedplot"))
  expect_equal(class(oneRasterPlot(divStack, land = land,
                                   varName = "Test")), c("recordedplot"))
  expect_equal(class(oneRasterPlot(divStack, land = land,
                                   legendRound = 1)), c("recordedplot"))
})

rast1 <- rast(ncol=10, nrow=10)
values(rast1) <- rep(0:1, 50)

rast2 <- rast(ncol=10, nrow=5)
values(rast2) <- c(rep(0, 25), rep(1,25))
rasterList <- list(rast1, rast2)

test_that("diversityStack works", {
  expect_warning(diversityStack(rasterList = "a"))
  expect_warning(diversityStack(rasterList = list("eggs", "bacon", "spam")))
  expect_warning(diversityStack(rasterList = rasterList, template = "b"))
  divStack <- diversityStack(rasterList = rasterList, template = rast2)
  expect_true(grepl("SpatRaster", class(divStack)))
})

rast1 <- rast(ncol=10, nrow=10)
values(rast1) <- rep(0:1, 50)

rast2 <- rast(ncol=10, nrow=10)
values(rast2) <- c(rep(0, 50), rep(1,50))

rast3 <- rast(ncol=10, nrow=10)
values(rast3) <- rep(c(1,0,0,1), 25)
distBrick <- c(rast1, rast2, rast3)
names(distBrick) <- c(1,2,3)

test_that("plotLayers works", {
  expect_warning(plotLayers(rast = "a"))
  expect_warning(plotLayers(rast = distBrick, land = "a"))
  expect_warning(plotLayers(rast = distBrick, landCol = "bacon"))
  expect_warning(plotLayers(rast = distBrick, graticule = 10))
  expect_equal(class(plotLayers(distBrick, land = land)), "recordedplot")
})

rast1 <- rast(ncol=10, nrow=10)
values(rast1) <- rep(0:3, 25)

rast2 <- rast(ncol=10, nrow=10)
values(rast2) <- c(rep(0, 50), rep(1,25), rep(2,25))

rast3 <- rast(ncol=10, nrow=10)
values(rast3) <- rep(c(1,3,2,1), 25)

distBrick <- c(rast1, rast2, rast3)
names(distBrick) <- c(0:2)

test_that("verticalSample works", {
  expect_warning(verticalSample())
  expect_warning(verticalSample(x = distBrick,
                                sampleAxis = "nope"))
  expect_warning(verticalSample(x = distBrick,
                                axisValue = "nope"))
  expect_warning(verticalSample(x = distBrick, axisValue = "nope"))
  expect_warning(verticalSample(x = distBrick, axisValue = 200))
  expect_warning(verticalSample(x = distBrick, sampleAxis = "lat",
                                axisValue = 200))
  sampleResult <- voluModel::verticalSample(distBrick)
  expect_equal(class(sampleResult), "data.frame")
  sampleResult <- voluModel::verticalSample(distBrick, sampleAxis = "lat")
  expect_equal(class(sampleResult), "data.frame")
})

test_that("transectPlot works", {
  expect_warning(transectPlot())
  expect_warning(transectPlot(rast = distBrick,
                                sampleAxis = "nope"))
  expect_warning(transectPlot(rast = distBrick,
                                axisValue = "nope"))
  expect_warning(transectPlot(rast = distBrick, axisValue = "nope"))
  expect_warning(transectPlot(rast = distBrick, axisValue = 200))
  expect_warning(transectPlot(rast = distBrick, sampleAxis = "lat",
                                axisValue = 200))
  distBrickBroken <- distBrick
  names(distBrickBroken) <- c("eggs", "bacon", "spam")
  expect_warning(transectPlot(rast = distBrickBroken, sampleAxis = "lat"))
  expect_warning(transectPlot(rast = distBrick, scaleRange = 1))
  expect_warning(transectPlot(rast = distBrick, scaleRange = 20))
  expect_message(voluModel::transectPlot(distBrick, scaleRange = c(1,2),
                                          verbose = T))
  sampleResult <- voluModel::transectPlot(distBrick, scaleRange = c(1,2),
                                          verbose = T)
  expect_true("ggplot" %in% class(sampleResult))
  sampleResult <- voluModel::transectPlot(distBrick, sampleAxis = "lat",
                                          option = "mako", n = 4,
                                          scaleRange = c(0,10),
                                          legendRound = 0)
  expect_true("ggplot" %in% class(sampleResult))
})

#' @title Test different threshold levels and produce presence/absence layers for
#' 3D models
#'
#' @description Creates 3D presence/absence layers by setting all values in a suitability
#' layer above a certain threshold to 1 and all values below that threshold to 0. The
#' threshold value is determined by the sensitivity given by the user, where the
#' sensitivity is the percentage of occurrences to be counted as present in the
#' resulting presence/absence layer. For example, with a sensitivity of 0.9 or 90%,
#' The threshold is the suitability value where 90% of the occurrence points fall on
#' grid cells with suitability scores above that value.
#'
#' The function can try multiple different sensitivity levels, and for each will output
#' a presence/absence spatRaster stack, as well as the specificity (proportion of
#' psuedoabsences correctly considered absent), the true skill statistic (TSS), which is
#' a measure of how well the threshold balances commission and omission error, and the
#' suitability value of the threshold.
#'
#' The user can also downweight sensitivity when calculating TSS with the "weights"
#' parameter. This may be important to do as 3D niche models often have a higher ratio
#' of psuedoabsences to occurrences than 2D models.
#'
#' @param predicted_layers A spatRaster stack of suitability layers where each layer
#' corresponds to a depth slice
#'
#' @param thresholding_vals A vector of sensitivity levels for creating different
#' thresholds and presence/absence layers. If one wanted to test sensitivities of
#' 90%, and 95%, this would be input as c(0.9, 0.95)
#'
#' @param maxent_df 'data.frame' where first column is a vector
#' of presences named "p" containing 1's and 0's. Each row represents a cell in the
#' spatRaster volume with an x, y, z coordinate, and 1's are presences while 0's are
#' absences, or background points. Other columns are environmental variable values
#' extracted at the occurrence and background points.
#'
#' @param coord_df A dataframe containing the longitude, latitude, and depth for each
#' cell in maxent_df, named "longitude," "latitude," and "depth"
#'
#' @param weights a numeric giving what the sensitivity should be downweighted by. If
#' no value is given, TSS will be calculated according to its original formula
#'
#' @references Allouche O, Tsoar A, and Kadmon R. 2006. Assessing the accuracy of species
#' distribution models: prevalence, kappa and the true skill statistic (TSS). *Journal of
#' Applied Ecology*, 43, 1223-1232.
#'
#' @return a 'list' with two components:
#'
#' $threshold_layers, a spatRaster stack of presence absence rasters thresholded at the
#' sensitivity with the highest TSS
#'
#' $tss_results, a 'data.frame' containing the specificity, TSS, and suitability score
#' for each sensitivity given in thresholding_vals
#'
#' @examples
#' library(terra)
#' library(dplyr)
#'
#' # creating list of spatRaster stacks where each element is a depth slice
#' r1_d1 <- rast(ncol = 100, nrow = 100)
#' set.seed(0)
#' values(r1_d1) <- sample(c(1:100), size = 1000, replace = TRUE)
#' r2_d1 <- rast(ncol = 100, nrow = 100)
#' set.seed(0)
#' values(r2_d1) <- sample(c(1:1000), size = 1000, replace = TRUE)
#' r1_d2 <- r1_d1
#' values(r1_d2) <- values(r1_d1)+10
#' r2_d2 <- r2_d1
#' values(r2_d2) <- values(r2_d1)+10
#' d1 <- c(r1_d1, r2_d1)
#' names(d1) <- c("valsr1", "valsr2")
#' d2 <- c(r1_d2, r2_d2)
#' names(d2) <- c("valsr1", "valsr2")
#' envlist <- list(d1, d2)
#'
#' # creating occs and bgs
#' set.seed(0)
#' occs <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 50, replace = FALSE)
#' bgs <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 500, replace = FALSE)
#'
#' occs_d1 <- crds(envlist[[1]][[1]])[occs[1:25],]
#' occs_d2 <- crds(envlist[[2]][[1]])[occs[26:50],]
#' bg_d1 <- crds(envlist[[1]][[1]])[bgs[1:250],]
#' bg_d2 <- crds(envlist[[1]][[1]])[bgs[251:500],]
#'
#' # extracting at occs and bgs
#' occ_valsr1_d1 <- extract(envlist[[1]][[1]], occs_d1)
#' occ_valsr1_d2 <- extract(envlist[[2]][[1]], occs_d2)
#' occ_valsr2_d1 <- extract(envlist[[1]][[2]], occs_d1)
#' occ_valsr2_d2 <- extract(envlist[[2]][[2]], occs_d2)
#'
#' occ_valsr1 <- rbind(occ_valsr1_d1, occ_valsr1_d2)
#' occ_valsr2 <- rbind(occ_valsr2_d1, occ_valsr2_d2)
#'
#' bg_valsr1_d1 <- extract(envlist[[1]][[1]], bg_d1)
#' bg_valsr1_d2 <- extract(envlist[[2]][[1]], bg_d2)
#' bg_valsr2_d1 <- extract(envlist[[1]][[2]], bg_d1)
#' bg_valsr2_d2 <- extract(envlist[[2]][[2]], bg_d2)
#'
#' bg_valsr1 <- rbind(bg_valsr1_d1, bg_valsr1_d2)
#' bg_valsr2 <- rbind(bg_valsr2_d1, bg_valsr2_d2)
#'
#' valsr1 <- rbind(occ_valsr1, bg_valsr1)
#' valsr2 <- rbind(occ_valsr2, bg_valsr2)
#'
#' p1 <- rep(1, times = 50)
#' p0 <- rep(0, times = 500)
#' p <- c(p1, p0)
#'
#' maxdf <- data.frame(p, valsr1, valsr2)
#'
#' # creating coord_df
#' coord_df <- rbind(occs_d1, occs_d2, bg_d1, bg_d2)
#' z <- c(rep(1, times = 25), rep(2, times = 25), rep(1, times = 250),
#' rep(2, times = 250))
#' coord_df <- cbind(coord_df, z)
#' colnames(coord_df) <- c("longitude", "latitude", "depth")
#'
#' # creating suitability rasters
#' suitd1 <- d1[[1]]
#' values(suitd1) <- runif(min = 0, max = 1, n = length(values(suitd1)))
#' suitd2 <- d2[[1]]
#' values(suitd2) <- runif(min = 0, max = 1, n = length(values(suitd2)))
#' suit <- c(suitd1, suitd2)
#'
#' # here's the function
#' result <- threshold_3D(predicted_layers = suit, thresholding_vals = c(0.9, 0.95),
#' maxent_df = maxdf, coord_df = coord_df, weights = 2/3)
#'
#' @importFrom terra extract
#'
#' @keywords threshold
#'
#' @export

threshold_3D <- function(predicted_layers,
                         thresholding_vals, maxent_df, coord_df, weights=NULL) {

  # initial check for proper formatting
  if(!(is.numeric(thresholding_vals))) {
    warning(message("Argument 'thresholding_vals' should be numeric.\n"))
    return(NULL)
  }

  if(any(thresholding_vals > 1)) {
    warning(message("Argument 'thresholding_vals' should be a fraction between 0 and 1.\n"))
    return(NULL)
  }

  if(!("p" %in% colnames(maxent_df))) {
    warning(message("Argument 'maxent_df' is missing column 'p'.\n"))
    return(NULL)
  }

  if(any(!(c("longitude", "latitude", "depth") %in% colnames(coord_df)))) {
    warning(message("Argument 'coord_df' should have columns named 'longitude',
                    'latitude' and 'depth'.\n"))
    return(NULL)
  }

  # making a dataframe occurrences and background
  full_df <- cbind(maxent_df, coord_df)
  occ_df <- full_df[full_df$p == 1,]
  bg_df <- full_df[full_df$p == 0,]

  depth_slices <- unique(bg_df$depth)

  sdm_suit_vals <- vector("list", length = length(depth_slices))
  for(i in 1:length(depth_slices)) {
    need_occ <- occ_df[occ_df$depth == depth_slices[i],]
    if(nrow(need_occ) < 1) {
      sdm_suit_vals[[i]] <- NA
    } else {
      sdm_suit_vals[[i]] <- terra::extract(predicted_layers[[i]],
                                           data.frame(need_occ$longitude, need_occ$latitude))
    }
  }
  sdm_suits <- do.call(rbind, sdm_suit_vals)[,2]
  sdm_suits <- sdm_suits[which(!(is.na(sdm_suits)))]
  sdm_suits <- sdm_suits[order(sdm_suits, decreasing = T)]
  thresh_sdm_list <- vector("list", length = length(thresholding_vals))
  tss_list <- vector(length = length(thresholding_vals))
  spec_list <- vector(length = length(thresholding_vals))
  suit_list <- vector(length = length(thresholding_vals))
  for(i in seq_along(thresholding_vals)) {
    thresholded_layers <- vector("list", length = length(predicted_layers))
    real_abs_list <- vector("list", length = length(depth_slices))
    thresh <- trunc(length(sdm_suits)*thresholding_vals[i])
    thresh_val <- sdm_suits[thresh]
    a <- thresh
    c <- length(sdm_suits) - thresh
    for(j in 1:dim(predicted_layers)[3]) {
      rclmat <- matrix(data = c(0, thresh_val, 0,
                                thresh_val, 1, 1), nrow = 2, ncol = 3, byrow = T)
      thresholded_layers[[j]] <- classify(predicted_layers[[j]],
                                          rcl = rclmat)
      need_bg <- bg_df[bg_df$depth == depth_slices[j],]
      real_abs_list[[j]] <- terra::extract(thresholded_layers[[j]],
                                           data.frame(need_bg$longitude, need_bg$latitude))
    }
    thresh_sdm_list[[i]] <- rast(thresholded_layers)
    real_abs <- do.call(rbind, real_abs_list)[,2]
    specificity <- length(which(real_abs == 0))/length(real_abs)
    sensitivity <- thresholding_vals[i]
    d <- length(which(real_abs == 0))
    b <- length(real_abs) - length(which(real_abs == 0))
    if(is.null(weights)) {
      tss <- ((a*b) + (a*d) - (c*b))/((a+c)*(b+d))
    } else {
      weightnum <- weights*100
      weightden <- 100
      tss <- ((-(weightnum)*a*b) + (a*d) - ((weightden)*c*b))/((weightden)*(a+c)*(b+d))
    }
    tss_list[i] <- tss
    spec_list[i] <- specificity
    suit_list[i] <- thresh_val
  }
  choose_final <- which(tss_list == max(tss_list))
  if(length(choose_final) > 1) {
    wanted <- thresholding_vals[choose_final][order(thresholding_vals[choose_final])]
    choose_final <- length(wanted)
  }
  tss_df <- data.frame(thresholding_vals, spec_list, tss_list, suit_list)
  colnames(tss_df) <- c("Sensitivity", "Specificity", "TSS",
                        "Suitability")
  final_threshold_output <- list(threshold_layers = thresh_sdm_list[[choose_final]],
                                 tss_results = tss_df)
  return(final_threshold_output)
}

#' Create 3D partitions
#'
#' @description Creates partition schemes in 3D for model training and testing.
#' Can create block or kfold partitions, with output as a `vector` or `list`
#'
#' @param maxent_df Data frame with column 'p' (1=presence, 0=absence).
#' @param coord_df  Data frame with 'longitude','latitude','depth' aligned with maxent_df.
#' @param which_partition "k.fold" (default) or "block".
#' @param kfolds Integer >= 2 for k.fold.
#' @param orientation For "block": "lon_lat" (default) or "lat_lon".
#' @param return_format "vector" (default) or "list".
#' @param ensure_all_folds (k.fold) Ensure all folds appear among presences with known depth (default TRUE).
#' @param max_attempts (k.fold) Retry cap when ensure_all_folds=TRUE (default 100).
#' @param na_strategy (k.fold) How to handle NA depth rows: "NA" (default) leaves them NA; "random" assigns a random fold.
#'
#' @details maxent_df and coord_df should be the same dataframes to be provided
#' to `maxent_3D()` for model production.
#' The spatial block partition, similarly to a traditional 2D partition,
#' will separate the occurrences into 4 groups of equal (or about equal) size,
#' with two groups making up two blocks on the upper half of the depth distribution
#' and two groups on the lower half. Background points are assigned to spatial groups
#' according to how they fall into the spatial partitions delimited by the occurrences.
#' Note that this means the number of background points in each partition may not be as
#' close to equal as the occurrences.
#'
#' @return If return_format="vector": integer vector (1..k or 1..4), length == nrow(maxent_df).
#' Unassignable rows get NA. If "list": list(occ_partitions, bg_partitions).
#'
#' @examples
#' \donttest{
#' # create test dataframe
#' occ <- rep(1, times = 10)
#' bg <- rep(0, times = 1000)
#' env1 <- sample(c(1:100), size = 1010, replace = TRUE)
#' env2 <- sample(c(1:1000), size = 1010, replace = TRUE)
#' p <- c(occ, bg)
#' testdf <- data.frame(p, env1, env2)
#'
#' # create test coord data
#' r <- terra::rast(ncol = 100, nrow = 100)
#' set.seed(0)
#' longitude <- sample(terra::ext(r)[1]:terra::ext(r)[2], size = 1010, replace = TRUE)
#' set.seed(0)
#' latitude <- sample(terra::ext(r)[3]:terra::ext(r)[4], size = 1010, replace = TRUE)
#' depth <- sample(c(0, 5, 10, 15, 20, 25, 30, 35, 40, 45), size = 1010, replace = TRUE)
#' test_coords <- data.frame(longitude, latitude, depth)
#'
#' # Here's the function
#' result_kfold <- partition_3D(maxent_df = testdf, coord_df = test_coords,
#' which_partition = 'k.fold', kfolds = 3)
#'
#' result_block <- partition_3D(maxent_df = testdf, coord_df = test_coords,
#' which_partition = 'block', orientation = 'lat_lon')
#' }
#'
#' @keywords partition validation
#'
#' @export
#'

partition_3D <- function(maxent_df, coord_df,
                         which_partition = "k.fold",
                         kfolds = NULL,
                         orientation = "lon_lat",
                         return_format = "vector",
                         ensure_all_folds = TRUE,
                         max_attempts = 100,
                         na_strategy = "NA") {

  if (!("p" %in% names(maxent_df))) {
    warning("maxent_df must contain column 'p' (1=presence, 0=absence).")
    return(NULL)
  }

  if(!(which_partition %in% c("k.fold", "block"))){
    warning("which_partition must be k.fold or block")
    return(NULL)
  }

  if(which_partition == "k.fold"){
    if(!is.numeric(kfolds)){
      warning("k folds must be numeric")
      return(NULL)
    }
  }

  need_cols <- c("longitude","latitude","depth")
  if (!all(need_cols %in% names(coord_df))) {
    warning("coord_df must have columns: 'longitude','latitude','depth'.")
    return(NULL)
  }
  if (nrow(maxent_df) != nrow(coord_df)) {
    warning("maxent_df and coord_df must have the same number of rows in the same order.")
    return(NULL)
  }

  # Meat of function
  n_all  <- nrow(maxent_df)
  fold   <- rep(NA_integer_, n_all)  # unified output

  is_pres <- maxent_df$p == 1
  is_bg   <- !is_pres

  occ_xyz <- coord_df[is_pres, , drop = FALSE]
  bg_xyz  <- coord_df[is_bg,  , drop = FALSE]

  if (which_partition == "k.fold") {
    split_into_k <- function(n, k) {
      if (n == 0) integer(0) else rep_len(seq_len(k), length.out = n)
    }
    if (is.null(kfolds) || !is.numeric(kfolds) ||
        length(kfolds) != 1 || kfolds < 2) {
      stop("For 'k.fold', provide numeric 'kfolds' >= 2.")
    }

    # --- Presences: per-depth where depth known; NA depth handled by na_strategy ---
    depth_levels_occ <- sort(unique(occ_xyz$depth[!is.na(occ_xyz$depth)]))
    n_known_occ <- sum(!is.na(occ_xyz$depth))

    assign_occ_once <- function() {
      occ_folds <- rep(NA_integer_, nrow(occ_xyz))
      for (d in depth_levels_occ) {
        idx <- which(occ_xyz$depth == d)
        n   <- length(idx)
        if (n >= kfolds) {
          f <- split_into_k(n, kfolds)
          occ_folds[idx] <- sample(f, size = n, replace = FALSE)
        } else {
          occ_folds[idx] <- sample.int(kfolds, size = n, replace = TRUE)
        }
      }
      # NA-depth handling
      na_idx <- which(is.na(occ_xyz$depth))
      if (length(na_idx) && na_strategy == "random") {
        occ_folds[na_idx] <- sample.int(kfolds, length(na_idx), replace = TRUE)
      } # else remain NA
      occ_folds
    }

    if (length(depth_levels_occ)) {
      attempt <- 1L
      repeat {
        occ_folds <- assign_occ_once()
        present_folds <- unique(occ_folds[!is.na(occ_folds)])
        if (ensure_all_folds && n_known_occ >= kfolds) {
          ok <- length(setdiff(seq_len(kfolds), present_folds)) == 0L
        } else {
          ok <- TRUE
        }
        if (ok || attempt >= max_attempts) break
        attempt <- attempt + 1L
      }
      if (ensure_all_folds && n_known_occ >= kfolds &&
          attempt >= max_attempts &&
          length(setdiff(seq_len(kfolds), unique(occ_folds[!is.na(occ_folds)]))) > 0L) {
        warning("Could not ensure all folds among presences with known depth; proceeding.")
      }
    } else {
      occ_folds <- rep(NA_integer_, nrow(occ_xyz))
      if (na_strategy == "random") {
        # All presences have NA depth — assign randomly if requested
        occ_folds[] <- sample.int(kfolds, length(occ_folds), replace = TRUE)
      } else {
        warning("No presences have known depth; k.fold assigns NA to all presences.")
      }
    }

    # --- Backgrounds: per-depth where depth known; NA depth via na_strategy ---
    depth_levels_bg <- sort(unique(bg_xyz$depth[!is.na(bg_xyz$depth)]))
    bg_folds <- rep(NA_integer_, nrow(bg_xyz))
    for (d in depth_levels_bg) {
      idx <- which(bg_xyz$depth == d)
      n   <- length(idx)
      if (n >= kfolds) {
        f <- split_into_k(n, kfolds)
        bg_folds[idx] <- sample(f, size = n, replace = FALSE)
      } else {
        bg_folds[idx] <- sample.int(kfolds, size = n, replace = TRUE)
      }
    }
    na_bg <- which(is.na(bg_xyz$depth))
    if (length(na_bg) && na_strategy == "random") {
      bg_folds[na_bg] <- sample.int(kfolds, length(na_bg), replace = TRUE)
    } # else remain NA

    # Map into unified vector
    fold[which(is_pres)] <- occ_folds
    fold[which(is_bg)]   <- bg_folds

    if (return_format == "list") {
      return(list(occ_partitions = occ_folds, bg_partitions = bg_folds))
    }
    return(fold)
  }

  # -------- BLOCK partitioning (missing coords -> NA) --------
  pres_with_depth <- which(!is.na(occ_xyz$depth))
  if (!length(pres_with_depth)) {
    warning("No presence rows have non-NA depth; cannot form 3D blocks. Returning NA folds.")
    if (return_format == "list") {
      return(list(occ_partitions = rep(NA_integer_, nrow(occ_xyz)),
                  bg_partitions  = rep(NA_integer_, nrow(bg_xyz))))
    }
    fold[which(is_pres)] <- NA_integer_
    fold[which(is_bg)]   <- NA_integer_
    return(fold)
  }

  # Depth-balanced split of presences with known depth
  occ_d <- occ_xyz[pres_with_depth, , drop = FALSE]
  ord_by_depth <- order(occ_d$depth, na.last = TRUE)
  occ_d <- occ_d[ord_by_depth, , drop = FALSE]

  n_occ <- nrow(occ_d)
  half  <- floor(n_occ / 2)

  top_df    <- if (half > 0) occ_d[seq_len(half), , drop = FALSE] else occ_d[0, , drop = FALSE]
  bottom_df <- if (n_occ > half) occ_d[seq(half + 1L, n_occ), , drop = FALSE] else occ_d[0, , drop = FALSE]

  # Robust depth cutoff calculation from presences
  d_top_max <- if (nrow(top_df) > 0) max(top_df$depth, na.rm = TRUE) else -Inf
  d_bot_min <- if (nrow(bottom_df) > 0) min(bottom_df$depth, na.rm = TRUE) else Inf

  if (d_top_max == d_bot_min) {
    # If the row split falls inside the same discrete depth layer due to skew,
    # use the midpoint of unique depth layers to ensure a geometric split for backgrounds
    unique_depths <- sort(unique(occ_xyz$depth[!is.na(occ_xyz$depth)]))
    if (length(unique_depths) >= 2) {
      depth_cutoff <- unique_depths[floor(length(unique_depths) / 2)]
    } else {
      depth_cutoff <- d_top_max
    }
  } else {
    # Clean separation between layers
    depth_cutoff <- (d_top_max + d_bot_min) / 2
  }

  axis_var <- if (orientation == "lon_lat") "longitude" else "latitude"

  split_by_axis <- function(df_half) {
    n <- nrow(df_half)
    if (n == 0L) return(integer(0))
    grp <- rep(NA_integer_, n)
    valid <- which(!is.na(df_half[[axis_var]]))
    if (length(valid)) {
      o <- order(df_half[[axis_var]][valid], na.last = NA)
      v_idx <- valid[o]
      m <- length(v_idx)
      cut <- floor(m / 2)
      if (m > 0) {
        grp[v_idx[seq_len(cut)]] <- 1L
        if (m > cut) grp[v_idx[seq(cut + 1L, m)]] <- 2L
      }
    }
    grp
  }

  top_grp12     <- split_by_axis(top_df)       # 1/2/NA
  bottom_grp12  <- split_by_axis(bottom_df)    # 1/2/NA
  bottom_grp34  <- bottom_grp12
  bottom_grp34[bottom_grp34 == 1L] <- 3L
  bottom_grp34[bottom_grp34 == 2L] <- 4L

  occ_groups_subset <- c(top_grp12, bottom_grp34)   # depth-ordered
  occ_groups_all <- rep(NA_integer_, nrow(occ_xyz))
  tmp <- integer(length = nrow(occ_d))
  tmp[] <- NA_integer_

  # FIX BUG 3: Correctly map back using original order vector
  tmp[ord_by_depth] <- occ_groups_subset
  occ_groups_all[pres_with_depth] <- tmp

  # Calculate separate spatial cutoffs for top and bottom layers
  cutoff_from_half <- function(df_half, grp_half12) {
    sel1 <- which(grp_half12 == 1L & !is.na(df_half[[axis_var]]))
    sel2 <- which(grp_half12 == 2L & !is.na(df_half[[axis_var]]))
    if (!length(sel1) || !length(sel2)) return(NA_real_)
    (max(df_half[[axis_var]][sel1], na.rm = TRUE) + min(df_half[[axis_var]][sel2], na.rm = TRUE)) / 2
  }

  axis_cutoff_top <- cutoff_from_half(top_df, top_grp12)
  axis_cutoff_bot <- cutoff_from_half(bottom_df, bottom_grp12)

  # Fallbacks if a layer has no spatial spread
  global_axis_median <- stats::median(occ_xyz[[axis_var]], na.rm = TRUE)
  if (is.na(axis_cutoff_top)) axis_cutoff_top <- global_axis_median
  if (is.na(axis_cutoff_bot)) axis_cutoff_bot <- global_axis_median

  # Background quadrants; missing depth or axis -> NA
  bg_axis <- bg_xyz[[axis_var]]
  cond_top    <- !is.na(bg_xyz$depth) & bg_xyz$depth <= depth_cutoff
  cond_bottom <- !is.na(bg_xyz$depth) & !cond_top

  # Calculate distinct spatial cutoffs for the top and bottom layers
  axis_cutoff_top <- cutoff_from_half(top_df, top_grp12)
  axis_cutoff_bot <- cutoff_from_half(bottom_df, bottom_grp12)

  # Robust fallbacks if a specific layer lacks spatial variation
  global_axis_median <- stats::median(occ_xyz[[axis_var]], na.rm = TRUE)
  if (is.na(axis_cutoff_top)) axis_cutoff_top <- global_axis_median
  if (is.na(axis_cutoff_bot)) axis_cutoff_bot <- global_axis_median

  # Evaluate top background points using the top cutoff line,
  # and bottom background points using the bottom cutoff line
  cond_left_top  <- !is.na(bg_axis) & bg_axis <= axis_cutoff_top
  cond_left_bot  <- !is.na(bg_axis) & bg_axis <= axis_cutoff_bot

  bg_groups <- rep(NA_integer_, nrow(bg_xyz))
  bg_groups[cond_top & cond_left_top]      <- 1L
  bg_groups[cond_top & !cond_left_top]     <- 2L
  bg_groups[cond_bottom & cond_left_bot]   <- 3L
  bg_groups[cond_bottom & !cond_left_bot]  <- 4L

  # Fill unified vector
  fold[which(is_pres)] <- occ_groups_all
  fold[which(is_bg)]   <- bg_groups

  if (return_format == "list") {
    return(list(occ_partitions = occ_groups_all, bg_partitions = bg_groups))
  } else {
    return(fold)
  }
}

#' @title Marine background shapefile generation
#'
#' @description Automatically generates background
#' shapefiles for sampling pseudoabsences and/or background
#' points for niche modeling or species distribution modeling.
#' Delineating background sampling regions can be one of the
#' trickiest parts of generating a good model. Automatically
#' generated background shapefiles should be inspected
#' carefully prior to model use.
#'
#' Useful references, among others:
#' \itemize{
#'   \item Barve N, Barve V, Jiménez-Valverde A, Lira-Noriega A,
#'   Maher SP, Peterson AT, Soberón J, Villalobos F. 2011. The
#'   crucial role of the accessible area in ecological niche
#'   modeling and species distribution modeling.
#'   \emph{Ecological modelling} 222:1810-9.
#'   \item Merow, C, Smith MJ, Silander JA. 2013. A practical
#'   guide to MaxEnt for modeling species’ distributions: what
#'   it does, and why inputs and settings matter." \emph{Ecography}
#'   36: 1058-69.
#'   \item Murphy SJ. 2021. Sampling units derived from geopolitical
#'   boundaries bias biodiversity analyses. \emph{Global Ecology
#'   and Biogeography} 30: 1876-88.
#' }
#'
#' @param occs A `data.frame` with at least two columns
#' named "longitude" and "latitude" or that
#' can be coerced into this format.
#'
#' @param clipToOcean `logical`. Clips shapefile to oceans where species
#' occurs. Useful in cases where buffers jump over narrow
#' peninsulas (e.g. Isthmus of Panama). Can be quite artificial at ocean
#' boundaries.
#'
#' @param verbose `logical`. Switching to `FALSE` mutes message describing
#' which columns in `occs` are interpreted as x and y coordinates.
#'
#' @param ... Additional optional arguments to pass to
#' `getDynamicAlphaHull`.
#'
#' @return A `SpatVector`
#'
#' @details The meat of this function is a special-case wrapper
#' around `getDynamicAlphaHull()` from the `rangeBuilder` package.
#' The function documented here is especially useful in cases where
#' one wants to automatically generate training regions that overlap
#' the international date line. Regions that exceed the line are cut
#' and pasted into the appropriate hemisphere instead of being
#' deleted.
#'
#' If the argument `buff` is not supplied, a buffer is
#' calculated by taking the mean between the 10th and 90th percentile
#' of horizontal distances between occurrence points.
#'
#' If `getDynamicAlphaHull()` cannot satisfy the provided conditions,
#' the occurrences are buffered and then a minimum convex hull is
#' drawn around the buffer polygons.
#'
#' @examples
#' \donttest{
#' library(terra)
#' # Create sample raster
#' r <- rast(ncol=10, nrow=10)
#' values(r) <- 1:100
#'
#' # Create test occurrences
#' set.seed(0)
#' longitude <- sample(-50:50,
#'                     size = 20, replace = FALSE)
#' set.seed(0)
#' latitude <- sample(-30:30,
#'                    size = 20, replace = FALSE)
#' occurrences <- as.data.frame(cbind(longitude,latitude))
#'
#' # Here's the function
#' result <- marineBackground(occs = occurrences, buff = 100000,
#'                            fraction = .9, partCount = 2, clipToOcean = FALSE)
#' }
#'
#' @import terra
#' @import sf
#' @importFrom rangeBuilder getDynamicAlphaHull
#' @importFrom methods as slot<-
#' @importFrom rnaturalearth ne_countries
#'
#' @seealso \code{\link[rangeBuilder:getDynamicAlphaHull]{getDynamicAlphaHull}}
#'
#' @keywords backgroundSampling
#'
#' @export


marineBackground <- function(occs, clipToOcean = TRUE, verbose = TRUE, ...){
  args <- list(...)

  if("fraction" %in% names(args)){
    fraction <- args$fraction
  } else {
    fraction <- 0.95
  }

  if("partCount" %in% names(args)){
    partCount <- args$partCount
  } else {
    partCount <- 1
  }

  if("initialAlpha" %in% names(args)){
    initialAlpha <- args$initialAlpha
  } else{
    initialAlpha <- 3
  }

  if("alphaIncrement" %in% names(args)){
    alphaIncrement <- args$alphaIncrement
  } else{
    alphaIncrement <- 1
  }

  # Input checking
  if(!is.data.frame(occs)){
    warning(paste0("'occs' must be an object of class 'data.frame'.\n"))
    return(NULL)
  }
  if (!is.logical(clipToOcean)) {
    warning(message("Argument 'clipToOcean' is not of type 'logical'.\n"))
    return(NULL)
  }
  if (!is.logical(verbose)) {
    warning(message("Argument 'verbose' is not of type 'logical'.\n"))
    return(NULL)
  }
  if (!is.numeric(fraction)) {
    warning(message("Argument 'fraction' is not of class 'numeric'.\n"))
    return(NULL)
  }
  if (!is.numeric(partCount)) {
    warning(message("Argument 'partCount' is not of type 'numeric'.\n"))
    return(NULL)
  }

  # Parse columns
  colNames <- colnames(occs)
  colParse <- columnParse(occs)
  if(is.null(colParse)){
    return(NULL)
  }
  xIndex <- colParse$xIndex
  yIndex <- colParse$yIndex
  interp <- colParse$reportMessage

  if(verbose){
    message(interp)
  }

  # Calculate buffer
  if("buff" %in% names(args)){
    buff <- args$buff
    if (!is.numeric(buff)) {
      warning(message("Argument 'buff' is not of type 'numeric'.\n"))
      return(NULL)
    }
  } else{
    pDist <- distance(as.matrix(occs[,c(xIndex, yIndex)]), lonlat = TRUE)
    buff <- mean(quantile(pDist, c(.1, .9), na.rm = TRUE))/2
  }

  if (!is.numeric(initialAlpha)) {
    warning(message("Argument 'initialAlpha' is not of type 'numeric'.\n"))
    return(NULL)
  }
  if (!(is.logical(clipToOcean))) {
    warning(message(clipToOcean, " is not valid for 'clipToOcean'.\n"))
    return(NULL)
  }
  if (!is.numeric(alphaIncrement)) {
    warning(message("Argument 'alphaIncrement' is not of type 'numeric'.\n"))
    return(NULL)
  }

  # Point part
  upj <- st_crs(4326) # Unprojected WGS84
  pj <- st_crs(4087) # Projected WGS84
  occsForM <- vect(x = occs[,c(xIndex, yIndex)],
                   geom = c(colNames[xIndex], colNames[yIndex]),
                   crs = upj$wkt)
  occsForM <- project(occsForM, y = pj$wkt)
  occBuff <- suppressWarnings(buffer(occsForM,
                                     width = buff))
  occBuff <- aggregate(occBuff)

  # Hull part
  hull <- try(getDynamicAlphaHull(occs, initialAlpha = initialAlpha,
                                  alphaIncrement = alphaIncrement,
                                  coordHeaders=colnames(occs)[c(xIndex,
                                                                yIndex)],
                                  clipToCoast = "no",
                                  fraction = fraction,
                                  partCount = partCount),
              silent = TRUE)
  gdahAlternative <- FALSE
  if("try-error" %in% class(hull)){
    gdahAlternative <- TRUE
  } else{
    if(hull$alpha == "alphaMCH"){
      gdahAlternative <- TRUE
    }
    hull <- st_transform(hull[[1]], crs = pj$wkt)
    if(!all(st_is_valid(hull))){
      gdahAlternative <- TRUE
    }
  }

  if(gdahAlternative){
    occBuffTmp <- disagg(occBuff)
    occBuffConv <- vector(mode = "list", length = length(occBuffTmp))
    for (i in 1:length(occBuffTmp)){
      occBuffConv[[i]] <- terra::hull(occBuffTmp[i],
                                      type = "concave_ratio",
                                      param = 0.1, allowHoles = F)
      values(occBuffConv[[i]]) <- 1
    }
    wholeM <- aggregate(vect(occBuffConv))
  } else{
    hullBuff <- buffer(vect(hull), width = buff)
    hullBuff <- aggregate(hullBuff)
    wholeM <- union(occBuff, hullBuff)
    wholeM <- aggregate(wholeM)
  }

  # Putting it all together and fixing the date line
  worldExtent <- ext(-20037508,
                     20037508,
                     -10018754,
                     10018754) # Plate-Carre world extent

  # Get rid of slop at poles
  wholeM <- crop(wholeM, ext(c(xmin(wholeM),
                               xmax(wholeM),
                               ymin(worldExtent),
                               ymax(worldExtent))))
  # Wrap shapefiles at 180th meridian
  middle <- intersect(wholeM, worldExtent)
  ends <- disagg(erase(wholeM, worldExtent))
  if(length(ends) > 0){
    if(length(ends) == 1){
      if(xmin(ends) < -20037508){
        ends <- shift(ends, dx = 20037508*2)
      }
      if(xmax(ends) > 20037508){
        ends <- shift(ends, dx = -20037508*2)
      }
    } else{
      result <- list()
      for(j in 1:length(ends)){
        if(xmin(ends[j,]) < -20037508){
          endTemp <- shift(ends[j,], dx = 20037508*2)
        } else if(xmax(ends[j,]) > 20037508){
          endTemp <- shift(ends[j,], dx = -20037508*2)
        }
        result[[j]] <- endTemp
      }
      ends <- vect(unlist(result))
      ends <- aggregate(ends)
    }
    wholeM <- aggregate(union(middle, ends))
  } else{
    wholeM <- middle
  }

  wholeM <- aggregate(wholeM)
  crs(wholeM) <- pj$wkt

  # Crop out land
  land <- aggregate(rnaturalearth::ne_countries(returnclass = "sv"))
  land <- project(land, crs(wholeM))
  wholeM <- erase(wholeM, land)

  # Optional removal of unoccupied polygons
  if(clipToOcean){
    # First, split up disjunct polygons
    wholeM <- disagg(wholeM)
    polysContainingPoints <- apply(relate(wholeM, occsForM, "contains"),
                                   MARGIN = 1, FUN = function(x) any(x))
    wholeM <- wholeM[polysContainingPoints]
  }
  wholeM <- aggregate(wholeM)
  wholeM <- terra::project(wholeM, y = upj$wkt)

  return(wholeM)
}

# Read occurrences for test
occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv",
                             package='voluModel'))

# Create test occurrences
set.seed(0)
occurrences <- occs[sample(1:nrow(occs),
            size = 24, replace = FALSE),]

test_that("marineBackground input warnings behave as expected", {
  expect_error(marineBackground())
  expect_warning(marineBackground(occs = "a"))
  badColNames <- occurrences
  colnames(badColNames) <- c("eggs", "spam")
  expect_warning(marineBackground(occs = badColNames))
  expect_warning(marineBackground(occs = occurrences, clipToOcean = "a"))
  expect_warning(marineBackground(occs = occurrences, fraction = "a",
                                  partCount = 3, buff = 10000,
                                  initialAlpha = 3, alphaIncrement = 1,
                                  clipToOcean = 'no'))
  expect_warning(marineBackground(occs = occurrences, fraction = 0.95,
                                  partCount = "a", buff = 10000,
                                  initialAlpha = 3, alphaIncrement = 1,
                                  clipToOcean = 'no'))
  expect_warning(marineBackground(occs = occurrences, fraction = 0.95,
                                  partCount = 3, buff = "a",
                                  initialAlpha = 3, alphaIncrement = 1,
                                  clipToOcean = 'no'))
  expect_warning(marineBackground(occs = occurrences, fraction = 0.95,
                                  partCount = 3, buff = 10000,
                                  initialAlpha = "a", alphaIncrement = 1,
                                  clipToOcean = 'no'))
  expect_warning(marineBackground(occs = occurrences, fraction = 0.95,
                                  partCount = 3, buff = 10000,
                                  initialAlpha = 3, alphaIncrement = "a",
                                  clipToOcean = 'no'))
  expect_warning(marineBackground(occs = occurrences, fraction = 0.95,
                                  partCount = 3, buff = 10000,
                                  initialAlpha = 3, alphaIncrement = 1,
                                  clipToOcean = 1))
  expect_warning(marineBackground(occs = occurrences, fraction = 0.95,
                                  partCount = 3, buff = 10000,
                                  initialAlpha = 3, alphaIncrement = 1,
                                  clipToOcean = 1, verbose = "rabbit"))
})

test_that("marineBackground results as expected", {
  skip_on_cran() # Function's a little slow for CRAN's tastes
#  skip_on_ci() # Something weird happens at line 58
  result <- marineBackground(occs = occurrences, buff = 100000,
                             fraction = .9, partCount = 2, clipToOcean = TRUE)
  expect_true("SpatVector" %in% class(result))

  result <- marineBackground(occs = occurrences,
                             fraction = .9, partCount = 2,
                             clipToOcean = TRUE)
  expect_true("SpatVector" %in% class(result))
})

test_that("marineBackground Pacific results as expected", {
  skip_on_cran() # Function's a little slow for CRAN's tastes
  # Both sides

  # Create test occurrences
  set.seed(0)
  longitude <- c(sample(-180:-175,
                        size = 10, replace = TRUE),
                 sample(165:180,
                        size = 10, replace = TRUE))
  set.seed(0)
  latitude <- sample(-20:20,
                     size = 20, replace = TRUE)
  occurrences <- data.frame(longitude,latitude)

  result <- marineBackground(occs = occurrences, buff = 1000000,
                             fraction = .95, partCount = 2, clipToOcean = TRUE,
                             verbose = FALSE)
  expect_true("SpatVector" %in% class(result))

  #One side
  result <- marineBackground(occs = occurrences[1:10,], buff = 1000000,
                             fraction = .95, partCount = 2, clipToOcean = TRUE,
                             verbose = FALSE)
  expect_true("SpatVector" %in% class(result))
})

#' @title Match the depth values given in an occurrence dataset to the depth slice
#' values of a user provided spatRaster stack
#'
#' @description assigns coordinates to depth slices given a user provided raster template.
#' if any points are deeper than the lowest depth slice, they are assigned to the
#' lowest depth slice.
#'
#' @param occs dataframe of occurrence records with colnames "longitude," "latitude,"
#' and "depth." Depth values should be positive.
#' @param rasterTemplate a spatRaster stack where the layer names correspond to the depth value,
#' as a positive number
#'
#' @details rasterTemplate names and values in the depth column of occs should be positive.
#' Points lower than the deepest depth slice are assigned to the deepest depth slice
#'
#' @return object of class dataframe, the same occs fed into the function but with the
#' depth values adjusted to match the spatRaster stack depth slices
#'
#' @examples

#' library(terra)
#' # Create test raster brick
#' r1 <- rast(ncol = 100, nrow = 100)
#' r2 <- rast(ncol = 100, nrow = 100)
#' r3 <- rast(ncol = 100, nrow = 100)
#' rbrick <- c(r1, r2, r3)
#' names(rbrick) <- c(1, 40, 90)
#'
#' # Create test occurrences
#' set.seed(0)
#' longitude <- sample(ext(r1)[1]:ext(r1)[2], size = 10, replace = FALSE)
#' set.seed(0)
#' latitude <- sample(ext(r1)[3]:ext(r1)[4], size = 10, replace = FALSE)
#' set.seed(0)
#' depth <- sample(1:100, size = 10, replace = TRUE)
#' occs <- data.frame(longitude, latitude, depth)
#'
#' # Here's the function
#' result <- depthMatch(occs = occs, rasterTemplate = rbrick)
#'
#'@importFrom terra rast
#'
#' @keywords occurrence cleaning
#'
#' @export

depthMatch <- function(occs, rasterTemplate) {
  wantdepths <- names(rasterTemplate)
  for(k in 1:length(wantdepths)) {
    if(k == 1){
      fixdepths <- which(occs$depth < wantdepths[k])
      occs$depth[fixdepths] <- wantdepths[k]
    }
    if(k < length(wantdepths)) {
      depthint <- as.numeric(c(wantdepths[k], wantdepths[k+1]))
      meandepth <- mean(depthint)
      fixdepths1 <- which(occs$depth > depthint[1] & occs$depth <= meandepth)
      fixdepths2 <- which(occs$depth > meandepth & occs$depth < depthint[2])
      if(length(fixdepths1) > 0) {
        occs$depth[fixdepths1] <- depthint[1]
      }
      if(length(fixdepths2) > 0)
        occs$depth[fixdepths2] <- depthint[2]
    } else {
      fixdepths <- which(occs$depth > as.numeric(wantdepths[k]))
      if(length(fixdepths) > 0) {
        occs$depth[fixdepths] <- as.numeric(wantdepths[k])
      }
    }
  }
  return(occs)
}

# testing depthMatch function

library(terra)
library(sf)

# reading in example data
occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv",
                             package='voluModel'))
occs <- occs %>% dplyr::select(scientificName, decimalLatitude, decimalLongitude, depth)
colnames(occs) <- c("species", "latitude", "longitude", "depth")
projcrs <- "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
occs_sf <- st_as_sf(x = occs,
               coords = c("longitude", "latitude"),
               crs = projcrs)

# creating raster brick
temp1 <- rast(ext(occs_sf))
temp2 <- rast(ext(occs_sf))
temp3 <- rast(ext(occs_sf))
temp <- c(temp1, temp2, temp3)
names(temp) <- c(10, 100, 1000)

# testing function

test_that("depthMatch outputs as expected", {
  depth_occs <- depthMatch(occs = occs, rasterTemplate = temp)
  expect_false(any(occs$depth == depth_occs$depth, na.rm = T))
  wantdepths <- as.numeric(depth_occs$depth[which(!(is.na(depth_occs$depth)))])
  expect_false(any(!(wantdepths %in% names(temp))))
}
)

#' @title Clean and flag coordinates based on intersection with a land polygon,
#' intersection with a bathymetry layer, and/or by a given depth range
#'
#' @description function to remove or flag coordinates on land, coordinates which are deeper
#' than the known bathymetry at a given xy location, and/or coordinates outside of
#' a specified depth range when cleaning coordinates for species in marine environments
#'
#' @param occs dataframe containing columns named "longitude", "latitude", and "depth," where
#' depth values are positive
#' @param bathy a spatRaster of bathymetry in the same units as the occurrence depth, where
#' values are negative
#' @param land_poly optional polygon of continents in same projection as occurrence data
#' @param depth_range optional range of depth values in the form of c(upper, lower) in which
#' all points should fall between. depth value should be positive, as in the case of the occs
#' @param flag default is F, if T, points are not removed, but flagged if they intersect, and a
#' column is added to the output indicating what points have been flagged
#' @param bottom_correct default is F, but if T, points are not removed, and a column is added
#' to the output displaying the bathymetry value at the flagged point for corrective purposes
#'
#' @details assumes depths are listed as positive in the occs and depth_range, while the
#' bathymetry layer lists them as negative when below the sea surface
#'
#' @return an object of class dataframe containing cleaned or flagged occurrences
#'
#' @examples
#' library(terra)
#' library(dplyr)
#'
#' # Create sample bathymetry
#' r1 <- rast(ncol=10, nrow=10)
#' values(r1) <- c(-100:0)
#'
#' # Create test occurrences
#' set.seed(0)
#' longitude <- sample(ext(r1)[1]:ext(r1)[2], size = 10, replace = FALSE)
#' set.seed(0)
#' latitude <- sample(ext(r1)[3]:ext(r1)[4], size = 10, replace = FALSE)
#' set.seed(0)
#' depth <- sample(1:100, size = 10, replace = TRUE)
#' occs <- data.frame(longitude, latitude, depth)
#'
#' #Heres the function
#' result <- cleanDepth(bathy = r1, occs = occs)
#'
#' @importFrom terra extract
#' @import sf
#'
#' @keywords occurrence cleaning
#'
#' @export

cleanDepth <- function(occs, bathy, land_poly, depth_range, flag = FALSE,
                       bottom_correct = FALSE) {

  # bathymetry cleaning
  occ_mat <- occs[,c("longitude", "latitude")]
  depth_at_points <- terra::extract(bathy, occ_mat)
  occs$depth[which(occs$depth > 0 & !(is.na(occs$depth)))] <- 0 -
    occs$depth[which(occs$depth > 0 & !(is.na(occs$depth)))]
  bath_flag <- vector(length = nrow(occs))
  for (k in 1:nrow(occs)) {
    if(is.na(occs$depth[k])) {
      bath_flag[k] < - 0
    } else if(occs$depth[k] < depth_at_points[k,2] | depth_at_points[k,2] > 0) {
      bath_flag[k] <- 1
    } else {
      bath_flag[k] <- 0
    }
  }
  if(flag == T & bottom_correct == T) {
    flagged_points_bathymetry <- bath_flag
    bottom_correct_values <- rep(NA, times = nrow(occs))
    bottom_correct_values[which(bath_flag > 0)] <- depth_at_points[which(bath_flag > 0),2]
    output_df <- cbind(occs, flagged_points_bathymetry, bottom_correct_values)
    print(paste0("flagged ", length(which(bath_flag > 0)), " points intersecting bathymetry"))
  } else if(flag == T & bottom_correct == F) {
    flagged_points_bathymetry <- bath_flag
    output_df <- cbind(occs, flagged_points_bathymetry)
    print(paste0("flagged ", length(which(bath_flag > 0)), " points intersecting bathymetry"))
  } else if(flag == F & bottom_correct == T) {
    bottom_correct_values <- rep(NA, times = nrow(occs))
    bottom_correct_values[which(bath_flag > 0)] <- depth_at_points[which(bath_flag > 0),2]
    output_df <- cbind(occs, bottom_correct_values)
  } else {
    output_df <- occs[-which(bath_flag > 0),]
    print(paste0("removed ", length(which(bath_flag > 0)), " points intersecting bathymetry"))
  }

  # in the case that land_poly is supplied
  if(!(missing(land_poly))) {
    x_sf <- st_as_sf(output_df, coords = c("longitude", "latitude"), crs = st_crs(land_poly))
    within_list <- st_within(x_sf, land_poly)
    within_list_real <- vector(length = length(within_list))
    for(k in 1:length(within_list)) {
      within_list_real[k] <- length(within_list[[k]])
    }
    flag_points <- which(within_list_real != 0)
    if(length(flag_points) > 0) {
      if(flag == T) {
        flagged_points_land <- within_list_real
        output_df <- cbind(output_df, flagged_points_land)
        print(paste0("flagged ", length(flag_points), " points within land polygon"))
      } else {
        output_df <- output_df[-flag_points,]
        print(paste0("removed ", length(flag_points), " points within land polygon"))
      }
    } else {
      print("no points found on land")
    }
  }

  # in the case depth_range is supplied
  if(!(missing(depth_range))) {
    rem1 <- which(!(output_df$depth <= min(depth_range)))
    rem2 <- which(!(output_df$depth >= -max(depth_range)))
    if(length(rem1) > 0) {
      output_df <- output_df[-rem1,]
    }
    if(length(rem2) > 0) {
      output_df <- output_df[-rem2,]
    }
    print(paste0("removed ", length(rem1) + length(rem2), " points outside of depth range"))
  }

  output_df$depth <- abs(output_df$depth)
  return(output_df)
}

# testing cleanDepth function

library(rnaturalearth)
library(terra)
library(sf)
library(dplyr)
library(voluModel)

# reading in example data
occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv",
                             package='voluModel'))
occs <- occs %>% dplyr::select(scientificName, decimalLatitude, decimalLongitude, depth)
colnames(occs) <- c("species", "latitude", "longitude", "depth")
target_crs <- "+proj=longlat +datum=WGS84"
occs_sf <- st_as_sf(x = occs,
                    coords = c("longitude", "latitude"),
                    crs = target_crs)

# creating example land polygon
land <- st_as_sf(ne_countries())[1] %>% st_set_crs(target_crs)
land <- land[-which(!(st_is_valid(land))),]

# creating example bathymetry layer
bath_data <- rast(extend(ext(occs_sf), 10),
                                   crs = target_crs, resolution = 1, vals = NA)
values(bath_data) <- sample(c(-4000:0), size = length(values(bath_data)))

# setting example depth_range
depth_range <- c(0, 300)

# testing that cleanDepth outputs as expected
test_that("cleanDepth outputs as expected", {
  expect_error(cleanDepth())
  testcase1 <- cleanDepth(occs = occs, bathy = bath_data)
  expect_false(nrow(occs) == nrow(testcase1))
  testcase2 <- cleanDepth(occs = occs, bathy = bath_data, flag = T)
  expect_true(nrow(occs) == nrow(testcase2))
  testcase3 <- cleanDepth(occs = occs, bathy = bath_data, flag = T, bottom_correct = T)
  expect_true("bottom_correct_values" %in% names(testcase3))
  testcase4 <- cleanDepth(occs = occs, bathy = bath_data, land_poly = land)
  expect_false(nrow(testcase4) == nrow(testcase1))
  testcase5 <- cleanDepth(occs = occs, bathy = bath_data, land_poly = land,
                          depth_range = depth_range,
                          flag = T,
                          bottom_correct = T)
  expect_false(nrow(testcase1) == nrow(testcase5))
}
)

#' @title Sampling from a `SpatRaster` vector using 3D coordinates
#'
#' @description Gets values at x,y,z occurrences from a
#' given 3D environmental variable brick
#'
#' @param occs A `data.frame` with at least three columns
#' named "longitude", "latitude", and "depth", or that
#' can be coerced into this format.
#'
#' @param envBrick A `SpatRaster` vector object with
#' one environmental variable. Each layer represents
#' a depth slice. See Details for more information.
#'
#' @param verbose `logical`. Switching to `FALSE` mutes message
#' describing which columns in `occs1` and `occs2` are interpreted
#' as x, y, and z coordinates.
#'
#' @details The `SpatRaster` vector object should
#' have numeric names that correspond with the beginning
#' depth of a particular depth slice. For example, one
#' might have three layers, one from 0 to 10m, one from
#' 10 to 30m, and one from 30 to 100m. You would name the
#' layers in this brick `names(envBrick) <- c(0, 10, 30`.
#' `xyzSample` identifies the layer name that is closest
#' to the depth layer value at a particular X, Y
#' coordinate, and samples the environmental value at that
#' 3D coordinate.
#'
#' @return Vector of environmental values equal in length
#' to number of rows of input `occs` `data.frame`.
#'
#' @examples
#' library(terra)
#'
#' # Create test raster
#' r1 <- rast(ncol=10, nrow=10)
#' values(r1) <- 1:100
#' r2 <- rast(ncol=10, nrow=10)
#' values(r2) <- c(rep(20, times = 50), rep(60, times = 50))
#' r3 <- rast(ncol=10, nrow=10)
#' values(r3) <- 8
#' envBrick <- c(r1, r2, r3)
#' names(envBrick) <- c(0, 10, 30)
#'
#' # Create test occurrences
#' set.seed(0)
#' longitude <- sample(ext(envBrick)[1]:ext(envBrick)[2],
#'                     size = 10, replace = FALSE)
#' set.seed(0)
#' latitude <- sample(ext(envBrick)[3]:ext(envBrick)[4],
#'                    size = 10, replace = FALSE)
#' set.seed(0)
#' depth <- sample(0:35, size = 10, replace = TRUE)
#' occurrences <- as.data.frame(cbind(longitude,latitude,depth))
#'
#' # Test function
#' occSample3d <- xyzSample(occurrences, envBrick)
#'
#' # How to use
#' occurrences$envtValue <- occSample3d
#'
#' @importFrom terra rast extract
#'
#' @keywords dataPrep
#'
#' @export

xyzSample <- function(occs, envBrick, verbose = TRUE){
  if(!is.data.frame(occs)){
    warning(paste0("'occs' must be an object of class 'data.frame'.\n"))
    return(NULL)
  }

  if(ncol(occs) < 3){
    warning(paste0("'occs' must have at least three columns.\n"))
    return(NULL)
  }

  if(!inherits(envBrick, what = "SpatRaster")){
    warning(paste0("'envBrick' must be of class 'SpatRaster'.\n"))
    return(NULL)
  }

  if (!is.logical(verbose)) {
    warning(message("Argument 'verbose' is not of type 'logical'.\n"))
    return(NULL)
  }

  # Checking for appropriate environmental layer names
  suppressWarnings(layerNames <- as.numeric(names(envBrick)))
  if(any(is.na(layerNames))){
    warning(message("\nInput RasterBrick names inappropriate: \n",
            paste(names(envBrick), collapse = ", "), "\n",
            "Names must be numeric.\n"))
    return(NULL)
  }

  # Parse columns
  colNames <- colnames(occs)
  colParse <- columnParse(occs, wDepth = TRUE)
  if(is.null(colParse)){
    return(NULL)
  }
  xIndex <- colParse$xIndex
  yIndex <- colParse$yIndex
  zIndex <- colParse$zIndex
  interp <- colParse$reportMessage

  if(verbose){
    message(interp)
  }

  # Sampling values
  sampledValues <- NULL
  index <- unlist(lapply(occs[,zIndex], FUN = function(x) which.min(abs(layerNames - x))))
  occs$index <- index
  occs$sampledValues <- rep(NA, times = nrow(occs))
  indices <- unique(occs$index)
  for(i in indices){
    occs[occs$index == i,]$sampledValues <- terra::extract(x = envBrick[[i]],
                                                           y = occs[occs$index == i,c(xIndex,
                                                                                      yIndex)])[,2]
  }
  return(occs$sampledValues)
}

library(terra)

# Create sample raster
r <- rast(ncol=10, nrow=10)
values(r) <- 1:100

# Create sample raster brick
rBrick <- c(r, r*10, r*100)
names(rBrick) <- c(0, 10, 100)

# Create test occurrences
set.seed(0)
longitude <- sample(ext(rBrick)[1]:ext(rBrick)[2],
                    size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(rBrick)[3]:ext(rBrick)[4],
                   size = 10, replace = FALSE)
set.seed(0)
depth <- sample(0:98, size = 10, replace = TRUE)
occurrences <- as.data.frame(cbind(longitude,latitude,depth))

test_that("xyzSample input warnings behave as expected", {
  # Tests
  expect_error(xyzSample())
  expect_warning(xyzSample(occs = "a", envBrick = rBrick))
  expect_warning(xyzSample(occs = occurrences[,1:2], envBrick = rBrick))
  expect_warning(xyzSample(occs = occurrences, envBrick = "a"))
  expect_warning(xyzSample(occs = occurrences, envBrick = rBrick,
                           verbose = "eggs"))

  colnames(occurrences) <- c("cheese", "eggs", "spam")
  expect_warning(xyzSample(occs = occurrences, envBrick = rBrick))
  colnames(occurrences) <- c("latitude", "longitude", "depth")

  names(rBrick) <- c("a", "b", "c")
  expect_warning(xyzSample(occs = occurrences, envBrick = rBrick))
  names(rBrick) <- c(0, 10, 100)
})


test_that("xyzSample returns appropriate information", {
  # Test function
  occSample3d <- xyzSample(occurrences, rBrick)

  expect_type(occSample3d, "integer")
  expect_length(occSample3d, 10)
  expect_equal(0, sum(is.na(occSample3d)))
})

test_that("xyzSample column parsing works", {
  # Test subset selection
  occurrences$"z" <- occurrences$depth
  occurrences$"zebra" <- occurrences$depth
  occSample3d <- xyzSample(occurrences, rBrick)
  expect_type(occSample3d, "integer")
  expect_length(occSample3d, 10)
  expect_equal(0, sum(is.na(occSample3d)))
})

library(terra)
library(dplyr)

# Create first rasterStack
r1 <- rast(ncol=10, nrow=10)
values(r1) <- 1:100
r2 <- rast(ncol=10, nrow=10)
values(r2) <- c(rep(20, times = 50), rep(60, times = 50))
r3 <- rast(ncol=10, nrow=10)
values(r3) <- 8
envBrick1 <- c(r1, r2, r3)
names(envBrick1) <- c(0, 10, 30)

# Create second rasterStack
r1 <- rast(ncol=10, nrow=10)
values(r1) <- 100:1
r2 <- rast(ncol=10, nrow=10)
values(r2) <- c(rep(10, times = 50), rep(20, times = 50))
r3 <- rast(ncol=10, nrow=10)
values(r3) <- rep(c(10,20,30,25), times = 25)
envBrick2 <- c(r1, r2, r3)
names(envBrick2) <- c(0, 10, 30)

rastList <- list("temperature" = envBrick1, "salinity" = envBrick2)

# Create test occurrences
set.seed(0)
longitude <- sample(ext(envBrick1)[1]:ext(envBrick1)[2],
                    size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(envBrick1)[3]:ext(envBrick1)[4],
                   size = 10, replace = FALSE)
set.seed(0)
depth <- sample(0:35, size = 10, replace = TRUE)
occurrences <- as.data.frame(cbind(longitude,latitude,depth))

# Sample data at occurrences
cal_temp <- xyzSample(occurrences, rastList$temperature)
cal_sal  <- xyzSample(occurrences, rastList$salinity)

# Calibration, assuming the sampled environmental value is the 4th column
calibration <- data.frame(
  temperature = cal_temp,
  salinity    = cal_sal)

test_that("MESS3D warnings work", {
  expect_error(MESS3D())
  expect_warning(MESS3D(calibration = "a", projection = rastList))
  expect_warning(MESS3D(calibration = calibration, projection = NULL))
  names(rastList) <- c("depecheMode", "fleetwoodMac")
  expect_warning(MESS3D(calibration = calibration, projection = rastList))
})

rastList <- list("temperature" = envBrick1, "salinity" = envBrick2)

test_that("MESS3D outputs as expected", {
  temporary <- MESS3D(calibration = calibration, projection = rastList)
  expect_equal(class(temporary)[[1]], "SpatRaster")
})

#' @title Calculate MESS
#'
#' @description Calculates multivariate environmental similarity
#' surface based on model calibration and projection data
#'
#' @param calibration A `data.frame` of environmental variables
#' used to calibrate an ecological niche model, one row for
#' measurements from each voxel included in the data used to
#' calibrate the model. Columns with names not corresponding to
#' `projection` `list` items are ignored.
#'
#' @param projection A named `list` of `SpatRaster` objects for
#' projection; names correspond to `calibration` column names.
#' Each `SpatRaster` should have the same number of layers,
#' corresponding to vertical depth slices.
#'
#' @details `MESS3D` is a wrapper around `MESS` from the `modEvA`
#' package. It calculates MESS for each depth layer. Negative values
#' indicate areas of extrapolation which should be interpreted with
#' caution (see Elith *et al*, 2010 in *MEE*).
#'
#' @note The calibration dataset should include both presences
#' and background/pseudoabsence points used to calibrate an
#' ecological niche model.
#'
#' @references Elith J, Kearney M, and Phillips S. 2010.
#' The art of modelling range-shifting species.
#' *Methods in Ecology and Evolution*, 1, 330-342.
#'
#' @return A `SpatRaster` vector with MESS scores for each
#' voxel; layer names correspond to layer names of first
#' `SpatRaster` vector in `projection` `list`.
#'
#' @examples
#' library(terra)
#' library(dplyr)
#'
#' # Create sample rasterBricks
#' r1 <- rast(ncol=10, nrow=10)
#' values(r1) <- 1:100
#' r2 <- rast(ncol=10, nrow=10)
#' values(r2) <- c(rep(20, times = 50), rep(60, times = 50))
#' r3 <- rast(ncol=10, nrow=10)
#' values(r3) <- 8
#' envBrick1 <- c(r1, r2, r3)
#' names(envBrick1) <- c(0, 10, 30)
#'
#' r1 <- rast(ncol=10, nrow=10)
#' values(r1) <- 100:1
#' r2 <- rast(ncol=10, nrow=10)
#' values(r2) <- c(rep(10, times = 50), rep(20, times = 50))
#' r3 <- rast(ncol=10, nrow=10)
#' values(r3) <- c(rep(c(10,20,30,25), times = 25))
#' envBrick2 <- c(r1, r2, r3)
#' names(envBrick2) <- c(0, 10, 30)
#'
#' rastList <- list("temperature" = envBrick1, "salinity" = envBrick2)
#'
#' # Create test reference set
#' set.seed(0)
#' longitude <- sample(ext(envBrick1)[1]:ext(envBrick1)[2],
#'                     size = 10, replace = FALSE)
#' set.seed(0)
#' latitude <- sample(ext(envBrick1)[3]:ext(envBrick1)[4],
#'                    size = 10, replace = FALSE)
#' set.seed(0)
#' depth <- sample(0:35, size = 10, replace = TRUE)
#' occurrences <- as.data.frame(cbind(longitude,latitude,depth))
#'
#' # Sample data at occurrences to characterize calibration region
#' cal_temp <- xyzSample(occurrences, rastList$temperature)
#' cal_sal  <- xyzSample(occurrences, rastList$salinity)
#'
#' calibration <- data.frame(
#'   temperature = cal_temp,
#'   salinity    = cal_sal)
#'
#' # Run the function
#' messStack <- MESS3D(calibration = calibration, projection = rastList)
#' plot(messStack)
#'
#' @importFrom predicts mess
#' @importFrom methods is
#' @importFrom terra rasterize
#'
#' @seealso \code{\link[predicts]{mess}}
#'
#' @keywords modelDiagnostics
#' @export

MESS3D <- function (calibration, projection) {
  if (!is.data.frame(calibration)) {
    warning(paste0("'calibration' must be an object of class 'data.frame'.\n"))
    return(NULL)
  }
  if (!is.list(projection) || !all(unlist(lapply(projection, FUN = function(x) is(x, "SpatRaster"))))) {
    warning(paste0("'projection' must be a list of 'SpatRaster' objects.\n"))
    return(NULL)
  }
  if(!all(names(projection) %in% colnames(calibration))){
    warning(paste0("'projection' names must correspond with column names in `calibration`.\n"))
    return(NULL)
  }

  # Go through the environmental variables and calculate MESS for each layer
  cal <- as.data.frame(calibration[,names(projection)])
  messStack <- projection
  for (i in 1:nlyr(projection[[1]])){
    proj <- rast(unlist(lapply(projection, FUN = function(x) x[[i]])))
    messStack[[i]] <- mess(x = proj, v = cal)
  }
  names(messStack) <- names(projection[[1]])
  messStack <- rast(messStack)
  return(messStack)
}

#' @title Create 3D Ecological Niche Models with Maxent
#'
#' @description Uses MaxEnt from `predicts` package to test multiple models with
#' different feature class combinations, regularization multipliers, and a user
#' supplied partitioning scheme for training and testing. The function outputs
#' model objects, model results, as well as prediction SpatRasters of each model
#' projected back onto geographic space using the supplied environmental SpatRaster
#' stacks. Note: you will need to install `rJava` to successfully run this function.
#'
#' @param maxent_df 'data.frame' where the first column is a vector
#' of presences named "p" containing 1's and 0's. Each row represents a cell in the
#' spatRaster volume with an x, y, z coordinate, and 1's are presences while 0's are
#' absences, or background points. Other columns are environmental variable values
#' extracted at the occurrence and background points, and should have the same names
#' as the names of the environmental layers in the projection_layers list of SpatRaster
#' stacks.
#'
#' @param wanted_fc a character vector giving what feature class
#' combinations should be tried. "L" refers to linear, "Q" refers to quadratic, "H"
#' refers to hinge, and "P" refers to product. Should be in the format c("L", "Q", "LQ")
#' etc.
#'
#' @param wanted_rm regularization multipliers to be tried in format c(1:4)
#'
#' @param wanted_partition optional, should be the output of 'partition_3D'. if no
#' partition is supplied, all points will be used for training
#'
#' @param projection_layers `list` of SpatRaster stacks by depth for predictions,
#' must all be cropped to the depth slice with the largest extent and masked
#' to the accessible area of the matching depth slice. Each element is a depth slice,
#' and each stack should contain all of the environmental variables used in the model
#' with the same names as that used in the model.
#'
#' @param occs `data.frame` of longitude, latitude, and depth occurrences with columns
#' named "longitude", "latitude", and "depth".
#'
#' @param depth_list vector of depths corresponding to depth slices of list elements of
#' projection_layers. Should be positive and go from shallowest depth to deepest depth
#'
#' @details The names of the projection_layers should be the same as the column names
#' of the environmental variables in maxent_df. The number of models output will be the
#' number of feature classes multiplied by the number of regularization multipliers.
#' For example, a wanted_fc of c("L", "Q", "P") and a wanted_rm of c(1:3) will output 9
#' total models.
#'
#' @return An object of class `list` with four components:
#'
#' $models, a list containing each model object produced.
#'
#' $results, a `data.frame` where each row is a model corresponding to the list element
#' in $models and $predictions. If there was no partition supplied for training and
#' testing, each column will report the feature class, regularization multiplier, AUC,
#' total coefficients, nonzero coefficients, AICc, and delta AICc. if a partition is
#' used, it will report the average of these statistics across all partitions.
#'
#' $predictions, a `list` of spatRaster stacks where each list element is a model
#' corresponding to the rows of $results and elements of $models projected onto the
#' supplied projection_layers. Each layer in the stack is a depth slice.
#'
#' $partition_results, a `list` object of the same length as the number of partition
#' groups containing a `data.frame` of results for each model for each partition. Only
#' produced if a partition is supplied.
#'
#' @examples
#'
#' library(dplyr)
#' library(predicts)
#' library(terra)
#'
#' # creating list of spatraster stacks where each element is a depth slice
#' r1_d1 <- rast(ncol = 100, nrow = 100)
#' set.seed(0)
#' values(r1_d1) <- as.numeric(sample(c(1:100), size = 1000, replace = TRUE))
#' r2_d1 <- rast(ncol = 100, nrow = 100)
#' set.seed(0)
#' values(r2_d1) <- as.numeric(sample(c(1:1000), size = 1000, replace = FALSE))
#' r1_d2 <- r1_d1
#' values(r1_d2) <- as.numeric(values(r1_d1)+10)
#' r2_d2 <- r2_d1
#' values(r2_d2) <- as.numeric(values(r2_d1)+10)
#' d1 <- c(r1_d1, r2_d1)
#' names(d1) <- c("valsr1", "valsr2")
#' d2 <- c(r1_d2, r2_d2)
#' names(d2) <- c("valsr1", "valsr2")
#' envlist <- list(d1, d2)
#'
#' # creating occs and bgs
#' set.seed(0)
#' occs <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 50, replace = FALSE)
#' bgs <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 500, replace = FALSE)
#'
#' occ_indices <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 50, replace = FALSE)
#' bg_indices  <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 500, replace = FALSE)
#'
#' occs_d1 <- crds(envlist[[1]][[1]])[occ_indices[1:25],]
#' occs_d2 <- crds(envlist[[2]][[1]])[occ_indices[26:50],]
#' bg_d1   <- crds(envlist[[1]][[1]])[bg_indices[1:250],]
#' bg_d2   <- crds(envlist[[2]][[1]])[bg_indices[251:500],]
#'
#' # extracting at occs and bgs
#' occ_valsr1_d1 <- extract(envlist[[1]][[1]], occs_d1)
#' occ_valsr1_d2 <- extract(envlist[[2]][[1]], occs_d2)
#' occ_valsr2_d1 <- extract(envlist[[1]][[2]], occs_d1)
#' occ_valsr2_d2 <- extract(envlist[[2]][[2]], occs_d2)
#'
#' occ_valsr1 <- rbind(occ_valsr1_d1, occ_valsr1_d2)
#' occ_valsr2 <- rbind(occ_valsr2_d1, occ_valsr2_d2)
#'
#' bg_valsr1_d1 <- extract(envlist[[1]][[1]], bg_d1)
#' bg_valsr1_d2 <- extract(envlist[[2]][[1]], bg_d2)
#' bg_valsr2_d1 <- extract(envlist[[1]][[2]], bg_d1)
#' bg_valsr2_d2 <- extract(envlist[[2]][[2]], bg_d2)
#'
#' bg_valsr1 <- rbind(bg_valsr1_d1, bg_valsr1_d2)
#' bg_valsr2 <- rbind(bg_valsr2_d1, bg_valsr2_d2)
#'
#' valsr1 <- rbind(occ_valsr1, bg_valsr1)
#' valsr2 <- rbind(occ_valsr2, bg_valsr2)
#'
#' p1 <- rep(1, times = 50)
#' p0 <- rep(0, times = 500)
#' p <- c(p1, p0)
#'
#' maxdf <- data.frame(p, valsr1, valsr2)
#'
#' coords <- rbind(occs_d1, occs_d2)
#' colnames(coords) <- c("longitude", "latitude")
#' depth_vector     <- c(rep(1, times = 25), rep(2, times = 25))
#'
#' # Use data.frame instead of cbind so $depth is completely valid inside maxent_3D
#' occs_dataframe <- data.frame(coords, depth = depth_vector)
#'
#'\donttest{
#' # Pass the clean data.frame to the function
#' if(requireNamespace("rJava", quietly = TRUE)){
#'   result <- maxent_3D(maxent_df = maxdf, wanted_fc = c("L", "Q"),
#'                       wanted_rm = c(1:2), projection_layers = envlist,
#'                       occs = occs_dataframe, depth_list = c(1,2))
#' }
#' }
#'
#' @importFrom terra extract
#'
#' @import predicts
#'
#' @keywords MaxEnt
#'
#' @export


maxent_3D <- function(maxent_df, wanted_fc, wanted_rm, wanted_partition=NULL,
                      projection_layers, occs, depth_list) {

  if(!requireNamespace("rJava", quietly = TRUE)){
    warning(message("rJava not available, cannot run MaxEnt."))
    return(NULL)
  }

  # initial check of proper formatting
  if(!("p" %in% colnames(maxent_df))) {
    warning(message("Argument 'maxent_df' is missing column 'p'.\n"))
    return(NULL)
  }

  if(any(!(c("longitude", "latitude", "depth") %in% colnames(occs)))) {
    warning(message("Argument 'occs' should have columns named 'longitude',
                    'latitude' and 'depth'.\n"))
    return(NULL)
  }

  if(!(is.numeric(wanted_rm))) {
    warning(message("Argument 'wanted_rm' must be a numeric vector.\n"))
    return(NULL)
  }

  if(any(!(names(projection_layers[[1]]) %in% colnames(maxent_df)))) {
    warning(message("Argument 'projection_layers' should contain the same names as
                    the column names for 'maxent_df'.\n"))
    return(NULL)
  }

  if(!(is.numeric(depth_list)) | any(depth_list < 0)) {
    warning(message("Argument 'depth_list' should be numeric, go from the shallowest
                    to the deepest depth, and should be all positive.\n"))
    return(NULL)
  }

  # Confirm predicts is using the correct .jar file
  pred_jar_dir <- system.file("java", package = "predicts")
  pred_jar <- file.path(pred_jar_dir, "maxent.jar")
  rJava::.jinit(classpath = pred_jar, force.init = TRUE)

  # lets remove incomplete cases from the maxent_df so all points match
  maxent_df_present <- maxent_df[maxent_df$p == 1,]
  maxent_df_absent <- maxent_df[maxent_df$p == 0,]
  maxent_df_present <- maxent_df_present[complete.cases(maxent_df_present),]
  df_for_maxent <- rbind(maxent_df_present, maxent_df_absent)

  # first we'll make the fc input readable to maxent
  new_wanted_fc <- vector("list", length = length(wanted_fc))
  for(i in 1:length(wanted_fc)) {
    if(wanted_fc[i] == "L") {
      wanted_fc1 <- c("noquadratic", "noproduct", "nohinge", "noautofeature")
    } else if(wanted_fc[i] == "Q") {
      wanted_fc1 <- c("nolinear", "noproduct", "nohinge", "noautofeature")
    } else if(wanted_fc[i] == "H") {
      wanted_fc1 <- c("nolinear", "noquadratic", "noproduct", "noautofeature")
    } else if(wanted_fc[i] == "P") {
      wanted_fc1 <- c("nolinear", "noquadratic", "nohinge", "noautofeature")
    } else if(wanted_fc[i] == "LQ") {
      wanted_fc1 <- c("nohinge", "noproduct", "noautofeature")
    } else if(wanted_fc[i] == "LH") {
      wanted_fc1 <- c("noproduct", "noquadratic", "noautofeature")
    } else if(wanted_fc[i] == "LP") {
      wanted_fc1 <- c("noquadratic", "nohinge", "noautofeature")
    } else if(wanted_fc[i] == "QH") {
      wanted_fc1 <- c("nolinear", "noproduct", "noautofeature")
    } else if(wanted_fc[i] == "QP") {
      wanted_fc1 <- c("nolinear", "nohinge", "noautofeature")
    } else if(wanted_fc[i] == "HP") {
      wanted_fc1 <- c("nolinear", "noquadratic", "noautofeature")
    } else if(wanted_fc[i] == "LQP") {
      wanted_fc1 <- c("nohinge", "noautofeature")
    } else if(wanted_fc[i] == "LQH") {
      wanted_fc1 <- c("noproduct", "noautofeature")
    } else if(wanted_fc[i] == "QHP") {
      wanted_fc1 <- c("nolinear", "noautofeature")
    } else if(wanted_fc[i] == "LHP") {
      wanted_fc1 <- c("noquadratic", "noautofeature")
    } else if(wanted_fc[i] == "LQHP") {
      wanted_fc1 <- c("noautofeature")
    } else {
      warning(message("Not a usable feature class.\n"))
      return(NULL)
    }
    new_wanted_fc[[i]] <- wanted_fc1
  }

  # next we'll make a full list of the fc and rm combinations
  # j loop for each rm, make a list the length of rm attached to fc[i]
  # so there will be i lists for each fc
  # concatenate the i lists into one list to make the full combined rm and fc list
  fc_perm_list <- vector("list", length = length(new_wanted_fc))
  for(i in 1:length(new_wanted_fc)) {
    rm_perm_list <- vector("list", length = length(wanted_rm))
    for(j in 1:length(wanted_rm)) {
      rm_element <- wanted_rm[j]
      fc_element <- new_wanted_fc[[i]]
      rm_perm_list[[j]] <- list(rm_element, fc_element)
    }
    fc_perm_list[[i]] <- rm_perm_list
  }
  final_perm_call <- vector("list", length = length(wanted_fc))
  for(i in 1:length(wanted_fc)) {
    rm_parts <- vector(length = length(wanted_rm))
    for(j in 1:length(wanted_rm)) {
      rm_parts[j] <- paste0("fc_perm_list[[", i, "]][[", j, "]]")
    }
    final_perm_call[[i]] <- rm_parts
  }
  final_perm_call_real <- unlist(final_perm_call)
  final_perm_list <- vector("list", length = length(final_perm_call_real))
  for(i in 1:length(final_perm_call_real)) {
    final_perm_list[[i]] <- eval(parse(text = final_perm_call_real[i]))
  }

  # if there is no partitioning scheme, a model will be run that is trained
  # on all of the data. These models will also be returned as the models
  # run for a partitioning scheme, but the partitioning scheme run will also
  # return validation statistics. it can only return AUC
  if(is.null(wanted_partition)) {
    model_list <- vector("list", length = length(final_perm_list))
    auc_list <- vector("list", length = length(final_perm_list))
    total_coef_list <- vector(length = length(final_perm_list))
    nonzero_coef_list <- vector(length = length(final_perm_list))
    predictions_list <- vector("list", length = length(final_perm_list))
    AICc_list <- vector(length = length(final_perm_list))
    print("running models")

    for(i in 1:length(final_perm_list)) {
      # create model
      mod1 <- predicts::MaxEnt(x = df_for_maxent[,!names(df_for_maxent) %in% "p"],
                               p = c(df_for_maxent[,"p"]),
                               args = c(paste0("betamultiplier=",
                                               final_perm_list[[i]][[1]]),
                                        paste0(final_perm_list[[i]][[2]])))
      # save model
      model_list[[i]] <- mod1
      # retrieve AUC
      auc_list[[i]] <- mod1@results[which(rownames(mod1@results) ==
                                            "Training.AUC"),1]
      # retrieve coefficents
      all_coef <- suppressWarnings(as.numeric(unlist(strsplit(mod1@lambdas, ","))))
      all_coef <- all_coef[-which(is.na(all_coef))]
      total_coef_list[i] <- length(all_coef)
      nonzero_coef <- all_coef[which(all_coef != 0)]
      nonzero_coef <- nonzero_coef[which(nonzero_coef != 0.000000e+00)]
      nonzero_coef_list[i] <- length(nonzero_coef)
      # generate predictions and pull out values at occurrences for AICc calc
      predicted_suit_list <- vector("list", length = length(projection_layers))
      wanted_val_list <- vector("list", length = length(projection_layers))
      for(j in 1:length(projection_layers)) {
        predicted_suit <- predict(mod1, projection_layers[[j]])
        predicted_suit_list[[j]] <- predicted_suit
        standard_suit <- predicted_suit
        values(standard_suit) <- values(standard_suit)/max(values(standard_suit),
                                                           na.rm = TRUE)
        needed_occs <- occs[occs$depth == depth_list[j],]
        if(nrow(needed_occs > 0)) {
          wanted_val_list[[j]] <- terra::extract(standard_suit,
                                                 data.frame(needed_occs$longitude,
                                                            needed_occs$latitude))
        } else {
          wanted_val_list[[j]] <- NA
        }
      }
      wanted_val_final <- do.call(rbind, wanted_val_list)[,1]
      wanted_val_final <- wanted_val_final[which(!(is.na(wanted_val_final)))]
      predictions_list[[i]] <- rast(predicted_suit_list)


      # calculating AICc
      bigk <- length(nonzero_coef)
      likelihood <- sum(log(wanted_val_final))
      littlen <- length(wanted_val_final)
      if(bigk == (littlen - 1)) {
        AICc_list[i] <- NA
      } else {
        AICc <- ((2*bigk) - (2*likelihood)) +
          (((2*bigk)*(bigk + 1))/(littlen - bigk - 1))
        AICc_list[i] <- AICc
      }
    }
    all_auc <- do.call(rbind, auc_list)
    rm <- rep(wanted_rm, times = length(wanted_fc))
    fc_list <- vector("list", length = length(wanted_fc))
    for(i in 1:length(wanted_fc)) {
      fc_list[[i]] <- rep(wanted_fc[i], times = length(wanted_rm))
    }
    fc <- unlist(fc_list)
    delta.AICc <- AICc_list - min(AICc_list, na.rm = T)
    mod_results <- data.frame(fc, rm, all_auc[,1], total_coef_list, nonzero_coef_list,
                              AICc_list, delta.AICc)
    colnames(mod_results) <- c("fc", "rm", "train.AUC", "toal.coef", "nonzero.coef",
                               "AICc", "delta.AICc")
    final_output <- list(models = model_list, predictions = predictions_list,
                         results = mod_results)

  # generating models for a given partition scheme
  } else {

    # partition warnings
    if(!(all(is.list(wanted_partition),
             c("occ_partitions", "bg_partitions") %in% names(wanted_partition)))) {
      warning(message("Argument 'wanted_partition' should be a list containing the
                      named elements 'occ_partitions' and 'bg_partitions', as output
                      by partition_3D().\n"))
      return(NULL)
    }

    if(length(wanted_partition$occ_partitions) != nrow(maxent_df[maxent_df$p == 1,])) {
      warning(message("Number of occurrences in partition does not match number of
                      occurrences in 'maxent_df'.\n"))
      return(NULL)
    }

    # full models for model part of output
    model_list <- vector("list", length = length(final_perm_list))
    print("running models")
    for(i in 1:length(final_perm_list)) {
      # create model
      mod1 <- MaxEnt(x = df_for_maxent[,-1], p = df_for_maxent[,1],
                     args = c(paste0("betamultiplier=",
                                     final_perm_list[[i]][[1]]),
                              paste0(final_perm_list[[i]][[2]])))
      # save model
      model_list[[i]] <- mod1
    }

    num_of_partitions <- unique(wanted_partition$bg_partitions)
    # generating training and testing models for each partition

    part_model_list <- vector("list", length = length(num_of_partitions))
    part_eval_list <- vector("list", length = length(num_of_partitions))
    part_predictions_list <- vector("list", length = length(num_of_partitions))
    present_only <- df_for_maxent[which(df_for_maxent$p == 1),]
    absent_only <- df_for_maxent[which(df_for_maxent$p == 0),]
    for (i in (1:length(num_of_partitions))) {
      test_p <- present_only[which(wanted_partition$occ_partitions ==
                                     num_of_partitions[i]),]
      train_p <- present_only[which(wanted_partition$occ_partitions !=
                                      num_of_partitions[i]),]
      test_a <- absent_only[which(wanted_partition$bg_partitions ==
                                    num_of_partitions[i]),]
      train_a <- absent_only[which(wanted_partition$bg_partitions !=
                                     num_of_partitions[i]),]
      test_full <- rbind(test_p, test_a)
      train_full <- rbind(train_p, train_a)
      mod_per_param_list <- vector("list", length = length(final_perm_list))
      auc_val_per_param_list <- vector(length = length(final_perm_list))
      auc_train_per_param_list <- vector("list", length = length(final_perm_list))
      total_coef_list <- vector(length = length(final_perm_list))
      nonzero_coef_list <- vector(length = length(final_perm_list))
      predictions_list <- vector("list", length = length(final_perm_list))
      print(paste0("training and testing models for partition ", i))
      for(j in 1:length(final_perm_list)) {
        # generating each model combination for partition i
        train_model <- MaxEnt(x = train_full[,-1], p = train_full[,1],
                              args = c(paste0("betamultiplier=",
                                              final_perm_list[[j]][[1]]),
                                       paste0(final_perm_list[[j]][[2]])))
        # saving model
        mod_per_param_list[[j]] <- train_model
        # retrieving validation auc
        test_full_present <- test_full[test_full$p == 1,]
        test_full_absent <- test_full[test_full$p == 0,]
        ev <- predicts::pa_evaluate(p = test_full_present[,-1],
                                    a = test_full_absent[,-1],
                                    model = train_model)
        auc_val_per_param_list[j] <- ev@stats$auc
        # retrieving training AUC
        auc_train_per_param_list[[j]] <-
          train_model@results[which(rownames(train_model@results) == "Training.AUC"),1]
        # retrieve coefficents
        all_coef <- suppressWarnings(as.numeric(unlist(strsplit(train_model@lambdas, ","))))
        all_coef <- all_coef[-which(is.na(all_coef))]
        total_coef_list[j] <- length(all_coef)
        nonzero_coef <- all_coef[which(all_coef != 0)]
        nonzero_coef <- nonzero_coef[which(nonzero_coef != 0.000000e+00)]
        nonzero_coef_list[j] <- length(nonzero_coef)
        # generate predictions; AICc will be calculated after model averaging across
        # partitions
        predicted_suit_list <- vector("list", length = length(projection_layers))
        wanted_val_list <- vector("list", length = length(projection_layers))
        for(k in 1:length(projection_layers)) {
          predicted_suit <- predict(train_model, projection_layers[[k]])
          predicted_suit_list[[k]] <- predicted_suit
        }
        predictions_list[[j]] <- predicted_suit_list
      }
      part_model_list[[i]] <- mod_per_param_list
      all_auc_train <- do.call(rbind, auc_train_per_param_list)
      rm <- rep(wanted_rm, times = length(wanted_fc))
      fc_list <- vector("list", length = length(wanted_fc))
      for(j in 1:length(wanted_fc)) {
        fc_list[[j]] <- rep(wanted_fc[j], times = length(wanted_rm))
      }
      fc <- unlist(fc_list)
      mod_results <- data.frame(fc, rm, all_auc_train, auc_val_per_param_list,
                                total_coef_list,
                                nonzero_coef_list)
      colnames(mod_results) <- c("fc", "rm", "train.AUC", "val.AUC", "total.coef",
                                 "nonzero.coef")
      part_eval_list[[i]] <- mod_results
      part_predictions_list[[i]] <- predictions_list
    }

    # creating a mod results dataframe averaged across all partitions
    wantdf <- part_eval_list[[1]][,-c(1:2)]
    cell_num <- (nrow(wantdf)*ncol(wantdf))
    avg_mod_result <- matrix(nrow = nrow(wantdf),
                             ncol = ncol(wantdf))
    for(i in 1:cell_num) {
      avg_vec <- vector(length = length(num_of_partitions))
      for(j in 1:length(num_of_partitions)) {
        wantdf <- part_eval_list[[j]][,-c(1:2)]
        avg_vec[j] <- as.matrix(wantdf)[i]
      }
      avg_mod_result[i] <- mean(avg_vec)
    }
    avg_mod_result <- data.frame(avg_mod_result)
    colnames(avg_mod_result) <- c("avg.train.AUC", "avg.val.AUC", "avg.total.coef",
                                  "avg.nonzero.coef")
    avg_mod_result <- cbind(part_eval_list[[1]][,1:2], avg_mod_result)

    # creating an average suitability list across partitions for final predictions
    # and average AICc
    avg_predictions_list <- vector("list", length = length(final_perm_list))
    for(i in 1:length(final_perm_list)) {
      avg_suit_list <- vector("list", length = length(projection_layers))
      for(j in 1:length(projection_layers)) {
        partition_layers <- vector("list", length = length(num_of_partitions))
        for(k in 1:length(num_of_partitions)) {
          partition_layers[[k]] <- part_predictions_list[[k]][[i]][[j]]
        }
        avg_across_depth_for_model_i <- terra::app(rast(partition_layers), mean)
        avg_suit_list[[j]] <- avg_across_depth_for_model_i
      }
      avg_predictions_list[[i]] <- rast(avg_suit_list)
    }

    # calculating average AICc for each model type
    avg.AICc <- vector(length = length(final_perm_list))
    for(i in 1:length(final_perm_list)) {
      wanted_val_list <- vector("list", length = length(projection_layers))
      for(j in 1:length(projection_layers)) {
        standard_suit <- avg_predictions_list[[i]][[j]]
        values(standard_suit) <- values(standard_suit)/max(values(standard_suit),
                                                           na.rm = T)
        needed_occs <- occs[occs$depth == depth_list[j],]
        if(nrow(needed_occs) > 0) {
          wanted_val_list[[j]] <- terra::extract(standard_suit,
                                    data.frame(needed_occs$longitude, needed_occs$latitude))
        } else {
          wanted_val_list[[j]] <- NA
        }
      }
      wanted_val_final <- do.call(rbind, wanted_val_list)[,1]
      wanted_val_final <- wanted_val_final[which(!(is.na(wanted_val_final)))]
      # calculating AICc
      bigk <- avg_mod_result$avg.nonzero.coef[i]
      likelihood <- sum(log(wanted_val_final))
      littlen <- length(wanted_val_final)
      if(bigk == (littlen - 1)) {
        avg.AICc[i] <- NA
      } else {
        AICc <- ((2*bigk) - (2*likelihood)) +
          (((2*bigk)*(bigk + 1))/(littlen - bigk - 1))
        avg.AICc[i] <- AICc
      }
    }
    avg.delta.AICc <- avg.AICc-min(avg.AICc)
    avg_mod_result <- cbind(avg_mod_result, avg.AICc, avg.delta.AICc)

    final_output <- list(models = model_list, predictions = avg_predictions_list,
                         results = avg_mod_result, partition_results = part_eval_list)
  }
  return(final_output)
}

#' @title Occurrence cell removal
#'
#' @description Removes cells from raster that contain occurrences
#'
#' @param occs A `data.frame` with at least two columns
#' named "longitude" and "latitude" or that
#' can be coerced into this format.
#'
#' @param rasterTemplate A `SpatRaster` object to serve
#' as a template for cells to be removed.
#'
#' @details This is an internal function to remove cells that
#' intersect with occurrences from a `SpatRaster` template object. This
#' template can then be overlaid onto a `SpatRaster` vector to
#' remove occurrences from all layers.
#'
#' @return A `SpatRaster`
#'
#' @examples
#'
#' library(terra)
#' # Create sample raster
#' r <- rast(ncol=10, nrow=10)
#' values(r) <- 1:100
#'
#' # Create test occurrences
#' set.seed(0)
#' longitude <- sample(ext(r)[1]:ext(r)[2],
#'                     size = 10, replace = FALSE)
#' set.seed(0)
#' latitude <- sample(ext(r)[3]:ext(r)[4],
#'                    size = 10, replace = FALSE)
#' occs <- data.frame(longitude, latitude)
#'
#' # Here's the function
#' result <- occCellRemoval(occs = occs, rasterTemplate = r)
#'
#' @import terra
#'
#' @keywords internal
#'
#' @noRd

occCellRemoval <- function(occs, rasterTemplate){
  # Handling alternative column names for occurrences
  colNames <- colnames(occs)
  cp <- columnParse(occs)
  xIndex <- cp$xIndex
  yIndex <- cp$yIndex

  # Meat of function
  occCells <- cellFromXY(object = rasterTemplate, occs[,c(xIndex,yIndex)])
  rasterTemplate[occCells] <- NA
  return(rasterTemplate)
}

#' @title 2D background sampling
#'
#' @description Samples in 2D at resolution of raster
#'
#' @param occs A dataframe with at least two columns
#' named "longitude" and "latitude", or that can be
#' coerced into this format.
#'
#' @param rasterTemplate A `SpatRaster` object to serve
#' as a template for generating background sampling
#' coordinates.
#'
#' @param mShp A shapefile defining the area from
#' which background points should be sampled.
#'
#' @param verbose `logical`. Switching to `FALSE` mutes message describing
#' which columns in `occs` are interpreted as x and y coordinates.
#'
#' @details This function is designed to sample background points
#' for distributional modeling in two dimensions. The returned
#' `data.frame` contains all points from across the designated
#' background. It is up to the user to determine how to
#' appropriately sample from those background points.
#'
#' @return A `data.frame` with 2D coordinates of points
#' for background sampling.
#'
#' @examples
#' library(terra)
#'
#' # Create sample raster
#' r <- rast(ncol=10, nrow=10)
#' values(r) <- 1:100
#'
#' # Create test occurrences
#' set.seed(0)
#' longitude <- sample(ext(r)[1]:ext(r)[2],
#'                     size = 10, replace = FALSE)
#' set.seed(0)
#' latitude <- sample(ext(r)[3]:ext(r)[4],
#'                    size = 10, replace = FALSE)
#' occurrences <- data.frame(longitude,latitude)
#'
#' # Generate background sampling buffer
#' buffPts <- vect(occurrences,
#'                 c("longitude", "latitude"))
#' crs(buffPts) <- crs(r)
#' mShp <- aggregate(buffer(buffPts, width = 1000000))
#'
#' # Here's the function
#' result <- mSampling2D(occs = occurrences, rasterTemplate = r, mShp = mShp)
#'
#' @import terra
#'
#' @keywords backgroundSampling
#'
#' @export

mSampling2D <- function(occs, rasterTemplate, mShp, verbose = TRUE){
  if(!is.data.frame(occs)){
    warning(paste0("'occs' must be an object of class 'data.frame'.\n"))
    return(NULL)
  }

  if(!grepl("SpatRaster", class(rasterTemplate))){
    warning(paste0("'rasterTemplate' must be of class 'SpatRaster'.\n"))
    return(NULL)
  }

  if(!is(mShp, "SpatVector")){
    warning(paste0("'mShp' must be of class 'SpatVector'.\n"))
    return(NULL)
  }

  if (!is.logical(verbose)) {
    warning(paste0("Argument 'verbose' is not of type 'logical'.\n"))
    return(NULL)
  }

  # Parse columns
  colNames <- colnames(occs)
  colParse <- columnParse(occs)
  if(is.null(colParse)){
    return(NULL)
  }
  xIndex <- colParse$xIndex
  yIndex <- colParse$yIndex
  interp <- colParse$reportMessage

  if(verbose){
    message(interp)
  }

  rasterTemplate <- crop(x = mask(x = rasterTemplate, mask = mShp), y = mShp)
  rawLayer <- occCellRemoval(occs = occs[,c(xIndex,yIndex)],
                             rasterTemplate)
  mPts <- data.frame(xyFromCell(rawLayer, cell = 1:ncell(rawLayer)))
  mPts$extract <- extract(x = rawLayer, y = mPts)[,2]
  mPts <- mPts[!is.na(mPts$extract),1:2]
  colnames(mPts) <- colNames[c(xIndex, yIndex)]
  return(mPts)
}

#' @title 3D background sampling
#'
#' @description Samples XYZ coordinates from a shapefile
#' from maximum to minimum occurrence depth at XYZ
#' resolution of envBrick.
#'
#' @param occs A `data.frame` with at least three columns
#' named "longitude", "latitude", and "depth", or that
#' can be coerced into this format.
#'
#' @param envBrick A `SpatRaster` vector object to serve
#' as a template for generating background sampling
#' coordinates.
#'
#' @param mShp A shapefile defining the area from
#' which background points should be sampled.
#'
#' @param depthLimit An argument controlling the depth
#' extent of sampling. Refer to `Details` for more information.
#'
#' @param verbose `logical`. Switching to `FALSE` mutes message describing
#' which columns in `occs` are interpreted as x, y, and z coordinates.
#'
#' @details This function is designed to sample background points for
#' distributional modeling in three dimensions. If a voxel (3D pixel)
#' in the `SpatRaster` vector intersects with an occurrence from
#' `occs`, it is removed. Note that this function returns points
#' representing every voxel in the background area within the
#' specified depth range. It is up to the user to downsample from
#' these data as necessary, depending on the model type being used.
#'
#' `depthLimit` argument options:
#' \itemize{
#'   \item `occs` Samples background from the full depth extent of `occs`.
#'   \item `all` Samples background from the full depth extent of `envBrick`.
#'   \item A `vector` of length 2 with maximum and minimum depth values from
#'   which to sample.
#' }
#'
#' @return A `data.frame` with 3D coordinates of points for background
#' sampling.
#'
#' @examples
#' library(terra)
#'
#' # Create test raster
#' r1 <- rast(ncol=10, nrow=10)
#' values(r1) <- 1:100
#' r2 <- rast(ncol=10, nrow=10)
#' values(r2) <- c(rep(20, times = 50), rep(60, times = 50))
#' r3 <- rast(ncol=10, nrow=10)
#' values(r3) <- 8
#' envBrick <- c(r1, r2, r3)
#' names(envBrick) <- c(0, 10, 30)
#'
#' # Create test occurrences
#' set.seed(0)
#' longitude <- sample(ext(envBrick)[1]:ext(envBrick)[2],
#'                     size = 10, replace = FALSE)
#' set.seed(0)
#' latitude <- sample(ext(envBrick)[3]:ext(envBrick)[4],
#'                    size = 10, replace = FALSE)
#' set.seed(0)
#' depth <- sample(0:35, size = 10, replace = TRUE)
#' occurrences <- data.frame(longitude,latitude,depth)
#'
#' # Generate background sampling buffer
#' buffPts <- vect(occurrences,
#'                 c("longitude", "latitude"))
#' crs(buffPts) <- crs(envBrick)
#' mShp <- aggregate(buffer(buffPts, width = 1000000))
#'
#' # Here's the function
#' occSample3d <- mSampling3D(occs = occurrences,
#'                            envBrick = envBrick,
#'                            mShp = mShp,
#'                            depthLimit = "occs")
#'
#' @import terra
#'
#' @keywords backgroundSampling
#'
#' @export

mSampling3D <- function(occs, envBrick, mShp, depthLimit = "all", verbose = TRUE){
  if(!is.data.frame(occs)){
    warning(paste0("'occs' must be an object of class 'data.frame'.\n"))
    return(NULL)
  }

  if(ncol(occs) < 3){
    warning(paste0("'occs' must have at least three columns.\n"))
    return(NULL)
  }

  if(!is(envBrick, "SpatRaster")){
    warning(paste0("'envBrick' must be of class 'SpatRaster'.\n"))
    return(NULL)
  }

  if(!is(mShp, "SpatVector")){
    warning(paste0("'mShp' must be of class 'SpatVector'.\n"))
    return(NULL)
  }

  if(!(class(depthLimit) %in% c("character","numeric"))){
    warning(paste0("'depthLimit' must be of class 'character' or 'numeric'.\n"))
    return(NULL)
  }

  if(is(depthLimit, "numeric")){
    if(length(depthLimit) != 2){
      warning(paste0("'depthLimit' arguments of 'numeric' must be of length 2.\n"))
      return(NULL)
    }
  }

  if(is(depthLimit, "character")){
    if(length(depthLimit) > 1){
      warning(paste0(depthLimit, " is not a valid value for 'depthLimit'\n"))
      return(NULL)
    } else if(!(depthLimit %in% c("all", "occs"))){
      warning(paste0(depthLimit, " is not a valid value for 'depthLimit'\n"))
      return(NULL)
    }
  }

  if (!is.logical(verbose)) {
    warning(paste0("Argument 'verbose' is not of type 'logical'.\n"))
    return(NULL)
  }

  # Parse columns
  colNames <- colnames(occs)
  colParse <- columnParse(occs, wDepth = TRUE)
  if(is.null(colParse)){
    return(NULL)
  }
  xIndex <- colParse$xIndex
  yIndex <- colParse$yIndex
  zIndex <- colParse$zIndex
  interp <- colParse$reportMessage

  if(verbose){
    message(interp)
  }

  # Checking for appropriate environmental layer names
  layerNames <- as.numeric(gsub("[X]", "", names(envBrick)))
  if(sum(is.na(layerNames)) > 0){
    message("\nInput SpatRaster vector names inappropriate: \n",
            paste(names(envBrick), collapse = ", "), "\n",
            "Names must follow the format 'X' ",
            "followed by a number corresponding to ",
            "the starting depth of the layer.")
    return(NULL)
  }

  # Depth slice indices for occurrences
  occs$index <- unlist(lapply(occs[,zIndex],
                              FUN = function(x) which.min(abs(layerNames - x))))

  # Get depth range
  if(is(depthLimit, "numeric")){
    depthRange <- c(which.min(abs(layerNames - min(depthLimit))),
                    which.min(abs(layerNames - max(depthLimit))))
  } else if(depthLimit == "occs"){
    depthRange <- c(min(occs$index), max(occs$index))
  } else {
    depthRange <- c(1, nlyr(envBrick))
  }

  envBrick <- crop(x = mask(x = envBrick[[depthRange[[1]]:depthRange[[2]]]],
                        mask = mShp), y = mShp)
  mPts <- data.frame()
  for(i in 1:length(names(envBrick))){
    rawLayer <- envBrick[[i]]
    layerDepth <- as.numeric(gsub("[X]", "", names(rawLayer)))
    occsAtLayerDepth <- occs[occs$index == match(layerDepth, layerNames),]
    if (nrow(occsAtLayerDepth) > 0){
      rawLayer <- occCellRemoval(occs = occsAtLayerDepth[,c(xIndex,yIndex)],
                                 rawLayer)
    }
    tempPoints <- data.frame(xyFromCell(rawLayer,
                                        cell = 1:ncell(rawLayer)))
    tempPoints$extract <- extract(x = rawLayer, y = tempPoints)[,2]
    tempPoints <- tempPoints[!is.na(tempPoints$extract),1:2]
    colnames(tempPoints) <- colNames[c(xIndex, yIndex)]
    tempPoints[,zIndex] <- rep(layerDepth, times = nrow(tempPoints))
    mPts <- rbind(mPts, tempPoints)
  }
  colnames(mPts)[[3]] <- colNames[[zIndex]]
  return(mPts)
}

library(terra)
# Create test brick
r1 <- rast(ncol=10, nrow=10)
values(r1) <- 1:100
r2 <- rast(ncol=10, nrow=10)
values(r2) <- c(rep(20, times = 50), rep(60, times = 50))
r3 <- rast(ncol=10, nrow=10)
values(r3) <- 8
rBrick <- c(r1, r2, r3)
names(rBrick) <- c(0, 10, 100)

# Create sample raster
r <- r1

# Create test occurrences
set.seed(1)
longitude <- sample(ext(r)[1]:ext(r)[2],
                    size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(r)[3]:ext(r)[4],
                   size = 10, replace = FALSE)

set.seed(0)
depth <- sample(0:35, size = 10, replace = TRUE)

occurrences <- data.frame(longitude, latitude, depth)

# Generate background sampling buffer
buffPts <- vect(occurrences,
                c("longitude", "latitude"))
crs(buffPts) <- crs(r)
mShp <- aggregate(buffer(buffPts, width = 1000000))

# occCellRemoval() tests ----
test_that("occCellRemoval behaves as expected", {
  expect_error(voluModel:::occCellRemoval())

  testResult <- voluModel:::occCellRemoval(occurrences, r)
  expect_equal(class(testResult)[[1]], "SpatRaster")
  expect_length(values(testResult), 100)

  naCount <- length(testResult[is.na(testResult)])
  expect_equal(9, naCount)
})

#mSampling2d() tests ----
test_that("mSampling2D input warnings behave as expected", {
  expect_error(mSampling2D())
  expect_warning(mSampling2D(occs = "a", rasterTemplate = r,
                             mShp = mShp))
  expect_warning(mSampling2D(occs = occurrences[,1], rasterTemplate = r,
                             mShp = mShp))
  expect_warning(mSampling2D(occs = occurrences, rasterTemplate = "a",
                             mShp = mShp))
  expect_warning(mSampling2D(occs = occurrences, rasterTemplate = r,
                             mShp = "a"))
  expect_warning(mSampling2D(occs = occurrences, rasterTemplate = r,
                             mShp = mShp, verbose = "dragon"))

  colnames(occurrences) <- c("spam", "eggs")
  expect_warning(mSampling2D(occs = occurrences, rasterTemplate = r,
                             mShp = mShp))
})

test_that("mSampling2D outputs as expected", {
  expect_error(mSampling2D())

  testResult <- mSampling2D(occurrences, r, mShp = mShp)
  expect_true(is.data.frame(testResult))
  expect_equal(nrow(testResult), 25)
})

# mSampling3D() tests ----
test_that("mSampling3D input warnings behave as expected", {
  expect_error(mSampling3D())
  expect_warning(mSampling3D(occs = "a", envBrick = rBrick, mShp = mShp))
  expect_warning(mSampling3D(occs = occurrences[,1:2], envBrick = rBrick))
  expect_warning(mSampling3D(occs = occurrences[,1], envBrick = rBrick))
  expect_warning(mSampling3D(occs = occurrences, envBrick = "a"))
  expect_warning(mSampling3D(occs = occurrences, envBrick = rBrick,
                             mShp = mShp, verbose = "dragon"))

  colnames(occurrences) <- c("spam", "eggs", "cheese")
  expect_warning(mSampling3D(occs = occurrences, envBrick = rBrick,
                             mShp = mShp))
  colnames(occurrences) <- c("longitude", "latitude", "depth")

  names(rBrick) <- c("a", "b", "c")
  expect_warning(mSampling3D(occs = occurrences, envBrick = rBrick,
                             mShp = mShp))

  names(rBrick) <- c(0, 10, 100)
  expect_warning(mSampling3D(occs = occurrences, envBrick = rBrick,
                             mShp = "cheese"))

  expect_warning(mSampling3D(occs = occurrences,
                             envBrick = rBrick, mShp = mShp,
                             depthLimit = rBrick))
  expect_warning(mSampling3D(occs = occurrences,
                             envBrick = rBrick, mShp = mShp,
                             depthLimit = c(0,1,100)))
  expect_warning(mSampling3D(occs = occurrences,
                             envBrick = rBrick,  mShp = mShp,
                             depthLimit = "cheese"))
  expect_warning(mSampling3D(occs = occurrences,
                             envBrick = rBrick,  mShp = mShp,
                             depthLimit = c("all", "occs")))

})

test_that("mSampling3D outputs as expected", {
  # Testing
  testResult <- mSampling3D(occurrences, rBrick, mShp = mShp,
                            verbose = FALSE)
  expect_true(is.data.frame(testResult))
  expect_equal(nrow(testResult), 93)

  testResult <- mSampling3D(occurrences, rBrick, mShp = mShp,
                            depthLimit = "occs", verbose = FALSE)
  expect_true(is.data.frame(testResult))
  expect_equal(nrow(testResult), 59)

  testResult <- mSampling3D(occurrences, rBrick, mShp = mShp,
                            depthLimit = "all", verbose = FALSE)
  expect_true(is.data.frame(testResult))
  expect_equal(nrow(testResult), 93)

  testResult <- mSampling3D(occurrences, rBrick, mShp = mShp,
                            depthLimit = c(5, 25), verbose = FALSE)
  expect_true(is.data.frame(testResult))
  expect_equal(nrow(testResult), 59)
})

#' @title Column Parsing
#'
#' @description Parses column names from input occurrence
#' `data.frame` for more seamless function
#'
#' @param occs A `data.frame` with at least two columns
#' named "longitude" and "latitude" or that
#' can be coerced into this format.
#'
#' @param wDepth Logical; flags whether a depth column should
#' also be sought.
#'
#' @details This is an internal function to return the putative
#' indices for latitude and longitude or x and y coordinates
#' of occurrences to allow for code that is more robust to
#' very common user error
#'
#' @return A `list` of length 2 with indices of the x and y
#' columns, respectively, followed by a message with a plain
#' text report of which columns were interpreted as x and y.
#'
#' @examples
#' library(terra)
#'
#' # Create sample raster
#' r <- rast(ncol=10, nrow=10)
#' values(r) <- 1:100
#'
#' # Create test occurrences
#' set.seed(0)
#' longitude <- sample(ext(r)[1]:ext(r)[2],
#'                     size = 10, replace = FALSE)
#' set.seed(0)
#' latitude <- sample(ext(r)[3]:ext(r)[4],
#'                    size = 10, replace = FALSE)
#' set.seed(0)
#' depth <- sample(0:35, size = 10, replace = TRUE)
#' occurrences <- as.data.frame(cbind(longitude,latitude,depth))
#'
#' # Here's the function
#' result <- columnParse(occs = occurrences[,1:2],
#'                       wDepth = FALSE)
#' result <- columnParse(occs = occurrences,
#'                       wDepth = TRUE)
#'
#' @keywords internal
#'
#' @export

columnParse <- function(occs, wDepth = FALSE){
  # Handling alternative column names for occurrences
  colNames <- colnames(occs)
  xIndex <- grep(tolower(colNames), pattern = "long")
  yIndex <- grep(tolower(colNames), pattern = "lat")
  if(length(xIndex) > 1){xIndex <- xIndex[[1]]}
  if(length(yIndex) > 1){yIndex <- yIndex[[1]]}
  if(length(xIndex) == 0){
    xIndex <- grep(tolower(colNames), pattern = "x")
    if(length(xIndex) > 1){xIndex <- xIndex[[1]]}
    if(length(xIndex) < 1){
      warning(message("Could not parse\n ",
                      paste0(colNames, collapse = ", "), "\n ",
                      "into x and/or y coordinates."))
      return(NULL)
    }
  }
  if(length(yIndex) == 0){
    yIndex <- grep(tolower(colNames), pattern = "y")
    if(length(yIndex) > 1){yIndex <- yIndex[[1]]}
    if(length(yIndex) < 1){
      warning(message("Could not parse\n ",
                      paste0(colNames, collapse = ", "), "\n ",
                      "into x and/or y coordinates."))
      return(NULL)
    }
  }

  # Depth parsing
  if(wDepth){
    zIndex <- grep(tolower(colNames), pattern = "depth")
    if(length(zIndex) > 1){zIndex <- zIndex[[1]]}
    if(length(zIndex) == 0){
      zIndex <- grep(tolower(colNames), pattern = "z")
      if(length(zIndex) > 1){zIndex <- zIndex[[1]]}
      if(length(zIndex) < 1){
        warning(message("Could not parse\n ",
                        paste0(colNames, collapse = ", "), "\n ",
                        "into x, y, and/or z coordinates."))
        return(NULL)
      }
    }
    reportMessage <- paste0("Using ", colNames[xIndex], ", ",
                            colNames[yIndex], ", and ", colNames[zIndex],
                            "\n as x, y, and z coordinates, respectively.")
    return(list(xIndex = xIndex, yIndex = yIndex, zIndex = zIndex,
                reportMessage = reportMessage))
  } else {
    reportMessage <- paste0("Using ", colNames[xIndex], " and ",
                            colNames[yIndex],
                            "\n as x and y coordinates, respectively.")
    return(list(xIndex = xIndex, yIndex = yIndex,
                reportMessage = reportMessage))
  }
}

#' @title Occurrence downsampling
#'
#' @description Reduces number of occurrences to resolution of input raster
#'
#' @param occs A `data.frame` with at least two columns
#' named "longitude" and "latitude" or that
#' can be coerced into this format.
#'
#' @param rasterTemplate A `SpatRaster` object to serve
#' as a template for the resolution at which `occs` should be
#' downsampled.
#'
#' @param verbose `logical`. Switching to `FALSE` mutes message describing
#' which columns in `occs` are interpreted as x and y coordinates.
#'
#' @return A `data.frame` with two columns named "longitude"
#' and "latitude" or with names that were used when coercing
#' input data into this format.
#'
#' @examples
#' library(terra)
#' # Create sample raster
#' r <- rast(ncol=10, nrow=10)
#' values(r) <- 1:100
#'
#' # Create test occurrences
#' set.seed(0)
#' longitude <- sample(ext(r)[1]:ext(r)[2],
#'                     size = 10, replace = FALSE)
#' set.seed(0)
#' latitude <- sample(ext(r)[3]:ext(r)[4],
#'                    size = 10, replace = FALSE)
#' occurrences <- as.data.frame(cbind(longitude,latitude))
#'
#' # Here's the function
#' result <- downsample(occs = occurrences, rasterTemplate = r)
#'
#' @import terra
#' @importFrom stats complete.cases
#'
#' @keywords inputProcessing
#' @export

downsample <- function(occs, rasterTemplate, verbose = TRUE){
  if(!is.data.frame(occs)){
    warning(paste0("'occs' must be an object of class 'data.frame'.\n"))
    return(NULL)
  }

  if(!grepl("SpatRaster", class(rasterTemplate))){
    warning(paste0("'rasterTemplate' must be of class 'SpatRaster'.\n"))
    return(NULL)
  }

  if (!is.logical(verbose)) {
    warning(message("Argument 'verbose' is not of type 'logical'.\n"))
    return(NULL)
  }

  # Parse columns
  colNames <- colnames(occs)
  colParse <- columnParse(occs)
  if(is.null(colParse)){
    return(NULL)
  }
  xIndex <- colParse$xIndex
  yIndex <- colParse$yIndex
  interp <- colParse$reportMessage

  if(verbose){
    message(interp)
  }

  occCells <- cellFromXY(object = rasterTemplate, occs[,c(xIndex,yIndex)])
  occCells <- unique(occCells)
  occs <- xyFromCell(object = rasterTemplate, cell = occCells)
  occs <- occs[complete.cases(occs),]
  if(is.null(nrow(occs))){
    names(occs) <- colNames[c(xIndex,yIndex)]
    occs <- data.frame(t(occs))
  } else {
    colnames(occs) <- colNames[c(xIndex,yIndex)]
    occs <- data.frame(occs)
  }
  return(occs)
}

#' @title Bottom raster generation
#'
#' @description Samples deepest depth values from a
#' `SpatVector` data frame and generates a `SpatRaster`.
#'
#' @param rawPointData A `SpatVector` object from which
#' bottom variables will be sampled. See Details for more about format.
#'
#' @return A `SpatRaster` designed to approximate sea bottom
#' measurements for modeling species' distributions and/or niches.
#'
#' @details `rawPointData` is a `SpatVector` object that
#' contains measurements of a single environmental variable (e.g.
#' salinity, temperature, etc.) with x, y, and z coordinates. The
#' measurements in the `data.frame` should be organized so that each
#' column is a depth slice, increasing in depth from left to right. The
#' function was designed around the oceanographic data shapefiles supplied
#' by the World Ocean Atlas
#' (\url{https://www.ncei.noaa.gov/access/world-ocean-atlas-2018/}).
#' The function selects the "deepest" (rightmost) measurement at each
#' x, y coordinate pair that contains data. These measurements are then
#' rasterized at the resolution and extent of the x,y coordinates, under
#' the assumption that x and y intervals are equal and represent the center
#' of a cell.
#'
#' @examples
#'
#' library(terra)
#'
#' # Create point grid
#' coords <- data.frame(x = rep(seq(1:5), times = 5),
#'                     y = unlist(lapply(1:5, FUN = function(x) {
#'                       rep(x, times = 5)})))
#'
#' # Create data and add NAs to simulate uneven bottom depths
#' dd <- data.frame(SURFACE = 1:25,
#'                 d5M = 6:30,
#'                 d10M = 11:35,
#'                 d25M = 16:40)
#' dd$d25M[c(1:5, 18:25)] <- NA
#' dd$d10M[c(3:5, 21:23)] <- NA
#' dd$d5M[c(4, 22)] <- NA
#'
#' dd[,c("x","y")] <- coords
#'
#' # Create SpatialPointsDataFrame
#' sp <- vect(dd, geom = c("x", "y"))
#'
#' # Here's the function
#' result <- bottomRaster(rawPointData = sp)
#' plot(result)
#'
#' @import terra
#'
#' @keywords inputProcessing
#' @export

# Samples bottom values from raster bricks
bottomRaster <- function(rawPointData){
  if(!is(rawPointData, "SpatVector")){
    warning(paste0("'rawPointData' must be class 'SpatVector'.\n"))
    return(NULL)
  }

  rpdf <- as.data.frame(rawPointData)

  template <- centerPointRasterTemplate(rawPointData)

  bottomSample <- apply(rpdf, MARGIN = 1,
                        FUN = function(x) tail(x[!is.na(x)],1))
  rawPointData$Bottom <- bottomSample

  bRaster <- rasterize(x = rawPointData, y = template, field = "Bottom")
  return(bRaster)
}

#' @title Center Point Raster Template
#'
#' @description Creates a `SpatRaster` template from a
#' `SpatVector` point object in which the raster cells
#' are centered on the vector points.
#'
#' @param rawPointData A `SpatVector` object with points
#' that will represent the center of each cell in the output
#' template.
#'
#' @return An empty `SpatRaster` designed to serve as a template for
#' rasterizing `SpatVector` objects.
#'
#' @details `rawPointData` is a `SpatVector` object that
#' contains x and y coordinates.
#'
#' @seealso \code{\link[terra:rasterize]{rasterize}}
#'
#' @examples
#'
#' library(terra)
#'
#' # Create point grid
#' coords <- data.frame(x = rep(seq(1:5), times = 5),
#'                     y = unlist(lapply(1:5, FUN = function(x) {
#'                       rep(x, times = 5)})))
#'
#' # Create data and add NAs to simulate uneven bottom depths
#' dd <- data.frame(SURFACE = 1:25,
#'                 d5M = 6:30,
#'                 d10M = 11:35,
#'                 d25M = 16:40)
#' dd$d25M[c(1:5, 18:25)] <- NA
#' dd$d10M[c(3:5, 21:23)] <- NA
#' dd$d5M[c(4, 22)] <- NA
#'
#' dd[,c("x","y")] <- coords
#'
#' # Create SpatialPointsDataFrame
#' sp <- vect(dd, geom = c("x", "y"))
#'
#' # Here's the function
#' template <- centerPointRasterTemplate(rawPointData = sp)
#' class(template)
#'
#' @import terra
#'
#' @keywords inputProcessing
#' @export

centerPointRasterTemplate <- function(rawPointData){
  if(!is(rawPointData, "SpatVector")){
    warning(paste0("'rawPointData' must be class 'SpatVector'.\n"))
    return(NULL)
  }
  rpdGeom <- geom(rawPointData)

  centeringAdjustment <- min(abs(diff(unique(rpdGeom[,"x"]))), abs(diff(unique(rpdGeom[,"y"]))))/2
  oldExtent <- ext(rawPointData)
  newExtent <- ext(oldExtent[1] - centeringAdjustment,
                   oldExtent[2] + centeringAdjustment,
                   oldExtent[3] - centeringAdjustment,
                   oldExtent[4] + centeringAdjustment)

  template <- rast(nrows=length(unique(rpdGeom[,"y"])),
                   ncols=length(unique(rpdGeom[,"x"])), extent = newExtent)
  return(template)
}

library(terra)

# voluModel:::columnParse() tests ----
# Create sample raster
r <- rast(ncol=10, nrow=10)
values(r) <- 1:100

# Create sample raster brick
rBrick <- c(r, r*10, r*100)
names(rBrick) <- c(0, 10, 100)

# Create test occurrences
set.seed(0)
longitude <- sample(ext(rBrick)[1]:ext(rBrick)[2],
                    size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(rBrick)[3]:ext(rBrick)[4],
                   size = 10, replace = FALSE)
set.seed(0)
depth <- sample(0:98, size = 10, replace = TRUE)
occurrences <- as.data.frame(cbind(longitude, latitude, depth))

test_that("columnParse input warnings behave as expected", {

  expect_error(voluModel:::columnParse())
  expect_warning(voluModel:::columnParse(occs = "a"))

  # 2D
  expect_warning(voluModel:::columnParse(occs = occurrences[,1]))

  colnames(occurrences) <- c("cheese", "eggs")
  expect_warning(voluModel:::columnParse(occs = occurrences))

  colnames(occurrences) <- c("longitude", "longitude")
  expect_warning(voluModel:::columnParse(occs = occurrences))
  colnames(occurrences) <- c("latitude", "latitude")
  expect_warning(voluModel:::columnParse(occs = occurrences))

  occurrences$extra <- occurrences$longitude
  colnames(occurrences) <- rep("yak", times = length(colnames(occurrences)))
  expect_warning(voluModel:::columnParse(occs = occurrences))
  colnames(occurrences) <- rep("xebu", times = length(colnames(occurrences)))
  expect_warning(voluModel:::columnParse(occs = occurrences))

  # 3D
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))

  colnames(occurrences) <- c("cheese", "eggs", "spam")
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))

  colnames(occurrences) <- c("longitude", "longitude", "depth")
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))
  colnames(occurrences) <- c("latitude", "latitude", "depth")
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))
  colnames(occurrences) <- c("longitude", "depth", "depth")
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))

  colnames(occurrences) <- c("y", "yum", "X")
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))
  colnames(occurrences) <- c("x", "exlax", "depth")
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))
  colnames(occurrences) <- c("z", "z", "sneeze")
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))

  names(rBrick) <- c("a", "b", "c")
  expect_warning(voluModel:::columnParse(occs = occurrences, wDepth = TRUE))
})

result2D <- voluModel:::columnParse(occs = occurrences)
result3D <- voluModel:::columnParse(occs = occurrences, wDepth = TRUE)

test_that("columnParse outputs as expected", {
  # 2D
  expect_equal(length(result2D), 3)
  expect_true(is.numeric(result2D$xIndex))
  expect_true(is.numeric(result2D$yIndex))
  expect_true(is.character(result2D$reportMessage))

  # 3D
  expect_equal(length(result3D), 4)
  expect_true(is.numeric(result3D$xIndex))
  expect_true(is.numeric(result3D$yIndex))
  expect_true(is.numeric(result3D$zIndex))
  expect_true(is.character(result3D$reportMessage))
})

# downsample() tests ----
# Create sample raster
r <- rast(ncol=5, nrow=5)
values(r) <- 1:25

# Create test occurrences
set.seed(0)
longitude <- sample(ext(r)[1]:ext(r)[2],
                    size = 5, replace = TRUE)
set.seed(0)
latitude <- sample(ext(r)[3]:ext(r)[4],
                   size = 5, replace = TRUE)
occurrences <- as.data.frame(cbind(longitude,latitude))

test_that("downsample input warnings behave as expected", {
  expect_error(downsample())
  expect_warning(downsample(occs = "a", rasterTemplate = r))
  expect_warning(downsample(occs = occurrences[,1], rasterTemplate = r))
  expect_warning(downsample(occs = occurrences, rasterTemplate = "a"))
  expect_warning(downsample(occs = occurrences, rasterTemplate = r, verbose = "potato"))

  colnames(occurrences) <- c("spam", "eggs")
  expect_warning(downsample(occs = occurrences, rasterTemplate = r))
})

# Here's the function
result <- downsample(occs = occurrences, rasterTemplate = r)

test_that("downsample outputs as expected", {
  expect_true(is.data.frame(result))
  expect_equal(ncol(result), 2)
  expect_true(nrow(result) == 4)
})

test_that("downsample special case checks", {
  result <- downsample(occs = occurrences[1,], rasterTemplate = r)
  expect_true(is.data.frame(result))
  expect_equal(ncol(result), 2)
  expect_true(nrow(result) == 1)
})

# centerPointRasterTemplate() tests ----

# Create point grid
coords <- data.frame(x = rep(seq(1:5), times = 5),
                     y = unlist(lapply(1:5, FUN = function(x) {
                       rep(x, times = 5)})))

# Create data and add NAs to simulate uneven bottom depths
dd <- data.frame(SURFACE = 1:25,
                 d5M = 6:30,
                 d10M = 11:35,
                 d25M = 16:40)
dd$d25M[c(1:5, 18:25)] <- NA
dd$d10M[c(3:4, 21:23)] <- NA
dd$d5M[c(4, 15, 22)] <- NA
dd <- cbind(coords, dd)

# Create SpatVector
sp <- vect(dd, geom = c("x", "y"))

test_that("centerPointRasterTemplate input warnings behave as expected", {
  expect_error(centerPointRasterTemplate())
  expect_warning(centerPointRasterTemplate(rawPointData = "a"))
})

test_that("centerPointRasterTemplate outputs as expected", {
  result <- centerPointRasterTemplate(rawPointData = sp)
  expect_true(class(result) == "SpatRaster")
  expect_warning(values(result))
  expect_true(suppressWarnings(sum(is.na(values(result)))) == 25)
})

# bottomRaster() tests ----
test_that("bottomRaster input warnings behave as expected", {
  expect_error(bottomRaster())
  expect_warning(bottomRaster(rawPointData = "a"))
})

result <- bottomRaster(rawPointData = sp)

test_that("bottomRaster outputs as expected", {
  expect_true(class(result) == "SpatRaster")
  expect_true(global(result, "isNA") == 0)
  expect_true(result[2] == 22)
})

---
title: "3. Environmental Data Sampling"
author:
- Hannah L. Owens
- Carsten Rahbek
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7
    fig_height: 7
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{3. Environmental Data Sampling}
  \usepackage[utf8]{inputenc}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)
```

# Introduction

As stated in the [introduction to this package](https://hannahlowens.github.io/voluModel/articles/a_Introduction.html), one of the key contributions of `voluModel` is the way it handles the extraction of environmental data for niche modeling. Specifically, environmental data at presences, absences, pseudoabsences, and/or background points can be extracted from environmental data multi-layer `SpatRaster` objects based on depth in addition to horizontal coordinates. Another useful tool provided with `voluModel` is `marineBackground()`, which helps the user generate training regions from which background or pseudoabsence data are drawn. This is where we will start, so let's get into it.

Here are the packages you will need.

```{r packages, message=FALSE, warning=FALSE}
library(voluModel) # Because of course
library(ggplot2) # For fancy plotting
library(rangeBuilder) # To compare marineBackground to getDynamicAlphaHull
library(dplyr) # To filter data
library(terra) # Now being transitioned in
library(sf) # Now being transitioned in
```

# 3D Data Extraction

First, we will cover how to extract environmental data from a multi-layer `SpatRaster` object using 3D occurrence coordinates. The species occurrences I will use are *Steindachneria argentea*, Luminous Hake, data downloaded via R (R Core Team, 2020) from GBIF (Chamberlain *et al.*, 2021; Chamberlain and Boettiger, 2017) and OBIS (Provoost and Bosch, 2019) via `occCite` (Owens *et al.*, 2021).

```{r occurrence dataset, message=FALSE, warning = FALSE}
# Get points
occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv", 
                             package='voluModel'))
occurrences <- occs %>% 
  dplyr::select(decimalLongitude, decimalLatitude, depth) %>%
  dplyr::distinct() %>% 
  dplyr::filter(depth %in% 1:2000)
```

The environmental dataset I will use as an example is temperature (Locarnini *et al.*, 2018) from the World Ocean Atlas (WOA; Garcia *et al.*, 2019). I have chosen to focus on temperature for simple illustrative purposes--we recommend you explore additional variables from the World Ocean Atlas and other sources. These data are supplied by the World Ocean Atlas as point shapefiles; the version supplied here has been cropped between -110 and -40 longitude and between -5 and 50 latitude to make it more memory-efficient. 

This chunk of code loads the example data shapefile, which results in a `SpatVector`:

```{r environmental data loading, eval=T, asis=T, message = FALSE, warning=FALSE}
# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Looking at the dataset
as.data.frame(temperature[1:5,1:10])
```

As you can see, each point in the `SpatVector` is a line in the associated `data.frame`; each column is a depth layer at that point--"d5M" translates to "5 meters deep". For more details on how to process environmental data, see [the raster data tutorial](https://hannahlowens.github.io/voluModel/articles/b_RasterProcessing.html). This next chunk converts the `SpatVector` to a multi-layer `SpatRaster`; the layers of the multi-layer `SpatRaster` are then named the same as the column names of the `SpatVector`. I strip the "d" and "M" from the column names so they are a little easier to use as depth coordinates. 

```{r temperature to compatible RasterBrick}
# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal

rm(tempTerVal)
```

At this point, you may have noticed a message popping up about which columns `voluModel` is interpreting as coordinates. `voluModel` is fairly flexible in how the coordinate columns are named, and it relies on names instead of column position to interpret x, y, and z (or latitude, longitude, and depth) coordinates. We do this to make `voluModel` as flexible as possible to a diversity of input formats, although English is the only supported language at the moment. Here are some examples of names that work.

```{r column interpretations, message=TRUE, warning=TRUE}
occsTest <- occurrences[19:24,]
xyzSample(occs = occsTest, envBrick = temperature)

colnames(occsTest) <- c("x", "y", "z")
xyzSample(occs = occsTest, envBrick = temperature)

rm(occsTest)
```

One common step in a niche modeling workflow is to down-sample occurrence data to the same resolution as the environmental data. We do this to avoid over-fitting the model due to biased sampling. The next chunk of code downsamples occurrence points so that there is only one per voxel (i.e. 3D pixel) of environmental data. It does this by looping through each layer of the multi-layer `SpatRaster`, selecting the occurrence points with depths closest to the depths represented by the corresponding layer in the `SpatRaster` vector. I then downsample these points using `voluModel`'s `downsample()` function.

```{r downsample to voxel, eval=TRUE, warning=FALSE, message=FALSE}
# Gets the layer index for each occurrence by matching to depth
layerNames <- as.numeric(names(temperature))
occurrences$index <- unlist(lapply(occurrences$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occurrences$index)
downsampledOccs <- data.frame()
for(i in indices){
  tempPoints <- occurrences[occurrences$index==i,]
  tempPoints <- downsample(tempPoints, temperature[[1]])
  tempPoints$depth <- rep(layerNames[[i]], times = nrow(tempPoints))
  downsampledOccs <- rbind(downsampledOccs, tempPoints)
}

occurrences <- downsampledOccs
head(occurrences)

print(paste0("Original number of points: ", nrow(occs), "; number of downsampled occs: ", nrow(occurrences)))
```

The resampled points returned by `downsample()` are horizontally centered in each voxel. The red dots in the plot show occurrences from the original dataset; the orange dots show the downsampled dataset. Not all of the orange dots may be visible underneath the red dots.

```{r plot downsample, warning=FALSE}
land <- rnaturalearth::ne_countries(scale = "small", returnclass = "sf")[1]

pointCompMap(occs1 = occs, occs2 = occurrences, 
             occs1Name = "original", occs2Name = "cleaned", 
             spName = "Steindachneria argentea", 
             land = land, verbose = FALSE)
```

At this point, we can demonstrate one of the key contributions of `voluModel`-- an improvement in how the environmental data are extracted at species' occurrences. `voluModel` samples environmental conditions at the depth where each record was located, instead of extrapolating to surface or bottom conditions. Finally, we add a column called "response" and fill it by repeating "1", to signify that these occurrences should be interpreted as presences for the purposes of modeling.

```{r temperature extraction}
# Extraction
occurrences$temperature <- xyzSample(occs = occurrences, envBrick = temperature)

# Add "response" column for modeling
occurrences$response <- rep(1, times = nrow(occurrences))
occurrences <- occurrences[complete.cases(occurrences),]

head(occurrences)
```

In a "real" modeling scenario, you would extract all your environmental variables using `xyzSample()`, then add the response column. Or add the response column first or in the middle. I'm not the boss of you.

# Generating Repeatable Training Regions

To generate a sample of pseudoabsences and/or background points, it is important to consider the environments the species of interest can access (Barve *et al.*, 2011). There is ample discussion in the literature on how to best delimit a niche model training region (also variously called M or the background region), and we will not review the issue here. In the absence of specific information on species’ dispersal capabilities, algorithmically generating a repeatable sampling background from a clear set of rules may be suitable approximation, or a jumping-off point for hand-curating training regions in a GIS software of choice.

We designed `marineBackground()` as a wrapper around \code{\link[rangeBuilder:getDynamicAlphaHull]{getDynamicAlphaHull}}, a function provided by the `rangeBuilder` package (Davis-Rabosky *et al.*, 2016) which generates background sampling regions by fitting an alpha hull polygon around an occurrence point dataset. Let's look at how `marineBackground()` (and by extension `getDynamicAlphaHull()`) fits alpha hull polygons, based on the cleaned species occurrence data from above. 

`getDynamicAlphaHull()`, which is run within `marineBackground()` generates a polygon (or polygons) that can be used as a training region by fitting an alpha hull (or hulls) around an occurrence dataset. Alpha is a parameter that controls the complexity of the polygon(s) that is being fit around the occurrences. `getDynamicAlphaHull()` adjusts the alpha parameter iteratively, until it meets criteria the user can set via the arguments `fraction`, which is the minimum fraction of occurrences that must be contained within the polygon(s), `partCount`, which is the maximum number of polygons generated to fit the data, and `buff`, which is the permissible distance from a point to the edge of a polyon in meters. `marineBackground()` takes these and many of the same other arguments as `getDynamicAlphaHull()`. Here's how it works.

```{r alpha hull demonstration, message=FALSE, warning=FALSE, eval=T}
trainingRegion <- marineBackground(occurrences, 
                                   fraction = 1, 
                                   partCount = 1, 
                                   buff = 1000000,
                                   clipToOcean = F)
plot(trainingRegion, border = F, col = "gray",
     main = "100% Points,\nMax 1 Polygon Permitted, 100 km Buffer",
     axes = T)
plot(land, col = "black", add = T)
points(occurrences[,c("decimalLongitude", "decimalLatitude")], 
       pch = 20, col = "red", cex = 1.5)
```

What parameters you choose to fit the alpha hull will depend on the biology of the organism you are trying to model (e.g. you may choose a much smaller buffer for a sessile organism than you would for a highly vagile migratory species), your data, and the particulars of your study system and the geography of your study area. You may need to experiment to find something that looks sensible. If you do not provide a pre-defined buffer, `marineBackground()` will calculate a buffer that is the mean of the top and bottom 10% of distances between points in the dataset.

You may have noticed that the training regions in the above examples include some areas of the Pacific where the species has not been observed, and cannot access. This could cause problems if it is used to train your model. You *could* delete this area by hand in your favorite GIS software, OR you could change `clipToOcean` to `TRUE` in `marineBackground()`, and any polygon fragments that do not contain an occurrence will be deleted automatically. This is a key difference from `getDynamicAlphaHull()`.

```{r clipToOcean demo, message=FALSE, warning=FALSE, eval=T}
trainingRegion <- marineBackground(occurrences, 
                                   buff = 1000000, 
                                   clipToOcean = T)
plot(trainingRegion, border = F, col = "gray",
     main = "100 km Buffer,\nClipped to Occupied Polygon",
     axes = T)
plot(land, col = "black", add = T)
points(occurrences[,c("decimalLongitude", "decimalLatitude")], 
       pch = 20, col = "red", cex = 1.5)
```

The last major difference between `marineBackground()` and `getDynamicAlphaHull()` is how the functions handle training regions that are adjacent to the antimeridian (i.e. 180° E or W). `getDynamicAlphaHull()` truncates polygons at this line, which can lead to some strange-looking training areas or fatal errors. `marineBackground()`, on the other hand, wraps polygons across the meridian and merges them if they overlap. For this example, I am going to manipulate the longitudinal coordinates of our Luminous Hake dataset to generate a quick fictional example.

```{r meridian wrap demo, warning=FALSE, message=FALSE, eval=T}
# Fictional example occurrences
pacificOccs <- occurrences
pacificOccs$decimalLongitude <- pacificOccs$decimalLongitude - 100
for (i in 1:length(pacificOccs$decimalLongitude)){
  if (pacificOccs$decimalLongitude[[i]] < -180){
    pacificOccs$decimalLongitude[[i]] <- pacificOccs$decimalLongitude[[i]] + 360
  }
}

# marine Background
pacificTrainingRegion <- marineBackground(pacificOccs, 
                                          fraction = 0.95, partCount = 3,
                                          buff = 1000000,
                                          clipToOcean = T)
plot(pacificTrainingRegion, border = F, col = "gray",
     main = "marineBackground\nAntimeridian Wrap",
     axes = T)
plot(land, col = "black", add = T)
points(pacificOccs[,c("decimalLongitude", "decimalLatitude")], 
       pch = 20, col = "red", cex = 1.5)
```

# Background Data Extraction

As discussed above, some modeling methods require the generation of a sampling region from which training/pseudoabsence/background points are drawn. The training region should approximately answer the question "What environments are most likely to have been experienced by Lumious Hakes?" Our answer to this question for the purposes of this tutorial is represented in `trainingRegion`, which we generated above using `marineBackground()`. Next, we generate 3D training points within this region using the `mSampling3D()` function. The function takes an occurrence `data.frame`, a multi-layer `SpatRaster` that will serve as a template for the resolution of the training points, a `SpatialPolygons` object representing the training region, and, optionally, information on the range of depths from which the function should sample. If you have reason to believe the species you are modeling can access depths higher or lower than those represented by occurrences, you can specify a maximum and minimum training depth from which to draw occurrence points. `mSampling3D()` by default draws training points from the full depth extent of the supplied multi-layer `SpatRaster`. 

For this example, I am sampling training points from 50 to 1500 m.

```{r training points}
# Background
backgroundVals <- mSampling3D(occs = occurrences, 
                              envBrick = temperature, 
                              mShp = trainingRegion, 
                              depthLimit = c(50, 1500))
backgroundVals$temperature <- xyzSample(occs = backgroundVals, temperature)

#Remove incomplete cases
backgroundVals <- backgroundVals[complete.cases(backgroundVals),]
```

Once the points are sampled, we add a column called "response" and fill it by repeating "0", to signify that these occurrences should be interpreted as absences for the purposes of modeling.

```{r }
# Add "response" column for modeling
backgroundVals$response <- rep(0, times = nrow(backgroundVals))

head(backgroundVals)
```

It is important to note that this background data represents EVERY voxel within the training region that is not occupied by observances. Depending on your study design and the modeling algorithm you are using, these data will need to be subsampled. You could choose a random sample, select points based on geography, or any number of other strategies. Below is just one example--weighting the background points by their environmental dissimilarity to the Luminous Hake occurrence points. This manner of downsampling may be appropriate for modeling methods that interpret training points as pseudoabsences, such as generalized linear modeling, but will lead to overfit models when using algorithms like Maxent. 

```{r glm data prep part 2}
# Sample background points weighted by distance from mean of occurrence temperatures
meanTemp <- mean(occurrences[,c("temperature")])
backgroundVals$distance <- abs(backgroundVals[,"temperature"] - meanTemp)
backgroundVals$sampleWeight <- (backgroundVals$distance - 
                                  min(backgroundVals$distance))/(max(backgroundVals$distance) -
                                                                  min(backgroundVals$distance))
sampleForAbsence <- sample(x = rownames(backgroundVals), 
                           size = nrow(occurrences) * 100, 
                           prob = backgroundVals$backgroundVals)
backgroundVals <- backgroundVals[match(sampleForAbsence, 
                                       rownames(backgroundVals)),]

backgroundVals$response <- as.factor(backgroundVals$response)

# Unite datasets and see how things look
dataForModeling <- rbind(occurrences, backgroundVals[,colnames(occurrences)])
ggplot(dataForModeling, aes(x = temperature, fill = response, color = response)) +
  geom_density(alpha = .6) +
  scale_color_manual(values=c("#999999", "#E69F00"), 
                     labels = c("Pseudoabsence", "Presence")) +
  scale_fill_manual(values=c("#999999", "#E69F00"), 
                    labels = c("Pseudoabsence", "Presence")) +
  labs(title="Temperature Sampling Density,\nOccurrences vs. Pseudoabsences",
       x="Temperature (C)", y = "Density")+
  theme_classic()
```

For a more complex, multivariate example of environmental dissimilarity weighting, refer to the [GLM workflow](https://hannahlowens.github.io/voluModel/articles/e_GLMWorkflow.html).

# Tidying up
Last, we need to close the temporary directory we opened when we opened the data.

```{r cleanup temporary directory}
unlink(td, recursive = T)
```

# References

Bakis Y (2021): TU_Fish. v1.1. No organization. Dataset/Occurrence. https://web.archive.org/web/20221202141213/https://bgnn.tulane.edu/ipt/resource?r=tu_fish&v=1.1 Accessed via OBIS on 2020-11-04.  

Bentley A (2022). KUBI Ichthyology Collection. Version 17.80. University of Kansas Biodiversity Institute. DOI: 10.15468/mgjasg. Accessed via GBIF on 2020-10-13.

Bentley A (2022). KUBI Ichthyology Tissue Collection. Version 18.68. University of Kansas Biodiversity Institute. DOI: 10.15468/jmsnwg. Accessed via GBIF on 2020-10-13.

Buckup P A (2022). Coleção Ictiológica (MNRJ), Museu Nacional (MN), Universidade Federal do Rio de Janeiro(UFRJ). Version 157.1487. Museu Nacional / UFRJ. DOI: 10.15468/lluzfl. Accessed via GBIF on 2020-10-13.

Catania D, Fong J (2022). CAS Ichthyology (ICH). Version 150.300. California Academy of Sciences. DOI: 10.15468/efh2ib. Accessed via GBIF on 2020-10-13.

Chakrabarty P (2019). LSUMZ (LSU MNS) Fishes Collection. Version 2.2. Louisiana State University Museum of Natural Science. DOI: 10.15468/gbnym3. Accessed via GBIF on 2020-10-13.

Chamberlain S, Barve V, Mcglinn D, Oldoni D, Desmet P, Geffert L, Ram K (2021). rgbif: Interface to the Global Biodiversity Information Facility API. R package version 3.6.0, https://CRAN.R-project.org/package=rgbif.

Chamberlain S, Boettiger C (2017). “R Python, and Ruby clients for GBIF species occurrence data.” PeerJ PrePrints. DOI: 10.7287/peerj.preprints.3304v1.

Davis-Rabosky AR, Cox CL, Rabosky DL, Title PO, Holmes IA, Feldman A, McGuire JA (2016). Coral snakes predict the evolution of mimicry across New World snakes. *Nature Communications* 7:11484. DOI: 10.1038/ncomms11484

Espinosa Pérez H, Comisión nacional para el conocimiento y uso de la biodiversidad C (2021). Computarización de la Colección Nacional de Peces del Instituto de Biología UNAM. Version 1.9. Comisión nacional para el conocimiento y uso de la biodiversidad. DOI: 10.15468/zb2odl. Accessed via GBIF on 2020-10-13.

Frable B (2019). SIO Marine Vertebrate Collection. Version 1.7. Scripps Institution of Oceanography. DOI: 10.15468/ad1ovc. Accessed via GBIF on 2020-10-13.

Fishnet2 Portal, http://www.fishnet2.net, Accessed via OBIS on 2020-11-04.

Froese, R. and D. Pauly. Editors. 2020. FishBase. World Wide Web electronic publication. www.fishbase.org, version (xx/200x). Accessed via OBIS on 2020-11-04.   

Gall L (2021). Vertebrate Zoology Division - Ichthyology, Yale Peabody Museum. Yale University Peabody Museum. DOI: 10.15468/mgyhok. Accessed via GBIF on 2020-10-13.

García, CB, and Duarte, LO, 'Columbian Caribbean Sea', in J.H. Nicholls (comp.) HMAP Data Pages https://figshare.com/articles/dataset/HMAP_Dataset_18_Colombian_Caribbean_Sea/23504046 Accessed via OBIS on 2020-11-04.   

Garcia HE, Weathers K, Paver CR, Smolyar I, Boyer TP, Locarnini RA, Zweng MM, Mishonov AV, Baranova OK, Seidov D, Reagan JR (2018). World Ocean Atlas 2018, Volume 3: Dissolved Oxygen, Apparent Oxygen Utilization, and Oxygen Saturation. A Mishonov Technical Ed.; NOAA Atlas NESDIS 83, 38pp.

GBIF.org (24 September 2020) GBIF Occurrence Download DOI: 10.15468/dl.efuutj.

Grant S, McMahan C (2020). Field Museum of Natural History (Zoology) Fish Collection. Version 13.12. Field Museum. DOI: 10.15468/alz7wu. Accessed via GBIF on 2020-10-13.

Harvard University M, Morris P J (2022). Museum of Comparative Zoology, Harvard University. Version 162.296. Museum of Comparative Zoology, Harvard University. DOI: 10.15468/p5rupv. Accessed via GBIF on 2020-10-13.

Hendrickson D A, Cohen A E, Casarez M J (2022). University of Texas, Biodiversity Center, Ichthyology Collection (TNHCi). Version 5.175. University of Texas at Austin, Biodiversity Collections. DOI: 10.15468/h8gxdr. Accessed via GBIF on 2020-10-13.

INVEMAR (2022). SIBM en línea: Sistema de Información sobre Biodiversidad Marina. Santa Marta: Instituto de investigaciones Marinas y Costeras José Benito Vives de Andréis,. https://siam.invemar.org.co/ Accessed via OBIS on 2020-11-04. 

Locarnini RA, Mishonov AV, Baranova OK, Boyer TP, Zweng MM, Garcia HE, Reagan JR, Seidov D, Weathers K, Paver CR, Smolyar I (2018). World Ocean Atlas 2018, Volume 1: Temperature. A. Mishonov Technical Ed.; NOAA Atlas NESDIS 81, 52pp.

McLean, M.W. (2014). Straightforward Bibliography Management in R Using the RefManager Package. NA, NA. https://arxiv.org/abs/1403.2036.

McLean, M.W. (2017). RefManageR: Import and Manage BibTeX and BibLaTeX References in R. The Journal of Open Source Software.

Mertz W (2021). LACM Vertebrate Collection. Version 18.9. Natural History Museum of Los Angeles County. DOI: 10.15468/77rmwd. Accessed via GBIF on 2020-10-13.

Nakae M, Shinohara G (2018). Fish collection of National Museum of Nature and Science. National Museum of Nature and Science, Japan. Occurrence dataset DOI: 10.15468/w3dzv1 accessed via GBIF.org on yyyy-mm-dd. Accessed via OBIS on 2020-11-04.    

Natural History Museum (2021). Natural History Museum (London) Collection Specimens. DOI: 10.5519/0002965. Accessed via GBIF on 2020-10-13.

National Museum of Natural History, Smithsonian Institution NMNH Fishes Collection Database (2007). National Museum of Natural History, Smithsonian Institution, 10th and Constitution Ave. N.W., Washington, DC 20560-0193. Accessed via OBIS on 2020-11-04.

Norén M, Shah M (2017). Fishbase. FishBase. DOI: 10.15468/wk3zk7. Accessed via GBIF on 2020-10-13.

Norton B, Hogue G (2021). NCSM Ichthyology Collection. Version 22.8. North Carolina State Museum of Natural Sciences. DOI: 10.36102/dwc.1. Accessed via GBIF on 2020-10-13.

Nychka D, Furrer R, Paige J, Sain S (2021). “fields: Tools for spatial data.” R package version 13.3.  https://github.com/dnychka/fieldsRPackage.

Orrell T, Informatics Office (2021). NMNH Extant Specimen Records (USNM, US). Version 1.49. National Museum of Natural History, Smithsonian Institution. DOI: 10.15468/hnhrg3. Accessed via GBIF on 2020-10-13.

Owens H, Merow C, Maitner B, Kass J, Barve V, Guralnick R (2021). occCite: Querying and Managing Large Biodiversity Occurrence Datasets. doi: 10.5281/zenodo.4726676, R package version 0.4.9.9000, https://CRAN.R-project.org/package=occCite.

Prestridge H (2021). Biodiversity Research and Teaching Collections - TCWC Vertebrates. Version 9.4. Texas A&M University Biodiversity Research and Teaching Collections. DOI: 10.15468/szomia. Accessed via GBIF on 2020-10-13.

Provoost P, Bosch S (2019). “robis: R Client to access data from the OBIS API.” Ocean Biogeographic Information System. Intergovernmental Oceanographic Commission of UNESCO. R package version 2.1.8, https://cran.r-project.org/package=robis.

Pugh W (2021). UAIC Ichthyological Collection. Version 3.3. University of Alabama Biodiversity and Systematics. DOI: 10.15468/a2laag. Accessed via GBIF on 2020-10-13.

R Core Team (2017). R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. URL https://www.R-project.org/.

Robins R (2021). UF FLMNH Ichthyology. Version 117.342. Florida Museum of Natural History. DOI: 10.15468/8mjsel. Accessed via GBIF on 2020-10-13.

Tavera L, Tavera M. L (2021). Biodiversidad de los recursos marinos y costeros entre Cartagena y el Golfo de Urabá, Caribe colombiano. Version 2.4. Instituto de Investigaciones Marinas y Costeras - Invemar. DOI: 10.15472/d2jusp. Accessed via GBIF on 2020-10-13.

UNIBIO, IBUNAM (2020). CNPE/Coleccion Nacional de Peces. DOI: 10.15468/o5d48z. Accessed via GBIF on 2020-10-13.

Wagner M (2017). MMNS Ichthyology Collection. Mississippi Museum of Natural Science. DOI: 10.15468/4c3qeq. Accessed via GBIF on 2020-10-13.

Werneke D (2019). AUM Fish Collection. Version 8.1. Auburn University Museum. DOI: 10.15468/dm3oyz. Accessed via GBIF on 2020-10-13.

---
title: "1. Introduction to `voluModel`"
author:
- Hannah L. Owens
- Carsten Rahbek
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7
    fig_height: 7
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{1. Introduction to `voluModel`}
  \usepackage[utf8]{inputenc}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)
```

# Introduction 
To date, distribution and ecological niche models of marine species have generally only considered distributions in two-dimensional space, not accounting for depth. This may lead to mis-estimation of species distributions and subsequent diversity estimates, especially among pelagic and benthic species. Here, we present a possible way forward: generating species distribution models based on environmental data extracted at the depths where individuals were observed, and calibrated with three-dimensional sampling of pseudo-absences. We also provide some methods to project these models back into three-dimensional space and visualize the results.

Here are the packages you will need.

```{r packages, message=FALSE, warning=FALSE}
library(voluModel) # Because of course
library(dplyr) # To filter data
library(ggplot2) # For fancy plotting
library(terra) # Now being transitioned in
```

# Data Inputs

First thing's first. Distributional models are based on coordinates of species' occurrences. We start with and example occurrence dataset of *Steindachneria argentea*, Luminous Hake, using data downloaded via R (R Core Team, 2020) from GBIF (Chamberlain *et al.*, 2021; Chamberlain and Boettiger, 2017) and OBIS (Provoost and Bosch, 2019) via occCite (Owens *et al.*, 2021). Here is how the occurrence data file is structured, and where the occurrences we will be using are located.

```{r show points, warning=FALSE, echo=TRUE, message=FALSE, eval=TRUE}
occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv", 
                             package='voluModel'))

# Filter points
occurrences <- occs %>% 
  dplyr::select(decimalLongitude, decimalLatitude, depth) %>%
  dplyr::distinct() %>% 
  filter(depth %in% 1:2000)

head(occurrences)
land <- rnaturalearth::ne_countries(scale = "small", returnclass = "sf")[1]
pointMap(occs = occurrences, ptCol = "orange", landCol = "black",
             spName = "Steindachneria argentea", ptSize = 3,
             land = land)
```

Next, we load two environmental datasets from the World Ocean Atlas (Garcia *et al.*, 2019): temperature (Locarnini *et al.*, 2018) and apparent oxygen utilization (AOU; Garcia *et al.*, 2019). I have chosen these variables for simplified illustrative purposes--we recommend you explore additional variables from the World Ocean Atlas and other sources. These data are supplied by the World Ocean Atlas as point shapefiles; the version supplied here has been cropped between -98 and -45 longitude, between -1 and 45 latitude, and between 0 and 4,400 m in depth to make it more space-efficient. 

```{r loading temperature data, message=FALSE, warning=FALSE, include=TRUE}
# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Creating a bottom raster
temperatureBottom <- bottomRaster(temperature)

# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- terra::rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal
rm(tempTerVal)
```

Let's see what the top and bottom temperature files look like.

``` {r plotting temperature, eval = TRUE}
# How do these files look?
par(mfrow=c(1,2))
p1 <- oneRasterPlot(temperature[[1]], land = land, landCol = "black", 
              title= "Surface Temperature (C)")

p2 <- oneRasterPlot(temperatureBottom,land = land, landCol = "black", 
              title = "Bottom Temperature (C)")
```

The AOU dataset has been interpolated and smoothed using a thin plate spline algorithm. For more details on how to process environmental data, see [the raster data tutorial](https://hannahlowens.github.io/voluModel/articles/b_RasterProcessing.html). First, we load the data.

```{r loading oxygen data, message=FALSE, warning=FALSE, include=TRUE}
# Oxygen processing, pre-baked to save time
oxygenSmooth <- rast(system.file("extdata/oxygenSmooth.tif", 
                                 package='voluModel'))
oxygenBottom <- rast(system.file("extdata/oxygenBottom.tif", 
                                 package="voluModel"))
names(oxygenSmooth) <- names(temperature)
```
Then we plot it.

```{r oxygen plotting, echo=TRUE, eval = TRUE, message=FALSE, warning=FALSE}
par(mfrow=c(1,2))
p3 <- oneRasterPlot(oxygenSmooth[[1]], land = land, landCol = "black", 
              title= "Surface Apparent Oxygen Utilization (µmol/kg),\ninterpolated and smoothed")
p4 <- oneRasterPlot(oxygenBottom, land = land, landCol = "black",
     title = "Bottom Apparent Oxygen Utilization (µmol/kg),\ninterpolated and smoothed")
```

Finally, we get data layer indices for each occurrence point by matching their depth of sampling to the appropriate environmental layer name.

```{r occurrence and depth matchup}
# Gets the layer index for each occurrence by matching to depth
layerNames <- as.numeric(names(temperature))
occurrences$index <- unlist(lapply(occurrences$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occurrences$index)

# Downsamples occurrences in each depth layer
downsampledOccs <- data.frame()
for(i in indices){
  tempPoints <- occurrences[occurrences$index==i,]
  tempPoints <- downsample(tempPoints, temperature[[1]], verbose = FALSE)
  tempPoints$depth <- rep(layerNames[[i]], times = nrow(tempPoints))
  downsampledOccs <- rbind(downsampledOccs, tempPoints)
}
occurrences <- downsampledOccs
occurrences <- occurrences[,c("decimalLatitude", "decimalLongitude", "depth")]

rm(indices, layerNames, tempPoints, i, downsampledOccs, occs)
```

# A point about 3D environmental data extraction

```{r data extraction, echo=FALSE, message=FALSE, warning=FALSE}
# Extract temperature
threeDimTemp <- xyzSample(occs = occurrences, temperature)
threeDimTemp <- cbind(rep("X, Y,\nZ", length(threeDimTemp)), threeDimTemp)
surfTemp <- extract(x = temperature[[1]], 
                    occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
surfTemp <- cbind(rep("X, Y,\nSurface", length(surfTemp)), surfTemp)
bottomTemp <- extract(x = temperatureBottom, 
                      occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
bottomTemp <- cbind(rep("X, Y,\nBottom", length(bottomTemp)), bottomTemp)

# Extract AOU
threeDimOxy <- xyzSample(occs = occurrences, oxygenSmooth)
threeDimOxy <- cbind(rep("X, Y,\nZ", length(threeDimOxy)), threeDimOxy)
surfOxy <- extract(x = oxygenSmooth[[1]], 
                   occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
surfOxy <- cbind(rep("X, Y,\nSurface", length(surfOxy)), surfOxy)
bottomOxy <- extract(x = oxygenBottom, 
                     occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
bottomOxy <- cbind(rep("X, Y,\nBottom", length(bottomOxy)), bottomOxy)
```

At this point, we can demonstrate one of the key contributions of `voluModel`-- an improvement in how the environmental data are extracted at species' occurrences. `voluModel` samples environmental conditions at the depth where each record was located, instead of extrapolating to surface or bottom conditions. How important is this? First, let's look at temperature.

```{r temperature violin plots, echo=FALSE, message=FALSE, warning=FALSE}
# Collation
allTemp <- rbind(threeDimTemp, surfTemp, bottomTemp)
colnames(allTemp) <- c("Group", "Temperature")
allTemp <- as.data.frame(allTemp)
allTemp <- allTemp[complete.cases(allTemp),]
allTemp$Temperature <- as.numeric(allTemp$Temperature)

allOxy <- rbind(threeDimOxy, surfOxy, bottomOxy)
colnames(allOxy) <- c("Group", "Oxygen")
allOxy <- as.data.frame(allOxy)
allOxy <- allOxy[complete.cases(allOxy),]
allOxy$Oxygen <- as.numeric(allOxy$Oxygen)

# Plotting
groups <- c("X, Y,\nSurface", "X, Y,\nBottom", "X, Y,\nZ")
tempPlot <- ggplot(allTemp, aes(x=Group, y=Temperature)) + 
  geom_boxplot(fill="#b2182b", notch = FALSE) +
  theme_classic(base_size = 15) +
  theme(axis.title.x = element_blank(), 
        text = element_text(family = "Arial"), 
        axis.text = element_text(size = rel(1.1))) +
  scale_x_discrete(limits = groups) +
  ylab("Temperature (C)")

oxyPlot <- ggplot(allOxy, aes(x=Group, y=Oxygen)) + 
  geom_boxplot(fill="#2166ac", notch = FALSE) +
  theme_classic(base_size = 15) +
  theme(axis.title.x = element_blank(), 
        text = element_text(family = "Arial"), 
        axis.text = element_text(size = rel(1.1))) +
  scale_x_discrete(limits = groups) +
  ylab("Apparent Oxygen Utilization (µmol/kg)")

gridExtra::grid.arrange(tempPlot, oxyPlot, nrow = 1)
```

Temperature extracted at the appropriate depth (3D) is significantly lower than temperature extracted at the surface. The mean value of temperature is similar between the bottom and the 3D extractions, but the range of 3D values is narrower than those extracted at the bottom.

AOU based on 3D extraction is significantly higher than AOU extracted at the surface. The mean value of AOU is again similar between the bottom and the 3D extractions; the range of 3D values is broader than those extracted at the bottom, but contains no statistical outliers, i.e. greater than 1.5 * interquartile range from the hinge of the boxplot (black dots).

# Data sampling

Some modeling methods require the generation of a sampling region from which pseudoabsence or background points are drawn. For this introduction, we are showing you a simple envelope method that does not require background sampling. Instead, I will be applying the same principle to define the study region to which I project the envelope model, so we have a product that makes biological sense in terms of the question "Where are we most likely to encounter Lumious Hakes?" See [the background sampling tutorial](https://hannahlowens.github.io/voluModel/articles/c_DataSampling.html) for more information on `voluModel`'s `marineBackground()` function, and [the generalized linear model tutorial](https://hannahlowens.github.io/voluModel/articles/e_GLMWorkflow.html) for an example of a modelling application that uses a background sampling region. 

```{r study region, message=FALSE, warning=FALSE}
studyRegion <- marineBackground(occurrences,
                                buff = 1000000,
                                clipToOcean = TRUE,
                                alpha = 1,
                                partCount = 1)
landVect <- vect(land)
landVect <- terra::project(landVect, y = studyRegion)
plot(studyRegion, border = F, col = "gray",
     main = "Points and Background Sampling",
     axes = T)
plot(landVect, col = "black", add = T)
points(occurrences[,c("decimalLongitude", "decimalLatitude")], 
       pch = 20, col = "red", cex = 1.5)
```

For our simple niche envelope models, we need to draw the environmental variable data from each occurrence voxel. For comparison with previous modeling approaches, I will extract data from the surface, from the bottom, and from the 3D coordinate for each variable.

```{r sampling}
# Surface Presences
oxyVals <- extract(x = oxygenSmooth[[1]], occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
tempVals <- extract(x = temperature[[1]], occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
vals <- cbind(occurrences, oxyVals, tempVals)
colnames(vals) <- c("decimalLongitude", "decimalLatitude", "depth", "AOU", "Temperature")
vals <- vals[complete.cases(vals),]
row.names(vals) <- NULL
occsWdataSurface <- vals

# Bottom Presences
oxyVals <- extract(x = oxygenBottom, occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
tempVals <- extract(x = temperatureBottom, occurrences[,c("decimalLongitude", "decimalLatitude")])[,2]
vals <- cbind(occurrences, oxyVals, tempVals)
colnames(vals) <- c("decimalLongitude", "decimalLatitude", "depth", "AOU", "Temperature")
vals <- vals[complete.cases(vals),]
row.names(vals) <- NULL
occsWdataBottom <- vals

# 3D Presences
oxyVals <- xyzSample(occs = occurrences, oxygenSmooth)
tempVals <- xyzSample(occs = occurrences, temperature)
vals <- cbind(occurrences, oxyVals, tempVals)
colnames(vals) <- c("decimalLongitude", "decimalLatitude", "depth", "AOU", "Temperature")
vals <- vals[complete.cases(vals),]
row.names(vals) <- NULL
occsWdata3D <- vals

rm(oxyVals, tempVals, vals)
```

# Niche envelope models

Now I will demonstrate how to construct simple 2D envelope models, in which we define suitable conditions as those within the maximum and minimum conditions at which *Steindachneria argentea* has been observed. The results will be simple 2D distribution maps--one based on surface conditions and one based on bottom conditions.

```{r generate 2D envelope niche models}
# Get limits, surface
tempLims <- quantile(occsWdataSurface$Temperature,c(0, 1))
aouLims <- quantile(occsWdataSurface$AOU,c(0, 1))

# Reclassify environmental bricks to presence/absence, surface
temperaturePresence <- classify(temperature[[1]], 
                                  rcl = matrix(c(-Inf,tempLims[[1]],0, 
                                                 tempLims[[1]], tempLims[[2]], 1,
                                                 tempLims[[2]], Inf, 0), 
                                               ncol = 3, byrow = TRUE))
AOUpresence <- classify(oxygenSmooth[[1]], 
                          rcl = matrix(c(-Inf, aouLims[[1]],0,
                                  aouLims[[1]], aouLims[[2]], 1,
                                  aouLims[[2]], Inf, 0), ncol = 3, byrow = TRUE))

# Put it all together, surface
envelopeModelSurface <- temperaturePresence * AOUpresence
envelopeModelSurface <- mask(crop(envelopeModelSurface, studyRegion), 
                             mask = studyRegion)

# Get limits, bottom
tempLims <- quantile(occsWdataBottom$Temperature,c(0, 1))
aouLims <- quantile(occsWdataBottom$AOU,c(0, 1))

# Reclassify environmental bricks to presence/absence, bottom
temperaturePresence <- classify(temperatureBottom, 
                                  rcl = matrix(c(-Inf,tempLims[[1]],0,
                                          tempLims[[1]], tempLims[[2]], 1,
                                          tempLims[[2]], Inf, 0), byrow = TRUE, ncol=3))
AOUpresence <- classify(oxygenBottom, 
                          rcl = matrix(c(-Inf, aouLims[[1]],0,
                                  aouLims[[1]], aouLims[[2]], 1,
                                  aouLims[[2]], Inf, 0), byrow = TRUE, ncol = 3))

# Put it all together, bottom
envelopeModelBottom <- temperaturePresence * AOUpresence
envelopeModelBottom <- mask(crop(envelopeModelBottom, studyRegion), mask = studyRegion)
```

How do these 2D models compare? `voluModel` has a function to conveniently generate a formatted comparison between two presence/absence distribution rasters `rasterComp()`.

```{r comparing 2D maps}
rasterComp(rast1 = envelopeModelSurface, rast2 = envelopeModelBottom, 
           rast1Name = "Surface", rast2Name = "Bottom", 
           land = land, landCol = "black", 
           title = "Comparison between surface and bottom envelope models")
```

Now I'll demonstrate how to make a 3D envelope model. Generating the model is exactly the same--any conditions between the maximum and minimum values at which Luminous Hakes were observed are considered suitable. The estimates are, of course, based on 3D environmental data extractions. The other key difference is that the envelope model is projected to each layer in each of the raster bricks (essentially generating a single variable model for each variable). The single-variable models are then multiplied together to generate a full model.

```{r generate 3D envelope model}
# Get limits
tempLims <- quantile(occsWdata3D$Temperature,c(0, 1))
aouLims <- quantile(occsWdata3D$AOU,c(0, 1))

# Reclassify environmental bricks to presence/absence
temperaturePresence <- classify(temperature, 
                                  rcl = matrix(c(-Inf,tempLims[[1]],0,
                                                 tempLims[[1]], tempLims[[2]], 1,
                                                 tempLims[[2]], Inf, 0), 
                                               byrow = TRUE, ncol = 3))
AOUpresence <- classify(oxygenSmooth, 
                          rcl = matrix(c(-Inf, aouLims[[1]],0,
                                         aouLims[[1]], aouLims[[2]], 1,
                                         aouLims[[2]], Inf, 0), 
                                       byrow = TRUE, ncol = 3))

# Put it all together
envelopeModel3D <- temperaturePresence * AOUpresence
envelopeModel3D <- mask(crop(envelopeModel3D, studyRegion), 
                        mask = studyRegion)
names(envelopeModel3D) <- names(temperature)
```

How does the projection look? `plotLayers` plots a transparent layer of suitable habitat for each depth layer. The redder the color, the shallower the layer, the bluer, the deeper. The more saturated the color, the more layers with suitable habitat. We will only look at projections within the study area and at depths the species was observed.

```{r plot 3D envelope model}
# Get indices of model-relevant layers
layerNames <- as.numeric(names(envelopeModel3D))
occurrences$index <- unlist(lapply(occurrences$depth, 
                                   FUN = function(x) 
                                     which.min(abs(layerNames - x))))
indices <- sort(unique(occurrences$index))

plotLayers(envelopeModel3D[[min(indices):max(indices)]],
           title = "Envelope Model of Luminous Hake,\n 20 to 700m",
           land = land)
```

Interestingly, the bottom model did not predict the suitable habitat recovered by the surface model in the Sargasso Sea. The 3D model predicts suitable habitat in the Sargasso Sea, but only at lower depths.

Of course, this is just the introduction. Please explore the other vignettes for more features and workflow examples. We welcome submissions from the community for other types of modeling algorithms, functions, and/or features you would like to see. Submit your suggestions as issues [here](https://github.com/hannahlowens/voluModel/issues).

# Tidying up
Last, we need to close the temporary directory we opened when we opened the data.

```{r cleanup temporary directory}
unlink(td, recursive = T)
```

# References

Bakis Y (2021): TU_Fish. v1.1. No organization. Dataset/Occurrence. https://web.archive.org/web/20221202141213/https://bgnn.tulane.edu/ipt/resource?r=tu_fish&v=1.1 Accessed via OBIS on 2020-11-04.  

Bentley A (2022). KUBI Ichthyology Collection. Version 17.80. University of Kansas Biodiversity Institute. DOI: 10.15468/mgjasg. Accessed via GBIF on 2020-10-13.

Bentley A (2022). KUBI Ichthyology Tissue Collection. Version 18.68. University of Kansas Biodiversity Institute. DOI: 10.15468/jmsnwg. Accessed via GBIF on 2020-10-13.

Buckup P A (2022). Coleção Ictiológica (MNRJ), Museu Nacional (MN), Universidade Federal do Rio de Janeiro(UFRJ). Version 157.1487. Museu Nacional / UFRJ. DOI: 10.15468/lluzfl. Accessed via GBIF on 2020-10-13.

Catania D, Fong J (2022). CAS Ichthyology (ICH). Version 150.300. California Academy of Sciences. DOI: 10.15468/efh2ib. Accessed via GBIF on 2020-10-13.

Chakrabarty P (2019). LSUMZ (LSU MNS) Fishes Collection. Version 2.2. Louisiana State University Museum of Natural Science. DOI: 10.15468/gbnym3. Accessed via GBIF on 2020-10-13.

Chamberlain S, Barve V, Mcglinn D, Oldoni D, Desmet P, Geffert L, Ram K (2021). rgbif: Interface to the Global Biodiversity Information Facility API. R package version 3.6.0, https://CRAN.R-project.org/package=rgbif.

Chamberlain S, Boettiger C (2017). “R Python, and Ruby clients for GBIF species occurrence data.” PeerJ PrePrints. DOI: 10.7287/peerj.preprints.3304v1.

Davis Rabosky AR, Cox CL, Rabosky DL, Title PO, Holmes IA, Feldman A, McGuire JA (2016). Coral snakes predict the evolution of mimicry across New World snakes. Nature Communications 7:11484.

Espinosa Pérez H, Comisión nacional para el conocimiento y uso de la biodiversidad C (2021). Computarización de la Colección Nacional de Peces del Instituto de Biología UNAM. Version 1.9. Comisión nacional para el conocimiento y uso de la biodiversidad. DOI: 10.15468/zb2odl. Accessed via GBIF on 2020-10-13.

Frable B (2019). SIO Marine Vertebrate Collection. Version 1.7. Scripps Institution of Oceanography. DOI: 10.15468/ad1ovc. Accessed via GBIF on 2020-10-13.

Fishnet2 Portal, http://www.fishnet2.net, [Access date] Accessed via OBIS on 2020-11-04.

Froese, R. and D. Pauly. Editors. 200x. FishBase. World Wide Web electronic publication. www.fishbase.org, version (xx/200x). Accessed via OBIS on 2020-11-04.   

Gall L (2021). Vertebrate Zoology Division - Ichthyology, Yale Peabody Museum. Yale University Peabody Museum. DOI: 10.15468/mgyhok. Accessed via GBIF on 2020-10-13.

García, CB, and Duarte, LO, 'Columbian Caribbean Sea', in J.H. Nicholls (comp.) HMAP Data Pages https://figshare.com/articles/dataset/HMAP_Dataset_18_Colombian_Caribbean_Sea/23504046 Accessed via OBIS on 2020-11-04.   

Garcia HE, Weathers K, Paver CR, Smolyar I, Boyer TP, Locarnini RA, Zweng MM, Mishonov AV, Baranova OK, Seidov D, Reagan JR (2018). World Ocean Atlas 2018, Volume 3: Dissolved Oxygen, Apparent Oxygen Utilization, and Oxygen Saturation. A Mishonov Technical Ed.; NOAA Atlas NESDIS 83, 38pp.

GBIF.org (24 September 2020) GBIF Occurrence Download DOI: 10.15468/dl.efuutj.

Grant S, McMahan C (2020). Field Museum of Natural History (Zoology) Fish Collection. Version 13.12. Field Museum. DOI: 10.15468/alz7wu. Accessed via GBIF on 2020-10-13.

Harvard University M, Morris P J (2022). Museum of Comparative Zoology, Harvard University. Version 162.296. Museum of Comparative Zoology, Harvard University. DOI: 10.15468/p5rupv. Accessed via GBIF on 2020-10-13.

Hendrickson D A, Cohen A E, Casarez M J (2022). University of Texas, Biodiversity Center, Ichthyology Collection (TNHCi). Version 5.175. University of Texas at Austin, Biodiversity Collections. DOI: 10.15468/h8gxdr. Accessed via GBIF on 2020-10-13.

INVEMAR. SIBM en línea: Sistema de Información sobre Biodiversidad Marina. Santa Marta: Instituto de investigaciones Marinas y Costeras José Benito Vives de Andréis,. https://siam.invemar.org.co/ Accessed via OBIS on 2020-11-04. 

Locarnini RA, Mishonov AV, Baranova OK, Boyer TP, Zweng MM, Garcia HE, Reagan JR, Seidov D, Weathers K, Paver CR, Smolyar I (2018). World Ocean Atlas 2018, Volume 1: Temperature. A. Mishonov Technical Ed.; NOAA Atlas NESDIS 81, 52pp.

McLean, M.W. (2014). Straightforward Bibliography Management in R Using the RefManager Package. NA, NA. https://arxiv.org/abs/1403.2036.

McLean, M.W. (2017). RefManageR: Import and Manage BibTeX and BibLaTeX References in R. The Journal of Open Source Software.

Mertz W (2021). LACM Vertebrate Collection. Version 18.9. Natural History Museum of Los Angeles County. DOI: 10.15468/77rmwd. Accessed via GBIF on 2020-10-13.

Nakae M, Shinohara G (2018). Fish collection of National Museum of Nature and Science. National Museum of Nature and Science, Japan. Occurrence dataset DOI: 10.15468/w3dzv1 accessed via GBIF.org on yyyy-mm-dd. Accessed via OBIS on 2020-11-04.    

Natural History Museum (2021). Natural History Museum (London) Collection Specimens. DOI: 10.5519/0002965. Accessed via GBIF on 2020-10-13.

National Museum of Natural History, Smithsonian Institution NMNH Fishes Collection Database. National Museum of Natural History, Smithsonian Institution, 10th and Constitution Ave. N.W., Washington, DC 20560-0193, 2007. Accessed via OBIS on 2020-11-04.

Norén M, Shah M (2017). Fishbase. FishBase. DOI: 10.15468/wk3zk7. Accessed via GBIF on 2020-10-13.

Norton B, Hogue G (2021). NCSM Ichthyology Collection. Version 22.8. North Carolina State Museum of Natural Sciences. DOI: 10.36102/dwc.1. Accessed via GBIF on 2020-10-13.

Nychka D, Furrer R, Paige J, Sain S (2021). “fields: Tools for spatial data.” R package version 13.3, <URL:
https://github.com/dnychka/fieldsRPackage>.

Orrell T, Informatics Office (2021). NMNH Extant Specimen Records (USNM, US). Version 1.49. National Museum of Natural History, Smithsonian Institution. DOI: 10.15468/hnhrg3. Accessed via GBIF on 2020-10-13.

Owens H, Merow C, Maitner B, Kass J, Barve V, Guralnick R (2021). occCite: Querying and Managing Large Biodiversity Occurrence Datasets_. DOI:  10.5281/zenodo.4726676 (URL: DOI: 10.5281/zenodo.4726676), R package version 0.4.9.9000, (<URL: https://CRAN.R-project.org/package=occCite>).

Prestridge H (2021). Biodiversity Research and Teaching Collections - TCWC Vertebrates. Version 9.4. Texas A&M University Biodiversity Research and Teaching Collections. DOI: 10.15468/szomia. Accessed via GBIF on 2020-10-13.

Provoost P, Bosch S (2019). “robis: R Client to access data from the OBIS API.” Ocean Biogeographic Information System. Intergovernmental Oceanographic Commission of UNESCO. R package version 2.1.8, https://cran.r-project.org/package=robis.

Pugh W (2021). UAIC Ichthyological Collection. Version 3.3. University of Alabama Biodiversity and Systematics. DOI: 10.15468/a2laag. Accessed via GBIF on 2020-10-13.

R Core Team (2017). R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. URL https://www.R-project.org/.

Robins R (2021). UF FLMNH Ichthyology. Version 117.342. Florida Museum of Natural History. DOI: 10.15468/8mjsel. Accessed via GBIF on 2020-10-13.

Tavera L, Tavera ML. (2021). Biodiversidad de los recursos marinos y costeros entre Cartagena y el Golfo de Urabá, Caribe colombiano. Version 2.4. Instituto de Investigaciones Marinas y Costeras - Invemar. DOI: 10.15472/d2jusp. Accessed via GBIF on 2020-10-13.

UNIBIO, IBUNAM. CNPE/Coleccion Nacional de Peces. DOI: 10.15468/o5d48z. Accessed via GBIF on 2020-10-13.

Wagner M (2017). MMNS Ichthyology Collection. Mississippi Museum of Natural Science. DOI: 10.15468/4c3qeq. Accessed via GBIF on 2020-10-13.

Werneke D (2019). AUM Fish Collection. Version 8.1. Auburn University Museum. DOI:  10.15468/dm3oyz. Accessed via GBIF on 2020-10-13.

---
title: "2. Processing Raster Data"
author:
- Hannah L. Owens
- Carsten Rahbek
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7
    fig_height: 7
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{2. Processing Raster Data}
  \usepackage[utf8]{inputenc}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)
```

# Introduction 

Naturally, to build 3D distribution models, one needs three-dimensionally structured environmental data. These data may come in the form of interpolated *in situ* measurements (e.g. [NOAA's World Ocean Atlas dataset](https://www.ncei.noaa.gov/access/world-ocean-atlas-2018/); Garcia et al. 2018a) or as outputs from climate models, either as stand-alone ocean components or as a component of coupled atmosphere-ocean models (e.g. CCSM3, Collins et al. 2006; HadCM3, Valdes et al. 2017). Typically, these data are organized as a series of horizontal layers that are stacked by depth. 

```{r load packages, message=FALSE, warning=FALSE}
library(voluModel) # Since this is the package this vignette is about.
library(tibble) # For data organization
library(ggplot2) # For supplementary visualization
library(fields) # For raster interpolation
library(terra) # Now being transitioned in
```

# Data Inputs

First, let's look at a relatively simple environmental variable from the WOA: temperature (Locarnini et al, 2018). These data are supplied by the World Ocean Atlas as point shapefiles; the version supplied here has been cropped between -98 and -45 longitude, between -1 and 45 latitude, and between 0 and 4,400 m in depth to make it more space-efficientf. You can download the full dataset via the WOA website. Our first task is to read in the shapefile as a `SpatVector`. Note that each row in `temperature` is a set of horizontal coordinates. Each column is a vertical position in the water column. Make sure to check the metadata of the data you use. Different sources may use vertical depth structures.

```{r environmental data loading temperature, eval=T, message=FALSE, warning=FALSE, asis=T}
# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Looking at the dataset
as.data.frame(temperature[1:5,1:10])

# Plotting the dataset
layout(matrix(c(1, 2), ncol=2, byrow=TRUE), widths=c(4, 1))
land <- rnaturalearth::ne_countries(scale = "small", 
                                    returnclass = "sf")[1]
temperatureForPlot <- temperature
crs(temperatureForPlot) <- crs(land) 
ext <- ext(temperatureForPlot)
plot(temperatureForPlot, main = "Distribution of voluModel Subset\nof WOA Temperature 2018",
     pch = 20, col = "red", xlim = ext[1:2], ylim = ext[3:4], cex = .6, mar = c(2,2,3,2))
plot(land, col = "black", add = T)

# What does the WOA depth structure look like?
depths <- names(temperatureForPlot)
depths <- as.numeric(gsub(depths[-1], pattern = "[d,M]", replacement = ""))
plot(0, xlim = c(0,1), ylim = c(0-max(depths), 0), axes=FALSE, type = "n", xlab = "", ylab = "Depth Intervals (m)")
axis(2, at = 0-depths, labels = depths)
```

Next, we convert temperature into a `SpatRaster`. While we are at it, we will generate a raster from the deepest value available for each point. While you might call this a "bottom" raster, it is important to note that in some cases, values are not available for the bottom. After the `SpatRaster` is created, we give it the same depth names as the columns in the `SpatVector`. This is important, because `voluModel` uses these names as z coordinates when handling 3D data. I am using `oneRasterPlot()` from voluModel to visualize the rasters using a uniform, high-contrast aesthetic.

```{r temperature processing, eval=TRUE}
# Creating a bottom raster
temperatureBottom <- bottomRaster(temperature)

# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal
rm(tempTerVal)

# How do these files look?
par(mfrow=c(1,2))
p1 <- oneRasterPlot(temperature[[1]], land = land, landCol = "black", 
              title= "Surface Temperature (C)")

p2 <- oneRasterPlot(temperatureBottom,land = land, landCol = "black", 
              title = "Bottom Temperature (C)")
```

# Interpolation

Next, we have a bit of a more complicated example: apparent oxygen utilization (AOU; Garcia et al, 2018b). Apparent oxygen usage is more patchily sampled than temperature (it's generally measured from instrument casts on research cruises, and the coverage isn't quite as dense as for temperature).

```{r environmental data loading oxygen, eval=T, message=FALSE, warning=FALSE, asis=T}
td <- tempdir()
unzip(system.file("extdata/woa18_all_A00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/oxygen"), junkpaths = T)

oxygen <- vect(paste0(td, "/oxygen/woa18_all_A00mn01_cropped.shp")) 

plot(oxygen, main = "Distribution of voluModel subset of WOA AOU 2018",
     pch = 20, col = "red", xlim = ext[1:2], ylim = ext[3:4], cex = .6)
plot(land, col = "black", add = T)
```

There is a workaround for this, since we can reasonably expect some degree of spatial autocorrelation in apparent oxygen utilization. We use the `interpolateRaster()` function to produce statistically-interpolated layers using `TPS()` from the `fields` package (this is a thin-plate spline interpolation). Be patient--this step can take a while, although as shown here, I am using the `fastTPS()` approximation, which only samples from nearby cells to speed things up. Maybe `fastTPS()` will work ok for your data, maybe it won't. It depends on the data. For the sake of efficiency for this example, I am going to limit this to the first 10 depth layers.

```{r interpolate oxygen, warning=FALSE, eval = T}
# Creating a SpatRaster vector for the first 10 depth layers
oxygen <- oxygen[,1:10] # Remove this line if you want to process the whole file
oxygen <- rasterize(x = oxygen, y = template,
                   field = names(oxygen)) #Uses same raster template as temperature

for (i in 1:nlyr(oxygen)){ 
  oxygen[[i]] <- interpolateRaster(oxygen[[i]], lon.lat = T, fast = T, aRange = 30) #Thin plate spline interpolation
  oxygen[[i]] <- crop(mask(x = oxygen[[i]], 
                           mask = temperature[[i]]), 
                      temperature[[i]])
}

# Change names to match tempT
names(oxygen) <- envtNames[1:nlyr(oxygen)]
```

Some environmental data rasters may also look "patchy", possibly as an artifact of uneven sampling. If you are confident that there should be a strong spatial correlation in a data layer that looks "patchy", you can also statistically smooth it, again using `TPS()`, like this:

```{r smoothing oxygen, eval=TRUE}
# Smoothing tempO and saving
oxygenSmooth <- oxygen
for (i in 1:nlyr(oxygenSmooth)){
  oxygenSmooth[[i]] <- smoothRaster(oxygenSmooth[[i]], lon.lat = T) #Thin plate spline interpolation
  oxygenSmooth[[i]] <- crop(mask(x = oxygenSmooth[[i]], mask = temperature[[i]]), temperature[[i]])
}

# Change names to match tempT and save
names(oxygenSmooth) <- names(oxygen)
oxygenSmooth <- oxygenSmooth

par(mfrow=c(1,2))
p3 <- oneRasterPlot(oxygen[[1]], land = land, landCol = "black", 
              title= "Surface Apparent Oxygen Utilization\n(µmol/kg), interpolated")
p4 <- oneRasterPlot(oxygenSmooth[[1]], land = land, landCol = "black",
     title = "Surface Apparent Oxygen Utilization\n(µmol/kg), interpolated and smoothed")
```

# Tidying up
Last, we need to close the temporary directory we opened when we opened the data.

```{r cleanup temporary directory}
unlink(td, recursive = T)
```

# References

Collins WD, Bitz CM, Blackmon ML, Bonan GB, Bretherton CS, Carton JA, Chang P, Doney SC, Hack JJ, Henderson TB, Kiehl JT, Large WG, McKenna DS, Santer BD, Smith RD. (2006) The Community Climate System Model Version 3 (CCSM3) Journal of Climate, 19(11), 2122–2143 DOI: 10.1175/jcli3761.1

Garcia HE, Boyer P, Baranova OK, Locarnini RA, Mishonov AV, Grodsky A, Paver CR, Weathers KW, Smolyar IV, Reagan JR, Seidov D, Zweng MM. (2018a). World Ocean Atlas 2018 (A. Mishonov, Ed.). NOAA National Centers for Environmental Information. https://doi.org/10.25923/tzyw-rp36

Garcia HE, Weathers K, Paver CR, Smolyar I, Boyer TP, Locarnini RA, Zweng MM, Mishonov AV, Baranova OK, Seidov D, Reagan JR (2018b). World Ocean Atlas 2018, Volume 3: Dissolved Oxygen, Apparent Oxygen Utilization, and Oxygen Saturation. A Mishonov Technical Ed.; NOAA Atlas NESDIS 83, 38pp.

Locarnini RA, Mishonov AV, Baranova OK, Boyer TP, Zweng MM, Garcia HE, Reagan JR, Seidov D, Weathers K, Paver CR, Smolyar I (2018). World Ocean Atlas 2018, Volume 1: Temperature. A. Mishonov Technical Ed.; NOAA Atlas NESDIS 81, 52pp.

Nychka D, Furrer R, Paige J, Sain S (2021). “fields: Tools for spatial data.” R package version 13.3, <URL:
https://github.com/dnychka/fieldsRPackage>.

R Core Team (2017). R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. URL https://www.R-project.org/.

Valdes PJ, Armstrong E, Badger MPS, Bradshaw CD, Bragg F, Crucifix M, Davies-Barnard T, Day JJ, Farnsworth A, Gordon C, Hopcroft PO, Kennedy AT, Lord NS, Lunt DJ, Marzocchi A, Parry LM, Pope V, Roberts WHG, Stone EJ, … Williams JHT. (2017). The BRIDGE HadCM3 family of climate models: HadCM3@Bristol v1.0. Geoscientific Model Development, 10(10), 3715–3743. DOI: 10.5194/gmd-10-3715-2017

---
title: "4. Visualization Tools"
author:
- Hannah L. Owens
- Carsten Rahbek
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7
    fig_height: 7
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{4. Visualization Tools}
  \usepackage[utf8]{inputenc}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)
```

# Introduction 

`voluModel` contains several functions to generate formatted maps of rasters and points automatically. This was to aid us in cleanly visualizing large numbers of maps efficiently using `ggplot2` (Wickam, 2016) and `lattice` (Sarkar, 2008) functionality. While there are quite a few features available in these functions, they were designed with fairly specific use cases in mind. You are welcome to gut the functions and rewrite them for your own custom needs. 

The data I will be using for the demos in this tutorial will be loaded and analyzed first. The Luminous Hake occurrence points, which you can see in the [data sampling tutorial](https://hannahlowens.github.io/voluModel/articles/c_DataSampling.html), were downloaded via R (R Core Team, 2020) from GBIF (Chamberlain *et al.*, 2021; Chamberlain and Boettiger, 2017) and OBIS (Provoost and Bosch, 2019) via `occCite` (Owens *et al.*, 2021). The simple envelope model is based on two environmental datasets from the World Ocean Atlas (Garcia *et al.*, 2019): temperature (Locarnini *et al.*, 2018) and apparent oxygen utilization (AOU; Garcia *et al.*, 2019). For more details on how to process environmental data, see [the raster data tutorial](https://hannahlowens.github.io/voluModel/articles/b_RasterProcessing.html). For the workflow used to generate the niche envelopes, refer to [the introduction vignette](https://hannahlowens.github.io/voluModel/articles/a_Introduction.html).

```{r generating data for plotting}
library(voluModel) # Because of course
library(ggplot2) # For fancy plotting
library(viridisLite) # For high-contrast plotting palettes
library(dplyr) # To filter data
library(terra) # Now being transitioned in
library(sf) # Now being transitioned in

# Load data
oxygenSmooth <- rast(system.file("extdata/oxygenSmooth.tif", 
                                 package='voluModel'))

occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv", 
                             package='voluModel'))

# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal

# Oxygen processing
names(oxygenSmooth) <- names(temperature)

# Clean points ----
occurrences <- occs %>% dplyr::select(decimalLongitude, decimalLatitude, depth) %>%
  distinct() %>% filter(dplyr::between(depth, 1, 2000))

# Gets the layer index for each occurrence by matching to depth
layerNames <- as.numeric(names(temperature))
occurrences$index <- unlist(lapply(occurrences$depth, 
                                   FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occurrences$index)
downsampledOccs <- data.frame()
for(i in indices){
  tempPoints <- occurrences[occurrences$index==i,]
  tempPoints <- downsample(tempPoints, temperature[[1]], verbose = FALSE)
  tempPoints$depth <- rep(layerNames[[i]], times = nrow(tempPoints))
  downsampledOccs <- rbind(downsampledOccs, tempPoints)
}
occsWdata <- downsampledOccs[,c("decimalLatitude", "decimalLongitude", "depth")] 

# Extract data ----
occsWdata$temperature <- xyzSample(occs = occsWdata, temperature)
occsWdata$AOU <- xyzSample(occs = occsWdata, oxygenSmooth)
occsWdata <- occsWdata[complete.cases(occsWdata),]

# Land shapefile
land <- rnaturalearth::ne_countries(scale = "small", returnclass = "sf")[1]

# Study region
studyRegion <- marineBackground(occsWdata, buff = 1000000)

# Get limits
tempLims <- quantile(occsWdata$temperature,c(0, 1))
aouLims <- quantile(occsWdata$AOU,c(0, 1))

# Reclassify environmental bricks to presence/absence
temperaturePresence <- classify(temperature, 
                                rcl = matrix(c(-Inf,tempLims[[1]],0,
                                               tempLims[[1]], tempLims[[2]], 1,
                                               tempLims[[2]], Inf, 0),
                                             ncol = 3, byrow = TRUE))
AOUpresence <- classify(oxygenSmooth, 
                        rcl = matrix(c(-Inf, aouLims[[1]],0,
                                       aouLims[[1]], aouLims[[2]], 1,
                                       aouLims[[2]], Inf, 0), 
                                     ncol = 3, byrow = TRUE))

# Put it all together
envelopeModel3D <- temperaturePresence * AOUpresence
envelopeModel3D <- mask(crop(envelopeModel3D, studyRegion), 
                        mask = studyRegion)
names(envelopeModel3D) <- names(temperature)
rm(AOUpresence, downsampledOccs, occurrences, temperaturePresence, 
   tempPoints, aouLims, envtNames, i, indices, layerNames, tempLims)
```

# Plotting points

There are two functions in `voluModel` for plotting occurrence points. The first, `pointMap()`, plots a single set of horizontal geographic points. The title generated is the species name the user supplies and a count of the number of points that have been plotted.

```{r pointMap, warning=FALSE, message=FALSE, eval=TRUE}
pointMap(occs = occs, land = land, landCol = "black", spName = "Steindachneria argentea", 
         ptSize = 2, ptCol = "orange")
```

The second, `pointCompMap()` plots a map that maps the extents of two sets of horizontal geographic points in different colors. This might be useful, for example, if you want to compare a raw dataset to a cleaned dataset. If any points overlap, they can be plotted as a third color.

```{r pointCompMap, warning=FALSE, message=FALSE}
pointCompMap(occs1 = occs, occs1Col = "red", occs1Name = "Raw", 
             occs2 = occsWdata, occs2Col = "orange", occs2Name = "Clean",
             spName = "Steindachneria argentea", agreeCol = "purple",
             land = land, landCol = "black", ptSize = 2, verbose = FALSE)
```

# Plotting rasters

If you have a horizontal raster you would like to map that contains continuous data, the function for you might be `oneRasterPlot()`. It is a wrapper around `spplot()` from `sp` (Pebesma and Bivand, 2005; Bivand *et al.* 2013) that implements the high-contrast, colorblind-friendly color palettes offered in `viridis()` from `viridisLite` (Garnier *et al.*, 2021). The function optionally takes arguments to the `spplot()` and `viridis()` functions (so, for example, you can choose a palette other than the default).

```{r oneRasterPlot}
oneRasterPlot(rast = temperature[[1]],
              land = land, title = "Sea Surface Temperature, WOA 2018",
              landCol = "black", n = 11, option = "mako",
              varName = "Temperature")
```

Suppose you want to compare the extents of two presence-absence rasters--perhaps you are comparing two species distribution model method outputs, or comparing potential distributions at two different depths. We have a function for that, too.

```{r rasterComp}
rasterComp(rast1 = envelopeModel3D[[1]], rast1Name = "Surface",
           rast2 = envelopeModel3D[[10]], rast2Name = "45m", 
           land = land, landCol = "black", 
           title = "Suitability of Habitat for Luminous Hake\nAt Two Different Depths")
```

Of course, 3D models such as the ones `voluModel` is designed to help you produce, are difficult to visualize. We have given it a try, though. `plotLayers()` plots a transparent layer of suitable habitat for each depth layer. The redder the color, the shallower the layer, the bluer, the deeper. The more saturated the color, the more layers with suitable habitat. Here, I am plotting suitability from 20 to 700 m, the depth range of occurrences used to train the envelope model.

```{r plotLayers}
layerNames <- as.numeric(names(envelopeModel3D))
occsWdata$index <- unlist(lapply(occsWdata$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occsWdata$index)

layerPlot <- plotLayers(envelopeModel3D[[min(indices):max(indices)]], 
                        title = "Envelope Model of Luminous Hake,\n 20 to 700m",
                        land = land, landCol = "black")
```

# Tidying up
Last, we need to close the temporary directory we opened when we opened the data.

```{r cleanup temporary directory}
unlink(td, recursive = T)
```

# References

Bakis Y (2021): TU_Fish. v1.1. No organization. Dataset/Occurrence. https://fishair.org/. Accessed via OBIS on 2020-11-04.  

Bentley A (2022). KUBI Ichthyology Collection. Version 17.80. University of Kansas Biodiversity Institute. DOI: 10.15468/mgjasg. Accessed via GBIF on 2020-10-13.

Bentley A (2022). KUBI Ichthyology Tissue Collection. Version 18.68. University of Kansas Biodiversity Institute. DOI: 10.15468/jmsnwg. Accessed via GBIF on 2020-10-13.

Buckup P A (2022). Coleção Ictiológica (MNRJ), Museu Nacional (MN), Universidade Federal do Rio de Janeiro(UFRJ). Version 157.1487. Museu Nacional / UFRJ. DOI: 10.15468/lluzfl. Accessed via GBIF on 2020-10-13.

Catania D, Fong J (2022). CAS Ichthyology (ICH). Version 150.300. California Academy of Sciences. DOI: 10.15468/efh2ib. Accessed via GBIF on 2020-10-13.

Chakrabarty P (2019). LSUMZ (LSU MNS) Fishes Collection. Version 2.2. Louisiana State University Museum of Natural Science. DOI: 10.15468/gbnym3. Accessed via GBIF on 2020-10-13.

Chamberlain S, Barve V, Mcglinn D, Oldoni D, Desmet P, Geffert L, Ram K (2021). rgbif: Interface to the Global Biodiversity Information Facility API. R package version 3.6.0, https://CRAN.R-project.org/package=rgbif.

Chamberlain S, Boettiger C (2017). “R Python, and Ruby clients for GBIF species occurrence data.” PeerJ PrePrints. DOI: 10.7287/peerj.preprints.3304v1.

Davis Rabosky AR, Cox CL, Rabosky DL, Title PO, Holmes IA, Feldman A, McGuire JA (2016). Coral snakes predict the evolution of mimicry across New World snakes. Nature Communications 7:11484.

Espinosa Pérez H, Comisión nacional para el conocimiento y uso de la biodiversidad C (2021). Computarización de la Colección Nacional de Peces del Instituto de Biología UNAM. Version 1.9. Comisión nacional para el conocimiento y uso de la biodiversidad. DOI: 10.15468/zb2odl. Accessed via GBIF on 2020-10-13.

Frable B (2019). SIO Marine Vertebrate Collection. Version 1.7. Scripps Institution of Oceanography. DOI: 10.15468/ad1ovc. Accessed via GBIF on 2020-10-13.

Fishnet2 Portal, http://www.fishnet2.net, [Access date] Accessed via OBIS on 2020-11-04.

Froese, R. and D. Pauly. Editors. 200x. FishBase. World Wide Web electronic publication. www.fishbase.org, version (xx/200x). Accessed via OBIS on 2020-11-04.   

Gall L (2021). Vertebrate Zoology Division - Ichthyology, Yale Peabody Museum. Yale University Peabody Museum. DOI: 10.15468/mgyhok. Accessed via GBIF on 2020-10-13.

García, CB, and Duarte, LO, 'Columbian Caribbean Sea', in J.H. Nicholls (comp.) HMAP Data Pages https://doi.org/10.6084/m9.figshare.23504046 Accessed via OBIS on 2020-11-04.   

Garcia HE, Weathers K, Paver CR, Smolyar I, Boyer TP, Locarnini RA, Zweng MM, Mishonov AV, Baranova OK, Seidov D, Reagan JR (2018). World Ocean Atlas 2018, Volume 3: Dissolved Oxygen, Apparent Oxygen Utilization, and Oxygen Saturation. A Mishonov Technical Ed.; NOAA Atlas NESDIS 83, 38pp.

Garnier S, Ross N, Rudis R, Camargo AP, Sciaini M, Scherer C (2021). Rvision - Colorblind-Friendly Color Maps for R. R package version 0.6.2.

GBIF.org (24 September 2020) GBIF Occurrence Download DOI: 10.15468/dl.efuutj.

Grant S, McMahan C (2020). Field Museum of Natural History (Zoology) Fish Collection. Version 13.12. Field Museum. DOI: 10.15468/alz7wu. Accessed via GBIF on 2020-10-13.

Harvard University M, Morris P J (2022). Museum of Comparative Zoology, Harvard University. Version 162.296. Museum of Comparative Zoology, Harvard University. DOI: 10.15468/p5rupv. Accessed via GBIF on 2020-10-13.

Hendrickson D A, Cohen A E, Casarez M J (2022). University of Texas, Biodiversity Center, Ichthyology Collection (TNHCi). Version 5.175. University of Texas at Austin, Biodiversity Collections. DOI: 10.15468/h8gxdr. Accessed via GBIF on 2020-10-13.

INVEMAR. SIBM en línea: Sistema de Información sobre Biodiversidad Marina. Santa Marta: Instituto de investigaciones Marinas y Costeras José Benito Vives de Andréis,. https://siam.invemar.org.co/ Accessed via OBIS on 2020-11-04. 

Locarnini RA, Mishonov AV, Baranova OK, Boyer TP, Zweng MM, Garcia HE, Reagan JR, Seidov D, Weathers K, Paver CR, Smolyar I (2018). World Ocean Atlas 2018, Volume 1: Temperature. A. Mishonov Technical Ed.; NOAA Atlas NESDIS 81, 52pp.

McLean, M.W. (2014). Straightforward Bibliography Management in R Using the RefManager Package. NA, NA. https://arxiv.org/abs/1403.2036.

McLean, M.W. (2017). RefManageR: Import and Manage BibTeX and BibLaTeX References in R. The Journal of Open Source Software.

Mertz W (2021). LACM Vertebrate Collection. Version 18.9. Natural History Museum of Los Angeles County. DOI: 10.15468/77rmwd. Accessed via GBIF on 2020-10-13.

Nakae M, Shinohara G (2018). Fish collection of National Museum of Nature and Science. National Museum of Nature and Science, Japan. Occurrence dataset DOI: 10.15468/w3dzv1 accessed via GBIF.org on yyyy-mm-dd. Accessed via OBIS on 2020-11-04.    

Natural History Museum (2021). Natural History Museum (London) Collection Specimens. DOI: 10.5519/0002965. Accessed via GBIF on 2020-10-13.

National Museum of Natural History, Smithsonian Institution NMNH Fishes Collection Database. National Museum of Natural History, Smithsonian Institution, 10th and Constitution Ave. N.W., Washington, DC 20560-0193, 2007. Accessed via OBIS on 2020-11-04.

Norén M, Shah M (2017). Fishbase. FishBase. DOI: 10.15468/wk3zk7. Accessed via GBIF on 2020-10-13.

Norton B, Hogue G (2021). NCSM Ichthyology Collection. Version 22.8. North Carolina State Museum of Natural Sciences. DOI: 10.36102/dwc.1. Accessed via GBIF on 2020-10-13.

Nychka D, Furrer R, Paige J, Sain S (2021). “fields: Tools for spatial data.” R package version 13.3, <URL:
https://github.com/dnychka/fieldsRPackage>.

Orrell T, Informatics Office (2021). NMNH Extant Specimen Records (USNM, US). Version 1.49. National Museum of Natural History, Smithsonian Institution. DOI: 10.15468/hnhrg3. Accessed via GBIF on 2020-10-13.

Owens H, Merow C, Maitner B, Kass J, Barve V, Guralnick R (2021). occCite: Querying and Managing Large Biodiversity Occurrence Datasets_. doi: 10.5281/zenodo.4726676 (URL: DOI: 10.5281/zenodo.4726676), R package version 0.4.9.9000, (<URL: https://CRAN.R-project.org/package=occCite>).

Pebesma, E.J., R.S. Bivand (2005). Classes and methods for spatial data in R. R News 5 (2),
  https://cran.r-project.org/doc/Rnews/.

Prestridge H (2021). Biodiversity Research and Teaching Collections - TCWC Vertebrates. Version 9.4. Texas A&M University Biodiversity Research and Teaching Collections. DOI: 10.15468/szomia. Accessed via GBIF on 2020-10-13.

Provoost P, Bosch S (2019). “robis: R Client to access data from the OBIS API.” Ocean Biogeographic Information System. Intergovernmental Oceanographic Commission of UNESCO. R package version 2.1.8, https://cran.r-project.org/package=robis.

Pugh W (2021). UAIC Ichthyological Collection. Version 3.3. University of Alabama Biodiversity and Systematics. DOI: 10.15468/a2laag. Accessed via GBIF on 2020-10-13.

R Core Team (2017). R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. URL https://www.R-project.org/.

Robins R (2021). UF FLMNH Ichthyology. Version 117.342. Florida Museum of Natural History. DOI: 10.15468/8mjsel. Accessed via GBIF on 2020-10-13.

Bivand RS, Pebesma E, Gomez-Rubio V (2013). Applied spatial data analysis with R, Second edition.
  Springer, NY. https://asdar-book.org/

Sarkar D (2008). Lattice: Multivariate Data Visualization with R. Springer, New York. ISBN
  978-0-387-75968-5

Tavera L, Tavera M. L (2021). Biodiversidad de los recursos marinos y costeros entre Cartagena y el Golfo de Urabá, Caribe colombiano. Version 2.4. Instituto de Investigaciones Marinas y Costeras - Invemar. DOI: 10.15472/d2jusp. Accessed via GBIF on 2020-10-13.

UNIBIO, IBUNAM. CNPE/Coleccion Nacional de Peces. DOI: 10.15468/o5d48z. Accessed via GBIF on 2020-10-13.

Wagner M (2017). MMNS Ichthyology Collection. Mississippi Museum of Natural Science. DOI: 10.15468/4c3qeq. Accessed via GBIF on 2020-10-13.

Werneke D (2019). AUM Fish Collection. Version 8.1. Auburn University Museum. DOI: 10.15468/dm3oyz. Accessed via GBIF on 2020-10-13.

Wickham H (2016). ggplot2: Elegant Graphics for Data Analysis. Springer-Verlag New York.

---
title: "4. Visualization Tools"
author:
- Hannah L. Owens
- Carsten Rahbek
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7
    fig_height: 7
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{4. Visualization Tools}
  \usepackage[utf8]{inputenc}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)
```

# Introduction 

`voluModel` contains several functions to generate formatted maps of rasters and points automatically. This was to aid us in cleanly visualizing large numbers of maps efficiently using `ggplot2` (Wickam, 2016) and `lattice` (Sarkar, 2008) functionality. While there are quite a few features available in these functions, they were designed with fairly specific use cases in mind. You are welcome to gut the functions and rewrite them for your own custom needs. 

The data I will be using for the demos in this tutorial will be loaded and analyzed first. The Luminous Hake occurrence points, which you can see in the [data sampling tutorial](https://hannahlowens.github.io/voluModel/articles/c_DataSampling.html), were downloaded via R (R Core Team, 2020) from GBIF (Chamberlain *et al.*, 2021; Chamberlain and Boettiger, 2017) and OBIS (Provoost and Bosch, 2019) via `occCite` (Owens *et al.*, 2021). The simple envelope model is based on two environmental datasets from the World Ocean Atlas (Garcia *et al.*, 2019): temperature (Locarnini *et al.*, 2018) and apparent oxygen utilization (AOU; Garcia *et al.*, 2019). For more details on how to process environmental data, see [the raster data tutorial](https://hannahlowens.github.io/voluModel/articles/b_RasterProcessing.html). For the workflow used to generate the niche envelopes, refer to [the introduction vignette](https://hannahlowens.github.io/voluModel/articles/a_Introduction.html).

```{r generating data for plotting}
library(voluModel) # Because of course
library(ggplot2) # For fancy plotting
library(viridisLite) # For high-contrast plotting palettes
library(dplyr) # To filter data
library(terra) # Now being transitioned in
library(sf) # Now being transitioned in

# Load data
oxygenSmooth <- rast(system.file("extdata/oxygenSmooth.tif", 
                                 package='voluModel'))

occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv", 
                             package='voluModel'))

# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal

# Oxygen processing
names(oxygenSmooth) <- names(temperature)

# Clean points ----
occurrences <- occs %>% dplyr::select(decimalLongitude, decimalLatitude, depth) %>%
  distinct() %>% filter(dplyr::between(depth, 1, 2000))

# Gets the layer index for each occurrence by matching to depth
layerNames <- as.numeric(names(temperature))
occurrences$index <- unlist(lapply(occurrences$depth, 
                                   FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occurrences$index)
downsampledOccs <- data.frame()
for(i in indices){
  tempPoints <- occurrences[occurrences$index==i,]
  tempPoints <- downsample(tempPoints, temperature[[1]], verbose = FALSE)
  tempPoints$depth <- rep(layerNames[[i]], times = nrow(tempPoints))
  downsampledOccs <- rbind(downsampledOccs, tempPoints)
}
occsWdata <- downsampledOccs[,c("decimalLatitude", "decimalLongitude", "depth")] 

# Extract data ----
occsWdata$temperature <- xyzSample(occs = occsWdata, temperature)
occsWdata$AOU <- xyzSample(occs = occsWdata, oxygenSmooth)
occsWdata <- occsWdata[complete.cases(occsWdata),]

# Land shapefile
land <- rnaturalearth::ne_countries(scale = "small", returnclass = "sf")[1]

# Study region
studyRegion <- marineBackground(occsWdata, buff = 1000000)

# Get limits
tempLims <- quantile(occsWdata$temperature,c(0, 1))
aouLims <- quantile(occsWdata$AOU,c(0, 1))

# Reclassify environmental bricks to presence/absence
temperaturePresence <- classify(temperature, 
                                rcl = matrix(c(-Inf,tempLims[[1]],0,
                                               tempLims[[1]], tempLims[[2]], 1,
                                               tempLims[[2]], Inf, 0),
                                             ncol = 3, byrow = TRUE))
AOUpresence <- classify(oxygenSmooth, 
                        rcl = matrix(c(-Inf, aouLims[[1]],0,
                                       aouLims[[1]], aouLims[[2]], 1,
                                       aouLims[[2]], Inf, 0), 
                                     ncol = 3, byrow = TRUE))

# Put it all together
envelopeModel3D <- temperaturePresence * AOUpresence
envelopeModel3D <- mask(crop(envelopeModel3D, studyRegion), 
                        mask = studyRegion)
names(envelopeModel3D) <- names(temperature)
rm(AOUpresence, downsampledOccs, occurrences, temperaturePresence, 
   tempPoints, aouLims, envtNames, i, indices, layerNames, tempLims)
```

# Plotting points

There are two functions in `voluModel` for plotting occurrence points. The first, `pointMap()`, plots a single set of horizontal geographic points. The title generated is the species name the user supplies and a count of the number of points that have been plotted.

```{r pointMap, warning=FALSE, message=FALSE, eval=TRUE}
pointMap(occs = occs, land = land, landCol = "black", spName = "Steindachneria argentea", 
         ptSize = 2, ptCol = "orange")
```

The second, `pointCompMap()` plots a map that maps the extents of two sets of horizontal geographic points in different colors. This might be useful, for example, if you want to compare a raw dataset to a cleaned dataset. If any points overlap, they can be plotted as a third color.

```{r pointCompMap, warning=FALSE, message=FALSE}
pointCompMap(occs1 = occs, occs1Col = "red", occs1Name = "Raw", 
             occs2 = occsWdata, occs2Col = "orange", occs2Name = "Clean",
             spName = "Steindachneria argentea", agreeCol = "purple",
             land = land, landCol = "black", ptSize = 2, verbose = FALSE)
```

# Plotting rasters

If you have a horizontal raster you would like to map that contains continuous data, the function for you might be `oneRasterPlot()`. It is a wrapper around `spplot()` from `sp` (Pebesma and Bivand, 2005; Bivand *et al.* 2013) that implements the high-contrast, colorblind-friendly color palettes offered in `viridis()` from `viridisLite` (Garnier *et al.*, 2021). The function optionally takes arguments to the `spplot()` and `viridis()` functions (so, for example, you can choose a palette other than the default).

```{r oneRasterPlot}
oneRasterPlot(rast = temperature[[1]],
              land = land, title = "Sea Surface Temperature, WOA 2018",
              landCol = "black", n = 11, option = "mako",
              varName = "Temperature")
```

Suppose you want to compare the extents of two presence-absence rasters--perhaps you are comparing two species distribution model method outputs, or comparing potential distributions at two different depths. We have a function for that, too.

```{r rasterComp}
rasterComp(rast1 = envelopeModel3D[[1]], rast1Name = "Surface",
           rast2 = envelopeModel3D[[10]], rast2Name = "45m", 
           land = land, landCol = "black", 
           title = "Suitability of Habitat for Luminous Hake\nAt Two Different Depths")
```

Of course, 3D models such as the ones `voluModel` is designed to help you produce, are difficult to visualize. We have given it a try, though. `plotLayers()` plots a transparent layer of suitable habitat for each depth layer. The redder the color, the shallower the layer, the bluer, the deeper. The more saturated the color, the more layers with suitable habitat. Here, I am plotting suitability from 20 to 700 m, the depth range of occurrences used to train the envelope model.

```{r plotLayers}
layerNames <- as.numeric(names(envelopeModel3D))
occsWdata$index <- unlist(lapply(occsWdata$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occsWdata$index)

layerPlot <- plotLayers(envelopeModel3D[[min(indices):max(indices)]], 
                        title = "Envelope Model of Luminous Hake,\n 20 to 700m",
                        land = land, landCol = "black")
```

# Tidying up
Last, we need to close the temporary directory we opened when we opened the data.

```{r cleanup temporary directory}
unlink(td, recursive = T)
```

# References

Bakis Y (2021): TU_Fish. v1.1. No organization. Dataset/Occurrence. https://fishair.org/. Accessed via OBIS on 2020-11-04.  

Bentley A (2022). KUBI Ichthyology Collection. Version 17.80. University of Kansas Biodiversity Institute. DOI: 10.15468/mgjasg. Accessed via GBIF on 2020-10-13.

Bentley A (2022). KUBI Ichthyology Tissue Collection. Version 18.68. University of Kansas Biodiversity Institute. DOI: 10.15468/jmsnwg. Accessed via GBIF on 2020-10-13.

Buckup P A (2022). Coleção Ictiológica (MNRJ), Museu Nacional (MN), Universidade Federal do Rio de Janeiro(UFRJ). Version 157.1487. Museu Nacional / UFRJ. DOI: 10.15468/lluzfl. Accessed via GBIF on 2020-10-13.

Catania D, Fong J (2022). CAS Ichthyology (ICH). Version 150.300. California Academy of Sciences. DOI: 10.15468/efh2ib. Accessed via GBIF on 2020-10-13.

Chakrabarty P (2019). LSUMZ (LSU MNS) Fishes Collection. Version 2.2. Louisiana State University Museum of Natural Science. DOI: 10.15468/gbnym3. Accessed via GBIF on 2020-10-13.

Chamberlain S, Barve V, Mcglinn D, Oldoni D, Desmet P, Geffert L, Ram K (2021). rgbif: Interface to the Global Biodiversity Information Facility API. R package version 3.6.0, https://CRAN.R-project.org/package=rgbif.

Chamberlain S, Boettiger C (2017). “R Python, and Ruby clients for GBIF species occurrence data.” PeerJ PrePrints. DOI: 10.7287/peerj.preprints.3304v1.

Davis Rabosky AR, Cox CL, Rabosky DL, Title PO, Holmes IA, Feldman A, McGuire JA (2016). Coral snakes predict the evolution of mimicry across New World snakes. Nature Communications 7:11484.

Espinosa Pérez H, Comisión nacional para el conocimiento y uso de la biodiversidad C (2021). Computarización de la Colección Nacional de Peces del Instituto de Biología UNAM. Version 1.9. Comisión nacional para el conocimiento y uso de la biodiversidad. DOI: 10.15468/zb2odl. Accessed via GBIF on 2020-10-13.

Frable B (2019). SIO Marine Vertebrate Collection. Version 1.7. Scripps Institution of Oceanography. DOI: 10.15468/ad1ovc. Accessed via GBIF on 2020-10-13.

Fishnet2 Portal, http://www.fishnet2.net, [Access date] Accessed via OBIS on 2020-11-04.

Froese, R. and D. Pauly. Editors. 200x. FishBase. World Wide Web electronic publication. www.fishbase.org, version (xx/200x). Accessed via OBIS on 2020-11-04.   

Gall L (2021). Vertebrate Zoology Division - Ichthyology, Yale Peabody Museum. Yale University Peabody Museum. DOI: 10.15468/mgyhok. Accessed via GBIF on 2020-10-13.

García, CB, and Duarte, LO, 'Columbian Caribbean Sea', in J.H. Nicholls (comp.) HMAP Data Pages https://figshare.com/articles/dataset/HMAP_Dataset_18_Colombian_Caribbean_Sea/23504046 Accessed via OBIS on 2020-11-04.   

Garcia HE, Weathers K, Paver CR, Smolyar I, Boyer TP, Locarnini RA, Zweng MM, Mishonov AV, Baranova OK, Seidov D, Reagan JR (2018). World Ocean Atlas 2018, Volume 3: Dissolved Oxygen, Apparent Oxygen Utilization, and Oxygen Saturation. A Mishonov Technical Ed.; NOAA Atlas NESDIS 83, 38pp.

Garnier S, Ross N, Rudis R, Camargo AP, Sciaini M, Scherer C (2021). Rvision - Colorblind-Friendly Color Maps for R. R package version 0.6.2.

GBIF.org (24 September 2020) GBIF Occurrence Download DOI: 10.15468/dl.efuutj.

Grant S, McMahan C (2020). Field Museum of Natural History (Zoology) Fish Collection. Version 13.12. Field Museum. DOI: 10.15468/alz7wu. Accessed via GBIF on 2020-10-13.

Harvard University M, Morris P J (2022). Museum of Comparative Zoology, Harvard University. Version 162.296. Museum of Comparative Zoology, Harvard University. DOI: 10.15468/p5rupv. Accessed via GBIF on 2020-10-13.

Hendrickson D A, Cohen A E, Casarez M J (2022). University of Texas, Biodiversity Center, Ichthyology Collection (TNHCi). Version 5.175. University of Texas at Austin, Biodiversity Collections. DOI: 10.15468/h8gxdr. Accessed via GBIF on 2020-10-13.

INVEMAR. SIBM en línea: Sistema de Información sobre Biodiversidad Marina. Santa Marta: Instituto de investigaciones Marinas y Costeras José Benito Vives de Andréis,. https://siam.invemar.org.co/ Accessed via OBIS on 2020-11-04. 

Locarnini RA, Mishonov AV, Baranova OK, Boyer TP, Zweng MM, Garcia HE, Reagan JR, Seidov D, Weathers K, Paver CR, Smolyar I (2018). World Ocean Atlas 2018, Volume 1: Temperature. A. Mishonov Technical Ed.; NOAA Atlas NESDIS 81, 52pp.

McLean, M.W. (2014). Straightforward Bibliography Management in R Using the RefManager Package. NA, NA. https://arxiv.org/abs/1403.2036.

McLean, M.W. (2017). RefManageR: Import and Manage BibTeX and BibLaTeX References in R. The Journal of Open Source Software.

Mertz W (2021). LACM Vertebrate Collection. Version 18.9. Natural History Museum of Los Angeles County. DOI: 10.15468/77rmwd. Accessed via GBIF on 2020-10-13.

Nakae M, Shinohara G (2018). Fish collection of National Museum of Nature and Science. National Museum of Nature and Science, Japan. Occurrence dataset DOI: 10.15468/w3dzv1 accessed via GBIF.org on yyyy-mm-dd. Accessed via OBIS on 2020-11-04.    

Natural History Museum (2021). Natural History Museum (London) Collection Specimens. DOI: 10.5519/0002965. Accessed via GBIF on 2020-10-13.

National Museum of Natural History, Smithsonian Institution NMNH Fishes Collection Database. National Museum of Natural History, Smithsonian Institution, 10th and Constitution Ave. N.W., Washington, DC 20560-0193, 2007. Accessed via OBIS on 2020-11-04.

Norén M, Shah M (2017). Fishbase. FishBase. DOI: 10.15468/wk3zk7. Accessed via GBIF on 2020-10-13.

Norton B, Hogue G (2021). NCSM Ichthyology Collection. Version 22.8. North Carolina State Museum of Natural Sciences. DOI: 10.36102/dwc.1. Accessed via GBIF on 2020-10-13.

Nychka D, Furrer R, Paige J, Sain S (2021). “fields: Tools for spatial data.” R package version 13.3, <URL:
https://github.com/dnychka/fieldsRPackage>.

Orrell T, Informatics Office (2021). NMNH Extant Specimen Records (USNM, US). Version 1.49. National Museum of Natural History, Smithsonian Institution. DOI: 10.15468/hnhrg3. Accessed via GBIF on 2020-10-13.

Owens H, Merow C, Maitner B, Kass J, Barve V, Guralnick R (2021). occCite: Querying and Managing Large Biodiversity Occurrence Datasets_. doi: 10.5281/zenodo.4726676 (URL: DOI: 10.5281/zenodo.4726676), R package version 0.4.9.9000, (<URL: https://CRAN.R-project.org/package=occCite>).

Pebesma, E.J., R.S. Bivand (2005). Classes and methods for spatial data in R. R News 5 (2),
  https://cran.r-project.org/doc/Rnews/.

Prestridge H (2021). Biodiversity Research and Teaching Collections - TCWC Vertebrates. Version 9.4. Texas A&M University Biodiversity Research and Teaching Collections. DOI: 10.15468/szomia. Accessed via GBIF on 2020-10-13.

Provoost P, Bosch S (2019). “robis: R Client to access data from the OBIS API.” Ocean Biogeographic Information System. Intergovernmental Oceanographic Commission of UNESCO. R package version 2.1.8, https://cran.r-project.org/package=robis.

Pugh W (2021). UAIC Ichthyological Collection. Version 3.3. University of Alabama Biodiversity and Systematics. DOI: 10.15468/a2laag. Accessed via GBIF on 2020-10-13.

R Core Team (2017). R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. URL https://www.R-project.org/.

Robins R (2021). UF FLMNH Ichthyology. Version 117.342. Florida Museum of Natural History. DOI: 10.15468/8mjsel. Accessed via GBIF on 2020-10-13.

Bivand RS, Pebesma E, Gomez-Rubio V (2013). Applied spatial data analysis with R, Second edition.
  Springer, NY. https://asdar-book.org/

Sarkar D (2008). Lattice: Multivariate Data Visualization with R. Springer, New York. ISBN
  978-0-387-75968-5

Tavera L, Tavera M. L (2021). Biodiversidad de los recursos marinos y costeros entre Cartagena y el Golfo de Urabá, Caribe colombiano. Version 2.4. Instituto de Investigaciones Marinas y Costeras - Invemar. DOI: 10.15472/d2jusp. Accessed via GBIF on 2020-10-13.

UNIBIO, IBUNAM. CNPE/Coleccion Nacional de Peces. DOI: 10.15468/o5d48z. Accessed via GBIF on 2020-10-13.

Wagner M (2017). MMNS Ichthyology Collection. Mississippi Museum of Natural Science. DOI: 10.15468/4c3qeq. Accessed via GBIF on 2020-10-13.

Werneke D (2019). AUM Fish Collection. Version 8.1. Auburn University Museum. DOI: 10.15468/dm3oyz. Accessed via GBIF on 2020-10-13.

Wickham H (2016). ggplot2: Elegant Graphics for Data Analysis. Springer-Verlag New York.

---
title: "5. 3D Niche Modeling with the GLM Algorithm"
author:
- Hannah L. Owens
- Carsten Rahbek
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
    fig_width: 7
    fig_height: 7
    toc: true
    toc_depth: 3
vignette: >
  %\VignetteIndexEntry{5. 3D Niche Modeling with the GLM Algorithm}
  \usepackage[utf8]{inputenc}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, error = FALSE, fig.retina = 1, dpi = 100)
```

# Introduction 
In this vignette, I will focus on demonstrating a basic workflow for generating and projecting a 3D ecological niche model using a generalized linear model, incorporating `voluModel` tools. 

Here are the packages you will need.

```{r packages, warning=FALSE}
library(voluModel) # Because of course
library(dplyr) # For occurrence data filtration
library(ggplot2) # For fancy plotting
library(terra) # Now being transitioned in
library(sf) # Now being transitioned in
library(viridisLite) # For high-contrast plotting palettes
```

# Data Inputs

First thing's first. Distributional models are based on coordinates of species' occurrences. Here is an example dataset of *Steindachneria argentea*, Luminous Hake, using data downloaded via R (R Core Team, 2020) from GBIF (Chamberlain *et al.*, 2021; Chamberlain and Boettiger, 2017) and OBIS (Provoost and Bosch, 2019) via `occCite` (Owens *et al.*, 2021). 

```{r occurrence data, eval=TRUE}
occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv", 
                             package='voluModel'))
summary(occs$depth)
boxplot.stats(occs$depth)
```

Based on this information and what we know about the biology of Luminous Hakes and how these occurrence data are collected, I am going to remove occurrences that have a depth of 0, as well as the extreme depth outlier, since these look like either collections of individuals that were not *in situ* or errors of some kind.

```{r clean occurrence data, eval=TRUE}
occsClean <- occs %>% 
  dplyr::select(decimalLongitude, decimalLatitude, depth) %>%
  dplyr::distinct() %>% 
  filter(dplyr::between(depth, 1, 2000))
head(occsClean)

ggplot(occsClean, aes(x = 1, y=depth)) +
  geom_violin(fill="yellow", ) +
  theme_classic(base_size = 15) +
  theme(axis.title = element_blank(),
        text = element_text(family = "Arial"),
        axis.text = element_text(size = rel(1.1))) +
  ggtitle("Depth") +
  geom_boxplot(width=0.1)
```

Next, we load two environmental datasets from the World Ocean Atlas (Garcia *et al.*, 2019): temperature (Locarnini *et al.*, 2018) and apparent oxygen utilization (Garcia *et al.*, 2019). I have chosen these variables for simplified illustrative purposes--we recommend you explore additional variables from the World Ocean Atlas and other sources. These data are supplied by the World Ocean Atlas as point shapefiles; the version supplied here has been cropped between -110 and -40 longitude and between -5 and 50 latitude to make it more memory-efficient. For more details on how to process environmental data, see [the raster data tutorial](https://hannahlowens.github.io/voluModel/articles/b_RasterProcessing.html). Temperature processing is shown below for illustrative purposes.

```{r environmental data loading, eval=T, asis=T, message=FALSE}
# Temperature
td <- tempdir()
unzip(system.file("extdata/woa18_decav_t00mn01_cropped.zip", 
                  package = "voluModel"),
      exdir = paste0(td, "/temperature"), junkpaths = T)
temperature <- vect(paste0(td, "/temperature/woa18_decav_t00mn01_cropped.shp"))

# Creating a SpatRaster vector
template <- centerPointRasterTemplate(temperature)
tempTerVal <- rasterize(x = temperature, y = template, field = names(temperature))

# Get names of depths
envtNames <- gsub("[d,M]", "", names(temperature))
envtNames[[1]] <- "0"
names(tempTerVal) <- envtNames
temperature <- tempTerVal
```

Pre-processed apparent oxygen utilization is loaded from an `.tif` file for speed.

```{r environmental data loading oxygen, eval=T, asis=T, warning=FALSE}
oxygenSmooth <- rast(system.file("extdata/oxygenSmooth.tif", 
                                 package='voluModel'))

# Change names to match temperature
names(oxygenSmooth) <- names(temperature)
```

# Sampling data for model generation

The first step is down-sampling the occurrence data so that there is only one occurrence per voxel (the 3D equivalent of a pixel) of environmental data. We do this to avoid over-fitting the model due to biased sampling. The resampled points are centered in each voxel. 

```{r downsample to voxel, eval=TRUE, warning=FALSE, message=FALSE}
# Gets the layer index for each occurrence by matching to depth
layerNames <- as.numeric(names(temperature))
occsClean$index <- unlist(lapply(occsClean$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(occsClean$index)

# Downsamples occsClean in each depth layer
downsampledOccs <- data.frame()
for(i in indices){
  tempPoints <- occsClean[occsClean$index==i,]
  tempPoints <- downsample(tempPoints, temperature[[1]], 
                           verbose = FALSE)
  tempPoints$depth <- rep(layerNames[[i]], times = nrow(tempPoints))
  downsampledOccs <- rbind(downsampledOccs, tempPoints)
}

occsClean <- downsampledOccs

print(paste0("Original number of points: ", nrow(occs), "; number of downsampled occs: ", nrow(occsClean)))
land <- rnaturalearth::ne_countries(scale = "small", returnclass = "sf")[1]
pointCompMap(occs1 = occs, occs2 = occsClean, 
             occs1Name = "Original", occs2Name = "Cleaned", 
             spName = "Steindachneria argentea", 
             land = land, verbose = FALSE)
```

Next, we generate a sampling region for the model based on occurrence points. Note that ideally when you're modeling you would carefully curate this background sampling region to make sure it truly approximates the area accessible to the species you are modeling. If you write the shapefile to disk, you can open and edit it in any GIS software you choose, if you have reason to believe `marineBackground()` falls short of estimating the true accessible are for a species. For more information, see [the environmental data processing tutorial](https://hannahlowens.github.io/voluModel/articles/c_DataSampling.html).

```{r environmental background sampling, warning=FALSE, eval = T}
backgroundSamplingRegions <- marineBackground(occsClean,
                                buff = 1000000,
                                clipToOcean = TRUE,
                                alpha = 1,
                                partCount = 1)
crs(backgroundSamplingRegions) <- crs(land)
plot(backgroundSamplingRegions, border = F, col = "gray",
     main = "Points and Background Sampling",
     axes = T)
plot(land, col = "black", add = T)
points(occsClean[,c("decimalLongitude", "decimalLatitude")], 
       pch = 20, col = "red", cex = 1.5)
```

Finally, we need to draw the environmental variable data that will be used to generate the ecological niche model, from presence voxels as well as from the background sampling region. First, we sample environmental data for occurrences and add a "response" column filled with 1s, signifying these are considered "presences".

```{r presence sampling}
# Presences
oxyVals <- xyzSample(occs = occsClean, oxygenSmooth)
tempVals <- xyzSample(occs = occsClean, temperature)
vals <- cbind(occsClean, oxyVals, tempVals)
colnames(vals) <- c("decimalLongitude", "decimalLatitude", "depth", "AOU", "Temperature")
vals <- vals[complete.cases(vals),]
row.names(vals) <- NULL
occsWdata <- vals

# Add response as a column
occsWdata$response <- rep(1, times = nrow(occsWdata))
```

Then, we generate background data by drawing occurrences from the XY extent of the background shapefile and from user-specified depths from 5 to 800m. Background points intersecting with occurrences are not returned. Then the environmental data at each background coordinate are drawn and stored in their own data object, and the "response" column is added and filled with 0s, signifying that these are to be considered "absences" when training the model.

```{r background sampling}
# Background
backgroundVals <- mSampling3D(occs = occsClean, 
                              envBrick = temperature, 
                              mShp = backgroundSamplingRegions, 
                              depthLimit = c(5, 800))
oxyVals <- xyzSample(occs = backgroundVals, oxygenSmooth)
tempVals <- xyzSample(occs = backgroundVals, temperature)
vals <- cbind(backgroundVals, oxyVals, tempVals)
colnames(vals) <- c("decimalLongitude", "decimalLatitude", "depth", "AOU", "Temperature")
vals <- vals[complete.cases(vals),]
row.names(vals) <- NULL
backgroundWdata <- vals

# Add response as a column
backgroundWdata$response <- rep(0, times = nrow(backgroundWdata))
```

# Generalized linear model

If you are concerned about overfitting the model because you generated a lot more background points than presence points, you can sample from the background data however you think is logical. The example below samples 100 times the number of occurrence points from the background, weighted by distance form the suitable centroid of occurrence points. That is, the more environmentally-different a background point is from an occurrence point, the more likely it is to be sampled. This is helpful for GLMs, because background points are interpreted as "absences" unlike methods like Maxent, where sampling the most dissimilar background may more likely lead to an over-fit model.

```{r glm data prep part 2}
# Sample background points weighted by distance from centroid of occurrence environments
suitableCentroid <- apply(occsWdata[,c("Temperature", "AOU")], 
                          MARGIN = 2, FUN = mean)
backgroundWdata$distance <- apply(backgroundWdata[,c("Temperature", "AOU")], MARGIN = 1, 
                                  FUN = function(x) dist(rbind(suitableCentroid, x)))
backgroundWdata$sampleWeight <- (backgroundWdata$distance - 
                                 min(backgroundWdata$distance))/(max(backgroundWdata$distance)-
                                                                 min(backgroundWdata$distance))
sampleForAbsence <- sample(x = rownames(backgroundWdata), 
                           size = nrow(occsWdata) * 100, 
                           prob = backgroundWdata$sampleWeight)
backgroundWdata <- backgroundWdata[match(sampleForAbsence, 
                                         rownames(backgroundWdata)),]
```

Then we unite the presence and absence data into a single data object.

```{r uniting datasets}
# Unite datasets
datForMod <- rbind(occsWdata, backgroundWdata[,colnames(occsWdata)])
rm(suitableCentroid, sampleForAbsence, backgroundWdata, occsWdata)
```

With the data cleaned, formatted and united into a single `data.frame`, generating a generalized linear model is very simple. How does it look?

```{r generate glm niche model}
glmModel <- glm(formula = response ~ Temperature *  AOU, 
                  family = binomial(link = "logit"),  data = datForMod)
summary(glmModel)
```

Now we project the model back into geographic space; at this stage, the niche model is being used to model the geographic distribution of putative suitable habit for Luminous Hake (an ENM becomes an SDM).

```{r project glm niche model}
layerNames <- as.numeric(names(temperature))
index <- seq(from = match(min(datForMod$depth), layerNames), 
             to = match(max(datForMod$depth), layerNames), by = 1)
depthPred <- NULL
for(j in index){
  depthPreds <- c(temperature[[j]], oxygenSmooth[[j]])
  crs(depthPreds) <- "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs"
  names(depthPreds) <- c("Temperature", "AOU")
  depthPred[[j]] <- mask(predict(depthPreds, glmModel), backgroundSamplingRegions)
  depthPred[[j]] <- crop(depthPred[[j]], backgroundSamplingRegions)
  names(depthPred[[j]]) <- layerNames[[j]]
}
glmPred <- rast(depthPred[!unlist(lapply(depthPred, FUN = function(x) is.null(x)))])
```

And here's what the SDM looks like threshholded to presences and absences; raw ENM suitability values above the threshold value become presences, values below the threshold become absences. This is useful in a variety of applications, including, but not limited to visualizing the model. In this case, the threshold is the bottom tenth percentile of suitability scores at occurrences. What this means is that we are interpreting about 10% of the occurrence points to be erroneous. It is important to remember that there are MANY different thresholds you may choose based on your dataset and the goals of your analysis. Try to choose wisely.

```{r glm niche threshold}
glmThreshold <- quantile(xyzSample(datForMod[datForMod$response == 1,
                                             c("depth", "decimalLongitude", "decimalLatitude")], 
                                   glmPred), .1, na.rm = T)[[1]] # MS90
glmThresholded <- glmPred > glmThreshold
rclMatrix <- matrix(c(NA, NA, 0), ncol = 3, byrow = TRUE)
glmThresholded <- classify(glmThresholded, rclMatrix, include.lowest = T)
```

`plotLayers()` plots a transparent layer of suitable habitat for each depth layer. The redder the color, the shallower the layer, the bluer, the deeper. The more saturated the color, the more layers with suitable habitat. Here, I am plotting suitability from 0m to 800m, the depth range of occurrences used to train the envelope model.

```{r thresholded glm niche model plotted}
plotLayers(glmThresholded, 
          land = land, landCol = "black",
          title = "Areas of suitable habitat for \n Luminous Hake, 5m to 800m")
```

Of course, this just a starting example. We welcome submissions from the community for examples of other types of models you would like to see. Submit your suggestions as issues [here](https://github.com/hannahlowens/voluModel/issues).

# Cropping out extreme extrapolation

Sometimes ecological models fail to extrapolate realistically when they are projected into environments that are not well-represented by the set of conditions used to calibrate the model (Owens *et al*, 2013). One convenient way to estimate extrapolation is via the Multivariate Environmental Suitability Surface (MESS; Elith *et al*. 2010). `voluModel` has a function, `MESS3D()`, which calculates MESS for each depth layer in a projection dataset (supplied to the function as a named list of `RasterBrick` objects used for the model projection).

```{r calculate MESS, warning=FALSE}
# Prepare environmental data
layerNames <- as.numeric(names(temperature))
datForMod$index <- unlist(lapply(datForMod$depth, FUN = function(x) which.min(abs(layerNames - x))))
indices <- unique(datForMod$index)

projList <- list("AOU" = oxygenSmooth[[min(indices):max(indices)]], 
                 "Temperature" = temperature[[min(indices):max(indices)]])

# Explicitly formatting calibration data
calibration <- data.frame(
  AOU = datForMod$AOU,
  Temperature = datForMod$Temperature)

# Calculate MESS
messBrick <- MESS3D(calibration = calibration, projection = projList)
```

Now I will calculate two `SpatRaster` objects. In the first, `extrapolation`, MESS values zero and below (i.e. signalling extrapolation) will be reclassified as 1, which will allow me to easily map where extrapolation is occurring.

```{r reclassify and plot extrapolation}
# Reclassify MESS 
extrapolation <- 0 >= messBrick

# Plot Extrapolation
plotLayers(extrapolation, land = land, landCol = "black", 
           title = "Areas of extrapolation according to MESS,\n 5m to 800m")
```

In the second `RasterBrick`, `noExtrapolation`, I reclassify the layers so that positive MESS values = 1 and MESS values less than or equal to 0 = 0. This will make plotting cropping out areas of extrapolation from the niche model projection easier. I simply multiply the thresholded GLM map by the `noExtrapolation` `SpatRaster`.

```{r reclassify MESS and plot GLM with no extrapolation, warning=FALSE}
# Reclassify MESS 
noExtrapolation <- messBrick > 0

glmThreshNoExtrapolation <- NULL
for(i in 1:nlyr(noExtrapolation)){
  extrapCut <- crop(noExtrapolation[[i]], y = glmThresholded[[i]])
  croppedLayer <- glmThresholded[[i]] * extrapCut
  glmThreshNoExtrapolation[[i]] <- croppedLayer
}

glmThreshNoExtrapolation <- rast(glmThreshNoExtrapolation)

# Plot MESS
plotLayers(glmThreshNoExtrapolation, land = land, 
           landCol = "black", 
           title = "Areas of suitable habitat with no model extrapolation,\n 5m to 800m")
```

# Tidying up
Last, we need to close the temporary directory we opened when we opened the data.

```{r cleanup temporary directory}
unlink(td, recursive = T)
```

# References

Bakis Y (2021): TU_Fish. v1.1. No organization. Dataset/Occurrence. https://web.archive.org/web/20221202141213/https://bgnn.tulane.edu/ipt/resource?r=tu_fish&v=1.1 Accessed via OBIS on 2020-11-04.  

Bentley A (2022). KUBI Ichthyology Collection. Version 17.80. University of Kansas Biodiversity Institute. DOI: 10.15468/mgjasg. Accessed via GBIF on 2020-10-13.

Bentley A (2022). KUBI Ichthyology Tissue Collection. Version 18.68. University of Kansas Biodiversity Institute. DOI: 10.15468/jmsnwg. Accessed via GBIF on 2020-10-13.

Buckup P A (2022). Coleção Ictiológica (MNRJ), Museu Nacional (MN), Universidade Federal do Rio de Janeiro(UFRJ). Version 157.1487. Museu Nacional / UFRJ. DOI: 10.15468/lluzfl. Accessed via GBIF on 2020-10-13.

Catania D, Fong J (2022). CAS Ichthyology (ICH). Version 150.300. California Academy of Sciences. DOI: 10.15468/efh2ib. Accessed via GBIF on 2020-10-13.

Chakrabarty P (2019). LSUMZ (LSU MNS) Fishes Collection. Version 2.2. Louisiana State University Museum of Natural Science. DOI: 10.15468/gbnym3. Accessed via GBIF on 2020-10-13.

Chamberlain S, Barve V, Mcglinn D, Oldoni D, Desmet P, Geffert L, Ram K (2021). rgbif: Interface to the Global Biodiversity Information Facility API. R package version 3.6.0, https://CRAN.R-project.org/package=rgbif.

Chamberlain S, Boettiger C (2017). “R Python, and Ruby clients for GBIF species occurrence data.” *PeerJ* PrePrints. DOI: 10.7287/peerj.preprints.3304v1.

Davis Rabosky AR, Cox CL, Rabosky DL, Title PO, Holmes IA, Feldman A, McGuire JA (2016). Coral snakes predict the evolution of mimicry across New World snakes. *Nature Communications* 7:11484.

Elith J, Kearney M, Phillips S. 2010. The art of modelling range-shifting species. *Methods in Ecology and Evolution*, 1, 330-342.

Espinosa Pérez H, Comisión nacional para el conocimiento y uso de la biodiversidad C (2021). Computarización de la Colección Nacional de Peces del Instituto de Biología UNAM. Version 1.9. Comisión nacional para el conocimiento y uso de la biodiversidad. DOI: 10.15468/zb2odl. Accessed via GBIF on 2020-10-13.

Frable B (2019). SIO Marine Vertebrate Collection. Version 1.7. Scripps Institution of Oceanography. DOI: 10.15468/ad1ovc. Accessed via GBIF on 2020-10-13.

Fishnet2 Portal, http://www.fishnet2.net, [Access date] Accessed via OBIS on 2020-11-04.

Froese, R. and D. Pauly. Editors. 200x. FishBase. World Wide Web electronic publication. www.fishbase.org, version (xx/200x). Accessed via OBIS on 2020-11-04.   

Gall L (2021). Vertebrate Zoology Division - Ichthyology, Yale Peabody Museum. Yale University Peabody Museum. DOI: 10.15468/mgyhok. Accessed via GBIF on 2020-10-13.

García, CB, and Duarte, LO, 'Columbian Caribbean Sea', in J.H. Nicholls (comp.) HMAP Data Pages https://figshare.com/articles/dataset/HMAP_Dataset_18_Colombian_Caribbean_Sea/23504046 Accessed via OBIS on 2020-11-04.   

Garcia HE, Weathers K, Paver CR, Smolyar I, Boyer TP, Locarnini RA, Zweng MM, Mishonov AV, Baranova OK, Seidov D, Reagan JR (2018). World Ocean Atlas 2018, Volume 3: Dissolved Oxygen, Apparent Oxygen Utilization, and Oxygen Saturation. A Mishonov Technical Ed.; NOAA Atlas NESDIS 83, 38pp.

GBIF.org (24 September 2020) GBIF Occurrence Download DOI: 10.15468/dl.efuutj.

Grant S, McMahan C (2020). Field Museum of Natural History (Zoology) Fish Collection. Version 13.12. Field Museum. DOI: 10.15468/alz7wu. Accessed via GBIF on 2020-10-13.

Harvard University M, Morris P J (2022). Museum of Comparative Zoology, Harvard University. Version 162.296. Museum of Comparative Zoology, Harvard University. DOI: 10.15468/p5rupv. Accessed via GBIF on 2020-10-13.

Hendrickson D A, Cohen A E, Casarez M J (2022). University of Texas, Biodiversity Center, Ichthyology Collection (TNHCi). Version 5.175. University of Texas at Austin, Biodiversity Collections. DOI: 10.15468/h8gxdr. Accessed via GBIF on 2020-10-13.

INVEMAR. SIBM en línea: Sistema de Información sobre Biodiversidad Marina. Santa Marta: Instituto de investigaciones Marinas y Costeras José Benito Vives de Andréis,. https://siam.invemar.org.co/ Accessed via OBIS on 2020-11-04. 

Locarnini RA, Mishonov AV, Baranova OK, Boyer TP, Zweng MM, Garcia HE, Reagan JR, Seidov D, Weathers K, Paver CR, Smolyar I (2018). World Ocean Atlas 2018, Volume 1: Temperature. A. Mishonov Technical Ed.; NOAA Atlas NESDIS 81, 52pp.

McLean, M.W. (2014). Straightforward Bibliography Management in R Using the RefManager Package. NA, NA. https://arxiv.org/abs/1403.2036.

McLean, M.W. (2017). RefManageR: Import and Manage BibTeX and BibLaTeX References in R. *The Journal of Open Source Software*.

Mertz W (2021). LACM Vertebrate Collection. Version 18.9. Natural History Museum of Los Angeles County. DOI: 10.15468/77rmwd. Accessed via GBIF on 2020-10-13.

Nakae M, Shinohara G (2018). Fish collection of National Museum of Nature and Science. National Museum of Nature and Science, Japan. Occurrence dataset DOI: 10.15468/w3dzv1 accessed via GBIF.org on yyyy-mm-dd. Accessed via OBIS on 2020-11-04.    

Natural History Museum (2021). Natural History Museum (London) Collection Specimens. DOI: 10.5519/0002965. Accessed via GBIF on 2020-10-13.

National Museum of Natural History, Smithsonian Institution NMNH Fishes Collection Database. National Museum of Natural History, Smithsonian Institution, 10th and Constitution Ave. N.W., Washington, DC 20560-0193, 2007. Accessed via OBIS on 2020-11-04.

Norén M, Shah M (2017). Fishbase. FishBase. DOI: 10.15468/wk3zk7. Accessed via GBIF on 2020-10-13.

Norton B, Hogue G (2021). NCSM Ichthyology Collection. Version 22.8. North Carolina State Museum of Natural Sciences. DOI: 10.36102/dwc.1. Accessed via GBIF on 2020-10-13.

Nychka D, Furrer R, Paige J, Sain S (2021). “fields: Tools for spatial data.” R package version 13.3, <URL: https://github.com/dnychka/fieldsRPackage>.

Orrell T, Informatics Office (2021). NMNH Extant Specimen Records (USNM, US). Version 1.49. National Museum of Natural History, Smithsonian Institution. DOI: 10.15468/hnhrg3. Accessed via GBIF on 2020-10-13.

Owens H, Campbell L, Dornak L, Saupe E, Barve N, Soberón J, Ingenloff K, Lira-Noriega A, Hensz C, Myers, C, Peterson AT (2013). Constraints on interpretation of ecological niche models by limited environmental ranges on calibration areas. *Ecological Modelling* 263: 10-18.

Owens H, Merow C, Maitner B, Kass J, Barve V, Guralnick R (2021). occCite: Querying and Managing Large Biodiversity Occurrence Datasets_. doi: 10.5281/zenodo.4726676 (URL: DOI: 10.5281/zenodo.4726676), R package version 0.4.9.9000, (<URL: https://CRAN.R-project.org/package=occCite>).

Prestridge H (2021). Biodiversity Research and Teaching Collections - TCWC Vertebrates. Version 9.4. Texas A&M University Biodiversity Research and Teaching Collections. DOI: 10.15468/szomia. Accessed via GBIF on 2020-10-13.

Provoost P, Bosch S (2019). “robis: R Client to access data from the OBIS API.” Ocean Biogeographic Information System. Intergovernmental Oceanographic Commission of UNESCO. R package version 2.1.8, https://cran.r-project.org/package=robis.

Pugh W (2021). UAIC Ichthyological Collection. Version 3.3. University of Alabama Biodiversity and Systematics. DOI: 10.15468/a2laag. Accessed via GBIF on 2020-10-13.

R Core Team (2017). R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. URL https://www.R-project.org/.

Robins R (2021). UF FLMNH Ichthyology. Version 117.342. Florida Museum of Natural History. DOI: 10.15468/8mjsel. Accessed via GBIF on 2020-10-13.

Tavera L, Tavera M. L (2021). Biodiversidad de los recursos marinos y costeros entre Cartagena y el Golfo de Urabá, Caribe colombiano. Version 2.4. Instituto de Investigaciones Marinas y Costeras - Invemar. DOI: 10.15472/d2jusp. Accessed via GBIF on 2020-10-13.

UNIBIO, IBUNAM. CNPE/Coleccion Nacional de Peces. DOI: 10.15468/o5d48z. Accessed via GBIF on 2020-10-13.

Wagner M (2017). MMNS Ichthyology Collection. Mississippi Museum of Natural Science. DOI: 10.15468/4c3qeq. Accessed via GBIF on 2020-10-13.

Werneke D (2019). AUM Fish Collection. Version 8.1. Auburn University Museum. DOI: 10.15468/dm3oyz. Accessed via GBIF on 2020-10-13.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interpolateRaster.R
\name{interpolateRaster}
\alias{interpolateRaster}
\title{Interpolate patchily sampled rasters}
\usage{
interpolateRaster(inputRaster, fast = FALSE, ...)
}
\arguments{
\item{inputRaster}{An object of class \code{SpatRaster}}

\item{fast}{A logical operator. Setting to \code{TRUE} triggers use
of \code{fastTps} instead of \code{Tps}.}

\item{...}{For any additional arguments passed to \code{Tps} or \code{fastTps}}
}
\value{
An object of class raster
}
\description{
Uses thin plate spline regression from
\code{fields} package to interpolate missing two-dimensional
raster values.
}
\details{
Missing data values from original raster
are replaced with interpolated values. User has the
option of choosing \code{fastTps} to speed calculation,
but be advised that this is only an approximation
of a true thin plate spline.
}
\examples{
\donttest{
library(terra)
library(fields)
# Create sample raster
r <- rast(ncol=50, nrow=50)
values(r) <- 1:2500

# Introduce a "hole"
values(r)[c(117:127, 167:177, 500:550)] <- NA
plot(r)

# Patch hole with interpolateRaster
interpolatedRaster <- interpolateRaster(r)
plot(interpolatedRaster)
fastInterp <- interpolateRaster(r, fast = TRUE, aRange = 3.0)
plot(fastInterp)
}

}
\seealso{
\code{\link[fields]{Tps}}, \code{\link[fields]{fastTps}}
}
\keyword{dataPrep}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{diversityStack}
\alias{diversityStack}
\title{Diversity stack}
\usage{
diversityStack(rasterList, template)
}
\arguments{
\item{rasterList}{A \code{list} of \code{SpatRaster} objects, which
are interpreted as species distributions (1 = presence,
0 = absence).}

\item{template}{A \code{SpatRaster} with the desired extent}
}
\value{
A \code{SpatRaster}
}
\description{
Takes list of rasters of species distributions
(interpreted as 1 = presence, 0 = absence), which do not
have to have the same extents, and stack them to create an
estimate of species richness that matches the extent and
resolution of a template.
}
\examples{
library(terra)
rast1 <- rast(ncol=10, nrow=10)
values(rast1) <- rep(0:1, 50)

rast2 <- rast(ncol=10, nrow=10)
values(rast2) <- c(rep(0, 50), rep(1,50))

rastList <- list(rast1, rast2)
result <- diversityStack(rasterList = rastList,
                         template = rast2)
result
plot(result)

}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BackgroundSamplingFunctions.R
\name{mSampling2D}
\alias{mSampling2D}
\title{2D background sampling}
\usage{
mSampling2D(occs, rasterTemplate, mShp, verbose = TRUE)
}
\arguments{
\item{occs}{A dataframe with at least two columns
named "longitude" and "latitude", or that can be
coerced into this format.}

\item{rasterTemplate}{A \code{SpatRaster} object to serve
as a template for generating background sampling
coordinates.}

\item{mShp}{A shapefile defining the area from
which background points should be sampled.}

\item{verbose}{\code{logical}. Switching to \code{FALSE} mutes message describing
which columns in \code{occs} are interpreted as x and y coordinates.}
}
\value{
A \code{data.frame} with 2D coordinates of points
for background sampling.
}
\description{
Samples in 2D at resolution of raster
}
\details{
This function is designed to sample background points
for distributional modeling in two dimensions. The returned
\code{data.frame} contains all points from across the designated
background. It is up to the user to determine how to
appropriately sample from those background points.
}
\examples{
library(terra)

# Create sample raster
r <- rast(ncol=10, nrow=10)
values(r) <- 1:100

# Create test occurrences
set.seed(0)
longitude <- sample(ext(r)[1]:ext(r)[2],
                    size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(r)[3]:ext(r)[4],
                   size = 10, replace = FALSE)
occurrences <- data.frame(longitude,latitude)

# Generate background sampling buffer
buffPts <- vect(occurrences,
                c("longitude", "latitude"))
crs(buffPts) <- crs(r)
mShp <- aggregate(buffer(buffPts, width = 1000000))

# Here's the function
result <- mSampling2D(occs = occurrences, rasterTemplate = r, mShp = mShp)

}
\keyword{backgroundSampling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interpolateRaster.R
\name{smoothRaster}
\alias{smoothRaster}
\title{Smooth rasters}
\usage{
smoothRaster(inputRaster, fast = FALSE, ...)
}
\arguments{
\item{inputRaster}{An object of class \code{SpatRaster}}

\item{fast}{A logical operator. Setting to \code{TRUE} triggers use
of \code{fastTps} instead of \code{Tps}.}

\item{...}{For any additional arguments passed to \code{Tps} or \code{fastTps}}
}
\value{
An object of class \code{SpatRaster}
}
\description{
Uses thin plate spline regression from
\code{fields} package to smooth raster values.
}
\details{
Original raster is smoothed using a thin
plate spline. This may be desirable in cases where
the user has a reasonable expectation of spatial autocorrelation,
but observes putative measurement errors in a raster. The user has
the option of choosing \code{fastTps} to speed calculation,
but be advised that this is only an approximation
of a true thin plate spline.
}
\examples{
library(terra)
library(fields)
# Create sample raster
r <- rast(ncol=100, nrow=100)
values(r) <- 1:10000

# Introduce a "bubble"
values(r)[720:725] <- 9999
plot(r)

# Smooth bubble with smoothRaster
fastSmooth <- smoothRaster(r, fast = TRUE, aRange = 10.0)
plot(fastSmooth)

}
\seealso{
\code{\link[fields]{Tps}}, \code{\link[fields]{fastTps}}
}
\keyword{dataPrep}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputProcessingUtilties.R
\name{bottomRaster}
\alias{bottomRaster}
\title{Bottom raster generation}
\usage{
bottomRaster(rawPointData)
}
\arguments{
\item{rawPointData}{A \code{SpatVector} object from which
bottom variables will be sampled. See Details for more about format.}
}
\value{
A \code{SpatRaster} designed to approximate sea bottom
measurements for modeling species' distributions and/or niches.
}
\description{
Samples deepest depth values from a
\code{SpatVector} data frame and generates a \code{SpatRaster}.
}
\details{
\code{rawPointData} is a \code{SpatVector} object that
contains measurements of a single environmental variable (e.g.
salinity, temperature, etc.) with x, y, and z coordinates. The
measurements in the \code{data.frame} should be organized so that each
column is a depth slice, increasing in depth from left to right. The
function was designed around the oceanographic data shapefiles supplied
by the World Ocean Atlas
(\url{https://www.ncei.noaa.gov/access/world-ocean-atlas-2018/}).
The function selects the "deepest" (rightmost) measurement at each
x, y coordinate pair that contains data. These measurements are then
rasterized at the resolution and extent of the x,y coordinates, under
the assumption that x and y intervals are equal and represent the center
of a cell.
}
\examples{

library(terra)

# Create point grid
coords <- data.frame(x = rep(seq(1:5), times = 5),
                    y = unlist(lapply(1:5, FUN = function(x) {
                      rep(x, times = 5)})))

# Create data and add NAs to simulate uneven bottom depths
dd <- data.frame(SURFACE = 1:25,
                d5M = 6:30,
                d10M = 11:35,
                d25M = 16:40)
dd$d25M[c(1:5, 18:25)] <- NA
dd$d10M[c(3:5, 21:23)] <- NA
dd$d5M[c(4, 22)] <- NA

dd[,c("x","y")] <- coords

# Create SpatialPointsDataFrame
sp <- vect(dd, geom = c("x", "y"))

# Here's the function
result <- bottomRaster(rawPointData = sp)
plot(result)

}
\keyword{inputProcessing}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modelDiagnostics.R
\name{MESS3D}
\alias{MESS3D}
\title{Calculate MESS}
\usage{
MESS3D(calibration, projection)
}
\arguments{
\item{calibration}{A \code{data.frame} of environmental variables
used to calibrate an ecological niche model, one row for
measurements from each voxel included in the data used to
calibrate the model. Columns with names not corresponding to
\code{projection} \code{list} items are ignored.}

\item{projection}{A named \code{list} of \code{SpatRaster} objects for
projection; names correspond to \code{calibration} column names.
Each \code{SpatRaster} should have the same number of layers,
corresponding to vertical depth slices.}
}
\value{
A \code{SpatRaster} vector with MESS scores for each
voxel; layer names correspond to layer names of first
\code{SpatRaster} vector in \code{projection} \code{list}.
}
\description{
Calculates multivariate environmental similarity
surface based on model calibration and projection data
}
\details{
\code{MESS3D} is a wrapper around \code{MESS} from the \code{modEvA}
package. It calculates MESS for each depth layer. Negative values
indicate areas of extrapolation which should be interpreted with
caution (see Elith \emph{et al}, 2010 in \emph{MEE}).
}
\note{
The calibration dataset should include both presences
and background/pseudoabsence points used to calibrate an
ecological niche model.
}
\examples{
library(terra)
library(dplyr)

# Create sample rasterBricks
r1 <- rast(ncol=10, nrow=10)
values(r1) <- 1:100
r2 <- rast(ncol=10, nrow=10)
values(r2) <- c(rep(20, times = 50), rep(60, times = 50))
r3 <- rast(ncol=10, nrow=10)
values(r3) <- 8
envBrick1 <- c(r1, r2, r3)
names(envBrick1) <- c(0, 10, 30)

r1 <- rast(ncol=10, nrow=10)
values(r1) <- 100:1
r2 <- rast(ncol=10, nrow=10)
values(r2) <- c(rep(10, times = 50), rep(20, times = 50))
r3 <- rast(ncol=10, nrow=10)
values(r3) <- c(rep(c(10,20,30,25), times = 25))
envBrick2 <- c(r1, r2, r3)
names(envBrick2) <- c(0, 10, 30)

rastList <- list("temperature" = envBrick1, "salinity" = envBrick2)

# Create test reference set
set.seed(0)
longitude <- sample(ext(envBrick1)[1]:ext(envBrick1)[2],
                    size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(envBrick1)[3]:ext(envBrick1)[4],
                   size = 10, replace = FALSE)
set.seed(0)
depth <- sample(0:35, size = 10, replace = TRUE)
occurrences <- as.data.frame(cbind(longitude,latitude,depth))

# Sample data at occurrences to characterize calibration region
cal_temp <- xyzSample(occurrences, rastList$temperature)
cal_sal  <- xyzSample(occurrences, rastList$salinity)

calibration <- data.frame(
  temperature = cal_temp,
  salinity    = cal_sal)

# Run the function
messStack <- MESS3D(calibration = calibration, projection = rastList)
plot(messStack)

}
\references{
Elith J, Kearney M, and Phillips S. 2010.
The art of modelling range-shifting species.
\emph{Methods in Ecology and Evolution}, 1, 330-342.
}
\seealso{
\code{\link[predicts]{mess}}
}
\keyword{modelDiagnostics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BackgroundSamplingFunctions.R
\name{mSampling3D}
\alias{mSampling3D}
\title{3D background sampling}
\usage{
mSampling3D(occs, envBrick, mShp, depthLimit = "all", verbose = TRUE)
}
\arguments{
\item{occs}{A \code{data.frame} with at least three columns
named "longitude", "latitude", and "depth", or that
can be coerced into this format.}

\item{envBrick}{A \code{SpatRaster} vector object to serve
as a template for generating background sampling
coordinates.}

\item{mShp}{A shapefile defining the area from
which background points should be sampled.}

\item{depthLimit}{An argument controlling the depth
extent of sampling. Refer to \code{Details} for more information.}

\item{verbose}{\code{logical}. Switching to \code{FALSE} mutes message describing
which columns in \code{occs} are interpreted as x, y, and z coordinates.}
}
\value{
A \code{data.frame} with 3D coordinates of points for background
sampling.
}
\description{
Samples XYZ coordinates from a shapefile
from maximum to minimum occurrence depth at XYZ
resolution of envBrick.
}
\details{
This function is designed to sample background points for
distributional modeling in three dimensions. If a voxel (3D pixel)
in the \code{SpatRaster} vector intersects with an occurrence from
\code{occs}, it is removed. Note that this function returns points
representing every voxel in the background area within the
specified depth range. It is up to the user to downsample from
these data as necessary, depending on the model type being used.

\code{depthLimit} argument options:
\itemize{
\item \code{occs} Samples background from the full depth extent of \code{occs}.
\item \code{all} Samples background from the full depth extent of \code{envBrick}.
\item A \code{vector} of length 2 with maximum and minimum depth values from
which to sample.
}
}
\examples{
library(terra)

# Create test raster
r1 <- rast(ncol=10, nrow=10)
values(r1) <- 1:100
r2 <- rast(ncol=10, nrow=10)
values(r2) <- c(rep(20, times = 50), rep(60, times = 50))
r3 <- rast(ncol=10, nrow=10)
values(r3) <- 8
envBrick <- c(r1, r2, r3)
names(envBrick) <- c(0, 10, 30)

# Create test occurrences
set.seed(0)
longitude <- sample(ext(envBrick)[1]:ext(envBrick)[2],
                    size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(envBrick)[3]:ext(envBrick)[4],
                   size = 10, replace = FALSE)
set.seed(0)
depth <- sample(0:35, size = 10, replace = TRUE)
occurrences <- data.frame(longitude,latitude,depth)

# Generate background sampling buffer
buffPts <- vect(occurrences,
                c("longitude", "latitude"))
crs(buffPts) <- crs(envBrick)
mShp <- aggregate(buffer(buffPts, width = 1000000))

# Here's the function
occSample3d <- mSampling3D(occs = occurrences,
                           envBrick = envBrick,
                           mShp = mShp,
                           depthLimit = "occs")

}
\keyword{backgroundSampling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/env_stack_transform.R
\name{env_stack_transform}
\alias{env_stack_transform}
\title{Coverts a list of 'SpatRaster' stacks where the elements are environmental
variables and the layers are depths to a list of SpatRaster stacks where the elements
are depths and the layers are environmental variables}
\usage{
env_stack_transform(envs_all, envs_names)
}
\arguments{
\item{envs_all}{a list of 'spatRaster' stacks where each element is a different
environmental variable and each layer is a depth slice.}

\item{envs_names}{A 'character' vector of the names of the environmental variables
to be applied to the layers of the resulting list of 'spatRaster' stacks}
}
\value{
A list of 'spatRaster' stacks where each list element is a depth slice and
each layer is an environmental variable.
}
\description{
A user will likely already have environmental data organized into a list
of 'spatRaster' stacks where each element is a different environmental variable and each
layer corresponds to a depth slice for background sampling and extracting. However, the
maxent_3D function requires the data be input as a list of 'spatRaster' stacks where
each element corresponds to a depth slice and each layer is an environmental variable
for the purpose of projecting the model back into geographic space, and this function
has been developed for ease of conversion.
}
\details{
The extents of each environmental variable layer should match on a depth slice
by depth slice basis.
}
\examples{
library(terra)

# creating a list of spatRaster stacks where each element is an environmental variable
# and each layer is a depth

env1_d1 <- rast(ncol = 50, nrow = 50)
values(env1_d1) <- sample(c(1:100), size = 2500, replace = TRUE)
env2_d1 <- rast(ncol = 50, nrow = 50)
values(env2_d1) <- sample(c(1:100), size = 2500, replace = TRUE)
env1_d2 <- rast(ncol = 50, nrow = 50)
values(env1_d2) <- sample(c(1:100), size = 2500, replace = TRUE)
env2_d2 <- rast(ncol = 50, nrow = 50)
values(env2_d2) <- sample(c(1:100), size = 2500, replace = TRUE)

env1 <- c(env1_d1, env1_d2)
env2 <- c(env2_d1, env2_d2)
envs <- list(env1, env2)
envnames <- c("env1", "env2")

# Here's the function
result <- env_stack_transform(envs_all = envs, envs_names = envnames)

}
\keyword{SpatRaster}
\keyword{list}
\keyword{stack}
\keyword{transform}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cleanDepth.R
\name{cleanDepth}
\alias{cleanDepth}
\title{Clean and flag coordinates based on intersection with a land polygon,
intersection with a bathymetry layer, and/or by a given depth range}
\usage{
cleanDepth(
  occs,
  bathy,
  land_poly,
  depth_range,
  flag = FALSE,
  bottom_correct = FALSE
)
}
\arguments{
\item{occs}{dataframe containing columns named "longitude", "latitude", and "depth," where
depth values are positive}

\item{bathy}{a spatRaster of bathymetry in the same units as the occurrence depth, where
values are negative}

\item{land_poly}{optional polygon of continents in same projection as occurrence data}

\item{depth_range}{optional range of depth values in the form of c(upper, lower) in which
all points should fall between. depth value should be positive, as in the case of the occs}

\item{flag}{default is F, if T, points are not removed, but flagged if they intersect, and a
column is added to the output indicating what points have been flagged}

\item{bottom_correct}{default is F, but if T, points are not removed, and a column is added
to the output displaying the bathymetry value at the flagged point for corrective purposes}
}
\value{
an object of class dataframe containing cleaned or flagged occurrences
}
\description{
function to remove or flag coordinates on land, coordinates which are deeper
than the known bathymetry at a given xy location, and/or coordinates outside of
a specified depth range when cleaning coordinates for species in marine environments
}
\details{
assumes depths are listed as positive in the occs and depth_range, while the
bathymetry layer lists them as negative when below the sea surface
}
\examples{
library(terra)
library(dplyr)

# Create sample bathymetry
r1 <- rast(ncol=10, nrow=10)
values(r1) <- c(-100:0)

# Create test occurrences
set.seed(0)
longitude <- sample(ext(r1)[1]:ext(r1)[2], size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(r1)[3]:ext(r1)[4], size = 10, replace = FALSE)
set.seed(0)
depth <- sample(1:100, size = 10, replace = TRUE)
occs <- data.frame(longitude, latitude, depth)

#Heres the function
result <- cleanDepth(bathy = r1, occs = occs)

}
\keyword{cleaning}
\keyword{occurrence}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{testIntersection}
\alias{testIntersection}
\title{Test Intersection}
\usage{
testIntersection(a, b)
}
\arguments{
\item{a}{The first \code{SpatRaster} object}

\item{b}{The second \code{SpatRaster} object}
}
\value{
A logical vector stating whether the two
inputs overlap
}
\description{
Tests whether two rasters overlap. Used in
\verb{\code{\link[voluModel:diversityStack]{diversityStack}}}
function to verify all rasters in list overlap with the
template raster.
}
\examples{

library(terra)
rast1 <- rast(ncol=10, nrow=10)
values(rast1) <- rep(0:1, 50)

rast2 <- rast(ncol=10, nrow=10)
values(rast2) <- c(rep(0, 50), rep(1,50))

testIntersection(rast1, rast2)

rast1 <- crop(rast1, ext(10, 20, 30, 40))
rast2 <- crop(rast2, ext(-20, -10, -40, -30))

testIntersection(rast1, rast2)

}
\keyword{internal}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{pointMap}
\alias{pointMap}
\title{Point mapping}
\usage{
pointMap(
  occs,
  spName,
  land = NA,
  ptCol = "#bd0026",
  landCol = "gray",
  waterCol = "steelblue",
  ptSize = 1,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{occs}{A \code{data.frame} with at least two columns
named "longitude" and "latitude" or that
can be coerced into this format.}

\item{spName}{A character string with the species
name to be used in the plot title.}

\item{land}{An optional coastline polygon shapefile
of types \code{sf} or \code{SpatRaster} to provide geographic
context for the occurrence points.}

\item{ptCol}{Color for occurrence points on map}

\item{landCol}{Color for land on map}

\item{waterCol}{Color for water on map}

\item{ptSize}{\code{numeric} value for \code{cex};
size of occurrence points on map.}

\item{verbose}{\code{logical}. Switching to \code{FALSE} mutes message describing
which columns in \code{occs} are interpreted as x and y coordinates.}

\item{...}{Additional optional arguments to pass to
\code{ggplot} initial plot object.}
}
\value{
A \code{ggplot} plot object.
}
\description{
A convenient wrapper around ggplot
to generate formatted occurrence point plots.
}
\examples{
occs <- read.csv(system.file("extdata/Steindachneria_argentea.csv",
                             package='voluModel'))
spName <- "Steindachneria argentea"
pointMap(occs = occs, spName = spName,
         land = rnaturalearth::ne_countries(scale = "small",
                                            returnclass = "sf")[1])

}
\seealso{
\code{\link[ggplot2:ggplot]{ggplot}}
}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{areColors}
\alias{areColors}
\title{Are Colors}
\usage{
areColors(col)
}
\arguments{
\item{col}{A vector of anything to be interpreted by \code{rgb}
as a color.}
}
\value{
A logical vector stating whether inputs
can be interpreted as colors.
}
\description{
Checks to see if a given vector can be
interpreted by R as a color or colors
}
\examples{

areColors(col = c("red", "prairie_chicken", 2))

}
\keyword{internal}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{rasterComp}
\alias{rasterComp}
\title{Comparative raster mapping}
\usage{
rasterComp(
  rast1 = NULL,
  rast2 = NULL,
  col1 = "#1b9e777F",
  col2 = "#7570b37F",
  rast1Name = "Set 1",
  rast2Name = "Set 2",
  land = NA,
  landCol = "black",
  title = "A Raster Comparison",
  graticule = TRUE,
  ...
)
}
\arguments{
\item{rast1}{A single \code{SpatRaster} showing the
distribution of the species corresponding to
\code{rast1Name}. Should have values of 0 (absence)
and 1 (presence). Can also be \code{NULL}.}

\item{rast2}{A single \code{SpatRaster} showing the
distribution of the species corresponding to
\code{rast2Name}. Should have values of 0 (absence)
and 1 (presence). Must match the extent and
resolution of \code{rast1}. Can also be \code{NULL.}}

\item{col1}{Color for \code{rast1} presences}

\item{col2}{Color for \code{rast2} presences}

\item{rast1Name}{An optional name for the first set
of occurrences, which will be color-coded to
\code{occs1Col} in the resulting plot.}

\item{rast2Name}{An optional name for the first set
of occurrences, which will be color-coded to
\code{occs2Col} in the resulting plot.}

\item{land}{An optional coastline polygon shapefile
of types \code{sf} or \code{SpatRaster} to provide geographic
context for the occurrence points.}

\item{landCol}{Color for land on map.}

\item{title}{A title for the plot.}

\item{graticule}{Do you want a grid of lon/lat lines?}

\item{...}{Additional optional arguments to pass to
\code{terra::plot()}.}
}
\value{
A plot of class \code{recordedplot} overlaying mapped,
semitransparent extents of the input rasters
}
\description{
A convenient wrapper around \code{terra::plot}
to generate formatted plots comparing two rasters.
This is used in the context of voluModel to
overlay semi-transparent distributions (coded as 1)
in two different \code{RasterLayers}.
}
\note{
The extents of \code{rast1} and \code{rast2}
must match.
}
\examples{
library(terra)
rast1 <- rast(ncol=10, nrow=10)
values(rast1) <- rep(0:1, 50)

rast2 <- rast(ncol=10, nrow=10)
values(rast2) <- c(rep(0, 50), rep(1,50))

rasterComp(rast1 = rast1, rast2 = rast2)

}
\seealso{
\code{\link[terra:plot]{plot}}
}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{pointCompMap}
\alias{pointCompMap}
\title{Comparative point mapping}
\usage{
pointCompMap(
  occs1,
  occs2,
  spName,
  land = NA,
  occs1Col = "#bd0026",
  occs2Col = "#fd8d3c",
  agreeCol = "black",
  occs1Name = "Set 1",
  occs2Name = "Set 2",
  landCol = "gray",
  waterCol = "steelblue",
  ptSize = 1,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{occs1}{A \code{data.frame} with at least two columns
named "longitude" and "latitude" or that
can be coerced into this format.}

\item{occs2}{A \code{data.frame} with at least two columns
named "longitude" and "latitude" or that
can be coerced into this format.}

\item{spName}{A character string with the species
name to be used in the plot title.}

\item{land}{An optional coastline polygon shapefile
of types \code{sf} or \code{SpatRaster} to provide geographic
context for the occurrence points.}

\item{occs1Col}{Color for occurrence points on map}

\item{occs2Col}{Color for occurrence points on map}

\item{agreeCol}{Color for occurrence points shared
between \code{occs1} and \code{occs2}.}

\item{occs1Name}{An optional name for the first set
of occurrences, which will be color-coded to
\code{occs1Col} in the resulting plot.}

\item{occs2Name}{An optional name for the first set
of occurrences, which will be color-coded to
\code{occs2Col} in the resulting plot.}

\item{landCol}{Color for land on map}

\item{waterCol}{Color for water on map}

\item{ptSize}{\code{numeric} value for \code{cex};
size of occurrence points on map.}

\item{verbose}{\code{logical}. Switching to \code{FALSE} mutes message describing
which columns in \code{occs1} and \code{occs2} are interpreted as x and y coordinates.}

\item{...}{Additional optional arguments to pass to
\code{ggplot} initial plot object.}
}
\value{
A \code{ggplot} plot object.
}
\description{
A convenient wrapper around \code{ggplot}
to generate formatted plots comparing two sets of
occurrence point plots.
}
\note{
The x and y column names of \code{occs1} and \code{occs2}
must match.
}
\examples{
set.seed(5)
occs <- data.frame(cbind(decimalLatitude = sample(seq(7,35), 24),
                         decimalLongitude = sample(seq(-97, -70), 24)))

set.seed(0)
occs1 <- occs[sample(1:nrow(occs),
                     size = 12, replace = FALSE),]
set.seed(10)
occs2 <- occs[sample(1:nrow(occs),
                     size = 12, replace = FALSE),]

pointCompMap(occs1 = occs1, occs2 = occs2,
             occs1Col = "red", occs2Col = "orange",
             agreeCol = "purple",
             occs1Name = "2D",
             occs2Name = "3D",
             waterCol = "steelblue",
             spName = "Steindachneria argentea",
             ptSize = 2,
             verbose = FALSE)

}
\seealso{
\code{\link[ggplot2:ggplot]{ggplot}}
}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/depthMatch.R
\name{depthMatch}
\alias{depthMatch}
\title{Match the depth values given in an occurrence dataset to the depth slice
values of a user provided spatRaster stack}
\usage{
depthMatch(occs, rasterTemplate)
}
\arguments{
\item{occs}{dataframe of occurrence records with colnames "longitude," "latitude,"
and "depth." Depth values should be positive.}

\item{rasterTemplate}{a spatRaster stack where the layer names correspond to the depth value,
as a positive number}
}
\value{
object of class dataframe, the same occs fed into the function but with the
depth values adjusted to match the spatRaster stack depth slices
}
\description{
assigns coordinates to depth slices given a user provided raster template.
if any points are deeper than the lowest depth slice, they are assigned to the
lowest depth slice.
}
\details{
rasterTemplate names and values in the depth column of occs should be positive.
Points lower than the deepest depth slice are assigned to the deepest depth slice
}
\examples{
library(terra)
# Create test raster brick
r1 <- rast(ncol = 100, nrow = 100)
r2 <- rast(ncol = 100, nrow = 100)
r3 <- rast(ncol = 100, nrow = 100)
rbrick <- c(r1, r2, r3)
names(rbrick) <- c(1, 40, 90)

# Create test occurrences
set.seed(0)
longitude <- sample(ext(r1)[1]:ext(r1)[2], size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(r1)[3]:ext(r1)[4], size = 10, replace = FALSE)
set.seed(0)
depth <- sample(1:100, size = 10, replace = TRUE)
occs <- data.frame(longitude, latitude, depth)

# Here's the function
result <- depthMatch(occs = occs, rasterTemplate = rbrick)

}
\keyword{cleaning}
\keyword{occurrence}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xyzSample.R
\name{xyzSample}
\alias{xyzSample}
\title{Sampling from a \code{SpatRaster} vector using 3D coordinates}
\usage{
xyzSample(occs, envBrick, verbose = TRUE)
}
\arguments{
\item{occs}{A \code{data.frame} with at least three columns
named "longitude", "latitude", and "depth", or that
can be coerced into this format.}

\item{envBrick}{A \code{SpatRaster} vector object with
one environmental variable. Each layer represents
a depth slice. See Details for more information.}

\item{verbose}{\code{logical}. Switching to \code{FALSE} mutes message
describing which columns in \code{occs1} and \code{occs2} are interpreted
as x, y, and z coordinates.}
}
\value{
Vector of environmental values equal in length
to number of rows of input \code{occs} \code{data.frame}.
}
\description{
Gets values at x,y,z occurrences from a
given 3D environmental variable brick
}
\details{
The \code{SpatRaster} vector object should
have numeric names that correspond with the beginning
depth of a particular depth slice. For example, one
might have three layers, one from 0 to 10m, one from
10 to 30m, and one from 30 to 100m. You would name the
layers in this brick \verb{names(envBrick) <- c(0, 10, 30}.
\code{xyzSample} identifies the layer name that is closest
to the depth layer value at a particular X, Y
coordinate, and samples the environmental value at that
3D coordinate.
}
\examples{
library(terra)

# Create test raster
r1 <- rast(ncol=10, nrow=10)
values(r1) <- 1:100
r2 <- rast(ncol=10, nrow=10)
values(r2) <- c(rep(20, times = 50), rep(60, times = 50))
r3 <- rast(ncol=10, nrow=10)
values(r3) <- 8
envBrick <- c(r1, r2, r3)
names(envBrick) <- c(0, 10, 30)

# Create test occurrences
set.seed(0)
longitude <- sample(ext(envBrick)[1]:ext(envBrick)[2],
                    size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(envBrick)[3]:ext(envBrick)[4],
                   size = 10, replace = FALSE)
set.seed(0)
depth <- sample(0:35, size = 10, replace = TRUE)
occurrences <- as.data.frame(cbind(longitude,latitude,depth))

# Test function
occSample3d <- xyzSample(occurrences, envBrick)

# How to use
occurrences$envtValue <- occSample3d

}
\keyword{dataPrep}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/threshold_3D.R
\name{threshold_3D}
\alias{threshold_3D}
\title{Test different threshold levels and produce presence/absence layers for
3D models}
\usage{
threshold_3D(
  predicted_layers,
  thresholding_vals,
  maxent_df,
  coord_df,
  weights = NULL
)
}
\arguments{
\item{predicted_layers}{A spatRaster stack of suitability layers where each layer
corresponds to a depth slice}

\item{thresholding_vals}{A vector of sensitivity levels for creating different
thresholds and presence/absence layers. If one wanted to test sensitivities of
90\%, and 95\%, this would be input as c(0.9, 0.95)}

\item{maxent_df}{'data.frame' where first column is a vector
of presences named "p" containing 1's and 0's. Each row represents a cell in the
spatRaster volume with an x, y, z coordinate, and 1's are presences while 0's are
absences, or background points. Other columns are environmental variable values
extracted at the occurrence and background points.}

\item{coord_df}{A dataframe containing the longitude, latitude, and depth for each
cell in maxent_df, named "longitude," "latitude," and "depth"}

\item{weights}{a numeric giving what the sensitivity should be downweighted by. If
no value is given, TSS will be calculated according to its original formula}
}
\value{
a 'list' with two components:

$threshold_layers, a spatRaster stack of presence absence rasters thresholded at the
sensitivity with the highest TSS

$tss_results, a 'data.frame' containing the specificity, TSS, and suitability score
for each sensitivity given in thresholding_vals
}
\description{
Creates 3D presence/absence layers by setting all values in a suitability
layer above a certain threshold to 1 and all values below that threshold to 0. The
threshold value is determined by the sensitivity given by the user, where the
sensitivity is the percentage of occurrences to be counted as present in the
resulting presence/absence layer. For example, with a sensitivity of 0.9 or 90\%,
The threshold is the suitability value where 90\% of the occurrence points fall on
grid cells with suitability scores above that value.

The function can try multiple different sensitivity levels, and for each will output
a presence/absence spatRaster stack, as well as the specificity (proportion of
psuedoabsences correctly considered absent), the true skill statistic (TSS), which is
a measure of how well the threshold balances commission and omission error, and the
suitability value of the threshold.

The user can also downweight sensitivity when calculating TSS with the "weights"
parameter. This may be important to do as 3D niche models often have a higher ratio
of psuedoabsences to occurrences than 2D models.
}
\examples{
library(terra)
library(dplyr)

# creating list of spatRaster stacks where each element is a depth slice
r1_d1 <- rast(ncol = 100, nrow = 100)
set.seed(0)
values(r1_d1) <- sample(c(1:100), size = 1000, replace = TRUE)
r2_d1 <- rast(ncol = 100, nrow = 100)
set.seed(0)
values(r2_d1) <- sample(c(1:1000), size = 1000, replace = TRUE)
r1_d2 <- r1_d1
values(r1_d2) <- values(r1_d1)+10
r2_d2 <- r2_d1
values(r2_d2) <- values(r2_d1)+10
d1 <- c(r1_d1, r2_d1)
names(d1) <- c("valsr1", "valsr2")
d2 <- c(r1_d2, r2_d2)
names(d2) <- c("valsr1", "valsr2")
envlist <- list(d1, d2)

# creating occs and bgs
set.seed(0)
occs <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 50, replace = FALSE)
bgs <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 500, replace = FALSE)

occs_d1 <- crds(envlist[[1]][[1]])[occs[1:25],]
occs_d2 <- crds(envlist[[2]][[1]])[occs[26:50],]
bg_d1 <- crds(envlist[[1]][[1]])[bgs[1:250],]
bg_d2 <- crds(envlist[[1]][[1]])[bgs[251:500],]

# extracting at occs and bgs
occ_valsr1_d1 <- extract(envlist[[1]][[1]], occs_d1)
occ_valsr1_d2 <- extract(envlist[[2]][[1]], occs_d2)
occ_valsr2_d1 <- extract(envlist[[1]][[2]], occs_d1)
occ_valsr2_d2 <- extract(envlist[[2]][[2]], occs_d2)

occ_valsr1 <- rbind(occ_valsr1_d1, occ_valsr1_d2)
occ_valsr2 <- rbind(occ_valsr2_d1, occ_valsr2_d2)

bg_valsr1_d1 <- extract(envlist[[1]][[1]], bg_d1)
bg_valsr1_d2 <- extract(envlist[[2]][[1]], bg_d2)
bg_valsr2_d1 <- extract(envlist[[1]][[2]], bg_d1)
bg_valsr2_d2 <- extract(envlist[[2]][[2]], bg_d2)

bg_valsr1 <- rbind(bg_valsr1_d1, bg_valsr1_d2)
bg_valsr2 <- rbind(bg_valsr2_d1, bg_valsr2_d2)

valsr1 <- rbind(occ_valsr1, bg_valsr1)
valsr2 <- rbind(occ_valsr2, bg_valsr2)

p1 <- rep(1, times = 50)
p0 <- rep(0, times = 500)
p <- c(p1, p0)

maxdf <- data.frame(p, valsr1, valsr2)

# creating coord_df
coord_df <- rbind(occs_d1, occs_d2, bg_d1, bg_d2)
z <- c(rep(1, times = 25), rep(2, times = 25), rep(1, times = 250),
rep(2, times = 250))
coord_df <- cbind(coord_df, z)
colnames(coord_df) <- c("longitude", "latitude", "depth")

# creating suitability rasters
suitd1 <- d1[[1]]
values(suitd1) <- runif(min = 0, max = 1, n = length(values(suitd1)))
suitd2 <- d2[[1]]
values(suitd2) <- runif(min = 0, max = 1, n = length(values(suitd2)))
suit <- c(suitd1, suitd2)

# here's the function
result <- threshold_3D(predicted_layers = suit, thresholding_vals = c(0.9, 0.95),
maxent_df = maxdf, coord_df = coord_df, weights = 2/3)

}
\references{
Allouche O, Tsoar A, and Kadmon R. 2006. Assessing the accuracy of species
distribution models: prevalence, kappa and the true skill statistic (TSS). \emph{Journal of
Applied Ecology}, 43, 1223-1232.
}
\keyword{threshold}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{blendColor}
\alias{blendColor}
\title{Blend Colors}
\usage{
blendColor(col1 = "#1b9e777F", col2 = "#7570b37F")
}
\arguments{
\item{col1}{Anything that can be interpreted by \code{rgb}
as a color.}

\item{col2}{Anything that can be interpreted by \code{rgb}
as a color.}
}
\value{
A \code{character} string with hex color, including
adjustment for transparency.
}
\description{
Generates a blended color from two transparent colors
}
\examples{

blendColor(col1 = "#1B9E777F", col2 = "#7570B37F")

}
\keyword{internal}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{oneRasterPlot}
\alias{oneRasterPlot}
\title{Single raster plot}
\usage{
oneRasterPlot(
  rast,
  land = NA,
  landCol = "black",
  scaleRange = NA,
  graticule = TRUE,
  title = "A Raster",
  verbose = TRUE,
  ...
)
}
\arguments{
\item{rast}{A single \code{SpatRaster} layer on a continuous
scale.}

\item{land}{An optional coastline polygon shapefile
of types \code{sf} or \code{SpatRaster} to provide geographic
context for the occurrence points.}

\item{landCol}{Color for land on map.}

\item{scaleRange}{Optional numeric vector containing
maximum and minimum values for color scale. Helpful
when making multiple plots for comparison. Defaults
to minimum and maximum of input \code{rast}.}

\item{graticule}{\code{logical}. Do you want a grid of lon/lat lines?}

\item{title}{A title for the plot.}

\item{verbose}{\code{logical}. Switching to \code{FALSE} mutes message alerting
user if input \code{rast} values exceed a specified \code{scaleRange}.}

\item{...}{Additional optional arguments to pass to
\code{plot} initial plot object or \code{viridis}.}
}
\value{
A plot of mapping the values of the input raster layer
}
\description{
A convenient wrapper around \code{ggplot}
to generate a formatted plot of a single raster.
}
\examples{
library(terra)
rast <- rast(ncol=10, nrow=10)
values(rast) <- seq(0,99, 1)

oneRasterPlot(rast = rast)

}
\seealso{
\code{\link[viridisLite:viridis]{viridis}} \code{\link[ggplot2:ggplot]{ggplot}}
}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputProcessingUtilties.R
\name{columnParse}
\alias{columnParse}
\title{Column Parsing}
\usage{
columnParse(occs, wDepth = FALSE)
}
\arguments{
\item{occs}{A \code{data.frame} with at least two columns
named "longitude" and "latitude" or that
can be coerced into this format.}

\item{wDepth}{Logical; flags whether a depth column should
also be sought.}
}
\value{
A \code{list} of length 2 with indices of the x and y
columns, respectively, followed by a message with a plain
text report of which columns were interpreted as x and y.
}
\description{
Parses column names from input occurrence
\code{data.frame} for more seamless function
}
\details{
This is an internal function to return the putative
indices for latitude and longitude or x and y coordinates
of occurrences to allow for code that is more robust to
very common user error
}
\examples{
library(terra)

# Create sample raster
r <- rast(ncol=10, nrow=10)
values(r) <- 1:100

# Create test occurrences
set.seed(0)
longitude <- sample(ext(r)[1]:ext(r)[2],
                    size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(r)[3]:ext(r)[4],
                   size = 10, replace = FALSE)
set.seed(0)
depth <- sample(0:35, size = 10, replace = TRUE)
occurrences <- as.data.frame(cbind(longitude,latitude,depth))

# Here's the function
result <- columnParse(occs = occurrences[,1:2],
                      wDepth = FALSE)
result <- columnParse(occs = occurrences,
                      wDepth = TRUE)

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{transpColor}
\alias{transpColor}
\title{Transparent Color}
\usage{
transpColor(color, percent = 50)
}
\arguments{
\item{color}{Anything that can be interpreted by \code{rgb}
as a color.}

\item{percent}{A whole number between 0 and 100 specifying
how transparent the color should be.}
}
\value{
A \code{character} string with hex color, including
adjustment for transparency.
}
\description{
Generates transparent colors
}
\examples{

transpColor(color = "red", percent = 50)

}
\keyword{internal}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputProcessingUtilties.R
\name{centerPointRasterTemplate}
\alias{centerPointRasterTemplate}
\title{Center Point Raster Template}
\usage{
centerPointRasterTemplate(rawPointData)
}
\arguments{
\item{rawPointData}{A \code{SpatVector} object with points
that will represent the center of each cell in the output
template.}
}
\value{
An empty \code{SpatRaster} designed to serve as a template for
rasterizing \code{SpatVector} objects.
}
\description{
Creates a \code{SpatRaster} template from a
\code{SpatVector} point object in which the raster cells
are centered on the vector points.
}
\details{
\code{rawPointData} is a \code{SpatVector} object that
contains x and y coordinates.
}
\examples{

library(terra)

# Create point grid
coords <- data.frame(x = rep(seq(1:5), times = 5),
                    y = unlist(lapply(1:5, FUN = function(x) {
                      rep(x, times = 5)})))

# Create data and add NAs to simulate uneven bottom depths
dd <- data.frame(SURFACE = 1:25,
                d5M = 6:30,
                d10M = 11:35,
                d25M = 16:40)
dd$d25M[c(1:5, 18:25)] <- NA
dd$d10M[c(3:5, 21:23)] <- NA
dd$d5M[c(4, 22)] <- NA

dd[,c("x","y")] <- coords

# Create SpatialPointsDataFrame
sp <- vect(dd, geom = c("x", "y"))

# Here's the function
template <- centerPointRasterTemplate(rawPointData = sp)
class(template)

}
\seealso{
\code{\link[terra:rasterize]{rasterize}}
}
\keyword{inputProcessing}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{transectPlot}
\alias{transectPlot}
\title{Plot vertical sample}
\usage{
transectPlot(
  rast = NULL,
  sampleAxis = "lon",
  axisValue = NA,
  scaleRange = NA,
  plotLegend = TRUE,
  depthLim = as.numeric(max(names(rast))),
  transRange = c(-90, 90),
  transTicks = 20,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{rast}{A multilayer \code{SpatRaster} object, with names
corresponding to the z coordinate represented by the layer.
These names must be interpretable by \code{as.numeric}.}

\item{sampleAxis}{Specifies whether a latitudinal ("lat") or
longitudinal ("long") transect is desired.}

\item{axisValue}{Numeric value specifying transect postion.}

\item{scaleRange}{A numeric vector of length 2, specifying
the range that should be used for the plot color scale.}

\item{plotLegend}{\code{logical}, controls whether legend is plotted.}

\item{depthLim}{A single vector of class \code{numeric}. How deep
should the plot go?}

\item{transRange}{A \code{numeric} vector of lenghth 2. How far
along the transect should be plotted?}

\item{transTicks}{\code{numeric}, spacing between breaks on x axis.}

\item{verbose}{\code{logical}. Switching to \code{FALSE} mutes message
alerting user if input \code{rast} values exceed specified \code{scaleRange}.}

\item{...}{Additional optional arguments to pass to \code{viridis}.}
}
\value{
A \code{ggplot} showing a vertical slice through the \code{SpatRaster}.
}
\description{
Plots cell values along a vertical transect
}
\note{
Only unprojected \code{SpatRaster} files are supported.
}
\examples{
library(terra)

rast1 <- rast(ncol=10, nrow=10)
values(rast1) <- rep(0:3, 50)

rast2 <- rast(ncol=10, nrow=10)
values(rast2) <- c(rep(0, 50), rep(1,25), rep(2,25))

rast3 <- rast(ncol=10, nrow=10)
values(rast3) <- rep(c(1,3,2,1), 25)

distBrick <- c(rast1, rast2, rast3)
names(distBrick) <- c(0:2)

transectPlot(distBrick, depthLim = 3)


}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{verticalSample}
\alias{verticalSample}
\title{Vertical sample}
\usage{
verticalSample(x = NULL, sampleAxis = "lon", axisValue = NA)
}
\arguments{
\item{x}{A multilayer \code{SpatRaster} object}

\item{sampleAxis}{Specifies whether a latitudinal ("lat") or
longitudinal ("long") transect is desired.}

\item{axisValue}{Numeric value specifying transect postion.}
}
\value{
A \code{data.frame} with values sampled across vertical
transect.
}
\description{
Samples data along a vertical transect
}
\note{
Only unprojected \code{SpatRaster} files are supported.
}
\keyword{internal}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/marineBackground.R
\name{marineBackground}
\alias{marineBackground}
\title{Marine background shapefile generation}
\usage{
marineBackground(occs, clipToOcean = TRUE, verbose = TRUE, ...)
}
\arguments{
\item{occs}{A \code{data.frame} with at least two columns
named "longitude" and "latitude" or that
can be coerced into this format.}

\item{clipToOcean}{\code{logical}. Clips shapefile to oceans where species
occurs. Useful in cases where buffers jump over narrow
peninsulas (e.g. Isthmus of Panama). Can be quite artificial at ocean
boundaries.}

\item{verbose}{\code{logical}. Switching to \code{FALSE} mutes message describing
which columns in \code{occs} are interpreted as x and y coordinates.}

\item{...}{Additional optional arguments to pass to
\code{getDynamicAlphaHull}.}
}
\value{
A \code{SpatVector}
}
\description{
Automatically generates background
shapefiles for sampling pseudoabsences and/or background
points for niche modeling or species distribution modeling.
Delineating background sampling regions can be one of the
trickiest parts of generating a good model. Automatically
generated background shapefiles should be inspected
carefully prior to model use.

Useful references, among others:
\itemize{
\item Barve N, Barve V, Jiménez-Valverde A, Lira-Noriega A,
Maher SP, Peterson AT, Soberón J, Villalobos F. 2011. The
crucial role of the accessible area in ecological niche
modeling and species distribution modeling.
\emph{Ecological modelling} 222:1810-9.
\item Merow, C, Smith MJ, Silander JA. 2013. A practical
guide to MaxEnt for modeling species’ distributions: what
it does, and why inputs and settings matter." \emph{Ecography}
36: 1058-69.
\item Murphy SJ. 2021. Sampling units derived from geopolitical
boundaries bias biodiversity analyses. \emph{Global Ecology
and Biogeography} 30: 1876-88.
}
}
\details{
The meat of this function is a special-case wrapper
around \code{getDynamicAlphaHull()} from the \code{rangeBuilder} package.
The function documented here is especially useful in cases where
one wants to automatically generate training regions that overlap
the international date line. Regions that exceed the line are cut
and pasted into the appropriate hemisphere instead of being
deleted.

If the argument \code{buff} is not supplied, a buffer is
calculated by taking the mean between the 10th and 90th percentile
of horizontal distances between occurrence points.

If \code{getDynamicAlphaHull()} cannot satisfy the provided conditions,
the occurrences are buffered and then a minimum convex hull is
drawn around the buffer polygons.
}
\examples{
\donttest{
library(terra)
# Create sample raster
r <- rast(ncol=10, nrow=10)
values(r) <- 1:100

# Create test occurrences
set.seed(0)
longitude <- sample(-50:50,
                    size = 20, replace = FALSE)
set.seed(0)
latitude <- sample(-30:30,
                   size = 20, replace = FALSE)
occurrences <- as.data.frame(cbind(longitude,latitude))

# Here's the function
result <- marineBackground(occs = occurrences, buff = 100000,
                           fraction = .9, partCount = 2, clipToOcean = FALSE)
}

}
\seealso{
\code{\link[rangeBuilder:getDynamicAlphaHull]{getDynamicAlphaHull}}
}
\keyword{backgroundSampling}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/partition_3D.R
\name{partition_3D}
\alias{partition_3D}
\title{Create 3D partitions}
\usage{
partition_3D(
  maxent_df,
  coord_df,
  which_partition = "k.fold",
  kfolds = NULL,
  orientation = "lon_lat",
  return_format = "vector",
  ensure_all_folds = TRUE,
  max_attempts = 100,
  na_strategy = "NA"
)
}
\arguments{
\item{maxent_df}{Data frame with column 'p' (1=presence, 0=absence).}

\item{coord_df}{Data frame with 'longitude','latitude','depth' aligned with maxent_df.}

\item{which_partition}{"k.fold" (default) or "block".}

\item{kfolds}{Integer >= 2 for k.fold.}

\item{orientation}{For "block": "lon_lat" (default) or "lat_lon".}

\item{return_format}{"vector" (default) or "list".}

\item{ensure_all_folds}{(k.fold) Ensure all folds appear among presences with known depth (default TRUE).}

\item{max_attempts}{(k.fold) Retry cap when ensure_all_folds=TRUE (default 100).}

\item{na_strategy}{(k.fold) How to handle NA depth rows: "NA" (default) leaves them NA; "random" assigns a random fold.}
}
\value{
If return_format="vector": integer vector (1..k or 1..4), length == nrow(maxent_df).
Unassignable rows get NA. If "list": list(occ_partitions, bg_partitions).
}
\description{
Creates partition schemes in 3D for model training and testing.
Can create block or kfold partitions, with output as a \code{vector} or \code{list}
}
\details{
maxent_df and coord_df should be the same dataframes to be provided
to \code{maxent_3D()} for model production.
The spatial block partition, similarly to a traditional 2D partition,
will separate the occurrences into 4 groups of equal (or about equal) size,
with two groups making up two blocks on the upper half of the depth distribution
and two groups on the lower half. Background points are assigned to spatial groups
according to how they fall into the spatial partitions delimited by the occurrences.
Note that this means the number of background points in each partition may not be as
close to equal as the occurrences.
}
\examples{
\donttest{
# create test dataframe
occ <- rep(1, times = 10)
bg <- rep(0, times = 1000)
env1 <- sample(c(1:100), size = 1010, replace = TRUE)
env2 <- sample(c(1:1000), size = 1010, replace = TRUE)
p <- c(occ, bg)
testdf <- data.frame(p, env1, env2)

# create test coord data
r <- terra::rast(ncol = 100, nrow = 100)
set.seed(0)
longitude <- sample(terra::ext(r)[1]:terra::ext(r)[2], size = 1010, replace = TRUE)
set.seed(0)
latitude <- sample(terra::ext(r)[3]:terra::ext(r)[4], size = 1010, replace = TRUE)
depth <- sample(c(0, 5, 10, 15, 20, 25, 30, 35, 40, 45), size = 1010, replace = TRUE)
test_coords <- data.frame(longitude, latitude, depth)

# Here's the function
result_kfold <- partition_3D(maxent_df = testdf, coord_df = test_coords,
which_partition = 'k.fold', kfolds = 3)

result_block <- partition_3D(maxent_df = testdf, coord_df = test_coords,
which_partition = 'block', orientation = 'lat_lon')
}

}
\keyword{partition}
\keyword{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/maxent_3D.R
\name{maxent_3D}
\alias{maxent_3D}
\title{Create 3D Ecological Niche Models with Maxent}
\usage{
maxent_3D(
  maxent_df,
  wanted_fc,
  wanted_rm,
  wanted_partition = NULL,
  projection_layers,
  occs,
  depth_list
)
}
\arguments{
\item{maxent_df}{'data.frame' where the first column is a vector
of presences named "p" containing 1's and 0's. Each row represents a cell in the
spatRaster volume with an x, y, z coordinate, and 1's are presences while 0's are
absences, or background points. Other columns are environmental variable values
extracted at the occurrence and background points, and should have the same names
as the names of the environmental layers in the projection_layers list of SpatRaster
stacks.}

\item{wanted_fc}{a character vector giving what feature class
combinations should be tried. "L" refers to linear, "Q" refers to quadratic, "H"
refers to hinge, and "P" refers to product. Should be in the format c("L", "Q", "LQ")
etc.}

\item{wanted_rm}{regularization multipliers to be tried in format c(1:4)}

\item{wanted_partition}{optional, should be the output of 'partition_3D'. if no
partition is supplied, all points will be used for training}

\item{projection_layers}{\code{list} of SpatRaster stacks by depth for predictions,
must all be cropped to the depth slice with the largest extent and masked
to the accessible area of the matching depth slice. Each element is a depth slice,
and each stack should contain all of the environmental variables used in the model
with the same names as that used in the model.}

\item{occs}{\code{data.frame} of longitude, latitude, and depth occurrences with columns
named "longitude", "latitude", and "depth".}

\item{depth_list}{vector of depths corresponding to depth slices of list elements of
projection_layers. Should be positive and go from shallowest depth to deepest depth}
}
\value{
An object of class \code{list} with four components:

$models, a list containing each model object produced.

$results, a \code{data.frame} where each row is a model corresponding to the list element
in $models and $predictions. If there was no partition supplied for training and
testing, each column will report the feature class, regularization multiplier, AUC,
total coefficients, nonzero coefficients, AICc, and delta AICc. if a partition is
used, it will report the average of these statistics across all partitions.

$predictions, a \code{list} of spatRaster stacks where each list element is a model
corresponding to the rows of $results and elements of $models projected onto the
supplied projection_layers. Each layer in the stack is a depth slice.

$partition_results, a \code{list} object of the same length as the number of partition
groups containing a \code{data.frame} of results for each model for each partition. Only
produced if a partition is supplied.
}
\description{
Uses MaxEnt from \code{predicts} package to test multiple models with
different feature class combinations, regularization multipliers, and a user
supplied partitioning scheme for training and testing. The function outputs
model objects, model results, as well as prediction SpatRasters of each model
projected back onto geographic space using the supplied environmental SpatRaster
stacks. Note: you will need to install \code{rJava} to successfully run this function.
}
\details{
The names of the projection_layers should be the same as the column names
of the environmental variables in maxent_df. The number of models output will be the
number of feature classes multiplied by the number of regularization multipliers.
For example, a wanted_fc of c("L", "Q", "P") and a wanted_rm of c(1:3) will output 9
total models.
}
\examples{

library(dplyr)
library(predicts)
library(terra)

# creating list of spatraster stacks where each element is a depth slice
r1_d1 <- rast(ncol = 100, nrow = 100)
set.seed(0)
values(r1_d1) <- as.numeric(sample(c(1:100), size = 1000, replace = TRUE))
r2_d1 <- rast(ncol = 100, nrow = 100)
set.seed(0)
values(r2_d1) <- as.numeric(sample(c(1:1000), size = 1000, replace = FALSE))
r1_d2 <- r1_d1
values(r1_d2) <- as.numeric(values(r1_d1)+10)
r2_d2 <- r2_d1
values(r2_d2) <- as.numeric(values(r2_d1)+10)
d1 <- c(r1_d1, r2_d1)
names(d1) <- c("valsr1", "valsr2")
d2 <- c(r1_d2, r2_d2)
names(d2) <- c("valsr1", "valsr2")
envlist <- list(d1, d2)

# creating occs and bgs
set.seed(0)
occs <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 50, replace = FALSE)
bgs <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 500, replace = FALSE)

occ_indices <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 50, replace = FALSE)
bg_indices  <- sample(c(1:nrow(crds(envlist[[1]][[1]]))), size = 500, replace = FALSE)

occs_d1 <- crds(envlist[[1]][[1]])[occ_indices[1:25],]
occs_d2 <- crds(envlist[[2]][[1]])[occ_indices[26:50],]
bg_d1   <- crds(envlist[[1]][[1]])[bg_indices[1:250],]
bg_d2   <- crds(envlist[[2]][[1]])[bg_indices[251:500],]

# extracting at occs and bgs
occ_valsr1_d1 <- extract(envlist[[1]][[1]], occs_d1)
occ_valsr1_d2 <- extract(envlist[[2]][[1]], occs_d2)
occ_valsr2_d1 <- extract(envlist[[1]][[2]], occs_d1)
occ_valsr2_d2 <- extract(envlist[[2]][[2]], occs_d2)

occ_valsr1 <- rbind(occ_valsr1_d1, occ_valsr1_d2)
occ_valsr2 <- rbind(occ_valsr2_d1, occ_valsr2_d2)

bg_valsr1_d1 <- extract(envlist[[1]][[1]], bg_d1)
bg_valsr1_d2 <- extract(envlist[[2]][[1]], bg_d2)
bg_valsr2_d1 <- extract(envlist[[1]][[2]], bg_d1)
bg_valsr2_d2 <- extract(envlist[[2]][[2]], bg_d2)

bg_valsr1 <- rbind(bg_valsr1_d1, bg_valsr1_d2)
bg_valsr2 <- rbind(bg_valsr2_d1, bg_valsr2_d2)

valsr1 <- rbind(occ_valsr1, bg_valsr1)
valsr2 <- rbind(occ_valsr2, bg_valsr2)

p1 <- rep(1, times = 50)
p0 <- rep(0, times = 500)
p <- c(p1, p0)

maxdf <- data.frame(p, valsr1, valsr2)

coords <- rbind(occs_d1, occs_d2)
colnames(coords) <- c("longitude", "latitude")
depth_vector     <- c(rep(1, times = 25), rep(2, times = 25))

# Use data.frame instead of cbind so $depth is completely valid inside maxent_3D
occs_dataframe <- data.frame(coords, depth = depth_vector)

\donttest{
# Pass the clean data.frame to the function
if(requireNamespace("rJava", quietly = TRUE)){
  result <- maxent_3D(maxent_df = maxdf, wanted_fc = c("L", "Q"),
                      wanted_rm = c(1:2), projection_layers = envlist,
                      occs = occs_dataframe, depth_list = c(1,2))
}
}

}
\keyword{MaxEnt}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{plotLayers}
\alias{plotLayers}
\title{Plotting 3D model in 2D}
\usage{
plotLayers(
  rast,
  land = NA,
  landCol = "black",
  title = NULL,
  graticule = TRUE,
  ...
)
}
\arguments{
\item{rast}{A \code{SpatRaster} vector with the 3D presence/absence
distribution of a species (interpreted as 1 = presence,
0 = absence).}

\item{land}{An optional coastline polygon shapefile
of types \code{sf} or \code{SpatRaster} to provide geographic
context for the occurrence points.}

\item{landCol}{Color for land on map.}

\item{title}{A title for the plot. If not title is
supplied, the title "Suitability from (MINIMUM
DEPTH) to (MAXIMUM DEPTH)" is inferred from
names of \code{rast}.}

\item{graticule}{Do you want a grid of lon/lat lines?}

\item{...}{Additional optional arguments.}
}
\value{
A plot of class \code{recordedplot}
}
\description{
This script plots a semitransparent layer
of suitable habitat for each depth layer. The redder
the color, the shallower the layer, the bluer, the
deeper. The more saturated the color, the more layers
with suitable habitat.
}
\note{
Only include the depth layers that you actually
want to plot.
}
\examples{
library(terra)

rast1 <- rast(ncol=10, nrow=10)
values(rast1) <- rep(0:1, 50)

rast2 <- rast(ncol=10, nrow=10)
values(rast2) <- c(rep(0, 50), rep(1,50))

rast3 <- rast(ncol=10, nrow=10)
values(rast3) <- rep(c(1,0,0,1), 25)

distBrick <- c(rast1, rast2, rast3)

plotLayers(distBrick)

}
\seealso{
\code{\link[viridisLite:viridis]{viridis}}
}
\keyword{plotting}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputProcessingUtilties.R
\name{downsample}
\alias{downsample}
\title{Occurrence downsampling}
\usage{
downsample(occs, rasterTemplate, verbose = TRUE)
}
\arguments{
\item{occs}{A \code{data.frame} with at least two columns
named "longitude" and "latitude" or that
can be coerced into this format.}

\item{rasterTemplate}{A \code{SpatRaster} object to serve
as a template for the resolution at which \code{occs} should be
downsampled.}

\item{verbose}{\code{logical}. Switching to \code{FALSE} mutes message describing
which columns in \code{occs} are interpreted as x and y coordinates.}
}
\value{
A \code{data.frame} with two columns named "longitude"
and "latitude" or with names that were used when coercing
input data into this format.
}
\description{
Reduces number of occurrences to resolution of input raster
}
\examples{
library(terra)
# Create sample raster
r <- rast(ncol=10, nrow=10)
values(r) <- 1:100

# Create test occurrences
set.seed(0)
longitude <- sample(ext(r)[1]:ext(r)[2],
                    size = 10, replace = FALSE)
set.seed(0)
latitude <- sample(ext(r)[3]:ext(r)[4],
                   size = 10, replace = FALSE)
occurrences <- as.data.frame(cbind(longitude,latitude))

# Here's the function
result <- downsample(occs = occurrences, rasterTemplate = r)

}
\keyword{inputProcessing}
